/*!
 * jQuery JavaScript Library v1.3.2
 * http://jquery.com/
 *
 * Copyright (c) 2009 John Resig
 * Dual licensed under the MIT and GPL licenses.
 * http://docs.jquery.com/License
 *
 * Date: 2009-02-19 17:34:21 -0500 (Thu, 19 Feb 2009)
 * Revision: 6246
 */
(function(){

var 
	// Will speed up references to window, and allows munging its name.
	window = this,
	// Will speed up references to undefined, and allows munging its name.
	undefined,
	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,
	// Map over the $ in case of overwrite
	_$ = window.$,

	jQuery = window.jQuery = window.$ = function( selector, context ) {
		// The jQuery object is actually just the init constructor 'enhanced'
		return new jQuery.fn.init( selector, context );
	},

	// A simple way to check for HTML strings or ID strings
	// (both of which we optimize for)
	quickExpr = /^[^<]*(<(.|\s)+>)[^>]*$|^#([\w-]+)$/,
	// Is it a simple selector
	isSimple = /^.[^:#\[\.,]*$/;

jQuery.fn = jQuery.prototype = {
	init: function( selector, context ) {
		// Make sure that a selection was provided
		selector = selector || document;

		// Handle $(DOMElement)
		if ( selector.nodeType ) {
			this[0] = selector;
			this.length = 1;
			this.context = selector;
			return this;
		}
		// Handle HTML strings
		if ( typeof selector === "string" ) {
			// Are we dealing with HTML string or an ID?
			var match = quickExpr.exec( selector );

			// Verify a match, and that no context was specified for #id
			if ( match && (match[1] || !context) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[1] )
					selector = jQuery.clean( [ match[1] ], context );

				// HANDLE: $("#id")
				else {
					var elem = document.getElementById( match[3] );

					// Handle the case where IE and Opera return items
					// by name instead of ID
					if ( elem && elem.id != match[3] )
						return jQuery().find( selector );

					// Otherwise, we inject the element directly into the jQuery object
					var ret = jQuery( elem || [] );
					ret.context = document;
					ret.selector = selector;
					return ret;
				}

			// HANDLE: $(expr, [context])
			// (which is just equivalent to: $(content).find(expr)
			} else
				return jQuery( context ).find( selector );

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) )
			return jQuery( document ).ready( selector );

		// Make sure that old selector state is passed along
		if ( selector.selector && selector.context ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return this.setArray(jQuery.isArray( selector ) ?
			selector :
			jQuery.makeArray(selector));
	},

	// Start with an empty selector
	selector: "",

	// The current version of jQuery being used
	jquery: "1.3.2",

	// The number of elements contained in the matched element set
	size: function() {
		return this.length;
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num === undefined ?

			// Return a 'clean' array
			Array.prototype.slice.call( this ) :

			// Return just the object
			this[ num ];
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems, name, selector ) {
		// Build a new jQuery matched element set
		var ret = jQuery( elems );

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;

		ret.context = this.context;

		if ( name === "find" )
			ret.selector = this.selector + (this.selector ? " " : "") + selector;
		else if ( name )
			ret.selector = this.selector + "." + name + "(" + selector + ")";

		// Return the newly-formed element set
		return ret;
	},

	// Force the current matched set of elements to become
	// the specified array of elements (destroying the stack in the process)
	// You should use pushStack() in order to do this, but maintain the stack
	setArray: function( elems ) {
		// Resetting the length to 0, then using the native Array push
		// is a super-fast way to populate an object with array-like properties
		this.length = 0;
		Array.prototype.push.apply( this, elems );

		return this;
	},

	// Execute a callback for every element in the matched set.
	// (You can seed the arguments with an array of args, but this is
	// only used internally.)
	each: function( callback, args ) {
		return jQuery.each( this, callback, args );
	},

	// Determine the position of an element within
	// the matched set of elements
	index: function( elem ) {
		// Locate the position of the desired element
		return jQuery.inArray(
			// If it receives a jQuery object, the first element is used
			elem && elem.jquery ? elem[0] : elem
		, this );
	},

	attr: function( name, value, type ) {
		var options = name;

		// Look for the case where we're accessing a style value
		if ( typeof name === "string" )
			if ( value === undefined )
				return this[0] && jQuery[ type || "attr" ]( this[0], name );

			else {
				options = {};
				options[ name ] = value;
			}

		// Check to see if we're setting style values
		return this.each(function(i){
			// Set all the styles
			for ( name in options )
				jQuery.attr(
					type ?
						this.style :
						this,
					name, jQuery.prop( this, options[ name ], type, i, name )
				);
		});
	},

	css: function( key, value ) {
		// ignore negative width and height values
		if ( (key == 'width' || key == 'height') && parseFloat(value) < 0 )
			value = undefined;
		return this.attr( key, value, "curCSS" );
	},

	text: function( text ) {
		if ( typeof text !== "object" && text != null )
			return this.empty().append( (this[0] && this[0].ownerDocument || document).createTextNode( text ) );

		var ret = "";

		jQuery.each( text || this, function(){
			jQuery.each( this.childNodes, function(){
				if ( this.nodeType != 8 )
					ret += this.nodeType != 1 ?
						this.nodeValue :
						jQuery.fn.text( [ this ] );
			});
		});

		return ret;
	},

	wrapAll: function( html ) {
		if ( this[0] ) {
			// The elements to wrap the target around
			var wrap = jQuery( html, this[0].ownerDocument ).clone();

			if ( this[0].parentNode )
				wrap.insertBefore( this[0] );

			wrap.map(function(){
				var elem = this;

				while ( elem.firstChild )
					elem = elem.firstChild;

				return elem;
			}).append(this);
		}

		return this;
	},

	wrapInner: function( html ) {
		return this.each(function(){
			jQuery( this ).contents().wrapAll( html );
		});
	},

	wrap: function( html ) {
		return this.each(function(){
			jQuery( this ).wrapAll( html );
		});
	},

	append: function() {
		return this.domManip(arguments, true, function(elem){
			if (this.nodeType == 1)
				this.appendChild( elem );
		});
	},

	prepend: function() {
		return this.domManip(arguments, true, function(elem){
			if (this.nodeType == 1)
				this.insertBefore( elem, this.firstChild );
		});
	},

	before: function() {
		return this.domManip(arguments, false, function(elem){
			this.parentNode.insertBefore( elem, this );
		});
	},

	after: function() {
		return this.domManip(arguments, false, function(elem){
			this.parentNode.insertBefore( elem, this.nextSibling );
		});
	},

	end: function() {
		return this.prevObject || jQuery( [] );
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: [].push,
	sort: [].sort,
	splice: [].splice,

	find: function( selector ) {
		if ( this.length === 1 ) {
			var ret = this.pushStack( [], "find", selector );
			ret.length = 0;
			jQuery.find( selector, this[0], ret );
			return ret;
		} else {
			return this.pushStack( jQuery.unique(jQuery.map(this, function(elem){
				return jQuery.find( selector, elem );
			})), "find", selector );
		}
	},

	clone: function( events ) {
		// Do the clone
		var ret = this.map(function(){
			if ( !jQuery.support.noCloneEvent && !jQuery.isXMLDoc(this) ) {
				// IE copies events bound via attachEvent when
				// using cloneNode. Calling detachEvent on the
				// clone will also remove the events from the orignal
				// In order to get around this, we use innerHTML.
				// Unfortunately, this means some modifications to
				// attributes in IE that are actually only stored
				// as properties will not be copied (such as the
				// the name attribute on an input).
				var html = this.outerHTML;
				if ( !html ) {
					var div = this.ownerDocument.createElement("div");
					div.appendChild( this.cloneNode(true) );
					html = div.innerHTML;
				}

				return jQuery.clean([html.replace(/ jQuery\d+="(?:\d+|null)"/g, "").replace(/^\s*/, "")])[0];
			} else
				return this.cloneNode(true);
		});

		// Copy the events from the original to the clone
		if ( events === true ) {
			var orig = this.find("*").andSelf(), i = 0;

			ret.find("*").andSelf().each(function(){
				if ( this.nodeName !== orig[i].nodeName )
					return;

				var events = jQuery.data( orig[i], "events" );

				for ( var type in events ) {
					for ( var handler in events[ type ] ) {
						jQuery.event.add( this, type, events[ type ][ handler ], events[ type ][ handler ].data );
					}
				}

				i++;
			});
		}

		// Return the cloned set
		return ret;
	},

	filter: function( selector ) {
		return this.pushStack(
			jQuery.isFunction( selector ) &&
			jQuery.grep(this, function(elem, i){
				return selector.call( elem, i );
			}) ||

			jQuery.multiFilter( selector, jQuery.grep(this, function(elem){
				return elem.nodeType === 1;
			}) ), "filter", selector );
	},

	closest: function( selector ) {
		var pos = jQuery.expr.match.POS.test( selector ) ? jQuery(selector) : null,
			closer = 0;

		return this.map(function(){
			var cur = this;
			while ( cur && cur.ownerDocument ) {
				if ( pos ? pos.index(cur) > -1 : jQuery(cur).is(selector) ) {
					jQuery.data(cur, "closest", closer);
					return cur;
				}
				cur = cur.parentNode;
				closer++;
			}
		});
	},

	not: function( selector ) {
		if ( typeof selector === "string" )
			// test special case where just one selector is passed in
			if ( isSimple.test( selector ) )
				return this.pushStack( jQuery.multiFilter( selector, this, true ), "not", selector );
			else
				selector = jQuery.multiFilter( selector, this );

		var isArrayLike = selector.length && selector[selector.length - 1] !== undefined && !selector.nodeType;
		return this.filter(function() {
			return isArrayLike ? jQuery.inArray( this, selector ) < 0 : this != selector;
		});
	},

	add: function( selector ) {
		return this.pushStack( jQuery.unique( jQuery.merge(
			this.get(),
			typeof selector === "string" ?
				jQuery( selector ) :
				jQuery.makeArray( selector )
		)));
	},

	is: function( selector ) {
		return !!selector && jQuery.multiFilter( selector, this ).length > 0;
	},

	hasClass: function( selector ) {
		return !!selector && this.is( "." + selector );
	},

	val: function( value ) {
		if ( value === undefined ) {			
			var elem = this[0];

			if ( elem ) {
				if( jQuery.nodeName( elem, 'option' ) )
					return (elem.attributes.value || {}).specified ? elem.value : elem.text;
				
				// We need to handle select boxes special
				if ( jQuery.nodeName( elem, "select" ) ) {
					var index = elem.selectedIndex,
						values = [],
						options = elem.options,
						one = elem.type == "select-one";

					// Nothing was selected
					if ( index < 0 )
						return null;

					// Loop through all the selected options
					for ( var i = one ? index : 0, max = one ? index + 1 : options.length; i < max; i++ ) {
						var option = options[ i ];

						if ( option.selected ) {
							// Get the specifc value for the option
							value = jQuery(option).val();

							// We don't need an array for one selects
							if ( one )
								return value;

							// Multi-Selects return an array
							values.push( value );
						}
					}

					return values;				
				}

				// Everything else, we just grab the value
				return (elem.value || "").replace(/\r/g, "");

			}

			return undefined;
		}

		if ( typeof value === "number" )
			value += '';

		return this.each(function(){
			if ( this.nodeType != 1 )
				return;

			if ( jQuery.isArray(value) && /radio|checkbox/.test( this.type ) )
				this.checked = (jQuery.inArray(this.value, value) >= 0 ||
					jQuery.inArray(this.name, value) >= 0);

			else if ( jQuery.nodeName( this, "select" ) ) {
				var values = jQuery.makeArray(value);

				jQuery( "option", this ).each(function(){
					this.selected = (jQuery.inArray( this.value, values ) >= 0 ||
						jQuery.inArray( this.text, values ) >= 0);
				});

				if ( !values.length )
					this.selectedIndex = -1;

			} else
				this.value = value;
		});
	},

	html: function( value ) {
		return value === undefined ?
			(this[0] ?
				this[0].innerHTML.replace(/ jQuery\d+="(?:\d+|null)"/g, "") :
				null) :
			this.empty().append( value );
	},

	replaceWith: function( value ) {
		return this.after( value ).remove();
	},

	eq: function( i ) {
		return this.slice( i, +i + 1 );
	},

	slice: function() {
		return this.pushStack( Array.prototype.slice.apply( this, arguments ),
			"slice", Array.prototype.slice.call(arguments).join(",") );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map(this, function(elem, i){
			return callback.call( elem, i, elem );
		}));
	},

	andSelf: function() {
		return this.add( this.prevObject );
	},

	domManip: function( args, table, callback ) {
		if ( this[0] ) {
			var fragment = (this[0].ownerDocument || this[0]).createDocumentFragment(),
				scripts = jQuery.clean( args, (this[0].ownerDocument || this[0]), fragment ),
				first = fragment.firstChild;

			if ( first )
				for ( var i = 0, l = this.length; i < l; i++ )
					callback.call( root(this[i], first), this.length > 1 || i > 0 ?
							fragment.cloneNode(true) : fragment );
		
			if ( scripts )
				jQuery.each( scripts, evalScript );
		}

		return this;
		
		function root( elem, cur ) {
			return table && jQuery.nodeName(elem, "table") && jQuery.nodeName(cur, "tr") ?
				(elem.getElementsByTagName("tbody")[0] ||
				elem.appendChild(elem.ownerDocument.createElement("tbody"))) :
				elem;
		}
	}
};

// Give the init function the jQuery prototype for later instantiation
jQuery.fn.init.prototype = jQuery.fn;

function evalScript( i, elem ) {
	if ( elem.src )
		jQuery.ajax({
			url: elem.src,
			async: false,
			dataType: "script"
		});

	else
		jQuery.globalEval( elem.text || elem.textContent || elem.innerHTML || "" );

	if ( elem.parentNode )
		elem.parentNode.removeChild( elem );
}

function now(){
	return +new Date;
}

jQuery.extend = jQuery.fn.extend = function() {
	// copy reference to target object
	var target = arguments[0] || {}, i = 1, length = arguments.length, deep = false, options;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;
		target = arguments[1] || {};
		// skip the boolean and the target
		i = 2;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction(target) )
		target = {};

	// extend jQuery itself if only one argument is passed
	if ( length == i ) {
		target = this;
		--i;
	}

	for ( ; i < length; i++ )
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null )
			// Extend the base object
			for ( var name in options ) {
				var src = target[ name ], copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy )
					continue;

				// Recurse if we're merging object values
				if ( deep && copy && typeof copy === "object" && !copy.nodeType )
					target[ name ] = jQuery.extend( deep, 
						// Never move original objects, clone them
						src || ( copy.length != null ? [ ] : { } )
					, copy );

				// Don't bring in undefined values
				else if ( copy !== undefined )
					target[ name ] = copy;

			}

	// Return the modified object
	return target;
};

// exclude the following css properties to add px
var	exclude = /z-?index|font-?weight|opacity|zoom|line-?height/i,
	// cache defaultView
	defaultView = document.defaultView || {},
	toString = Object.prototype.toString;

jQuery.extend({
	noConflict: function( deep ) {
		window.$ = _$;

		if ( deep )
			window.jQuery = _jQuery;

		return jQuery;
	},

	// See test/unit/core.js for details concerning isFunction.
	// Since version 1.3, DOM methods and functions like alert
	// aren't supported. They return false on IE (#2968).
	isFunction: function( obj ) {
		return toString.call(obj) === "[object Function]";
	},

	isArray: function( obj ) {
		return toString.call(obj) === "[object Array]";
	},

	// check if an element is in a (or is an) XML document
	isXMLDoc: function( elem ) {
		return elem.nodeType === 9 && elem.documentElement.nodeName !== "HTML" ||
			!!elem.ownerDocument && jQuery.isXMLDoc( elem.ownerDocument );
	},

	// Evalulates a script in a global context
	globalEval: function( data ) {
		if ( data && /\S/.test(data) ) {
			// Inspired by code by Andrea Giammarchi
			// http://webreflection.blogspot.com/2007/08/global-scope-evaluation-and-dom.html
			var head = document.getElementsByTagName("head")[0] || document.documentElement,
				script = document.createElement("script");

			script.type = "text/javascript";
			if ( jQuery.support.scriptEval )
				script.appendChild( document.createTextNode( data ) );
			else
				script.text = data;

			// Use insertBefore instead of appendChild  to circumvent an IE6 bug.
			// This arises when a base node is used (#2709).
			head.insertBefore( script, head.firstChild );
			head.removeChild( script );
		}
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toUpperCase() == name.toUpperCase();
	},

	// args is for internal usage only
	each: function( object, callback, args ) {
		var name, i = 0, length = object.length;

		if ( args ) {
			if ( length === undefined ) {
				for ( name in object )
					if ( callback.apply( object[ name ], args ) === false )
						break;
			} else
				for ( ; i < length; )
					if ( callback.apply( object[ i++ ], args ) === false )
						break;

		// A special, fast, case for the most common use of each
		} else {
			if ( length === undefined ) {
				for ( name in object )
					if ( callback.call( object[ name ], name, object[ name ] ) === false )
						break;
			} else
				for ( var value = object[0];
					i < length && callback.call( value, i, value ) !== false; value = object[++i] ){}
		}

		return object;
	},

	prop: function( elem, value, type, i, name ) {
		// Handle executable functions
		if ( jQuery.isFunction( value ) )
			value = value.call( elem, i );

		// Handle passing in a number to a CSS property
		return typeof value === "number" && type == "curCSS" && !exclude.test( name ) ?
			value + "px" :
			value;
	},

	className: {
		// internal only, use addClass("class")
		add: function( elem, classNames ) {
			jQuery.each((classNames || "").split(/\s+/), function(i, className){
				if ( elem.nodeType == 1 && !jQuery.className.has( elem.className, className ) )
					elem.className += (elem.className ? " " : "") + className;
			});
		},

		// internal only, use removeClass("class")
		remove: function( elem, classNames ) {
			if (elem.nodeType == 1)
				elem.className = classNames !== undefined ?
					jQuery.grep(elem.className.split(/\s+/), function(className){
						return !jQuery.className.has( classNames, className );
					}).join(" ") :
					"";
		},

		// internal only, use hasClass("class")
		has: function( elem, className ) {
			return elem && jQuery.inArray( className, (elem.className || elem).toString().split(/\s+/) ) > -1;
		}
	},

	// A method for quickly swapping in/out CSS properties to get correct calculations
	swap: function( elem, options, callback ) {
		var old = {};
		// Remember the old values, and insert the new ones
		for ( var name in options ) {
			old[ name ] = elem.style[ name ];
			elem.style[ name ] = options[ name ];
		}

		callback.call( elem );

		// Revert the old values
		for ( var name in options )
			elem.style[ name ] = old[ name ];
	},

	css: function( elem, name, force, extra ) {
		if ( name == "width" || name == "height" ) {
			var val, props = { position: "absolute", visibility: "hidden", display:"block" }, which = name == "width" ? [ "Left", "Right" ] : [ "Top", "Bottom" ];

			function getWH() {
				val = name == "width" ? elem.offsetWidth : elem.offsetHeight;

				if ( extra === "border" )
					return;

				jQuery.each( which, function() {
					if ( !extra )
						val -= parseFloat(jQuery.curCSS( elem, "padding" + this, true)) || 0;
					if ( extra === "margin" )
						val += parseFloat(jQuery.curCSS( elem, "margin" + this, true)) || 0;
					else
						val -= parseFloat(jQuery.curCSS( elem, "border" + this + "Width", true)) || 0;
				});
			}

			if ( elem.offsetWidth !== 0 )
				getWH();
			else
				jQuery.swap( elem, props, getWH );

			return Math.max(0, Math.round(val));
		}

		return jQuery.curCSS( elem, name, force );
	},

	curCSS: function( elem, name, force ) {
		var ret, style = elem.style;

		// We need to handle opacity special in IE
		if ( name == "opacity" && !jQuery.support.opacity ) {
			ret = jQuery.attr( style, "opacity" );

			return ret == "" ?
				"1" :
				ret;
		}

		// Make sure we're using the right name for getting the float value
		if ( name.match( /float/i ) )
			name = styleFloat;

		if ( !force && style && style[ name ] )
			ret = style[ name ];

		else if ( defaultView.getComputedStyle ) {

			// Only "float" is needed here
			if ( name.match( /float/i ) )
				name = "float";

			name = name.replace( /([A-Z])/g, "-$1" ).toLowerCase();

			var computedStyle = defaultView.getComputedStyle( elem, null );

			if ( computedStyle )
				ret = computedStyle.getPropertyValue( name );

			// We should always get a number back from opacity
			if ( name == "opacity" && ret == "" )
				ret = "1";

		} else if ( elem.currentStyle ) {
			var camelCase = name.replace(/\-(\w)/g, function(all, letter){
				return letter.toUpperCase();
			});

			ret = elem.currentStyle[ name ] || elem.currentStyle[ camelCase ];

			// From the awesome hack by Dean Edwards
			// http://erik.eae.net/archives/2007/07/27/18.54.15/#comment-102291

			// If we're not dealing with a regular pixel number
			// but a number that has a weird ending, we need to convert it to pixels
			if ( !/^\d+(px)?$/i.test( ret ) && /^\d/.test( ret ) ) {
				// Remember the original values
				var left = style.left, rsLeft = elem.runtimeStyle.left;

				// Put in the new values to get a computed value out
				elem.runtimeStyle.left = elem.currentStyle.left;
				style.left = ret || 0;
				ret = style.pixelLeft + "px";

				// Revert the changed values
				style.left = left;
				elem.runtimeStyle.left = rsLeft;
			}
		}

		return ret;
	},

	clean: function( elems, context, fragment ) {
		context = context || document;

		// !context.createElement fails in IE with an error but returns typeof 'object'
		if ( typeof context.createElement === "undefined" )
			context = context.ownerDocument || context[0] && context[0].ownerDocument || document;

		// If a single string is passed in and it's a single tag
		// just do a createElement and skip the rest
		if ( !fragment && elems.length === 1 && typeof elems[0] === "string" ) {
			var match = /^<(\w+)\s*\/?>$/.exec(elems[0]);
			if ( match )
				return [ context.createElement( match[1] ) ];
		}

		var ret = [], scripts = [], div = context.createElement("div");

		jQuery.each(elems, function(i, elem){
			if ( typeof elem === "number" )
				elem += '';

			if ( !elem )
				return;

			// Convert html string into DOM nodes
			if ( typeof elem === "string" ) {
				// Fix "XHTML"-style tags in all browsers
				elem = elem.replace(/(<(\w+)[^>]*?)\/>/g, function(all, front, tag){
					return tag.match(/^(abbr|br|col|img|input|link|meta|param|hr|area|embed)$/i) ?
						all :
						front + "></" + tag + ">";
				});

				// Trim whitespace, otherwise indexOf won't work as expected
				var tags = elem.replace(/^\s+/, "").substring(0, 10).toLowerCase();

				var wrap =
					// option or optgroup
					!tags.indexOf("<opt") &&
					[ 1, "<select multiple='multiple'>", "</select>" ] ||

					!tags.indexOf("<leg") &&
					[ 1, "<fieldset>", "</fieldset>" ] ||

					tags.match(/^<(thead|tbody|tfoot|colg|cap)/) &&
					[ 1, "<table>", "</table>" ] ||

					!tags.indexOf("<tr") &&
					[ 2, "<table><tbody>", "</tbody></table>" ] ||

				 	// <thead> matched above
					(!tags.indexOf("<td") || !tags.indexOf("<th")) &&
					[ 3, "<table><tbody><tr>", "</tr></tbody></table>" ] ||

					!tags.indexOf("<col") &&
					[ 2, "<table><tbody></tbody><colgroup>", "</colgroup></table>" ] ||

					// IE can't serialize <link> and <script> tags normally
					!jQuery.support.htmlSerialize &&
					[ 1, "div<div>", "</div>" ] ||

					[ 0, "", "" ];

				// Go to html and back, then peel off extra wrappers
				div.innerHTML = wrap[1] + elem + wrap[2];

				// Move to the right depth
				while ( wrap[0]-- )
					div = div.lastChild;

				// Remove IE's autoinserted <tbody> from table fragments
				if ( !jQuery.support.tbody ) {

					// String was a <table>, *may* have spurious <tbody>
					var hasBody = /<tbody/i.test(elem),
						tbody = !tags.indexOf("<table") && !hasBody ?
							div.firstChild && div.firstChild.childNodes :

						// String was a bare <thead> or <tfoot>
						wrap[1] == "<table>" && !hasBody ?
							div.childNodes :
							[];

					for ( var j = tbody.length - 1; j >= 0 ; --j )
						if ( jQuery.nodeName( tbody[ j ], "tbody" ) && !tbody[ j ].childNodes.length )
							tbody[ j ].parentNode.removeChild( tbody[ j ] );

					}

				// IE completely kills leading whitespace when innerHTML is used
				if ( !jQuery.support.leadingWhitespace && /^\s/.test( elem ) )
					div.insertBefore( context.createTextNode( elem.match(/^\s*/)[0] ), div.firstChild );
				
				elem = jQuery.makeArray( div.childNodes );
			}

			if ( elem.nodeType )
				ret.push( elem );
			else
				ret = jQuery.merge( ret, elem );

		});

		if ( fragment ) {
			for ( var i = 0; ret[i]; i++ ) {
				if ( jQuery.nodeName( ret[i], "script" ) && (!ret[i].type || ret[i].type.toLowerCase() === "text/javascript") ) {
					scripts.push( ret[i].parentNode ? ret[i].parentNode.removeChild( ret[i] ) : ret[i] );
				} else {
					if ( ret[i].nodeType === 1 )
						ret.splice.apply( ret, [i + 1, 0].concat(jQuery.makeArray(ret[i].getElementsByTagName("script"))) );
					fragment.appendChild( ret[i] );
				}
			}
			
			return scripts;
		}

		return ret;
	},

	attr: function( elem, name, value ) {
		// don't set attributes on text and comment nodes
		if (!elem || elem.nodeType == 3 || elem.nodeType == 8)
			return undefined;

		var notxml = !jQuery.isXMLDoc( elem ),
			// Whether we are setting (or getting)
			set = value !== undefined;

		// Try to normalize/fix the name
		name = notxml && jQuery.props[ name ] || name;

		// Only do all the following if this is a node (faster for style)
		// IE elem.getAttribute passes even for style
		if ( elem.tagName ) {

			// These attributes require special treatment
			var special = /href|src|style/.test( name );

			// Safari mis-reports the default selected property of a hidden option
			// Accessing the parent's selectedIndex property fixes it
			if ( name == "selected" && elem.parentNode )
				elem.parentNode.selectedIndex;

			// If applicable, access the attribute via the DOM 0 way
			if ( name in elem && notxml && !special ) {
				if ( set ){
					// We can't allow the type property to be changed (since it causes problems in IE)
					if ( name == "type" && jQuery.nodeName( elem, "input" ) && elem.parentNode )
						throw "type property can't be changed";

					elem[ name ] = value;
				}

				// browsers index elements by id/name on forms, give priority to attributes.
				if( jQuery.nodeName( elem, "form" ) && elem.getAttributeNode(name) )
					return elem.getAttributeNode( name ).nodeValue;

				// elem.tabIndex doesn't always return the correct value when it hasn't been explicitly set
				// http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
				if ( name == "tabIndex" ) {
					var attributeNode = elem.getAttributeNode( "tabIndex" );
					return attributeNode && attributeNode.specified
						? attributeNode.value
						: elem.nodeName.match(/(button|input|object|select|textarea)/i)
							? 0
							: elem.nodeName.match(/^(a|area)$/i) && elem.href
								? 0
								: undefined;
				}

				return elem[ name ];
			}

			if ( !jQuery.support.style && notxml &&  name == "style" )
				return jQuery.attr( elem.style, "cssText", value );

			if ( set )
				// convert the value to a string (all browsers do this but IE) see #1070
				elem.setAttribute( name, "" + value );

			var attr = !jQuery.support.hrefNormalized && notxml && special
					// Some attributes require a special call on IE
					? elem.getAttribute( name, 2 )
					: elem.getAttribute( name );

			// Non-existent attributes return null, we normalize to undefined
			return attr === null ? undefined : attr;
		}

		// elem is actually elem.style ... set the style

		// IE uses filters for opacity
		if ( !jQuery.support.opacity && name == "opacity" ) {
			if ( set ) {
				// IE has trouble with opacity if it does not have layout
				// Force it by setting the zoom level
				elem.zoom = 1;

				// Set the alpha filter to set the opacity
				elem.filter = (elem.filter || "").replace( /alpha\([^)]*\)/, "" ) +
					(parseInt( value ) + '' == "NaN" ? "" : "alpha(opacity=" + value * 100 + ")");
			}

			return elem.filter && elem.filter.indexOf("opacity=") >= 0 ?
				(parseFloat( elem.filter.match(/opacity=([^)]*)/)[1] ) / 100) + '':
				"";
		}

		name = name.replace(/-([a-z])/ig, function(all, letter){
			return letter.toUpperCase();
		});

		if ( set )
		   if (value=="NaNpx")
            {
				value="1px";
            }
			elem[ name ] = value;

		return elem[ name ];
	},

	trim: function( text ) {
		return (text || "").replace( /^\s+|\s+$/g, "" );
	},

	makeArray: function( array ) {
		var ret = [];

		if( array != null ){
			var i = array.length;
			// The window, strings (and functions) also have 'length'
			if( i == null || typeof array === "string" || jQuery.isFunction(array) || array.setInterval )
				ret[0] = array;
			else
				while( i )
					ret[--i] = array[i];
		}

		return ret;
	},

	inArray: function( elem, array ) {
		for ( var i = 0, length = array.length; i < length; i++ )
		// Use === because on IE, window == document
			if ( array[ i ] === elem )
				return i;

		return -1;
	},

	merge: function( first, second ) {
		// We have to loop this way because IE & Opera overwrite the length
		// expando of getElementsByTagName
		var i = 0, elem, pos = first.length;
		// Also, we need to make sure that the correct elements are being returned
		// (IE returns comment nodes in a '*' query)
		if ( !jQuery.support.getAll ) {
			while ( (elem = second[ i++ ]) != null )
				if ( elem.nodeType != 8 )
					first[ pos++ ] = elem;

		} else
			while ( (elem = second[ i++ ]) != null )
				first[ pos++ ] = elem;

		return first;
	},

	unique: function( array ) {
		var ret = [], done = {};

		try {

			for ( var i = 0, length = array.length; i < length; i++ ) {
				var id = jQuery.data( array[ i ] );

				if ( !done[ id ] ) {
					done[ id ] = true;
					ret.push( array[ i ] );
				}
			}

		} catch( e ) {
			ret = array;
		}

		return ret;
	},

	grep: function( elems, callback, inv ) {
		var ret = [];

		// Go through the array, only saving the items
		// that pass the validator function
		for ( var i = 0, length = elems.length; i < length; i++ )
			if ( !inv != !callback( elems[ i ], i ) )
				ret.push( elems[ i ] );

		return ret;
	},

	map: function( elems, callback ) {
		var ret = [];

		// Go through the array, translating each of the items to their
		// new value (or values).
		for ( var i = 0, length = elems.length; i < length; i++ ) {
			var value = callback( elems[ i ], i );

			if ( value != null )
				ret[ ret.length ] = value;
		}

		return ret.concat.apply( [], ret );
	}
});

// Use of jQuery.browser is deprecated.
// It's included for backwards compatibility and plugins,
// although they should work to migrate away.

var userAgent = navigator.userAgent.toLowerCase();

// Figure out what browser is being used
jQuery.browser = {
	version: (userAgent.match( /.+(?:rv|it|ra|ie)[\/: ]([\d.]+)/ ) || [0,'0'])[1],
	safari: /webkit/.test( userAgent ),
	opera: /opera/.test( userAgent ),
	msie: /msie/.test( userAgent ) && !/opera/.test( userAgent ),
	mozilla: /mozilla/.test( userAgent ) && !/(compatible|webkit)/.test( userAgent )
};

jQuery.each({
	parent: function(elem){return elem.parentNode;},
	parents: function(elem){return jQuery.dir(elem,"parentNode");},
	next: function(elem){return jQuery.nth(elem,2,"nextSibling");},
	prev: function(elem){return jQuery.nth(elem,2,"previousSibling");},
	nextAll: function(elem){return jQuery.dir(elem,"nextSibling");},
	prevAll: function(elem){return jQuery.dir(elem,"previousSibling");},
	siblings: function(elem){return jQuery.sibling(elem.parentNode.firstChild,elem);},
	children: function(elem){return jQuery.sibling(elem.firstChild);},
	contents: function(elem){return jQuery.nodeName(elem,"iframe")?elem.contentDocument||elem.contentWindow.document:jQuery.makeArray(elem.childNodes);}
}, function(name, fn){
	jQuery.fn[ name ] = function( selector ) {
		var ret = jQuery.map( this, fn );

		if ( selector && typeof selector == "string" )
			ret = jQuery.multiFilter( selector, ret );

		return this.pushStack( jQuery.unique( ret ), name, selector );
	};
});

jQuery.each({
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function(name, original){
	jQuery.fn[ name ] = function( selector ) {
		var ret = [], insert = jQuery( selector );

		for ( var i = 0, l = insert.length; i < l; i++ ) {
			var elems = (i > 0 ? this.clone(true) : this).get();
			jQuery.fn[ original ].apply( jQuery(insert[i]), elems );
			ret = ret.concat( elems );
		}

		return this.pushStack( ret, name, selector );
	};
});

jQuery.each({
	removeAttr: function( name ) {
		jQuery.attr( this, name, "" );
		if (this.nodeType == 1)
			this.removeAttribute( name );
	},

	addClass: function( classNames ) {
		jQuery.className.add( this, classNames );
	},

	removeClass: function( classNames ) {
		jQuery.className.remove( this, classNames );
	},

	toggleClass: function( classNames, state ) {
		if( typeof state !== "boolean" )
			state = !jQuery.className.has( this, classNames );
		jQuery.className[ state ? "add" : "remove" ]( this, classNames );
	},

	remove: function( selector ) {
		if ( !selector || jQuery.filter( selector, [ this ] ).length ) {
			// Prevent memory leaks
			jQuery( "*", this ).add([this]).each(function(){
				jQuery.event.remove(this);
				jQuery.removeData(this);
			});
			if (this.parentNode)
				this.parentNode.removeChild( this );
		}
	},

	empty: function() {
		// Remove element nodes and prevent memory leaks
		jQuery(this).children().remove();

		// Remove any remaining nodes
		while ( this.firstChild )
			this.removeChild( this.firstChild );
	}
}, function(name, fn){
	jQuery.fn[ name ] = function(){
		return this.each( fn, arguments );
	};
});

// Helper function used by the dimensions and offset modules
function num(elem, prop) {
	return elem[0] && parseInt( jQuery.curCSS(elem[0], prop, true), 10 ) || 0;
}
var expando = "jQuery" + now(), uuid = 0, windowData = {};

jQuery.extend({
	cache: {},

	data: function( elem, name, data ) {
		elem = elem == window ?
			windowData :
			elem;

		var id = elem[ expando ];

		// Compute a unique ID for the element
		if ( !id )
			id = elem[ expando ] = ++uuid;

		// Only generate the data cache if we're
		// trying to access or manipulate it
		if ( name && !jQuery.cache[ id ] )
			jQuery.cache[ id ] = {};

		// Prevent overriding the named cache with undefined values
		if ( data !== undefined )
			jQuery.cache[ id ][ name ] = data;

		// Return the named cache data, or the ID for the element
		return name ?
			jQuery.cache[ id ][ name ] :
			id;
	},

	removeData: function( elem, name ) {
		elem = elem == window ?
			windowData :
			elem;

		var id = elem[ expando ];

		// If we want to remove a specific section of the element's data
		if ( name ) {
			if ( jQuery.cache[ id ] ) {
				// Remove the section of cache data
				delete jQuery.cache[ id ][ name ];

				// If we've removed all the data, remove the element's cache
				name = "";

				for ( name in jQuery.cache[ id ] )
					break;

				if ( !name )
					jQuery.removeData( elem );
			}

		// Otherwise, we want to remove all of the element's data
		} else {
			// Clean up the element expando
			try {
				delete elem[ expando ];
			} catch(e){
				// IE has trouble directly removing the expando
				// but it's ok with using removeAttribute
				if ( elem.removeAttribute )
					elem.removeAttribute( expando );
			}

			// Completely remove the data cache
			delete jQuery.cache[ id ];
		}
	},
	queue: function( elem, type, data ) {
		if ( elem ){
	
			type = (type || "fx") + "queue";
	
			var q = jQuery.data( elem, type );
	
			if ( !q || jQuery.isArray(data) )
				q = jQuery.data( elem, type, jQuery.makeArray(data) );
			else if( data )
				q.push( data );
	
		}
		return q;
	},

	dequeue: function( elem, type ){
		var queue = jQuery.queue( elem, type ),
			fn = queue.shift();
		
		if( !type || type === "fx" )
			fn = queue[0];
			
		if( fn !== undefined )
			fn.call(elem);
	}
});

jQuery.fn.extend({
	data: function( key, value ){
		var parts = key.split(".");
		parts[1] = parts[1] ? "." + parts[1] : "";

		if ( value === undefined ) {
			var data = this.triggerHandler("getData" + parts[1] + "!", [parts[0]]);

			if ( data === undefined && this.length )
				data = jQuery.data( this[0], key );

			return data === undefined && parts[1] ?
				this.data( parts[0] ) :
				data;
		} else
			return this.trigger("setData" + parts[1] + "!", [parts[0], value]).each(function(){
				jQuery.data( this, key, value );
			});
	},

	removeData: function( key ){
		return this.each(function(){
			jQuery.removeData( this, key );
		});
	},
	queue: function(type, data){
		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
		}

		if ( data === undefined )
			return jQuery.queue( this[0], type );

		return this.each(function(){
			var queue = jQuery.queue( this, type, data );
			
			 if( type == "fx" && queue.length == 1 )
				queue[0].call(this);
		});
	},
	dequeue: function(type){
		return this.each(function(){
			jQuery.dequeue( this, type );
		});
	}
});/*!
 * Sizzle CSS Selector Engine - v0.9.3
 *  Copyright 2009, The Dojo Foundation
 *  Released under the MIT, BSD, and GPL Licenses.
 *  More information: http://sizzlejs.com/
 */
(function(){

var chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^[\]]*\]|['"][^'"]*['"]|[^[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?/g,
	done = 0,
	toString = Object.prototype.toString;

var Sizzle = function(selector, context, results, seed) {
	results = results || [];
	context = context || document;

	if ( context.nodeType !== 1 && context.nodeType !== 9 )
		return [];
	
	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	var parts = [], m, set, checkSet, check, mode, extra, prune = true;
	
	// Reset the position of the chunker regexp (start from head)
	chunker.lastIndex = 0;
	
	while ( (m = chunker.exec(selector)) !== null ) {
		parts.push( m[1] );
		
		if ( m[2] ) {
			extra = RegExp.rightContext;
			break;
		}
	}

	if ( parts.length > 1 && origPOS.exec( selector ) ) {
		if ( parts.length === 2 && Expr.relative[ parts[0] ] ) {
			set = posProcess( parts[0] + parts[1], context );
		} else {
			set = Expr.relative[ parts[0] ] ?
				[ context ] :
				Sizzle( parts.shift(), context );

			while ( parts.length ) {
				selector = parts.shift();

				if ( Expr.relative[ selector ] )
					selector += parts.shift();

				set = posProcess( selector, set );
			}
		}
	} else {
		var ret = seed ?
			{ expr: parts.pop(), set: makeArray(seed) } :
			Sizzle.find( parts.pop(), parts.length === 1 && context.parentNode ? context.parentNode : context, isXML(context) );
		set = Sizzle.filter( ret.expr, ret.set );

		if ( parts.length > 0 ) {
			checkSet = makeArray(set);
		} else {
			prune = false;
		}

		while ( parts.length ) {
			var cur = parts.pop(), pop = cur;

			if ( !Expr.relative[ cur ] ) {
				cur = "";
			} else {
				pop = parts.pop();
			}

			if ( pop == null ) {
				pop = context;
			}

			Expr.relative[ cur ]( checkSet, pop, isXML(context) );
		}
	}

	if ( !checkSet ) {
		checkSet = set;
	}

	if ( !checkSet ) {
		throw "Syntax error, unrecognized expression: " + (cur || selector);
	}

	if ( toString.call(checkSet) === "[object Array]" ) {
		if ( !prune ) {
			results.push.apply( results, checkSet );
		} else if ( context.nodeType === 1 ) {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && (checkSet[i] === true || checkSet[i].nodeType === 1 && contains(context, checkSet[i])) ) {
					results.push( set[i] );
				}
			}
		} else {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && checkSet[i].nodeType === 1 ) {
					results.push( set[i] );
				}
			}
		}
	} else {
		makeArray( checkSet, results );
	}

	if ( extra ) {
		Sizzle( extra, context, results, seed );

		if ( sortOrder ) {
			hasDuplicate = false;
			results.sort(sortOrder);

			if ( hasDuplicate ) {
				for ( var i = 1; i < results.length; i++ ) {
					if ( results[i] === results[i-1] ) {
						results.splice(i--, 1);
					}
				}
			}
		}
	}

	return results;
};

Sizzle.matches = function(expr, set){
	return Sizzle(expr, null, null, set);
};

Sizzle.find = function(expr, context, isXML){
	var set, match;

	if ( !expr ) {
		return [];
	}

	for ( var i = 0, l = Expr.order.length; i < l; i++ ) {
		var type = Expr.order[i], match;
		
		if ( (match = Expr.match[ type ].exec( expr )) ) {
			var left = RegExp.leftContext;

			if ( left.substr( left.length - 1 ) !== "\\" ) {
				match[1] = (match[1] || "").replace(/\\/g, "");
				set = Expr.find[ type ]( match, context, isXML );
				if ( set != null ) {
					expr = expr.replace( Expr.match[ type ], "" );
					break;
				}
			}
		}
	}

	if ( !set ) {
		set = context.getElementsByTagName("*");
	}

	return {set: set, expr: expr};
};

Sizzle.filter = function(expr, set, inplace, not){
	var old = expr, result = [], curLoop = set, match, anyFound,
		isXMLFilter = set && set[0] && isXML(set[0]);

	while ( expr && set.length ) {
		for ( var type in Expr.filter ) {
			if ( (match = Expr.match[ type ].exec( expr )) != null ) {
				var filter = Expr.filter[ type ], found, item;
				anyFound = false;

				if ( curLoop == result ) {
					result = [];
				}

				if ( Expr.preFilter[ type ] ) {
					match = Expr.preFilter[ type ]( match, curLoop, inplace, result, not, isXMLFilter );

					if ( !match ) {
						anyFound = found = true;
					} else if ( match === true ) {
						continue;
					}
				}

				if ( match ) {
					for ( var i = 0; (item = curLoop[i]) != null; i++ ) {
						if ( item ) {
							found = filter( item, match, i, curLoop );
							var pass = not ^ !!found;

							if ( inplace && found != null ) {
								if ( pass ) {
									anyFound = true;
								} else {
									curLoop[i] = false;
								}
							} else if ( pass ) {
								result.push( item );
								anyFound = true;
							}
						}
					}
				}

				if ( found !== undefined ) {
					if ( !inplace ) {
						curLoop = result;
					}

					expr = expr.replace( Expr.match[ type ], "" );

					if ( !anyFound ) {
						return [];
					}

					break;
				}
			}
		}

		// Improper expression
		if ( expr == old ) {
			if ( anyFound == null ) {
				throw "Syntax error, unrecognized expression: " + expr;
			} else {
				break;
			}
		}

		old = expr;
	}

	return curLoop;
};

var Expr = Sizzle.selectors = {
	order: [ "ID", "NAME", "TAG" ],
	match: {
		ID: /#((?:[\w\u00c0-\uFFFF_-]|\\.)+)/,
		CLASS: /\.((?:[\w\u00c0-\uFFFF_-]|\\.)+)/,
		NAME: /\[name=['"]*((?:[\w\u00c0-\uFFFF_-]|\\.)+)['"]*\]/,
		ATTR: /\[\s*((?:[\w\u00c0-\uFFFF_-]|\\.)+)\s*(?:(\S?=)\s*(['"]*)(.*?)\3|)\s*\]/,
		TAG: /^((?:[\w\u00c0-\uFFFF\*_-]|\\.)+)/,
		CHILD: /:(only|nth|last|first)-child(?:\((even|odd|[\dn+-]*)\))?/,
		POS: /:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^-]|$)/,
		PSEUDO: /:((?:[\w\u00c0-\uFFFF_-]|\\.)+)(?:\((['"]*)((?:\([^\)]+\)|[^\2\(\)]*)+)\2\))?/
	},
	attrMap: {
		"class": "className",
		"for": "htmlFor"
	},
	attrHandle: {
		href: function(elem){
			return elem.getAttribute("href");
		}
	},
	relative: {
		"+": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string",
				isTag = isPartStr && !/\W/.test(part),
				isPartStrNotTag = isPartStr && !isTag;

			if ( isTag && !isXML ) {
				part = part.toUpperCase();
			}

			for ( var i = 0, l = checkSet.length, elem; i < l; i++ ) {
				if ( (elem = checkSet[i]) ) {
					while ( (elem = elem.previousSibling) && elem.nodeType !== 1 ) {}

					checkSet[i] = isPartStrNotTag || elem && elem.nodeName === part ?
						elem || false :
						elem === part;
				}
			}

			if ( isPartStrNotTag ) {
				Sizzle.filter( part, checkSet, true );
			}
		},
		">": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string";

			if ( isPartStr && !/\W/.test(part) ) {
				part = isXML ? part : part.toUpperCase();

				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						var parent = elem.parentNode;
						checkSet[i] = parent.nodeName === part ? parent : false;
					}
				}
			} else {
				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						checkSet[i] = isPartStr ?
							elem.parentNode :
							elem.parentNode === part;
					}
				}

				if ( isPartStr ) {
					Sizzle.filter( part, checkSet, true );
				}
			}
		},
		"": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( !part.match(/\W/) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("parentNode", part, doneName, checkSet, nodeCheck, isXML);
		},
		"~": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( typeof part === "string" && !part.match(/\W/) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("previousSibling", part, doneName, checkSet, nodeCheck, isXML);
		}
	},
	find: {
		ID: function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? [m] : [];
			}
		},
		NAME: function(match, context, isXML){
			if ( typeof context.getElementsByName !== "undefined" ) {
				var ret = [], results = context.getElementsByName(match[1]);

				for ( var i = 0, l = results.length; i < l; i++ ) {
					if ( results[i].getAttribute("name") === match[1] ) {
						ret.push( results[i] );
					}
				}

				return ret.length === 0 ? null : ret;
			}
		},
		TAG: function(match, context){
			return context.getElementsByTagName(match[1]);
		}
	},
	preFilter: {
		CLASS: function(match, curLoop, inplace, result, not, isXML){
			match = " " + match[1].replace(/\\/g, "") + " ";

			if ( isXML ) {
				return match;
			}

			for ( var i = 0, elem; (elem = curLoop[i]) != null; i++ ) {
				if ( elem ) {
					if ( not ^ (elem.className && (" " + elem.className + " ").indexOf(match) >= 0) ) {
						if ( !inplace )
							result.push( elem );
					} else if ( inplace ) {
						curLoop[i] = false;
					}
				}
			}

			return false;
		},
		ID: function(match){
			return match[1].replace(/\\/g, "");
		},
		TAG: function(match, curLoop){
			for ( var i = 0; curLoop[i] === false; i++ ){}
			return curLoop[i] && isXML(curLoop[i]) ? match[1] : match[1].toUpperCase();
		},
		CHILD: function(match){
			if ( match[1] == "nth" ) {
				// parse equations like 'even', 'odd', '5', '2n', '3n+2', '4n-1', '-n+6'
				var test = /(-?)(\d*)n((?:\+|-)?\d*)/.exec(
					match[2] == "even" && "2n" || match[2] == "odd" && "2n+1" ||
					!/\D/.test( match[2] ) && "0n+" + match[2] || match[2]);

				// calculate the numbers (first)n+(last) including if they are negative
				match[2] = (test[1] + (test[2] || 1)) - 0;
				match[3] = test[3] - 0;
			}

			// TODO: Move to normal caching system
			match[0] = done++;

			return match;
		},
		ATTR: function(match, curLoop, inplace, result, not, isXML){
			var name = match[1].replace(/\\/g, "");
			
			if ( !isXML && Expr.attrMap[name] ) {
				match[1] = Expr.attrMap[name];
			}

			if ( match[2] === "~=" ) {
				match[4] = " " + match[4] + " ";
			}

			return match;
		},
		PSEUDO: function(match, curLoop, inplace, result, not){
			if ( match[1] === "not" ) {
				// If we're dealing with a complex expression, or a simple one
				if ( match[3].match(chunker).length > 1 || /^\w/.test(match[3]) ) {
					match[3] = Sizzle(match[3], null, null, curLoop);
				} else {
					var ret = Sizzle.filter(match[3], curLoop, inplace, true ^ not);
					if ( !inplace ) {
						result.push.apply( result, ret );
					}
					return false;
				}
			} else if ( Expr.match.POS.test( match[0] ) || Expr.match.CHILD.test( match[0] ) ) {
				return true;
			}
			
			return match;
		},
		POS: function(match){
			match.unshift( true );
			return match;
		}
	},
	filters: {
		enabled: function(elem){
			return elem.disabled === false && elem.type !== "hidden";
		},
		disabled: function(elem){
			return elem.disabled === true;
		},
		checked: function(elem){
			return elem.checked === true;
		},
		selected: function(elem){
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			elem.parentNode.selectedIndex;
			return elem.selected === true;
		},
		parent: function(elem){
			return !!elem.firstChild;
		},
		empty: function(elem){
			return !elem.firstChild;
		},
		has: function(elem, i, match){
			return !!Sizzle( match[3], elem ).length;
		},
		header: function(elem){
			return /h\d/i.test( elem.nodeName );
		},
		text: function(elem){
			return "text" === elem.type;
		},
		radio: function(elem){
			return "radio" === elem.type;
		},
		checkbox: function(elem){
			return "checkbox" === elem.type;
		},
		file: function(elem){
			return "file" === elem.type;
		},
		password: function(elem){
			return "password" === elem.type;
		},
		submit: function(elem){
			return "submit" === elem.type;
		},
		image: function(elem){
			return "image" === elem.type;
		},
		reset: function(elem){
			return "reset" === elem.type;
		},
		button: function(elem){
			return "button" === elem.type || elem.nodeName.toUpperCase() === "BUTTON";
		},
		input: function(elem){
			return /input|select|textarea|button/i.test(elem.nodeName);
		}
	},
	setFilters: {
		first: function(elem, i){
			return i === 0;
		},
		last: function(elem, i, match, array){
			return i === array.length - 1;
		},
		even: function(elem, i){
			return i % 2 === 0;
		},
		odd: function(elem, i){
			return i % 2 === 1;
		},
		lt: function(elem, i, match){
			return i < match[3] - 0;
		},
		gt: function(elem, i, match){
			return i > match[3] - 0;
		},
		nth: function(elem, i, match){
			return match[3] - 0 == i;
		},
		eq: function(elem, i, match){
			return match[3] - 0 == i;
		}
	},
	filter: {
		PSEUDO: function(elem, match, i, array){
			var name = match[1], filter = Expr.filters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			} else if ( name === "contains" ) {
				return (elem.textContent || elem.innerText || "").indexOf(match[3]) >= 0;
			} else if ( name === "not" ) {
				var not = match[3];

				for ( var i = 0, l = not.length; i < l; i++ ) {
					if ( not[i] === elem ) {
						return false;
					}
				}

				return true;
			}
		},
		CHILD: function(elem, match){
			var type = match[1], node = elem;
			switch (type) {
				case 'only':
				case 'first':
					while (node = node.previousSibling)  {
						if ( node.nodeType === 1 ) return false;
					}
					if ( type == 'first') return true;
					node = elem;
				case 'last':
					while (node = node.nextSibling)  {
						if ( node.nodeType === 1 ) return false;
					}
					return true;
				case 'nth':
					var first = match[2], last = match[3];

					if ( first == 1 && last == 0 ) {
						return true;
					}
					
					var doneName = match[0],
						parent = elem.parentNode;
	
					if ( parent && (parent.sizcache !== doneName || !elem.nodeIndex) ) {
						var count = 0;
						for ( node = parent.firstChild; node; node = node.nextSibling ) {
							if ( node.nodeType === 1 ) {
								node.nodeIndex = ++count;
							}
						} 
						parent.sizcache = doneName;
					}
					
					var diff = elem.nodeIndex - last;
					if ( first == 0 ) {
						return diff == 0;
					} else {
						return ( diff % first == 0 && diff / first >= 0 );
					}
			}
		},
		ID: function(elem, match){
			return elem.nodeType === 1 && elem.getAttribute("id") === match;
		},
		TAG: function(elem, match){
			return (match === "*" && elem.nodeType === 1) || elem.nodeName === match;
		},
		CLASS: function(elem, match){
			return (" " + (elem.className || elem.getAttribute("class")) + " ")
				.indexOf( match ) > -1;
		},
		ATTR: function(elem, match){
			var name = match[1],
				result = Expr.attrHandle[ name ] ?
					Expr.attrHandle[ name ]( elem ) :
					elem[ name ] != null ?
						elem[ name ] :
						elem.getAttribute( name ),
				value = result + "",
				type = match[2],
				check = match[4];

			return result == null ?
				type === "!=" :
				type === "=" ?
				value === check :
				type === "*=" ?
				value.indexOf(check) >= 0 :
				type === "~=" ?
				(" " + value + " ").indexOf(check) >= 0 :
				!check ?
				value && result !== false :
				type === "!=" ?
				value != check :
				type === "^=" ?
				value.indexOf(check) === 0 :
				type === "$=" ?
				value.substr(value.length - check.length) === check :
				type === "|=" ?
				value === check || value.substr(0, check.length + 1) === check + "-" :
				false;
		},
		POS: function(elem, match, i, array){
			var name = match[2], filter = Expr.setFilters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			}
		}
	}
};

var origPOS = Expr.match.POS;

for ( var type in Expr.match ) {
	Expr.match[ type ] = RegExp( Expr.match[ type ].source + /(?![^\[]*\])(?![^\(]*\))/.source );
}

var makeArray = function(array, results) {
	array = Array.prototype.slice.call( array );

	if ( results ) {
		results.push.apply( results, array );
		return results;
	}
	
	return array;
};

// Perform a simple check to determine if the browser is capable of
// converting a NodeList to an array using builtin methods.
try {
	Array.prototype.slice.call( document.documentElement.childNodes );

// Provide a fallback method if it does not work
} catch(e){
	makeArray = function(array, results) {
		var ret = results || [];

		if ( toString.call(array) === "[object Array]" ) {
			Array.prototype.push.apply( ret, array );
		} else {
			if ( typeof array.length === "number" ) {
				for ( var i = 0, l = array.length; i < l; i++ ) {
					ret.push( array[i] );
				}
			} else {
				for ( var i = 0; array[i]; i++ ) {
					ret.push( array[i] );
				}
			}
		}

		return ret;
	};
}

var sortOrder;

if ( document.documentElement.compareDocumentPosition ) {
	sortOrder = function( a, b ) {
		var ret = a.compareDocumentPosition(b) & 4 ? -1 : a === b ? 0 : 1;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( "sourceIndex" in document.documentElement ) {
	sortOrder = function( a, b ) {
		var ret = a.sourceIndex - b.sourceIndex;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( document.createRange ) {
	sortOrder = function( a, b ) {
		var aRange = a.ownerDocument.createRange(), bRange = b.ownerDocument.createRange();
		aRange.selectNode(a);
		aRange.collapse(true);
		bRange.selectNode(b);
		bRange.collapse(true);
		var ret = aRange.compareBoundaryPoints(Range.START_TO_END, bRange);
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
}

// Check to see if the browser returns elements by name when
// querying by getElementById (and provide a workaround)
(function(){
	// We're going to inject a fake input element with a specified name
	var form = document.createElement("form"),
		id = "script" + (new Date).getTime();
	form.innerHTML = "<input name='" + id + "'/>";

	// Inject it into the root element, check its status, and remove it quickly
	var root = document.documentElement;
	root.insertBefore( form, root.firstChild );

	// The workaround has to do additional checks after a getElementById
	// Which slows things down for other browsers (hence the branching)
	if ( !!document.getElementById( id ) ) {
		Expr.find.ID = function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? m.id === match[1] || typeof m.getAttributeNode !== "undefined" && m.getAttributeNode("id").nodeValue === match[1] ? [m] : undefined : [];
			}
		};

		Expr.filter.ID = function(elem, match){
			var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");
			return elem.nodeType === 1 && node && node.nodeValue === match;
		};
	}

	root.removeChild( form );
})();

(function(){
	// Check to see if the browser returns only elements
	// when doing getElementsByTagName("*")

	// Create a fake element
	var div = document.createElement("div");
	div.appendChild( document.createComment("") );

	// Make sure no comments are found
	if ( div.getElementsByTagName("*").length > 0 ) {
		Expr.find.TAG = function(match, context){
			var results = context.getElementsByTagName(match[1]);

			// Filter out possible comments
			if ( match[1] === "*" ) {
				var tmp = [];

				for ( var i = 0; results[i]; i++ ) {
					if ( results[i].nodeType === 1 ) {
						tmp.push( results[i] );
					}
				}

				results = tmp;
			}

			return results;
		};
	}

	// Check to see if an attribute returns normalized href attributes
	div.innerHTML = "<a href='#'></a>";
	if ( div.firstChild && typeof div.firstChild.getAttribute !== "undefined" &&
			div.firstChild.getAttribute("href") !== "#" ) {
		Expr.attrHandle.href = function(elem){
			return elem.getAttribute("href", 2);
		};
	}
})();

if ( document.querySelectorAll ) (function(){
	var oldSizzle = Sizzle, div = document.createElement("div");
	div.innerHTML = "<p class='TEST'></p>";

	// Safari can't handle uppercase or unicode characters when
	// in quirks mode.
	if ( div.querySelectorAll && div.querySelectorAll(".TEST").length === 0 ) {
		return;
	}
	
	Sizzle = function(query, context, extra, seed){
		context = context || document;

		// Only use querySelectorAll on non-XML documents
		// (ID selectors don't work in non-HTML documents)
		if ( !seed && context.nodeType === 9 && !isXML(context) ) {
			try {
				return makeArray( context.querySelectorAll(query), extra );
			} catch(e){}
		}
		
		return oldSizzle(query, context, extra, seed);
	};

	Sizzle.find = oldSizzle.find;
	Sizzle.filter = oldSizzle.filter;
	Sizzle.selectors = oldSizzle.selectors;
	Sizzle.matches = oldSizzle.matches;
})();

if ( document.getElementsByClassName && document.documentElement.getElementsByClassName ) (function(){
	var div = document.createElement("div");
	div.innerHTML = "<div class='test e'></div><div class='test'></div>";

	// Opera can't find a second classname (in 9.6)
	if ( div.getElementsByClassName("e").length === 0 )
		return;

	// Safari caches class attributes, doesn't catch changes (in 3.2)
	div.lastChild.className = "e";

	if ( div.getElementsByClassName("e").length === 1 )
		return;

	Expr.order.splice(1, 0, "CLASS");
	Expr.find.CLASS = function(match, context, isXML) {
		if ( typeof context.getElementsByClassName !== "undefined" && !isXML ) {
			return context.getElementsByClassName(match[1]);
		}
	};
})();

function dirNodeCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ){
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 && !isXML ){
					elem.sizcache = doneName;
					elem.sizset = i;
				}

				if ( elem.nodeName === cur ) {
					match = elem;
					break;
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

function dirCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ) {
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 ) {
					if ( !isXML ) {
						elem.sizcache = doneName;
						elem.sizset = i;
					}
					if ( typeof cur !== "string" ) {
						if ( elem === cur ) {
							match = true;
							break;
						}

					} else if ( Sizzle.filter( cur, [elem] ).length > 0 ) {
						match = elem;
						break;
					}
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

var contains = document.compareDocumentPosition ?  function(a, b){
	return a.compareDocumentPosition(b) & 16;
} : function(a, b){
	return a !== b && (a.contains ? a.contains(b) : true);
};

var isXML = function(elem){
	return elem.nodeType === 9 && elem.documentElement.nodeName !== "HTML" ||
		!!elem.ownerDocument && isXML( elem.ownerDocument );
};

var posProcess = function(selector, context){
	var tmpSet = [], later = "", match,
		root = context.nodeType ? [context] : context;

	// Position selectors must be done after the filter
	// And so must :not(positional) so we move all PSEUDOs to the end
	while ( (match = Expr.match.PSEUDO.exec( selector )) ) {
		later += match[0];
		selector = selector.replace( Expr.match.PSEUDO, "" );
	}

	selector = Expr.relative[selector] ? selector + "*" : selector;

	for ( var i = 0, l = root.length; i < l; i++ ) {
		Sizzle( selector, root[i], tmpSet );
	}

	return Sizzle.filter( later, tmpSet );
};

// EXPOSE
jQuery.find = Sizzle;
jQuery.filter = Sizzle.filter;
jQuery.expr = Sizzle.selectors;
jQuery.expr[":"] = jQuery.expr.filters;

Sizzle.selectors.filters.hidden = function(elem){
	return elem.offsetWidth === 0 || elem.offsetHeight === 0;
};

Sizzle.selectors.filters.visible = function(elem){
	return elem.offsetWidth > 0 || elem.offsetHeight > 0;
};

Sizzle.selectors.filters.animated = function(elem){
	return jQuery.grep(jQuery.timers, function(fn){
		return elem === fn.elem;
	}).length;
};

jQuery.multiFilter = function( expr, elems, not ) {
	if ( not ) {
		expr = ":not(" + expr + ")";
	}

	return Sizzle.matches(expr, elems);
};

jQuery.dir = function( elem, dir ){
	var matched = [], cur = elem[dir];
	while ( cur && cur != document ) {
		if ( cur.nodeType == 1 )
			matched.push( cur );
		cur = cur[dir];
	}
	return matched;
};

jQuery.nth = function(cur, result, dir, elem){
	result = result || 1;
	var num = 0;

	for ( ; cur; cur = cur[dir] )
		if ( cur.nodeType == 1 && ++num == result )
			break;

	return cur;
};

jQuery.sibling = function(n, elem){
	var r = [];

	for ( ; n; n = n.nextSibling ) {
		if ( n.nodeType == 1 && n != elem )
			r.push( n );
	}

	return r;
};

return;

window.Sizzle = Sizzle;

})();
/*
 * A number of helper functions used for managing events.
 * Many of the ideas behind this code originated from
 * Dean Edwards' addEvent library.
 */
jQuery.event = {

	// Bind an event to an element
	// Original by Dean Edwards
	add: function(elem, types, handler, data) {
		if ( elem.nodeType == 3 || elem.nodeType == 8 )
			return;

		// For whatever reason, IE has trouble passing the window object
		// around, causing it to be cloned in the process
		if ( elem.setInterval && elem != window )
			elem = window;

		// Make sure that the function being executed has a unique ID
		if ( !handler.guid )
			handler.guid = this.guid++;

		// if data is passed, bind to handler
		if ( data !== undefined ) {
			// Create temporary function pointer to original handler
			var fn = handler;

			// Create unique handler function, wrapped around original handler
			handler = this.proxy( fn );

			// Store data in unique handler
			handler.data = data;
		}

		// Init the element's event structure
		var events = jQuery.data(elem, "events") || jQuery.data(elem, "events", {}),
			handle = jQuery.data(elem, "handle") || jQuery.data(elem, "handle", function(){
				// Handle the second event of a trigger and when
				// an event is called after a page has unloaded
				return typeof jQuery !== "undefined" && !jQuery.event.triggered ?
					jQuery.event.handle.apply(arguments.callee.elem, arguments) :
					undefined;
			});
		// Add elem as a property of the handle function
		// This is to prevent a memory leak with non-native
		// event in IE.
		handle.elem = elem;

		// Handle multiple events separated by a space
		// jQuery(...).bind("mouseover mouseout", fn);
		jQuery.each(types.split(/\s+/), function(index, type) {
			// Namespaced event handlers
			var namespaces = type.split(".");
			type = namespaces.shift();
			handler.type = namespaces.slice().sort().join(".");

			// Get the current list of functions bound to this event
			var handlers = events[type];
			
			if ( jQuery.event.specialAll[type] )
				jQuery.event.specialAll[type].setup.call(elem, data, namespaces);

			// Init the event handler queue
			if (!handlers) {
				handlers = events[type] = {};

				// Check for a special event handler
				// Only use addEventListener/attachEvent if the special
				// events handler returns false
				if ( !jQuery.event.special[type] || jQuery.event.special[type].setup.call(elem, data, namespaces) === false ) {
					// Bind the global event handler to the element
					if (elem.addEventListener)
						elem.addEventListener(type, handle, false);
					else if (elem.attachEvent)
						elem.attachEvent("on" + type, handle);
				}
			}

			// Add the function to the element's handler list
			handlers[handler.guid] = handler;

			// Keep track of which events have been used, for global triggering
			jQuery.event.global[type] = true;
		});

		// Nullify elem to prevent memory leaks in IE
		elem = null;
	},

	guid: 1,
	global: {},

	// Detach an event or set of events from an element
	remove: function(elem, types, handler) {
		// don't do events on text and comment nodes
		if ( elem.nodeType == 3 || elem.nodeType == 8 )
			return;

		var events = jQuery.data(elem, "events"), ret, index;

		if ( events ) {
			// Unbind all events for the element
			if ( types === undefined || (typeof types === "string" && types.charAt(0) == ".") )
				for ( var type in events )
					this.remove( elem, type + (types || "") );
			else {
				// types is actually an event object here
				if ( types.type ) {
					handler = types.handler;
					types = types.type;
				}

				// Handle multiple events seperated by a space
				// jQuery(...).unbind("mouseover mouseout", fn);
				jQuery.each(types.split(/\s+/), function(index, type){
					// Namespaced event handlers
					var namespaces = type.split(".");
					type = namespaces.shift();
					var namespace = RegExp("(^|\\.)" + namespaces.slice().sort().join(".*\\.") + "(\\.|$)");

					if ( events[type] ) {
						// remove the given handler for the given type
						if ( handler )
							delete events[type][handler.guid];

						// remove all handlers for the given type
						else
							for ( var handle in events[type] )
								// Handle the removal of namespaced events
								if ( namespace.test(events[type][handle].type) )
									delete events[type][handle];
									
						if ( jQuery.event.specialAll[type] )
							jQuery.event.specialAll[type].teardown.call(elem, namespaces);

						// remove generic event handler if no more handlers exist
						for ( ret in events[type] ) break;
						if ( !ret ) {
							if ( !jQuery.event.special[type] || jQuery.event.special[type].teardown.call(elem, namespaces) === false ) {
								if (elem.removeEventListener)
									elem.removeEventListener(type, jQuery.data(elem, "handle"), false);
								else if (elem.detachEvent)
									elem.detachEvent("on" + type, jQuery.data(elem, "handle"));
							}
							ret = null;
							delete events[type];
						}
					}
				});
			}

			// Remove the expando if it's no longer used
			for ( ret in events ) break;
			if ( !ret ) {
				var handle = jQuery.data( elem, "handle" );
				if ( handle ) handle.elem = null;
				jQuery.removeData( elem, "events" );
				jQuery.removeData( elem, "handle" );
			}
		}
	},

	// bubbling is internal
	trigger: function( event, data, elem, bubbling ) {
		// Event object or event type
		var type = event.type || event;

		if( !bubbling ){
			event = typeof event === "object" ?
				// jQuery.Event object
				event[expando] ? event :
				// Object literal
				jQuery.extend( jQuery.Event(type), event ) :
				// Just the event type (string)
				jQuery.Event(type);

			if ( type.indexOf("!") >= 0 ) {
				event.type = type = type.slice(0, -1);
				event.exclusive = true;
			}

			// Handle a global trigger
			if ( !elem ) {
				// Don't bubble custom events when global (to avoid too much overhead)
				event.stopPropagation();
				// Only trigger if we've ever bound an event for it
				if ( this.global[type] )
					jQuery.each( jQuery.cache, function(){
						if ( this.events && this.events[type] )
							jQuery.event.trigger( event, data, this.handle.elem );
					});
			}

			// Handle triggering a single element

			// don't do events on text and comment nodes
			if ( !elem || elem.nodeType == 3 || elem.nodeType == 8 )
				return undefined;
			
			// Clean up in case it is reused
			event.result = undefined;
			event.target = elem;
			
			// Clone the incoming data, if any
			data = jQuery.makeArray(data);
			data.unshift( event );
		}

		event.currentTarget = elem;

		// Trigger the event, it is assumed that "handle" is a function
		var handle = jQuery.data(elem, "handle");
		if ( handle )
			handle.apply( elem, data );

		// Handle triggering native .onfoo handlers (and on links since we don't call .click() for links)
		if ( (!elem[type] || (jQuery.nodeName(elem, 'a') && type == "click")) && elem["on"+type] && elem["on"+type].apply( elem, data ) === false )
			event.result = false;

		// Trigger the native events (except for clicks on links)
		if ( !bubbling && elem[type] && !event.isDefaultPrevented() && !(jQuery.nodeName(elem, 'a') && type == "click") ) {
			this.triggered = true;
			try {
				elem[ type ]();
			// prevent IE from throwing an error for some hidden elements
			} catch (e) {}
		}

		this.triggered = false;

		if ( !event.isPropagationStopped() ) {
			var parent = elem.parentNode || elem.ownerDocument;
			if ( parent )
				jQuery.event.trigger(event, data, parent, true);
		}
	},

	handle: function(event) {
		// returned undefined or false
		var all, handlers;

		event = arguments[0] = jQuery.event.fix( event || window.event );
		event.currentTarget = this;
		
		// Namespaced event handlers
		var namespaces = event.type.split(".");
		event.type = namespaces.shift();

		// Cache this now, all = true means, any handler
		all = !namespaces.length && !event.exclusive;
		
		var namespace = RegExp("(^|\\.)" + namespaces.slice().sort().join(".*\\.") + "(\\.|$)");

		handlers = ( jQuery.data(this, "events") || {} )[event.type];

		for ( var j in handlers ) {
			var handler = handlers[j];

			// Filter the functions by class
			if ( all || namespace.test(handler.type) ) {
				// Pass in a reference to the handler function itself
				// So that we can later remove it
				event.handler = handler;
				event.data = handler.data;

				var ret = handler.apply(this, arguments);

				if( ret !== undefined ){
					event.result = ret;
					if ( ret === false ) {
						event.preventDefault();
						event.stopPropagation();
					}
				}

				if( event.isImmediatePropagationStopped() )
					break;

			}
		}
	},

	props: "altKey attrChange attrName bubbles button cancelable charCode clientX clientY ctrlKey currentTarget data detail eventPhase fromElement handler keyCode metaKey newValue originalTarget pageX pageY prevValue relatedNode relatedTarget screenX screenY shiftKey srcElement target toElement view wheelDelta which".split(" "),

	fix: function(event) {
		if ( event[expando] )
			return event;

		// store a copy of the original event object
		// and "clone" to set read-only properties
		var originalEvent = event;
		event = jQuery.Event( originalEvent );

		for ( var i = this.props.length, prop; i; ){
			prop = this.props[ --i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Fix target property, if necessary
		if ( !event.target )
			event.target = event.srcElement || document; // Fixes #1925 where srcElement might not be defined either

		// check if target is a textnode (safari)
		if ( event.target.nodeType == 3 )
			event.target = event.target.parentNode;

		// Add relatedTarget, if necessary
		if ( !event.relatedTarget && event.fromElement )
			event.relatedTarget = event.fromElement == event.target ? event.toElement : event.fromElement;

		// Calculate pageX/Y if missing and clientX/Y available
		if ( event.pageX == null && event.clientX != null ) {
			var doc = document.documentElement, body = document.body;
			event.pageX = event.clientX + (doc && doc.scrollLeft || body && body.scrollLeft || 0) - (doc.clientLeft || 0);
			event.pageY = event.clientY + (doc && doc.scrollTop || body && body.scrollTop || 0) - (doc.clientTop || 0);
		}

		// Add which for key events
		if ( !event.which && ((event.charCode || event.charCode === 0) ? event.charCode : event.keyCode) )
			event.which = event.charCode || event.keyCode;

		// Add metaKey to non-Mac browsers (use ctrl for PC's and Meta for Macs)
		if ( !event.metaKey && event.ctrlKey )
			event.metaKey = event.ctrlKey;

		// Add which for click: 1 == left; 2 == middle; 3 == right
		// Note: button is not normalized, so don't use it
		if ( !event.which && event.button )
			event.which = (event.button & 1 ? 1 : ( event.button & 2 ? 3 : ( event.button & 4 ? 2 : 0 ) ));

		return event;
	},

	proxy: function( fn, proxy ){
		proxy = proxy || function(){ return fn.apply(this, arguments); };
		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || proxy.guid || this.guid++;
		// So proxy can be declared as an argument
		return proxy;
	},

	special: {
		ready: {
			// Make sure the ready event is setup
			setup: bindReady,
			teardown: function() {}
		}
	},
	
	specialAll: {
		live: {
			setup: function( selector, namespaces ){
				jQuery.event.add( this, namespaces[0], liveHandler );
			},
			teardown:  function( namespaces ){
				if ( namespaces.length ) {
					var remove = 0, name = RegExp("(^|\\.)" + namespaces[0] + "(\\.|$)");
					
					jQuery.each( (jQuery.data(this, "events").live || {}), function(){
						if ( name.test(this.type) )
							remove++;
					});
					
					if ( remove < 1 )
						jQuery.event.remove( this, namespaces[0], liveHandler );
				}
			}
		}
	}
};

jQuery.Event = function( src ){
	// Allow instantiation without the 'new' keyword
	if( !this.preventDefault )
		return new jQuery.Event(src);
	
	// Event object
	if( src && src.type ){
		this.originalEvent = src;
		this.type = src.type;
	// Event type
	}else
		this.type = src;

	// timeStamp is buggy for some events on Firefox(#3843)
	// So we won't rely on the native value
	this.timeStamp = now();
	
	// Mark it as fixed
	this[expando] = true;
};

function returnFalse(){
	return false;
}
function returnTrue(){
	return true;
}

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	preventDefault: function() {
		this.isDefaultPrevented = returnTrue;

		var e = this.originalEvent;
		if( !e )
			return;
		// if preventDefault exists run it on the original event
		if (e.preventDefault)
			e.preventDefault();
		// otherwise set the returnValue property of the original event to false (IE)
		e.returnValue = false;
	},
	stopPropagation: function() {
		this.isPropagationStopped = returnTrue;

		var e = this.originalEvent;
		if( !e )
			return;
		// if stopPropagation exists run it on the original event
		if (e.stopPropagation)
			e.stopPropagation();
		// otherwise set the cancelBubble property of the original event to true (IE)
		e.cancelBubble = true;
	},
	stopImmediatePropagation:function(){
		this.isImmediatePropagationStopped = returnTrue;
		this.stopPropagation();
	},
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse
};
// Checks if an event happened on an element within another element
// Used in jQuery.event.special.mouseenter and mouseleave handlers
var withinElement = function(event) {
	// Check if mouse(over|out) are still within the same parent element
	var parent = event.relatedTarget;
	// Traverse up the tree
	while ( parent && parent != this )
		try { parent = parent.parentNode; }
		catch(e) { parent = this; }
	
	if( parent != this ){
		// set the correct event type
		event.type = event.data;
		// handle event if we actually just moused on to a non sub-element
		jQuery.event.handle.apply( this, arguments );
	}
};
	
jQuery.each({ 
	mouseover: 'mouseenter', 
	mouseout: 'mouseleave'
}, function( orig, fix ){
	jQuery.event.special[ fix ] = {
		setup: function(){
			jQuery.event.add( this, orig, withinElement, fix );
		},
		teardown: function(){
			jQuery.event.remove( this, orig, withinElement );
		}
	};			   
});

jQuery.fn.extend({
	bind: function( type, data, fn ) {
		return type == "unload" ? this.one(type, data, fn) : this.each(function(){
			jQuery.event.add( this, type, fn || data, fn && data );
		});
	},

	one: function( type, data, fn ) {
		var one = jQuery.event.proxy( fn || data, function(event) {
			jQuery(this).unbind(event, one);
			return (fn || data).apply( this, arguments );
		});
		return this.each(function(){
			jQuery.event.add( this, type, one, fn && data);
		});
	},

	unbind: function( type, fn ) {
		return this.each(function(){
			jQuery.event.remove( this, type, fn );
		});
	},

	trigger: function( type, data ) {
		return this.each(function(){
			jQuery.event.trigger( type, data, this );
		});
	},

	triggerHandler: function( type, data ) {
		if( this[0] ){
			var event = jQuery.Event(type);
			event.preventDefault();
			event.stopPropagation();
			jQuery.event.trigger( event, data, this[0] );
			return event.result;
		}		
	},

	toggle: function( fn ) {
		// Save reference to arguments for access in closure
		var args = arguments, i = 1;

		// link all the functions, so any of them can unbind this click handler
		while( i < args.length )
			jQuery.event.proxy( fn, args[i++] );

		return this.click( jQuery.event.proxy( fn, function(event) {
			// Figure out which function to execute
			this.lastToggle = ( this.lastToggle || 0 ) % i;

			// Make sure that clicks stop
			event.preventDefault();

			// and execute the function
			return args[ this.lastToggle++ ].apply( this, arguments ) || false;
		}));
	},

	hover: function(fnOver, fnOut) {
		return this.mouseenter(fnOver).mouseleave(fnOut);
	},

	ready: function(fn) {
		// Attach the listeners
		bindReady();

		// If the DOM is already ready
		if ( jQuery.isReady )
			// Execute the function immediately
			fn.call( document, jQuery );

		// Otherwise, remember the function for later
		else
			// Add the function to the wait list
			jQuery.readyList.push( fn );

		return this;
	},
	
	live: function( type, fn ){
		var proxy = jQuery.event.proxy( fn );
		proxy.guid += this.selector + type;

		jQuery(document).bind( liveConvert(type, this.selector), this.selector, proxy );

		return this;
	},
	
	die: function( type, fn ){
		jQuery(document).unbind( liveConvert(type, this.selector), fn ? { guid: fn.guid + this.selector + type } : null );
		return this;
	}
});

function liveHandler( event ){
	var check = RegExp("(^|\\.)" + event.type + "(\\.|$)"),
		stop = true,
		elems = [];

	jQuery.each(jQuery.data(this, "events").live || [], function(i, fn){
		if ( check.test(fn.type) ) {
			var elem = jQuery(event.target).closest(fn.data)[0];
			if ( elem )
				elems.push({ elem: elem, fn: fn });
		}
	});

	elems.sort(function(a,b) {
		return jQuery.data(a.elem, "closest") - jQuery.data(b.elem, "closest");
	});
	
	jQuery.each(elems, function(){
		if ( this.fn.call(this.elem, event, this.fn.data) === false )
			return (stop = false);
	});

	return stop;
}

function liveConvert(type, selector){
	return ["live", type, selector.replace(/\./g, "`").replace(/ /g, "|")].join(".");
}

jQuery.extend({
	isReady: false,
	readyList: [],
	// Handle when the DOM is ready
	ready: function() {
		// Make sure that the DOM is not already loaded
		if ( !jQuery.isReady ) {
			// Remember that the DOM is ready
			jQuery.isReady = true;

			// If there are functions bound, to execute
			if ( jQuery.readyList ) {
				// Execute all of them
				jQuery.each( jQuery.readyList, function(){
					this.call( document, jQuery );
				});

				// Reset the list of functions
				jQuery.readyList = null;
			}

			// Trigger any bound ready events
			jQuery(document).triggerHandler("ready");
		}
	}
});

var readyBound = false;

function bindReady(){
	if ( readyBound ) return;
	readyBound = true;

	// Mozilla, Opera and webkit nightlies currently support this event
	if ( document.addEventListener ) {
		// Use the handy event callback
		document.addEventListener( "DOMContentLoaded", function(){
			document.removeEventListener( "DOMContentLoaded", arguments.callee, false );
			jQuery.ready();
		}, false );

	// If IE event model is used
	} else if ( document.attachEvent ) {
		// ensure firing before onload,
		// maybe late but safe also for iframes
		document.attachEvent("onreadystatechange", function(){
			if ( document.readyState === "complete" ) {
				document.detachEvent( "onreadystatechange", arguments.callee );
				jQuery.ready();
			}
		});

		// If IE and not an iframe
		// continually check to see if the document is ready
		if ( document.documentElement.doScroll && window == window.top ) (function(){
			if ( jQuery.isReady ) return;

			try {
				// If IE is used, use the trick by Diego Perini
				// http://javascript.nwbox.com/IEContentLoaded/
				document.documentElement.doScroll("left");
			} catch( error ) {
				setTimeout( arguments.callee, 0 );
				return;
			}

			// and execute any waiting functions
			jQuery.ready();
		})();
	}

	// A fallback to window.onload, that will always work
	jQuery.event.add( window, "load", jQuery.ready );
}

jQuery.each( ("blur,focus,load,resize,scroll,unload,click,dblclick," +
	"mousedown,mouseup,mousemove,mouseover,mouseout,mouseenter,mouseleave," +
	"change,select,submit,keydown,keypress,keyup,error").split(","), function(i, name){

	// Handle event binding
	jQuery.fn[name] = function(fn){
		return fn ? this.bind(name, fn) : this.trigger(name);
	};
});

// Prevent memory leaks in IE
// And prevent errors on refresh with events like mouseover in other browsers
// Window isn't included so as not to unbind existing unload events
jQuery( window ).bind( 'unload', function(){ 
	for ( var id in jQuery.cache )
		// Skip the window
		if ( id != 1 && jQuery.cache[ id ].handle )
			jQuery.event.remove( jQuery.cache[ id ].handle.elem );
}); 
(function(){

	jQuery.support = {};

	var root = document.documentElement,
		script = document.createElement("script"),
		div = document.createElement("div"),
		id = "script" + (new Date).getTime();

	div.style.display = "none";
	div.innerHTML = '   <link/><table></table><a href="/a" style="color:red;float:left;opacity:.5;">a</a><select><option>text</option></select><object><param/></object>';

	var all = div.getElementsByTagName("*"),
		a = div.getElementsByTagName("a")[0];

	// Can't get basic test support
	if ( !all || !all.length || !a ) {
		return;
	}

	jQuery.support = {
		// IE strips leading whitespace when .innerHTML is used
		leadingWhitespace: div.firstChild.nodeType == 3,
		
		// Make sure that tbody elements aren't automatically inserted
		// IE will insert them into empty tables
		tbody: !div.getElementsByTagName("tbody").length,
		
		// Make sure that you can get all elements in an <object> element
		// IE 7 always returns no results
		objectAll: !!div.getElementsByTagName("object")[0]
			.getElementsByTagName("*").length,
		
		// Make sure that link elements get serialized correctly by innerHTML
		// This requires a wrapper element in IE
		htmlSerialize: !!div.getElementsByTagName("link").length,
		
		// Get the style information from getAttribute
		// (IE uses .cssText insted)
		style: /red/.test( a.getAttribute("style") ),
		
		// Make sure that URLs aren't manipulated
		// (IE normalizes it by default)
		hrefNormalized: a.getAttribute("href") === "/a",
		
		// Make sure that element opacity exists
		// (IE uses filter instead)
		opacity: a.style.opacity === "0.5",
		
		// Verify style float existence
		// (IE uses styleFloat instead of cssFloat)
		cssFloat: !!a.style.cssFloat,

		// Will be defined later
		scriptEval: false,
		noCloneEvent: true,
		boxModel: null
	};
	
	script.type = "text/javascript";
	try {
		script.appendChild( document.createTextNode( "window." + id + "=1;" ) );
	} catch(e){}

	root.insertBefore( script, root.firstChild );
	
	// Make sure that the execution of code works by injecting a script
	// tag with appendChild/createTextNode
	// (IE doesn't support this, fails, and uses .text instead)
	if ( window[ id ] ) {
		jQuery.support.scriptEval = true;
		delete window[ id ];
	}

	root.removeChild( script );

	if ( div.attachEvent && div.fireEvent ) {
		div.attachEvent("onclick", function(){
			// Cloning a node shouldn't copy over any
			// bound event handlers (IE does this)
			jQuery.support.noCloneEvent = false;
			div.detachEvent("onclick", arguments.callee);
		});
		div.cloneNode(true).fireEvent("onclick");
	}

	// Figure out if the W3C box model works as expected
	// document.body must exist before we can do this
	jQuery(function(){
		var div = document.createElement("div");
		div.style.width = div.style.paddingLeft = "1px";

		document.body.appendChild( div );
		jQuery.boxModel = jQuery.support.boxModel = div.offsetWidth === 2;
		document.body.removeChild( div ).style.display = 'none';
	});
})();

var styleFloat = jQuery.support.cssFloat ? "cssFloat" : "styleFloat";

jQuery.props = {
	"for": "htmlFor",
	"class": "className",
	"float": styleFloat,
	cssFloat: styleFloat,
	styleFloat: styleFloat,
	readonly: "readOnly",
	maxlength: "maxLength",
	cellspacing: "cellSpacing",
	rowspan: "rowSpan",
	tabindex: "tabIndex"
};
jQuery.fn.extend({
	// Keep a copy of the old load
	_load: jQuery.fn.load,

	load: function( url, params, callback ) {
		if ( typeof url !== "string" )
			return this._load( url );

		var off = url.indexOf(" ");
		if ( off >= 0 ) {
			var selector = url.slice(off, url.length);
			url = url.slice(0, off);
		}

		// Default to a GET request
		var type = "GET";

		// If the second parameter was provided
		if ( params )
			// If it's a function
			if ( jQuery.isFunction( params ) ) {
				// We assume that it's the callback
				callback = params;
				params = null;

			// Otherwise, build a param string
			} else if( typeof params === "object" ) {
				params = jQuery.param( params );
				type = "POST";
			}

		var self = this;

		// Request the remote document
		jQuery.ajax({
			url: url,
			type: type,
			dataType: "html",
			data: params,
			complete: function(res, status){
				// If successful, inject the HTML into all the matched elements
				if ( status == "success" || status == "notmodified" )
					// See if a selector was specified
					self.html( selector ?
						// Create a dummy div to hold the results
						jQuery("<div/>")
							// inject the contents of the document in, removing the scripts
							// to avoid any 'Permission Denied' errors in IE
							.append(res.responseText.replace(/<script(.|\s)*?\/script>/g, ""))

							// Locate the specified elements
							.find(selector) :

						// If not, just inject the full result
						res.responseText );

				if( callback )
					self.each( callback, [res.responseText, status, res] );
			}
		});
		return this;
	},

	serialize: function() {
		return jQuery.param(this.serializeArray());
	},
	serializeArray: function() {
		return this.map(function(){
			return this.elements ? jQuery.makeArray(this.elements) : this;
		})
		.filter(function(){
			return this.name && !this.disabled &&
				(this.checked || /select|textarea/i.test(this.nodeName) ||
					/text|hidden|password|search/i.test(this.type));
		})
		.map(function(i, elem){
			var val = jQuery(this).val();
			return val == null ? null :
				jQuery.isArray(val) ?
					jQuery.map( val, function(val, i){
						return {name: elem.name, value: val};
					}) :
					{name: elem.name, value: val};
		}).get();
	}
});

// Attach a bunch of functions for handling common AJAX events
jQuery.each( "ajaxStart,ajaxStop,ajaxComplete,ajaxError,ajaxSuccess,ajaxSend".split(","), function(i,o){
	jQuery.fn[o] = function(f){
		return this.bind(o, f);
	};
});

var jsc = now();

jQuery.extend({
  
	get: function( url, data, callback, type ) {
		// shift arguments if data argument was ommited
		if ( jQuery.isFunction( data ) ) {
			callback = data;
			data = null;
		}

		return jQuery.ajax({
			type: "GET",
			url: url,
			data: data,
			success: callback,
			dataType: type
		});
	},

	getScript: function( url, callback ) {
		return jQuery.get(url, null, callback, "script");
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get(url, data, callback, "json");
	},

	post: function( url, data, callback, type ) {
		if ( jQuery.isFunction( data ) ) {
			callback = data;
			data = {};
		}

		return jQuery.ajax({
			type: "POST",
			url: url,
			data: data,
			success: callback,
			dataType: type
		});
	},

	ajaxSetup: function( settings ) {
		jQuery.extend( jQuery.ajaxSettings, settings );
	},

	ajaxSettings: {
		url: location.href,
		global: true,
		type: "GET",
		contentType: "application/x-www-form-urlencoded",
		processData: true,
		async: true,
		/*
		timeout: 0,
		data: null,
		username: null,
		password: null,
		*/
		// Create the request object; Microsoft failed to properly
		// implement the XMLHttpRequest in IE7, so we use the ActiveXObject when it is available
		// This function can be overriden by calling jQuery.ajaxSetup
		xhr:function(){
			return window.ActiveXObject ? new ActiveXObject("Microsoft.XMLHTTP") : new XMLHttpRequest();
		},
		accepts: {
			xml: "application/xml, text/xml",
			html: "text/html",
			script: "text/javascript, application/javascript",
			json: "application/json, text/javascript",
			text: "text/plain",
			_default: "*/*"
		}
	},

	// Last-Modified header cache for next request
	lastModified: {},

	ajax: function( s ) {
		// Extend the settings, but re-extend 's' so that it can be
		// checked again later (in the test suite, specifically)
		s = jQuery.extend(true, s, jQuery.extend(true, {}, jQuery.ajaxSettings, s));

		var jsonp, jsre = /=\?(&|$)/g, status, data,
			type = s.type.toUpperCase();

		// convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" )
			s.data = jQuery.param(s.data);

		// Handle JSONP Parameter Callbacks
		if ( s.dataType == "jsonp" ) {
			if ( type == "GET" ) {
				if ( !s.url.match(jsre) )
					s.url += (s.url.match(/\?/) ? "&" : "?") + (s.jsonp || "callback") + "=?";
			} else if ( !s.data || !s.data.match(jsre) )
				s.data = (s.data ? s.data + "&" : "") + (s.jsonp || "callback") + "=?";
			s.dataType = "json";
		}

		// Build temporary JSONP function
		if ( s.dataType == "json" && (s.data && s.data.match(jsre) || s.url.match(jsre)) ) {
			jsonp = "jsonp" + jsc++;

			// Replace the =? sequence both in the query string and the data
			if ( s.data )
				s.data = (s.data + "").replace(jsre, "=" + jsonp + "$1");
			s.url = s.url.replace(jsre, "=" + jsonp + "$1");

			// We need to make sure
			// that a JSONP style response is executed properly
			s.dataType = "script";

			// Handle JSONP-style loading
			window[ jsonp ] = function(tmp){
				data = tmp;
				success();
				complete();
				// Garbage collect
				window[ jsonp ] = undefined;
				try{ delete window[ jsonp ]; } catch(e){}
				if ( head )
					head.removeChild( script );
			};
		}

		if ( s.dataType == "script" && s.cache == null )
			s.cache = false;

		if ( s.cache === false && type == "GET" ) {
			var ts = now();
			// try replacing _= if it is there
			var ret = s.url.replace(/(\?|&)_=.*?(&|$)/, "$1_=" + ts + "$2");
			// if nothing was replaced, add timestamp to the end
			s.url = ret + ((ret == s.url) ? (s.url.match(/\?/) ? "&" : "?") + "_=" + ts : "");
		}

		// If data is available, append data to url for get requests
		if ( s.data && type == "GET" ) {
			s.url += (s.url.match(/\?/) ? "&" : "?") + s.data;

			// IE likes to send both get and post data, prevent this
			s.data = null;
		}

		// Watch for a new set of requests
		if ( s.global && ! jQuery.active++ )
			jQuery.event.trigger( "ajaxStart" );

		// Matches an absolute URL, and saves the domain
		var parts = /^(\w+:)?\/\/([^\/?#]+)/.exec( s.url );

		// If we're requesting a remote document
		// and trying to load JSON or Script with a GET
		if ( s.dataType == "script" && type == "GET" && parts
			&& ( parts[1] && parts[1] != location.protocol || parts[2] != location.host )){

			var head = document.getElementsByTagName("head")[0];
			var script = document.createElement("script");
			script.src = s.url;
			if (s.scriptCharset)
				script.charset = s.scriptCharset;

			// Handle Script loading
			if ( !jsonp ) {
				var done = false;

				// Attach handlers for all browsers
				script.onload = script.onreadystatechange = function(){
					if ( !done && (!this.readyState ||
							this.readyState == "loaded" || this.readyState == "complete") ) {
						done = true;
						success();
						complete();

						// Handle memory leak in IE
						script.onload = script.onreadystatechange = null;
						head.removeChild( script );
					}
				};
			}

			head.appendChild(script);

			// We handle everything using the script element injection
			return undefined;
		}

		var requestDone = false;

		// Create the request object
		var xhr = s.xhr();

		// Open the socket
		// Passing null username, generates a login popup on Opera (#2865)
		if( s.username )
			xhr.open(type, s.url, s.async, s.username, s.password);
		else
			xhr.open(type, s.url, s.async);

		// Need an extra try/catch for cross domain requests in Firefox 3
		try {
			// Set the correct header, if data is being sent
			if ( s.data )
				xhr.setRequestHeader("Content-Type", s.contentType);

			// Set the If-Modified-Since header, if ifModified mode.
			if ( s.ifModified )
				xhr.setRequestHeader("If-Modified-Since",
					jQuery.lastModified[s.url] || "Thu, 01 Jan 1970 00:00:00 GMT" );

			// Set header so the called script knows that it's an XMLHttpRequest
			xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");

			// Set the Accepts header for the server, depending on the dataType
			xhr.setRequestHeader("Accept", s.dataType && s.accepts[ s.dataType ] ?
				s.accepts[ s.dataType ] + ", */*" :
				s.accepts._default );
		} catch(e){}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend && s.beforeSend(xhr, s) === false ) {
			// Handle the global AJAX counter
			if ( s.global && ! --jQuery.active )
				jQuery.event.trigger( "ajaxStop" );
			// close opended socket
			xhr.abort();
			return false;
		}

		if ( s.global )
			jQuery.event.trigger("ajaxSend", [xhr, s]);

		// Wait for a response to come back
		var onreadystatechange = function(isTimeout){
			// The request was aborted, clear the interval and decrement jQuery.active
			if (xhr.readyState == 0) {
				if (ival) {
					// clear poll interval
					clearInterval(ival);
					ival = null;
					// Handle the global AJAX counter
					if ( s.global && ! --jQuery.active )
						jQuery.event.trigger( "ajaxStop" );
				}
			// The transfer is complete and the data is available, or the request timed out
			} else if ( !requestDone && xhr && (xhr.readyState == 4 || isTimeout == "timeout") ) {
				requestDone = true;

				// clear poll interval
				if (ival) {
					clearInterval(ival);
					ival = null;
				}

				status = isTimeout == "timeout" ? "timeout" :
					!jQuery.httpSuccess( xhr ) ? "error" :
					s.ifModified && jQuery.httpNotModified( xhr, s.url ) ? "notmodified" :
					"success";

				if ( status == "success" ) {
					// Watch for, and catch, XML document parse errors
					try {
						// process the data (runs the xml through httpData regardless of callback)
						data = jQuery.httpData( xhr, s.dataType, s );
					} catch(e) {
						status = "parsererror";
					}
				}

				// Make sure that the request was successful or notmodified
				if ( status == "success" ) {
					// Cache Last-Modified header, if ifModified mode.
					var modRes;
					try {
						modRes = xhr.getResponseHeader("Last-Modified");
					} catch(e) {} // swallow exception thrown by FF if header is not available

					if ( s.ifModified && modRes )
						jQuery.lastModified[s.url] = modRes;

					// JSONP handles its own success callback
					if ( !jsonp )
						success();
				} else
					jQuery.handleError(s, xhr, status);

				// Fire the complete handlers
				complete();

				if ( isTimeout )
					xhr.abort();

				// Stop memory leaks
				if ( s.async )
					xhr = null;
			}
		};

		if ( s.async ) {
			// don't attach the handler to the request, just poll it instead
			var ival = setInterval(onreadystatechange, 13);

			// Timeout checker
			if ( s.timeout > 0 )
				setTimeout(function(){
					// Check to see if the request is still happening
					if ( xhr && !requestDone )
						onreadystatechange( "timeout" );
				}, s.timeout);
		}

		// Send the data
		try {
			xhr.send(s.data);
		} catch(e) {
			jQuery.handleError(s, xhr, null, e);
		}

		// firefox 1.5 doesn't fire statechange for sync requests
		if ( !s.async )
			onreadystatechange();

		function success(){
			// If a local callback was specified, fire it and pass it the data
			if ( s.success )
				s.success( data, status );

			// Fire the global callback
			if ( s.global )
				jQuery.event.trigger( "ajaxSuccess", [xhr, s] );
		}

		function complete(){
			// Process result
			if ( s.complete )
				s.complete(xhr, status);

			// The request was completed
			if ( s.global )
				jQuery.event.trigger( "ajaxComplete", [xhr, s] );

			// Handle the global AJAX counter
			if ( s.global && ! --jQuery.active )
				jQuery.event.trigger( "ajaxStop" );
		}

		// return XMLHttpRequest to allow aborting the request etc.
		return xhr;
	},

	handleError: function( s, xhr, status, e ) {
		// If a local callback was specified, fire it
		if ( s.error ) s.error( xhr, status, e );

		// Fire the global callback
		if ( s.global )
			jQuery.event.trigger( "ajaxError", [xhr, s, e] );
	},

	// Counter for holding the number of active queries
	active: 0,

	// Determines if an XMLHttpRequest was successful or not
	httpSuccess: function( xhr ) {
		try {
			// IE error sometimes returns 1223 when it should be 204 so treat it as success, see #1450
			return !xhr.status && location.protocol == "file:" ||
				( xhr.status >= 200 && xhr.status < 300 ) || xhr.status == 304 || xhr.status == 1223;
		} catch(e){}
		return false;
	},

	// Determines if an XMLHttpRequest returns NotModified
	httpNotModified: function( xhr, url ) {
		try {
			var xhrRes = xhr.getResponseHeader("Last-Modified");

			// Firefox always returns 200. check Last-Modified date
			return xhr.status == 304 || xhrRes == jQuery.lastModified[url];
		} catch(e){}
		return false;
	},

	httpData: function( xhr, type, s ) {
		var ct = xhr.getResponseHeader("content-type"),
			xml = type == "xml" || !type && ct && ct.indexOf("xml") >= 0,
			data = xml ? xhr.responseXML : xhr.responseText;

		if ( xml && data.documentElement.tagName == "parsererror" )
			throw "parsererror";
			
		// Allow a pre-filtering function to sanitize the response
		// s != null is checked to keep backwards compatibility
		if( s && s.dataFilter )
			data = s.dataFilter( data, type );

		// The filter can actually parse the response
		if( typeof data === "string" ){

			// If the type is "script", eval it in global context
			if ( type == "script" )
				jQuery.globalEval( data );

			// Get the JavaScript object, if JSON is used.
			if ( type == "json" )
				data = window["eval"]("(" + data + ")");
		}
		
		return data;
	},

	// Serialize an array of form elements or a set of
	// key/values into a query string
	param: function( a ) {
		var s = [ ];

		function add( key, value ){
			s[ s.length ] = encodeURIComponent(key) + '=' + encodeURIComponent(value);
		};

		// If an array was passed in, assume that it is an array
		// of form elements
		if ( jQuery.isArray(a) || a.jquery )
			// Serialize the form elements
			jQuery.each( a, function(){
				add( this.name, this.value );
			});

		// Otherwise, assume that it's an object of key/value pairs
		else
			// Serialize the key/values
			for ( var j in a )
				// If the value is an array then the key names need to be repeated
				if ( jQuery.isArray(a[j]) )
					jQuery.each( a[j], function(){
						add( j, this );
					});
				else
					add( j, jQuery.isFunction(a[j]) ? a[j]() : a[j] );

		// Return the resulting serialization
		return s.join("&").replace(/%20/g, "+");
	}

});
var elemdisplay = {},
	timerId,
	fxAttrs = [
		// height animations
		[ "height", "marginTop", "marginBottom", "paddingTop", "paddingBottom" ],
		// width animations
		[ "width", "marginLeft", "marginRight", "paddingLeft", "paddingRight" ],
		// opacity animations
		[ "opacity" ]
	];

function genFx( type, num ){
	var obj = {};
	jQuery.each( fxAttrs.concat.apply([], fxAttrs.slice(0,num)), function(){
		obj[ this ] = type;
	});
	return obj;
}

jQuery.fn.extend({
	show: function(speed,callback){
		if ( speed ) {
			return this.animate( genFx("show", 3), speed, callback);
		} else {
			for ( var i = 0, l = this.length; i < l; i++ ){
				var old = jQuery.data(this[i], "olddisplay");
				
				this[i].style.display = old || "";
				
				if ( jQuery.css(this[i], "display") === "none" ) {
					var tagName = this[i].tagName, display;
					
					if ( elemdisplay[ tagName ] ) {
						display = elemdisplay[ tagName ];
					} else {
						var elem = jQuery("<" + tagName + " />").appendTo("body");
						
						display = elem.css("display");
						if ( display === "none" )
							display = "block";
						
						elem.remove();
						
						elemdisplay[ tagName ] = display;
					}
					
					jQuery.data(this[i], "olddisplay", display);
				}
			}

			// Set the display of the elements in a second loop
			// to avoid the constant reflow
			for ( var i = 0, l = this.length; i < l; i++ ){
				this[i].style.display = jQuery.data(this[i], "olddisplay") || "";
			}
			
			return this;
		}
	},

	hide: function(speed,callback){
		if ( speed ) {
			return this.animate( genFx("hide", 3), speed, callback);
		} else {
			for ( var i = 0, l = this.length; i < l; i++ ){
				var old = jQuery.data(this[i], "olddisplay");
				if ( !old && old !== "none" )
					jQuery.data(this[i], "olddisplay", jQuery.css(this[i], "display"));
			}

			// Set the display of the elements in a second loop
			// to avoid the constant reflow
			for ( var i = 0, l = this.length; i < l; i++ ){
				this[i].style.display = "none";
			}

			return this;
		}
	},

	// Save the old toggle function
	_toggle: jQuery.fn.toggle,

	toggle: function( fn, fn2 ){
		var bool = typeof fn === "boolean";

		return jQuery.isFunction(fn) && jQuery.isFunction(fn2) ?
			this._toggle.apply( this, arguments ) :
			fn == null || bool ?
				this.each(function(){
					var state = bool ? fn : jQuery(this).is(":hidden");
					jQuery(this)[ state ? "show" : "hide" ]();
				}) :
				this.animate(genFx("toggle", 3), fn, fn2);
	},

	fadeTo: function(speed,to,callback){
		return this.animate({opacity: to}, speed, callback);
	},

	animate: function( prop, speed, easing, callback ) {
		var optall = jQuery.speed(speed, easing, callback);

		return this[ optall.queue === false ? "each" : "queue" ](function(){
		
			var opt = jQuery.extend({}, optall), p,
				hidden = this.nodeType == 1 && jQuery(this).is(":hidden"),
				self = this;
	
			for ( p in prop ) {
				if ( prop[p] == "hide" && hidden || prop[p] == "show" && !hidden )
					return opt.complete.call(this);

				if ( ( p == "height" || p == "width" ) && this.style ) {
					// Store display property
					opt.display = jQuery.css(this, "display");

					// Make sure that nothing sneaks out
					opt.overflow = this.style.overflow;
				}
			}

			if ( opt.overflow != null )
				this.style.overflow = "hidden";

			opt.curAnim = jQuery.extend({}, prop);

			jQuery.each( prop, function(name, val){
				var e = new jQuery.fx( self, opt, name );

				if ( /toggle|show|hide/.test(val) )
					e[ val == "toggle" ? hidden ? "show" : "hide" : val ]( prop );
				else {
					var parts = val.toString().match(/^([+-]=)?([\d+-.]+)(.*)$/),
						start = e.cur(true) || 0;

					if ( parts ) {
						var end = parseFloat(parts[2]),
							unit = parts[3] || "px";

						// We need to compute starting value
						if ( unit != "px" ) {
							self.style[ name ] = (end || 1) + unit;
							start = ((end || 1) / e.cur(true)) * start;
							self.style[ name ] = start + unit;
						}

						// If a +=/-= token was provided, we're doing a relative animation
						if ( parts[1] )
							end = ((parts[1] == "-=" ? -1 : 1) * end) + start;

						e.custom( start, end, unit );
					} else
						e.custom( start, val, "" );
				}
			});

			// For JS strict compliance
			return true;
		});
	},

	stop: function(clearQueue, gotoEnd){
		var timers = jQuery.timers;

		if (clearQueue)
			this.queue([]);

		this.each(function(){
			// go in reverse order so anything added to the queue during the loop is ignored
			for ( var i = timers.length - 1; i >= 0; i-- )
				if ( timers[i].elem == this ) {
					if (gotoEnd)
						// force the next step to be the last
						timers[i](true);
					timers.splice(i, 1);
				}
		});

		// start the next in the queue if the last step wasn't forced
		if (!gotoEnd)
			this.dequeue();

		return this;
	}

});

// Generate shortcuts for custom animations
jQuery.each({
	slideDown: genFx("show", 1),
	slideUp: genFx("hide", 1),
	slideToggle: genFx("toggle", 1),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" }
}, function( name, props ){
	jQuery.fn[ name ] = function( speed, callback ){
		return this.animate( props, speed, callback );
	};
});

jQuery.extend({

	speed: function(speed, easing, fn) {
		var opt = typeof speed === "object" ? speed : {
			complete: fn || !fn && easing ||
				jQuery.isFunction( speed ) && speed,
			duration: speed,
			easing: fn && easing || easing && !jQuery.isFunction(easing) && easing
		};

		opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ? opt.duration :
			jQuery.fx.speeds[opt.duration] || jQuery.fx.speeds._default;

		// Queueing
		opt.old = opt.complete;
		opt.complete = function(){
			if ( opt.queue !== false )
				jQuery(this).dequeue();
			if ( jQuery.isFunction( opt.old ) )
				opt.old.call( this );
		};

		return opt;
	},

	easing: {
		linear: function( p, n, firstNum, diff ) {
			return firstNum + diff * p;
		},
		swing: function( p, n, firstNum, diff ) {
			return ((-Math.cos(p*Math.PI)/2) + 0.5) * diff + firstNum;
		}
	},

	timers: [],

	fx: function( elem, options, prop ){
		this.options = options;
		this.elem = elem;
		this.prop = prop;

		if ( !options.orig )
			options.orig = {};
	}

});

jQuery.fx.prototype = {

	// Simple function for setting a style value
	update: function(){
		if ( this.options.step )
			this.options.step.call( this.elem, this.now, this );

		(jQuery.fx.step[this.prop] || jQuery.fx.step._default)( this );

		// Set display property to block for height/width animations
		if ( ( this.prop == "height" || this.prop == "width" ) && this.elem.style )
			this.elem.style.display = "block";
	},

	// Get the current size
	cur: function(force){
		if ( this.elem[this.prop] != null && (!this.elem.style || this.elem.style[this.prop] == null) )
			return this.elem[ this.prop ];

		var r = parseFloat(jQuery.css(this.elem, this.prop, force));
		return r && r > -10000 ? r : parseFloat(jQuery.curCSS(this.elem, this.prop)) || 0;
	},

	// Start an animation from one number to another
	custom: function(from, to, unit){
		this.startTime = now();
		this.start = from;
		this.end = to;
		this.unit = unit || this.unit || "px";
		this.now = this.start;
		this.pos = this.state = 0;

		var self = this;
		function t(gotoEnd){
			return self.step(gotoEnd);
		}

		t.elem = this.elem;

		if ( t() && jQuery.timers.push(t) && !timerId ) {
			timerId = setInterval(function(){
				var timers = jQuery.timers;

				for ( var i = 0; i < timers.length; i++ )
					if ( !timers[i]() )
						timers.splice(i--, 1);

				if ( !timers.length ) {
					clearInterval( timerId );
					timerId = undefined;
				}
			}, 13);
		}
	},

	// Simple 'show' function
	show: function(){
		// Remember where we started, so that we can go back to it later
		this.options.orig[this.prop] = jQuery.attr( this.elem.style, this.prop );
		this.options.show = true;

		// Begin the animation
		// Make sure that we start at a small width/height to avoid any
		// flash of content
		this.custom(this.prop == "width" || this.prop == "height" ? 1 : 0, this.cur());

		// Start by showing the element
		jQuery(this.elem).show();
	},

	// Simple 'hide' function
	hide: function(){
		// Remember where we started, so that we can go back to it later
		this.options.orig[this.prop] = jQuery.attr( this.elem.style, this.prop );
		this.options.hide = true;

		// Begin the animation
		this.custom(this.cur(), 0);
	},

	// Each step of an animation
	step: function(gotoEnd){
		var t = now();

		if ( gotoEnd || t >= this.options.duration + this.startTime ) {
			this.now = this.end;
			this.pos = this.state = 1;
			this.update();

			this.options.curAnim[ this.prop ] = true;

			var done = true;
			for ( var i in this.options.curAnim )
				if ( this.options.curAnim[i] !== true )
					done = false;

			if ( done ) {
				if ( this.options.display != null ) {
					// Reset the overflow
					this.elem.style.overflow = this.options.overflow;

					// Reset the display
					this.elem.style.display = this.options.display;
					if ( jQuery.css(this.elem, "display") == "none" )
						this.elem.style.display = "block";
				}

				// Hide the element if the "hide" operation was done
				if ( this.options.hide )
					jQuery(this.elem).hide();

				// Reset the properties, if the item has been hidden or shown
				if ( this.options.hide || this.options.show )
					for ( var p in this.options.curAnim )
						jQuery.attr(this.elem.style, p, this.options.orig[p]);
					
				// Execute the complete function
				this.options.complete.call( this.elem );
			}

			return false;
		} else {
			var n = t - this.startTime;
			this.state = n / this.options.duration;

			// Perform the easing function, defaults to swing
			this.pos = jQuery.easing[this.options.easing || (jQuery.easing.swing ? "swing" : "linear")](this.state, n, 0, 1, this.options.duration);
			this.now = this.start + ((this.end - this.start) * this.pos);

			// Perform the next step of the animation
			this.update();
		}

		return true;
	}

};

jQuery.extend( jQuery.fx, {
	speeds:{
		slow: 600,
 		fast: 200,
 		// Default speed
 		_default: 400
	},
	step: {

		opacity: function(fx){
			jQuery.attr(fx.elem.style, "opacity", fx.now);
		},

		_default: function(fx){
			if ( fx.elem.style && fx.elem.style[ fx.prop ] != null )
				fx.elem.style[ fx.prop ] = fx.now + fx.unit;
			else
				fx.elem[ fx.prop ] = fx.now;
		}
	}
});
if ( document.documentElement["getBoundingClientRect"] )
	jQuery.fn.offset = function() {
		if ( !this[0] ) return { top: 0, left: 0 };
		if ( this[0] === this[0].ownerDocument.body ) return jQuery.offset.bodyOffset( this[0] );
		var box  = this[0].getBoundingClientRect(), doc = this[0].ownerDocument, body = doc.body, docElem = doc.documentElement,
			clientTop = docElem.clientTop || body.clientTop || 0, clientLeft = docElem.clientLeft || body.clientLeft || 0,
			top  = box.top  + (self.pageYOffset || jQuery.boxModel && docElem.scrollTop  || body.scrollTop ) - clientTop,
			left = box.left + (self.pageXOffset || jQuery.boxModel && docElem.scrollLeft || body.scrollLeft) - clientLeft;
		return { top: top, left: left };
	};
else 
	jQuery.fn.offset = function() {
		if ( !this[0] ) return { top: 0, left: 0 };
		if ( this[0] === this[0].ownerDocument.body ) return jQuery.offset.bodyOffset( this[0] );
		jQuery.offset.initialized || jQuery.offset.initialize();

		var elem = this[0], offsetParent = elem.offsetParent, prevOffsetParent = elem,
			doc = elem.ownerDocument, computedStyle, docElem = doc.documentElement,
			body = doc.body, defaultView = doc.defaultView,
			prevComputedStyle = defaultView.getComputedStyle(elem, null),
			top = elem.offsetTop, left = elem.offsetLeft;

		while ( (elem = elem.parentNode) && elem !== body && elem !== docElem ) {
			computedStyle = defaultView.getComputedStyle(elem, null);
			top -= elem.scrollTop, left -= elem.scrollLeft;
			if ( elem === offsetParent ) {
				top += elem.offsetTop, left += elem.offsetLeft;
				if ( jQuery.offset.doesNotAddBorder && !(jQuery.offset.doesAddBorderForTableAndCells && /^t(able|d|h)$/i.test(elem.tagName)) )
					top  += parseInt( computedStyle.borderTopWidth,  10) || 0,
					left += parseInt( computedStyle.borderLeftWidth, 10) || 0;
				prevOffsetParent = offsetParent, offsetParent = elem.offsetParent;
			}
			if ( jQuery.offset.subtractsBorderForOverflowNotVisible && computedStyle.overflow !== "visible" )
				top  += parseInt( computedStyle.borderTopWidth,  10) || 0,
				left += parseInt( computedStyle.borderLeftWidth, 10) || 0;
			prevComputedStyle = computedStyle;
		}

		if ( prevComputedStyle.position === "relative" || prevComputedStyle.position === "static" )
			top  += body.offsetTop,
			left += body.offsetLeft;

		if ( prevComputedStyle.position === "fixed" )
			top  += Math.max(docElem.scrollTop, body.scrollTop),
			left += Math.max(docElem.scrollLeft, body.scrollLeft);

		return { top: top, left: left };
	};

jQuery.offset = {
	initialize: function() {
		if ( this.initialized ) return;
		var body = document.body, container = document.createElement('div'), innerDiv, checkDiv, table, td, rules, prop, bodyMarginTop = body.style.marginTop,
			html = '<div style="position:absolute;top:0;left:0;margin:0;border:5px solid #000;padding:0;width:1px;height:1px;"><div></div></div><table style="position:absolute;top:0;left:0;margin:0;border:5px solid #000;padding:0;width:1px;height:1px;" cellpadding="0" cellspacing="0"><tr><td></td></tr></table>';

		rules = { position: 'absolute', top: 0, left: 0, margin: 0, border: 0, width: '1px', height: '1px', visibility: 'hidden' };
		for ( prop in rules ) container.style[prop] = rules[prop];

		container.innerHTML = html;
		body.insertBefore(container, body.firstChild);
		innerDiv = container.firstChild, checkDiv = innerDiv.firstChild, td = innerDiv.nextSibling.firstChild.firstChild;

		this.doesNotAddBorder = (checkDiv.offsetTop !== 5);
		this.doesAddBorderForTableAndCells = (td.offsetTop === 5);

		innerDiv.style.overflow = 'hidden', innerDiv.style.position = 'relative';
		this.subtractsBorderForOverflowNotVisible = (checkDiv.offsetTop === -5);

		body.style.marginTop = '1px';
		this.doesNotIncludeMarginInBodyOffset = (body.offsetTop === 0);
		body.style.marginTop = bodyMarginTop;

		body.removeChild(container);
		this.initialized = true;
	},

	bodyOffset: function(body) {
		jQuery.offset.initialized || jQuery.offset.initialize();
		var top = body.offsetTop, left = body.offsetLeft;
		if ( jQuery.offset.doesNotIncludeMarginInBodyOffset )
			top  += parseInt( jQuery.curCSS(body, 'marginTop',  true), 10 ) || 0,
			left += parseInt( jQuery.curCSS(body, 'marginLeft', true), 10 ) || 0;
		return { top: top, left: left };
	}
};


jQuery.fn.extend({
	position: function() {
		var left = 0, top = 0, results;

		if ( this[0] ) {
			// Get *real* offsetParent
			var offsetParent = this.offsetParent(),

			// Get correct offsets
			offset       = this.offset(),
			parentOffset = /^body|html$/i.test(offsetParent[0].tagName) ? { top: 0, left: 0 } : offsetParent.offset();

			// Subtract element margins
			// note: when an element has margin: auto the offsetLeft and marginLeft 
			// are the same in Safari causing offset.left to incorrectly be 0
			offset.top  -= num( this, 'marginTop'  );
			offset.left -= num( this, 'marginLeft' );

			// Add offsetParent borders
			parentOffset.top  += num( offsetParent, 'borderTopWidth'  );
			parentOffset.left += num( offsetParent, 'borderLeftWidth' );

			// Subtract the two offsets
			results = {
				top:  offset.top  - parentOffset.top,
				left: offset.left - parentOffset.left
			};
		}

		return results;
	},

	offsetParent: function() {
		var offsetParent = this[0].offsetParent || document.body;
		while ( offsetParent && (!/^body|html$/i.test(offsetParent.tagName) && jQuery.css(offsetParent, 'position') == 'static') )
			offsetParent = offsetParent.offsetParent;
		return jQuery(offsetParent);
	}
});


// Create scrollLeft and scrollTop methods
jQuery.each( ['Left', 'Top'], function(i, name) {
	var method = 'scroll' + name;
	
	jQuery.fn[ method ] = function(val) {
		if (!this[0]) return null;

		return val !== undefined ?

			// Set the scroll offset
			this.each(function() {
				this == window || this == document ?
					window.scrollTo(
						!i ? val : jQuery(window).scrollLeft(),
						 i ? val : jQuery(window).scrollTop()
					) :
					this[ method ] = val;
			}) :

			// Return the scroll offset
			this[0] == window || this[0] == document ?
				self[ i ? 'pageYOffset' : 'pageXOffset' ] ||
					jQuery.boxModel && document.documentElement[ method ] ||
					document.body[ method ] :
				this[0][ method ];
	};
});
// Create innerHeight, innerWidth, outerHeight and outerWidth methods
jQuery.each([ "Height", "Width" ], function(i, name){

	var tl = i ? "Left"  : "Top",  // top or left
		br = i ? "Right" : "Bottom", // bottom or right
		lower = name.toLowerCase();

	// innerHeight and innerWidth
	jQuery.fn["inner" + name] = function(){
		return this[0] ?
			jQuery.css( this[0], lower, false, "padding" ) :
			null;
	};

	// outerHeight and outerWidth
	jQuery.fn["outer" + name] = function(margin) {
		return this[0] ?
			jQuery.css( this[0], lower, false, margin ? "margin" : "border" ) :
			null;
	};
	
	var type = name.toLowerCase();

	jQuery.fn[ type ] = function( size ) {
		// Get window width or height
		return this[0] == window ?
			// Everyone else use document.documentElement or document.body depending on Quirks vs Standards mode
			document.compatMode == "CSS1Compat" && document.documentElement[ "client" + name ] ||
			document.body[ "client" + name ] :

			// Get document width or height
			this[0] == document ?
				// Either scroll[Width/Height] or offset[Width/Height], whichever is greater
				Math.max(
					document.documentElement["client" + name],
					document.body["scroll" + name], document.documentElement["scroll" + name],
					document.body["offset" + name], document.documentElement["offset" + name]
				) :

				// Get or set width or height on the element
				size === undefined ?
					// Get width or height on the element
					(this.length ? jQuery.css( this[0], type ) : null) :

					// Set the width or height on the element (default to pixels if value is unitless)
					this.css( type, typeof size === "string" ? size : size + "px" );
	};

});
})();
;
// $Id: drupal.js,v 1.41.2.4 2009/07/21 08:59:10 goba Exp $

var Drupal = Drupal || { 'settings': {}, 'behaviors': {}, 'themes': {}, 'locale': {} };

/**
 * Set the variable that indicates if JavaScript behaviors should be applied
 */
Drupal.jsEnabled = true;

/**
 * Attach all registered behaviors to a page element.
 *
 * Behaviors are event-triggered actions that attach to page elements, enhancing
 * default non-Javascript UIs. Behaviors are registered in the Drupal.behaviors
 * object as follows:
 * @code
 *    Drupal.behaviors.behaviorName = function () {
 *      ...
 *    };
 * @endcode
 *
 * Drupal.attachBehaviors is added below to the jQuery ready event and so
 * runs on initial page load. Developers implementing AHAH/AJAX in their
 * solutions should also call this function after new page content has been
 * loaded, feeding in an element to be processed, in order to attach all
 * behaviors to the new content.
 *
 * Behaviors should use a class in the form behaviorName-processed to ensure
 * the behavior is attached only once to a given element. (Doing so enables
 * the reprocessing of given elements, which may be needed on occasion despite
 * the ability to limit behavior attachment to a particular element.)
 *
 * @param context
 *   An element to attach behaviors to. If none is given, the document element
 *   is used.
 */
Drupal.attachBehaviors = function(context) {
  context = context || document;
  // Execute all of them.
  jQuery.each(Drupal.behaviors, function() {
    this(context);
  });
};

/**
 * Encode special characters in a plain-text string for display as HTML.
 */
Drupal.checkPlain = function(str) {
  str = String(str);
  var replace = { '&': '&amp;', '"': '&quot;', '<': '&lt;', '>': '&gt;' };
  for (var character in replace) {
    var regex = new RegExp(character, 'g');
    str = str.replace(regex, replace[character]);
  }
  return str;
};

/**
 * Translate strings to the page language or a given language.
 *
 * See the documentation of the server-side t() function for further details.
 *
 * @param str
 *   A string containing the English string to translate.
 * @param args
 *   An object of replacements pairs to make after translation. Incidences
 *   of any key in this array are replaced with the corresponding value.
 *   Based on the first character of the key, the value is escaped and/or themed:
 *    - !variable: inserted as is
 *    - @variable: escape plain text to HTML (Drupal.checkPlain)
 *    - %variable: escape text and theme as a placeholder for user-submitted
 *      content (checkPlain + Drupal.theme('placeholder'))
 * @return
 *   The translated string.
 */
Drupal.t = function(str, args) {
  // Fetch the localized version of the string.
  if (Drupal.locale.strings && Drupal.locale.strings[str]) {
    str = Drupal.locale.strings[str];
  }

  if (args) {
    // Transform arguments before inserting them
    for (var key in args) {
      switch (key.charAt(0)) {
        // Escaped only
        case '@':
          args[key] = Drupal.checkPlain(args[key]);
        break;
        // Pass-through
        case '!':
          break;
        // Escaped and placeholder
        case '%':
        default:
          args[key] = Drupal.theme('placeholder', args[key]);
          break;
      }
      str = str.replace(key, args[key]);
    }
  }
  return str;
};

/**
 * Format a string containing a count of items.
 *
 * This function ensures that the string is pluralized correctly. Since Drupal.t() is
 * called by this function, make sure not to pass already-localized strings to it.
 *
 * See the documentation of the server-side format_plural() function for further details.
 *
 * @param count
 *   The item count to display.
 * @param singular
 *   The string for the singular case. Please make sure it is clear this is
 *   singular, to ease translation (e.g. use "1 new comment" instead of "1 new").
 *   Do not use @count in the singular string.
 * @param plural
 *   The string for the plural case. Please make sure it is clear this is plural,
 *   to ease translation. Use @count in place of the item count, as in "@count
 *   new comments".
 * @param args
 *   An object of replacements pairs to make after translation. Incidences
 *   of any key in this array are replaced with the corresponding value.
 *   Based on the first character of the key, the value is escaped and/or themed:
 *    - !variable: inserted as is
 *    - @variable: escape plain text to HTML (Drupal.checkPlain)
 *    - %variable: escape text and theme as a placeholder for user-submitted
 *      content (checkPlain + Drupal.theme('placeholder'))
 *   Note that you do not need to include @count in this array.
 *   This replacement is done automatically for the plural case.
 * @return
 *   A translated string.
 */
Drupal.formatPlural = function(count, singular, plural, args) {
  var args = args || {};
  args['@count'] = count;
  // Determine the index of the plural form.
  var index = Drupal.locale.pluralFormula ? Drupal.locale.pluralFormula(args['@count']) : ((args['@count'] == 1) ? 0 : 1);

  if (index == 0) {
    return Drupal.t(singular, args);
  }
  else if (index == 1) {
    return Drupal.t(plural, args);
  }
  else {
    args['@count['+ index +']'] = args['@count'];
    delete args['@count'];
    return Drupal.t(plural.replace('@count', '@count['+ index +']'));
  }
};

/**
 * Generate the themed representation of a Drupal object.
 *
 * All requests for themed output must go through this function. It examines
 * the request and routes it to the appropriate theme function. If the current
 * theme does not provide an override function, the generic theme function is
 * called.
 *
 * For example, to retrieve the HTML that is output by theme_placeholder(text),
 * call Drupal.theme('placeholder', text).
 *
 * @param func
 *   The name of the theme function to call.
 * @param ...
 *   Additional arguments to pass along to the theme function.
 * @return
 *   Any data the theme function returns. This could be a plain HTML string,
 *   but also a complex object.
 */
Drupal.theme = function(func) {
  for (var i = 1, args = []; i < arguments.length; i++) {
    args.push(arguments[i]);
  }

  return (Drupal.theme[func] || Drupal.theme.prototype[func]).apply(this, args);
};

/**
 * Parse a JSON response.
 *
 * The result is either the JSON object, or an object with 'status' 0 and 'data' an error message.
 */
Drupal.parseJson = function (data) {
  if ((data.substring(0, 1) != '{') && (data.substring(0, 1) != '[')) {
    return { status: 0, data: data.length ? data : Drupal.t('Unspecified error') };
  }
  return eval('(' + data + ');');
};

/**
 * Freeze the current body height (as minimum height). Used to prevent
 * unnecessary upwards scrolling when doing DOM manipulations.
 */
Drupal.freezeHeight = function () {
  Drupal.unfreezeHeight();
  var div = document.createElement('div');
  $(div).css({
    position: 'absolute',
    top: '0px',
    left: '0px',
    width: '1px',
    height: $('body').css('height')
  }).attr('id', 'freeze-height');
  $('body').append(div);
};

/**
 * Unfreeze the body height
 */
Drupal.unfreezeHeight = function () {
  $('#freeze-height').remove();
};

/**
 * Wrapper around encodeURIComponent() which avoids Apache quirks (equivalent of
 * drupal_urlencode() in PHP). This function should only be used on paths, not
 * on query string arguments.
 */
Drupal.encodeURIComponent = function (item, uri) {
  uri = uri || location.href;
  item = encodeURIComponent(item).replace(/%2F/g, '/');
  return (uri.indexOf('?q=') != -1) ? item : item.replace(/%26/g, '%2526').replace(/%23/g, '%2523').replace(/\/\//g, '/%252F');
};

/**
 * Get the text selection in a textarea.
 */
Drupal.getSelection = function (element) {
  if (typeof(element.selectionStart) != 'number' && document.selection) {
    // The current selection
    var range1 = document.selection.createRange();
    var range2 = range1.duplicate();
    // Select all text.
    range2.moveToElementText(element);
    // Now move 'dummy' end point to end point of original range.
    range2.setEndPoint('EndToEnd', range1);
    // Now we can calculate start and end points.
    var start = range2.text.length - range1.text.length;
    var end = start + range1.text.length;
    return { 'start': start, 'end': end };
  }
  return { 'start': element.selectionStart, 'end': element.selectionEnd };
};

/**
 * Build an error message from ahah response.
 */
Drupal.ahahError = function(xmlhttp, uri) {
  if (xmlhttp.status == 200) {
    if (jQuery.trim(xmlhttp.responseText)) {
      var message = Drupal.t("An error occurred. \n@uri\n@text", {'@uri': uri, '@text': xmlhttp.responseText });
    }
    else {
      var message = Drupal.t("An error occurred. \n@uri\n(no information available).", {'@uri': uri });
    }
  }
  else {
    var message = Drupal.t("An HTTP error @status occurred. \n@uri", {'@uri': uri, '@status': xmlhttp.status });
  }
  return message.replace(/\n/g, '<br />');
}

// Global Killswitch on the <html> element
$(document.documentElement).addClass('js');
// Attach all behaviors.
$(document).ready(function() {
  Drupal.attachBehaviors(this);
});

/**
 * The default themes.
 */
Drupal.theme.prototype = {

  /**
   * Formats text for emphasized display in a placeholder inside a sentence.
   *
   * @param str
   *   The text to format (plain-text).
   * @return
   *   The formatted text (html).
   */
  placeholder: function(str) {
    return '<em>' + Drupal.checkPlain(str) + '</em>';
  }
};
;
// $Id: googleanalytics.js,v 1.9.2.4 2010/09/19 11:39:20 hass Exp $

$(document).ready(function() {

  // Attach onclick event to document only and catch clicks on all elements.
  $(document.body).click(function(event) {
    // Catch only the first parent link of a clicked element.
    $(event.target).parents("a:first,area:first").andSelf().filter("a,area").each(function() {

      var ga = Drupal.settings.googleanalytics;
      // Expression to check for absolute internal links.
      var isInternal = new RegExp("^(https?):\/\/" + window.location.host, "i");
      // Expression to check for special links like gotwo.module /go/* links.
      var isInternalSpecial = new RegExp("(\/go\/.*)$", "i");
      // Expression to check for download links.
      var isDownload = new RegExp("\\.(" + ga.trackDownloadExtensions + ")$", "i");

      // Is the clicked URL internal?
      if (isInternal.test(this.href)) {
        // Is download tracking activated and the file extension configured for download tracking?
        if (ga.trackDownload && isDownload.test(this.href)) {
          // Download link clicked.
          var extension = isDownload.exec(this.href);
          _gaq.push(["_trackEvent", "Downloads", extension[1].toUpperCase(), this.href.replace(isInternal, '')]);
        }
        else if (isInternalSpecial.test(this.href)) {
          // Keep the internal URL for Google Analytics website overlay intact.
          _gaq.push(["_trackPageview", this.href.replace(isInternal, '')]);
        }
      }
      else {
        if (ga.trackMailto && $(this).is("a[href^=mailto:],area[href^=mailto:]")) {
          // Mailto link clicked.
          _gaq.push(["_trackEvent", "Mails", "Click", this.href.substring(7)]);
        }
        else if (ga.trackOutgoing && this.href) {
          // External link clicked.
          _gaq.push(["_trackEvent", "Outgoing links", "Click", this.href]);
        }
      }

    });
  });
});
;
// $Id: panels.js,v 1.2.4.1 2009/10/05 22:40:35 merlinofchaos Exp $

(function ($) {
  Drupal.Panels = {};

  Drupal.Panels.autoAttach = function() {
    if ($.browser.msie) {
      // If IE, attach a hover event so we can see our admin links.
      $("div.panel-pane").hover(
        function() {
          $('div.panel-hide', this).addClass("panel-hide-hover"); return true;
        },
        function() {
          $('div.panel-hide', this).removeClass("panel-hide-hover"); return true;
        }
      );
      $("div.admin-links").hover(
        function() {
          $(this).addClass("admin-links-hover"); return true;
        },
        function(){
          $(this).removeClass("admin-links-hover"); return true;
        }
      );
    }
  };

  $(Drupal.Panels.autoAttach);
})(jQuery);
;
/*
 * Copyright (c) 2009 Simo Kinnunen.
 * Licensed under the MIT license.
 *
 * @version 1.09i
 */
var Cufon=(function(){var m=function(){return m.replace.apply(null,arguments)};var x=m.DOM={ready:(function(){var C=false,E={loaded:1,complete:1};var B=[],D=function(){if(C){return}C=true;for(var F;F=B.shift();F()){}};if(document.addEventListener){document.addEventListener("DOMContentLoaded",D,false);window.addEventListener("pageshow",D,false)}if(!window.opera&&document.readyState){(function(){E[document.readyState]?D():setTimeout(arguments.callee,10)})()}if(document.readyState&&document.createStyleSheet){(function(){try{document.body.doScroll("left");D()}catch(F){setTimeout(arguments.callee,1)}})()}q(window,"load",D);return function(F){if(!arguments.length){D()}else{C?F():B.push(F)}}})(),root:function(){return document.documentElement||document.body}};var n=m.CSS={Size:function(C,B){this.value=parseFloat(C);this.unit=String(C).match(/[a-z%]*$/)[0]||"px";this.convert=function(D){return D/B*this.value};this.convertFrom=function(D){return D/this.value*B};this.toString=function(){return this.value+this.unit}},addClass:function(C,B){var D=C.className;C.className=D+(D&&" ")+B;return C},color:j(function(C){var B={};B.color=C.replace(/^rgba\((.*?),\s*([\d.]+)\)/,function(E,D,F){B.opacity=parseFloat(F);return"rgb("+D+")"});return B}),fontStretch:j(function(B){if(typeof B=="number"){return B}if(/%$/.test(B)){return parseFloat(B)/100}return{"ultra-condensed":0.5,"extra-condensed":0.625,condensed:0.75,"semi-condensed":0.875,"semi-expanded":1.125,expanded:1.25,"extra-expanded":1.5,"ultra-expanded":2}[B]||1}),getStyle:function(C){var B=document.defaultView;if(B&&B.getComputedStyle){return new a(B.getComputedStyle(C,null))}if(C.currentStyle){return new a(C.currentStyle)}return new a(C.style)},gradient:j(function(F){var G={id:F,type:F.match(/^-([a-z]+)-gradient\(/)[1],stops:[]},C=F.substr(F.indexOf("(")).match(/([\d.]+=)?(#[a-f0-9]+|[a-z]+\(.*?\)|[a-z]+)/ig);for(var E=0,B=C.length,D;E<B;++E){D=C[E].split("=",2).reverse();G.stops.push([D[1]||E/(B-1),D[0]])}return G}),quotedList:j(function(E){var D=[],C=/\s*((["'])([\s\S]*?[^\\])\2|[^,]+)\s*/g,B;while(B=C.exec(E)){D.push(B[3]||B[1])}return D}),recognizesMedia:j(function(G){var E=document.createElement("style"),D,C,B;E.type="text/css";E.media=G;try{E.appendChild(document.createTextNode("/**/"))}catch(F){}C=g("head")[0];C.insertBefore(E,C.firstChild);D=(E.sheet||E.styleSheet);B=D&&!D.disabled;C.removeChild(E);return B}),removeClass:function(D,C){var B=RegExp("(?:^|\\s+)"+C+"(?=\\s|$)","g");D.className=D.className.replace(B,"");return D},supports:function(D,C){var B=document.createElement("span").style;if(B[D]===undefined){return false}B[D]=C;return B[D]===C},textAlign:function(E,D,B,C){if(D.get("textAlign")=="right"){if(B>0){E=" "+E}}else{if(B<C-1){E+=" "}}return E},textShadow:j(function(F){if(F=="none"){return null}var E=[],G={},B,C=0;var D=/(#[a-f0-9]+|[a-z]+\(.*?\)|[a-z]+)|(-?[\d.]+[a-z%]*)|,/ig;while(B=D.exec(F)){if(B[0]==","){E.push(G);G={};C=0}else{if(B[1]){G.color=B[1]}else{G[["offX","offY","blur"][C++]]=B[2]}}}E.push(G);return E}),textTransform:(function(){var B={uppercase:function(C){return C.toUpperCase()},lowercase:function(C){return C.toLowerCase()},capitalize:function(C){return C.replace(/\b./g,function(D){return D.toUpperCase()})}};return function(E,D){var C=B[D.get("textTransform")];return C?C(E):E}})(),whiteSpace:(function(){var D={inline:1,"inline-block":1,"run-in":1};var C=/^\s+/,B=/\s+$/;return function(H,F,G,E){if(E){if(E.nodeName.toLowerCase()=="br"){H=H.replace(C,"")}}if(D[F.get("display")]){return H}if(!G.previousSibling){H=H.replace(C,"")}if(!G.nextSibling){H=H.replace(B,"")}return H}})()};n.ready=(function(){var B=!n.recognizesMedia("all"),E=false;var D=[],H=function(){B=true;for(var K;K=D.shift();K()){}};var I=g("link"),J=g("style");function C(K){return K.disabled||G(K.sheet,K.media||"screen")}function G(M,P){if(!n.recognizesMedia(P||"all")){return true}if(!M||M.disabled){return false}try{var Q=M.cssRules,O;if(Q){search:for(var L=0,K=Q.length;O=Q[L],L<K;++L){switch(O.type){case 2:break;case 3:if(!G(O.styleSheet,O.media.mediaText)){return false}break;default:break search}}}}catch(N){}return true}function F(){if(document.createStyleSheet){return true}var L,K;for(K=0;L=I[K];++K){if(L.rel.toLowerCase()=="stylesheet"&&!C(L)){return false}}for(K=0;L=J[K];++K){if(!C(L)){return false}}return true}x.ready(function(){if(!E){E=n.getStyle(document.body).isUsable()}if(B||(E&&F())){H()}else{setTimeout(arguments.callee,10)}});return function(K){if(B){K()}else{D.push(K)}}})();function s(D){var C=this.face=D.face,B={"\u0020":1,"\u00a0":1,"\u3000":1};this.glyphs=D.glyphs;this.w=D.w;this.baseSize=parseInt(C["units-per-em"],10);this.family=C["font-family"].toLowerCase();this.weight=C["font-weight"];this.style=C["font-style"]||"normal";this.viewBox=(function(){var F=C.bbox.split(/\s+/);var E={minX:parseInt(F[0],10),minY:parseInt(F[1],10),maxX:parseInt(F[2],10),maxY:parseInt(F[3],10)};E.width=E.maxX-E.minX;E.height=E.maxY-E.minY;E.toString=function(){return[this.minX,this.minY,this.width,this.height].join(" ")};return E})();this.ascent=-parseInt(C.ascent,10);this.descent=-parseInt(C.descent,10);this.height=-this.ascent+this.descent;this.spacing=function(L,N,E){var O=this.glyphs,M,K,G,P=[],F=0,J=-1,I=-1,H;while(H=L[++J]){M=O[H]||this.missingGlyph;if(!M){continue}if(K){F-=G=K[H]||0;P[I]-=G}F+=P[++I]=~~(M.w||this.w)+N+(B[H]?E:0);K=M.k}P.total=F;return P}}function f(){var C={},B={oblique:"italic",italic:"oblique"};this.add=function(D){(C[D.style]||(C[D.style]={}))[D.weight]=D};this.get=function(H,I){var G=C[H]||C[B[H]]||C.normal||C.italic||C.oblique;if(!G){return null}I={normal:400,bold:700}[I]||parseInt(I,10);if(G[I]){return G[I]}var E={1:1,99:0}[I%100],K=[],F,D;if(E===undefined){E=I>400}if(I==500){I=400}for(var J in G){if(!k(G,J)){continue}J=parseInt(J,10);if(!F||J<F){F=J}if(!D||J>D){D=J}K.push(J)}if(I<F){I=F}if(I>D){I=D}K.sort(function(M,L){return(E?(M>=I&&L>=I)?M<L:M>L:(M<=I&&L<=I)?M>L:M<L)?-1:1});return G[K[0]]}}function r(){function D(F,G){if(F.contains){return F.contains(G)}return F.compareDocumentPosition(G)&16}function B(G){var F=G.relatedTarget;if(!F||D(this,F)){return}C(this,G.type=="mouseover")}function E(F){C(this,F.type=="mouseenter")}function C(F,G){setTimeout(function(){var H=d.get(F).options;m.replace(F,G?h(H,H.hover):H,true)},10)}this.attach=function(F){if(F.onmouseenter===undefined){q(F,"mouseover",B);q(F,"mouseout",B)}else{q(F,"mouseenter",E);q(F,"mouseleave",E)}}}function u(){var C=[],D={};function B(H){var E=[],G;for(var F=0;G=H[F];++F){E[F]=C[D[G]]}return E}this.add=function(F,E){D[F]=C.push(E)-1};this.repeat=function(){var E=arguments.length?B(arguments):C,F;for(var G=0;F=E[G++];){m.replace(F[0],F[1],true)}}}function A(){var D={},B=0;function C(E){return E.cufid||(E.cufid=++B)}this.get=function(E){var F=C(E);return D[F]||(D[F]={})}}function a(B){var D={},C={};this.extend=function(E){for(var F in E){if(k(E,F)){D[F]=E[F]}}return this};this.get=function(E){return D[E]!=undefined?D[E]:B[E]};this.getSize=function(F,E){return C[F]||(C[F]=new n.Size(this.get(F),E))};this.isUsable=function(){return !!B}}function q(C,B,D){if(C.addEventListener){C.addEventListener(B,D,false)}else{if(C.attachEvent){C.attachEvent("on"+B,function(){return D.call(C,window.event)})}}}function v(C,B){var D=d.get(C);if(D.options){return C}if(B.hover&&B.hoverables[C.nodeName.toLowerCase()]){b.attach(C)}D.options=B;return C}function j(B){var C={};return function(D){if(!k(C,D)){C[D]=B.apply(null,arguments)}return C[D]}}function c(F,E){var B=n.quotedList(E.get("fontFamily").toLowerCase()),D;for(var C=0;D=B[C];++C){if(i[D]){return i[D].get(E.get("fontStyle"),E.get("fontWeight"))}}return null}function g(B){return document.getElementsByTagName(B)}function k(C,B){return C.hasOwnProperty(B)}function h(){var C={},B,F;for(var E=0,D=arguments.length;B=arguments[E],E<D;++E){for(F in B){if(k(B,F)){C[F]=B[F]}}}return C}function o(E,M,C,N,F,D){var K=document.createDocumentFragment(),H;if(M===""){return K}var L=N.separate;var I=M.split(p[L]),B=(L=="words");if(B&&t){if(/^\s/.test(M)){I.unshift("")}if(/\s$/.test(M)){I.push("")}}for(var J=0,G=I.length;J<G;++J){H=z[N.engine](E,B?n.textAlign(I[J],C,J,G):I[J],C,N,F,D,J<G-1);if(H){K.appendChild(H)}}return K}function l(D,M){var C=D.nodeName.toLowerCase();if(M.ignore[C]){return}var E=!M.textless[C];var B=n.getStyle(v(D,M)).extend(M);var F=c(D,B),G,K,I,H,L,J;if(!F){return}for(G=D.firstChild;G;G=I){K=G.nodeType;I=G.nextSibling;if(E&&K==3){if(H){H.appendData(G.data);D.removeChild(G)}else{H=G}if(I){continue}}if(H){D.replaceChild(o(F,n.whiteSpace(H.data,B,H,J),B,M,G,D),H);H=null}if(K==1){if(G.firstChild){if(G.nodeName.toLowerCase()=="cufon"){z[M.engine](F,null,B,M,G,D)}else{arguments.callee(G,M)}}J=G}}}var t=" ".split(/\s+/).length==0;var d=new A();var b=new r();var y=new u();var e=false;var z={},i={},w={autoDetect:false,engine:null,forceHitArea:false,hover:false,hoverables:{a:true},ignore:{applet:1,canvas:1,col:1,colgroup:1,head:1,iframe:1,map:1,optgroup:1,option:1,script:1,select:1,style:1,textarea:1,title:1,pre:1},printable:true,selector:(window.Sizzle||(window.jQuery&&function(B){return jQuery(B)})||(window.dojo&&dojo.query)||(window.Ext&&Ext.query)||(window.YAHOO&&YAHOO.util&&YAHOO.util.Selector&&YAHOO.util.Selector.query)||(window.$$&&function(B){return $$(B)})||(window.$&&function(B){return $(B)})||(document.querySelectorAll&&function(B){return document.querySelectorAll(B)})||g),separate:"words",textless:{dl:1,html:1,ol:1,table:1,tbody:1,thead:1,tfoot:1,tr:1,ul:1},textShadow:"none"};var p={words:/\s/.test("\u00a0")?/[^\S\u00a0]+/:/\s+/,characters:"",none:/^/};m.now=function(){x.ready();return m};m.refresh=function(){y.repeat.apply(y,arguments);return m};m.registerEngine=function(C,B){if(!B){return m}z[C]=B;return m.set("engine",C)};m.registerFont=function(D){if(!D){return m}var B=new s(D),C=B.family;if(!i[C]){i[C]=new f()}i[C].add(B);return m.set("fontFamily",'"'+C+'"')};m.replace=function(D,C,B){C=h(w,C);if(!C.engine){return m}if(!e){n.addClass(x.root(),"cufon-active cufon-loading");n.ready(function(){n.addClass(n.removeClass(x.root(),"cufon-loading"),"cufon-ready")});e=true}if(C.hover){C.forceHitArea=true}if(C.autoDetect){delete C.fontFamily}if(typeof C.textShadow=="string"){C.textShadow=n.textShadow(C.textShadow)}if(typeof C.color=="string"&&/^-/.test(C.color)){C.textGradient=n.gradient(C.color)}else{delete C.textGradient}if(!B){y.add(D,arguments)}if(D.nodeType||typeof D=="string"){D=[D]}n.ready(function(){for(var F=0,E=D.length;F<E;++F){var G=D[F];if(typeof G=="string"){m.replace(C.selector(G),C,true)}else{l(G,C)}}});return m};m.set=function(B,C){w[B]=C;return m};return m})();Cufon.registerEngine("vml",(function(){var e=document.namespaces;if(!e){return}e.add("cvml","urn:schemas-microsoft-com:vml");e=null;var b=document.createElement("cvml:shape");b.style.behavior="url(#default#VML)";if(!b.coordsize){return}b=null;var h=(document.documentMode||0)<8;document.write(('<style type="text/css">cufoncanvas{text-indent:0;}@media screen{cvml\\:shape,cvml\\:rect,cvml\\:fill,cvml\\:shadow{behavior:url(#default#VML);display:block;antialias:true;position:absolute;}cufoncanvas{position:absolute;text-align:left;}cufon{display:inline-block;position:relative;vertical-align:'+(h?"middle":"text-bottom")+";}cufon cufontext{position:absolute;left:-10000in;font-size:1px;}a cufon{cursor:pointer}}@media print{cufon cufoncanvas{display:none;}}</style>").replace(/;/g,"!important;"));function c(i,j){return a(i,/(?:em|ex|%)$|^[a-z-]+$/i.test(j)?"1em":j)}function a(l,m){if(m==="0"){return 0}if(/px$/i.test(m)){return parseFloat(m)}var k=l.style.left,j=l.runtimeStyle.left;l.runtimeStyle.left=l.currentStyle.left;l.style.left=m.replace("%","em");var i=l.style.pixelLeft;l.style.left=k;l.runtimeStyle.left=j;return i}function f(l,k,j,n){var i="computed"+n,m=k[i];if(isNaN(m)){m=k.get(n);k[i]=m=(m=="normal")?0:~~j.convertFrom(a(l,m))}return m}var g={};function d(p){var q=p.id;if(!g[q]){var n=p.stops,o=document.createElement("cvml:fill"),i=[];o.type="gradient";o.angle=180;o.focus="0";o.method="sigma";o.color=n[0][1];for(var m=1,l=n.length-1;m<l;++m){i.push(n[m][0]*100+"% "+n[m][1])}o.colors=i.join(",");o.color2=n[l][1];g[q]=o}return g[q]}return function(ac,G,Y,C,K,ad,W){var n=(G===null);if(n){G=K.alt}var I=ac.viewBox;var p=Y.computedFontSize||(Y.computedFontSize=new Cufon.CSS.Size(c(ad,Y.get("fontSize"))+"px",ac.baseSize));var y,q;if(n){y=K;q=K.firstChild}else{y=document.createElement("cufon");y.className="cufon cufon-vml";y.alt=G;q=document.createElement("cufoncanvas");y.appendChild(q);if(C.printable){var Z=document.createElement("cufontext");Z.appendChild(document.createTextNode(G));y.appendChild(Z)}if(!W){y.appendChild(document.createElement("cvml:shape"))}}var ai=y.style;var R=q.style;var l=p.convert(I.height),af=Math.ceil(l);var V=af/l;var P=V*Cufon.CSS.fontStretch(Y.get("fontStretch"));var U=I.minX,T=I.minY;R.height=af;R.top=Math.round(p.convert(T-ac.ascent));R.left=Math.round(p.convert(U));ai.height=p.convert(ac.height)+"px";var F=Y.get("color");var ag=Cufon.CSS.textTransform(G,Y).split("");var L=ac.spacing(ag,f(ad,Y,p,"letterSpacing"),f(ad,Y,p,"wordSpacing"));if(!L.length){return null}var k=L.total;var x=-U+k+(I.width-L[L.length-1]);var ah=p.convert(x*P),X=Math.round(ah);var O=x+","+I.height,m;var J="r"+O+"ns";var u=C.textGradient&&d(C.textGradient);var o=ac.glyphs,S=0;var H=C.textShadow;var ab=-1,aa=0,w;while(w=ag[++ab]){var D=o[ag[ab]]||ac.missingGlyph,v;if(!D){continue}if(n){v=q.childNodes[aa];while(v.firstChild){v.removeChild(v.firstChild)}}else{v=document.createElement("cvml:shape");q.appendChild(v)}v.stroked="f";v.coordsize=O;v.coordorigin=m=(U-S)+","+T;v.path=(D.d?"m"+D.d+"xe":"")+"m"+m+J;v.fillcolor=F;if(u){v.appendChild(u.cloneNode(false))}var ae=v.style;ae.width=X;ae.height=af;if(H){var s=H[0],r=H[1];var B=Cufon.CSS.color(s.color),z;var N=document.createElement("cvml:shadow");N.on="t";N.color=B.color;N.offset=s.offX+","+s.offY;if(r){z=Cufon.CSS.color(r.color);N.type="double";N.color2=z.color;N.offset2=r.offX+","+r.offY}N.opacity=B.opacity||(z&&z.opacity)||1;v.appendChild(N)}S+=L[aa++]}var M=v.nextSibling,t,A;if(C.forceHitArea){if(!M){M=document.createElement("cvml:rect");M.stroked="f";M.className="cufon-vml-cover";t=document.createElement("cvml:fill");t.opacity=0;M.appendChild(t);q.appendChild(M)}A=M.style;A.width=X;A.height=af}else{if(M){q.removeChild(M)}}ai.width=Math.max(Math.ceil(p.convert(k*P)),0);if(h){var Q=Y.computedYAdjust;if(Q===undefined){var E=Y.get("lineHeight");if(E=="normal"){E="1em"}else{if(!isNaN(E)){E+="em"}}Y.computedYAdjust=Q=0.5*(a(ad,E)-parseFloat(ai.height))}if(Q){ai.marginTop=Math.ceil(Q)+"px";ai.marginBottom=Q+"px"}}return y}})());Cufon.registerEngine("canvas",(function(){var b=document.createElement("canvas");if(!b||!b.getContext||!b.getContext.apply){return}b=null;var a=Cufon.CSS.supports("display","inline-block");var e=!a&&(document.compatMode=="BackCompat"||/frameset|transitional/i.test(document.doctype.publicId));var f=document.createElement("style");f.type="text/css";f.appendChild(document.createTextNode(("cufon{text-indent:0;}@media screen,projection{cufon{display:inline;display:inline-block;position:relative;vertical-align:middle;"+(e?"":"font-size:1px;line-height:1px;")+"}cufon cufontext{display:-moz-inline-box;display:inline-block;width:0;height:0;overflow:hidden;text-indent:-10000in;}"+(a?"cufon canvas{position:relative;}":"cufon canvas{position:absolute;}")+"}@media print{cufon{padding:0;}cufon canvas{display:none;}}").replace(/;/g,"!important;")));document.getElementsByTagName("head")[0].appendChild(f);function d(p,h){var n=0,m=0;var g=[],o=/([mrvxe])([^a-z]*)/g,k;generate:for(var j=0;k=o.exec(p);++j){var l=k[2].split(",");switch(k[1]){case"v":g[j]={m:"bezierCurveTo",a:[n+~~l[0],m+~~l[1],n+~~l[2],m+~~l[3],n+=~~l[4],m+=~~l[5]]};break;case"r":g[j]={m:"lineTo",a:[n+=~~l[0],m+=~~l[1]]};break;case"m":g[j]={m:"moveTo",a:[n=~~l[0],m=~~l[1]]};break;case"x":g[j]={m:"closePath"};break;case"e":break generate}h[g[j].m].apply(h,g[j].a)}return g}function c(m,k){for(var j=0,h=m.length;j<h;++j){var g=m[j];k[g.m].apply(k,g.a)}}return function(V,w,P,t,C,W){var k=(w===null);if(k){w=C.getAttribute("alt")}var A=V.viewBox;var m=P.getSize("fontSize",V.baseSize);var B=0,O=0,N=0,u=0;var z=t.textShadow,L=[];if(z){for(var U=z.length;U--;){var F=z[U];var K=m.convertFrom(parseFloat(F.offX));var I=m.convertFrom(parseFloat(F.offY));L[U]=[K,I];if(I<B){B=I}if(K>O){O=K}if(I>N){N=I}if(K<u){u=K}}}var Z=Cufon.CSS.textTransform(w,P).split("");var E=V.spacing(Z,~~m.convertFrom(parseFloat(P.get("letterSpacing"))||0),~~m.convertFrom(parseFloat(P.get("wordSpacing"))||0));if(!E.length){return null}var h=E.total;O+=A.width-E[E.length-1];u+=A.minX;var s,n;if(k){s=C;n=C.firstChild}else{s=document.createElement("cufon");s.className="cufon cufon-canvas";s.setAttribute("alt",w);n=document.createElement("canvas");s.appendChild(n);if(t.printable){var S=document.createElement("cufontext");S.appendChild(document.createTextNode(w));s.appendChild(S)}}var aa=s.style;var H=n.style;var j=m.convert(A.height);var Y=Math.ceil(j);var M=Y/j;var G=M*Cufon.CSS.fontStretch(P.get("fontStretch"));var J=h*G;var Q=Math.ceil(m.convert(J+O-u));var o=Math.ceil(m.convert(A.height-B+N));n.width=Q;n.height=o;H.width=Q+"px";H.height=o+"px";B+=A.minY;H.top=Math.round(m.convert(B-V.ascent))+"px";H.left=Math.round(m.convert(u))+"px";var r=Math.max(Math.ceil(m.convert(J)),0)+"px";if(a){aa.width=r;aa.height=m.convert(V.height)+"px"}else{aa.paddingLeft=r;aa.paddingBottom=(m.convert(V.height)-1)+"px"}var X=n.getContext("2d"),D=j/A.height;X.scale(D,D*M);X.translate(-u,-B);X.save();function T(){var x=V.glyphs,ab,l=-1,g=-1,y;X.scale(G,1);while(y=Z[++l]){var ab=x[Z[l]]||V.missingGlyph;if(!ab){continue}if(ab.d){X.beginPath();if(ab.code){c(ab.code,X)}else{ab.code=d("m"+ab.d,X)}X.fill()}X.translate(E[++g],0)}X.restore()}if(z){for(var U=z.length;U--;){var F=z[U];X.save();X.fillStyle=F.color;X.translate.apply(X,L[U]);T()}}var q=t.textGradient;if(q){var v=q.stops,p=X.createLinearGradient(0,A.minY,0,A.maxY);for(var U=0,R=v.length;U<R;++U){p.addColorStop.apply(p,v[U])}X.fillStyle=p}else{X.fillStyle=P.get("color")}T();return s}})());;
/*!
 * The following copyright notice may not be removed under any circumstances.
 * 
 * Copyright:
 * Copyright ?1988, 1990, 1993, 2002 Adobe Systems Incorporated.  All Rights
 * Reserved.
 * 
 * Trademark:
 * Helvetica is a Trademark of Heidelberger Druckmaschinen AG exclusively licensed
 * through Linotype Library GmbH, and may be registered in certain jurisdictions.
 * 
 * Full name:
 * HelveticaNeueLTStd-Roman
 * 
 * Designer:
 * Linotype Staff
 * 
 * Vendor URL:
 * http://www.adobe.com/type
 * 
 * License information:
 * http://www.adobe.com/type/legal.html
 */
Cufon.registerFont({"w":556,"face":{"font-family":"Helvetica Neue LT Std 55 Roman","font-weight":400,"font-stretch":"normal","units-per-em":"1000","panose-1":"2 11 6 4 2 2 2 2 2 4","ascent":"714","descent":"-286","x-height":"12","bbox":"-166 -952 1077.17 214","underline-thickness":"50","underline-position":"-50","stemh":"75","stemv":"85","unicode-range":"U+0020-U+FB02"},"glyphs":{" ":{"w":278},"\ufb01":{"d":"99,0r0,-442r-87,0r0,-75r87,0r0,-77v-7,-103,94,-147,198,-120r0,74v-51,-14,-113,-14,-113,51r0,72r100,0r0,75r-100,0r0,442r-85,0xm450,-610r-85,0r0,-104r85,0r0,104xm450,0r-85,0r0,-517r85,0r0,517","w":518,"k":{"\u2019":-18,"f":18,"\u00df":18,"\ufb01":18,"\ufb02":18}},"\ufb02":{"d":"99,0r0,-442r-87,0r0,-75r87,0r0,-77v-7,-103,94,-147,198,-120r0,74v-51,-14,-113,-14,-113,51r0,72r100,0r0,75r-100,0r0,442r-85,0xm365,0r0,-714r85,0r0,714r-85,0","w":518,"k":{"\u2019":-18,"f":18,"\u00df":18,"\ufb01":18,"\ufb02":18}},"!":{"d":"177,-714r0,220r-25,311r-45,0r-25,-311r0,-220r95,0xm74,0r0,-111r111,0r0,111r-111,0","w":259},"\"":{"d":"109,-456r0,-258r68,0r0,258r-68,0xm249,-456r0,-258r68,0r0,258r-68,0","w":426},"#":{"d":"102,0r29,-206r-97,0r0,-68r107,0r21,-150r-102,0r0,-68r111,0r29,-205r69,0r-29,205r124,0r29,-205r69,0r-29,205r89,0r0,68r-98,0r-21,150r91,0r0,68r-101,0r-29,206r-69,0r29,-206r-124,0r-29,206r-69,0xm355,-424r-124,0r-21,150r124,0"},"$":{"d":"251,-417r0,-234v-64,7,-115,45,-115,121v0,71,48,93,115,113xm306,-314r0,251v75,-6,129,-42,129,-128v0,-76,-56,-102,-129,-123xm25,-218r85,0v3,89,61,142,141,155r0,-265v-141,-34,-205,-76,-205,-193v0,-121,90,-203,205,-210r0,-66r55,0r0,67v108,13,171,83,197,198r-85,0v-13,-65,-52,-105,-112,-116r0,246r60,16v51,13,159,63,159,186v0,129,-92,207,-219,217r0,82r-55,0r0,-82v-131,-13,-215,-97,-226,-235"},"%":{"d":"586,-186v0,-107,39,-203,162,-203v124,0,160,91,160,200v0,104,-44,201,-160,201v-121,0,-162,-90,-162,-198xm661,-188v0,58,9,145,84,145v76,0,88,-84,88,-146v0,-59,-10,-145,-85,-145v-74,0,-87,86,-87,146xm92,-506v0,-107,39,-203,162,-203v124,0,160,91,160,200v0,104,-44,201,-160,201v-121,0,-162,-90,-162,-198xm167,-508v0,58,9,145,84,145v76,0,88,-84,88,-146v0,-59,-10,-145,-85,-145v-74,0,-87,86,-87,146xm188,22r538,-744r85,0r-537,744r-86,0","w":1000},"&":{"d":"411,-148r-164,-197v-67,38,-124,76,-124,161v0,72,67,121,136,121v65,0,117,-33,152,-85xm534,0r-76,-89v-43,67,-124,101,-202,101v-193,0,-223,-137,-223,-195v0,-112,76,-168,167,-216v-40,-57,-79,-95,-79,-168v0,-78,58,-147,161,-147v89,0,173,49,173,146v0,85,-65,146,-134,185r129,157v13,-35,21,-73,23,-111r81,0v-7,84,-18,113,-48,176r139,161r-111,0xm275,-439v41,-26,95,-64,95,-124v0,-41,-31,-76,-82,-76v-40,0,-82,24,-82,76v0,43,38,85,69,124","w":630},"\u2019":{"d":"83,-603r0,-111r111,0v4,123,7,257,-110,258r0,-50v45,0,58,-51,58,-97r-59,0","w":278,"k":{"\u2019":95,"s":74,"\u0161":74,"t":18}},"(":{"d":"269,198r-65,0v-96,-150,-157,-293,-157,-468v0,-156,55,-325,157,-461r65,0v-95,150,-132,299,-132,461v0,168,37,315,132,468","w":259},")":{"d":"55,198r-65,0v95,-150,132,-299,132,-461v0,-168,-37,-315,-132,-468r65,0v96,150,157,293,157,468v0,156,-55,325,-157,461","w":259},"*":{"d":"201,-714r0,113r107,-40r20,48r-109,38r66,91r-41,31r-69,-96r-65,96r-45,-31r65,-91r-106,-38r18,-48r106,40r0,-113r53,0","w":352},"+":{"d":"334,-506r0,219r218,0r0,68r-218,0r0,219r-68,0r0,-219r-218,0r0,-68r218,0r0,-219r68,0","w":600},",":{"d":"83,0r0,-111r111,0v5,130,2,227,-110,258r0,-50v45,-15,60,-65,58,-97r-59,0","w":278},"-":{"d":"50,-238r0,-80r289,0r0,80r-289,0","w":389},".":{"d":"83,0r0,-111r111,0r0,111r-111,0","w":278},"\/":{"d":"-18,17r296,-748r73,0r-296,748r-73,0","w":333},"0":{"d":"42,-348v0,-157,14,-361,236,-361v222,0,236,204,236,361v0,156,-14,360,-236,360v-222,0,-236,-204,-236,-360xm132,-349v0,106,1,286,146,286v145,0,146,-180,146,-286v0,-105,-1,-285,-146,-285v-145,0,-146,180,-146,285"},"1":{"d":"356,-709r0,709r-85,0r0,-508r-184,0r0,-68v97,0,185,-26,204,-133r65,0"},"2":{"d":"494,-505v0,242,-327,249,-371,430r364,0r0,75r-463,0v14,-168,99,-228,199,-290v120,-74,181,-118,181,-216v0,-77,-62,-128,-137,-128v-99,0,-141,91,-138,176r-85,0v-7,-144,78,-251,228,-251v124,0,222,72,222,204"},"3":{"d":"218,-333r0,-72v94,9,172,-25,172,-115v0,-74,-54,-114,-126,-114v-93,0,-133,71,-134,154r-85,0v5,-135,77,-229,219,-229v123,0,216,59,216,190v0,64,-48,114,-98,143v86,19,128,87,128,172v0,142,-111,216,-244,216v-144,0,-239,-85,-236,-231r85,0v3,92,54,156,151,156v83,0,154,-51,154,-137v-1,-107,-92,-150,-202,-133"},"4":{"d":"419,-709r0,468r96,0r0,75r-96,0r0,166r-80,0r0,-166r-311,0r0,-82r323,-461r68,0xm101,-241r238,0r0,-347r-2,0"},"5":{"d":"470,-697r0,75r-286,0r-38,204r2,2v31,-35,84,-51,132,-51v126,0,229,81,229,243v0,120,-83,236,-248,236v-123,0,-223,-75,-226,-203r85,0v5,73,64,128,149,128v81,0,150,-55,150,-168v0,-94,-65,-161,-160,-161v-54,0,-99,27,-132,67r-73,-4r66,-368r350,0"},"6":{"d":"286,-381v-95,0,-147,71,-147,160v0,88,54,158,147,158v92,0,143,-73,143,-157v0,-88,-46,-161,-143,-161xm499,-528r-85,0v-8,-64,-52,-106,-119,-106v-144,-1,-173,165,-170,271v40,-65,100,-93,177,-93v136,0,217,99,217,230v0,103,-58,238,-237,238v-210,0,-244,-192,-244,-341v0,-195,59,-380,261,-380v117,0,190,66,200,181"},"7":{"d":"509,-697r0,73v-143,142,-259,402,-272,624r-95,0v16,-240,131,-459,280,-617r-372,0r0,-80r459,0"},"8":{"d":"130,-200v0,85,63,137,151,137v85,0,145,-59,145,-138v0,-77,-66,-135,-146,-135v-84,0,-150,54,-150,136xm65,-528v0,-117,99,-181,208,-181v244,0,276,249,117,328v85,27,126,89,126,177v0,137,-104,216,-235,216v-136,0,-241,-71,-241,-216v0,-84,45,-149,124,-177v-59,-24,-99,-83,-99,-147xm155,-525v0,78,56,114,126,114v67,0,117,-42,117,-114v0,-71,-50,-109,-119,-109v-68,0,-124,38,-124,109"},"9":{"d":"409,-472v0,-87,-51,-162,-146,-162v-101,0,-139,81,-139,168v0,78,58,150,140,150v89,0,145,-72,145,-156xm49,-171r85,0v6,69,62,108,130,108v103,0,161,-99,158,-276v-32,60,-95,98,-164,98v-144,0,-224,-97,-224,-237v0,-137,96,-231,233,-231v132,0,243,71,243,339v0,245,-74,382,-246,382v-119,0,-204,-61,-215,-183"},":":{"d":"83,0r0,-111r111,0r0,111r-111,0xm194,-517r0,111r-111,0r0,-111r111,0","w":278},";":{"d":"83,0r0,-111r111,0v5,130,2,227,-110,258r0,-50v45,-15,60,-65,58,-97r-59,0xm194,-517r0,111r-111,0r0,-111r111,0","w":278},"<":{"d":"554,-514r0,68r-425,193r425,193r0,68r-508,-230r0,-62","w":600},"=":{"d":"552,-388r0,68r-504,0r0,-68r504,0xm48,-118r0,-68r504,0r0,68r-504,0","w":600},">":{"d":"46,8r0,-68r425,-193r-425,-193r0,-68r508,230r0,62","w":600},"?":{"d":"490,-531v0,173,-192,173,-173,357r-85,0v-11,-109,40,-167,90,-214v40,-38,83,-77,78,-151v-5,-73,-50,-117,-121,-117v-95,0,-140,73,-139,165r-85,0v-2,-137,84,-240,224,-240v123,0,211,73,211,200xm218,0r0,-111r111,0r0,111r-111,0"},"@":{"d":"420,-505v-94,0,-160,122,-160,209v0,56,36,97,86,97v87,0,155,-131,155,-213v0,-48,-39,-93,-81,-93xm618,-554r-84,284v-9,29,-25,82,13,82v61,0,138,-123,138,-230v0,-158,-131,-258,-271,-258v-171,0,-304,146,-304,321v0,178,141,317,309,317v95,0,192,-48,248,-121r63,0v-64,110,-185,176,-313,176v-211,0,-375,-166,-375,-377v0,-206,168,-371,371,-371v185,0,340,128,340,308v0,174,-140,290,-234,290v-34,0,-62,-21,-64,-65r-3,0v-30,35,-75,67,-124,67v-85,0,-143,-70,-143,-158v0,-139,94,-284,235,-284v46,0,89,22,112,82r23,-63r63,0","w":800},"A":{"d":"-7,0r279,-714r105,0r279,714r-105,0r-78,-215r-302,0r-80,215r-98,0xm322,-628r-121,333r242,0r-119,-333r-2,0","w":648},"B":{"d":"173,-327r0,247r238,0v82,0,134,-50,134,-128v0,-168,-216,-108,-372,-119xm78,0r0,-714r259,0v63,0,146,0,189,26v47,28,84,79,84,149v0,78,-44,131,-106,162v88,18,136,84,136,172v0,105,-75,205,-217,205r-345,0xm173,-634r0,227r190,0v108,0,152,-39,152,-114v0,-99,-70,-113,-152,-113r-190,0","w":685},"C":{"d":"675,-499r-95,0v-20,-101,-101,-152,-200,-152v-170,0,-242,140,-242,289v0,163,72,299,243,299v125,0,197,-88,206,-206r95,0v-18,178,-129,286,-309,286v-222,0,-330,-164,-330,-372v0,-207,118,-376,338,-376v149,0,272,80,294,232","w":722},"D":{"d":"78,0r0,-714r245,0v220,0,343,110,343,340v0,239,-105,374,-343,374r-245,0xm173,-634r0,554r158,0v65,0,240,-18,240,-281v0,-170,-63,-273,-238,-273r-160,0","w":704},"E":{"d":"78,0r0,-714r493,0r0,80r-398,0r0,227r371,0r0,80r-371,0r0,247r401,0r0,80r-496,0","w":611},"F":{"d":"78,0r0,-714r469,0r0,80r-374,0r0,227r328,0r0,80r-328,0r0,327r-95,0","w":574,"k":{"A":55,"\u00c6":55,"\u00c1":55,"\u00c2":55,"\u00c4":55,"\u00c0":55,"\u00c5":55,"\u00c3":55,",":157,".":157}},"G":{"d":"637,0r-25,-89v-55,75,-145,106,-229,106v-213,0,-340,-174,-340,-360v0,-208,114,-388,340,-388v156,0,280,70,307,237r-95,0v-19,-108,-106,-157,-212,-157v-173,0,-245,150,-245,303v0,149,86,285,245,285v147,0,235,-103,229,-233r-228,0r0,-80r313,0r0,376r-60,0","w":759},"H":{"d":"78,0r0,-714r95,0r0,307r376,0r0,-307r95,0r0,714r-95,0r0,-327r-376,0r0,327r-95,0","w":722},"I":{"d":"82,0r0,-714r95,0r0,714r-95,0","w":259},"J":{"d":"441,-714r0,536v0,97,-51,195,-211,195v-149,0,-220,-91,-208,-252r95,0v-3,98,11,172,115,172v87,0,114,-50,114,-129r0,-522r95,0","w":519},"K":{"d":"78,0r0,-714r95,0r0,356r362,-356r123,0r-301,288r313,426r-120,0r-258,-361r-119,111r0,250r-95,0","w":667},"L":{"d":"78,0r0,-714r95,0r0,634r378,0r0,80r-473,0","k":{"T":92,"V":92,"W":55,"y":37,"\u00fd":37,"\u00ff":37,"Y":111,"\u00dd":111,"\u0178":111,"\u2019":74}},"M":{"d":"80,0r0,-714r130,0r225,600r226,-600r130,0r0,714r-90,0r0,-594r-2,0r-223,594r-81,0r-223,-594r-2,0r0,594r-90,0","w":871},"N":{"d":"77,0r0,-714r101,0r376,579r2,0r0,-579r90,0r0,714r-104,0r-373,-573r-2,0r0,573r-90,0","w":722},"O":{"d":"38,-357v0,-194,115,-374,342,-374v227,0,342,180,342,374v0,194,-115,374,-342,374v-227,0,-342,-180,-342,-374xm133,-357v0,144,73,294,247,294v174,0,247,-150,247,-294v0,-144,-73,-294,-247,-294v-174,0,-247,150,-247,294","w":760},"P":{"d":"78,0r0,-714r313,0v142,0,219,79,219,211v0,132,-77,211,-219,211r-218,0r0,292r-95,0xm173,-634r0,262r186,0v107,1,156,-46,156,-131v0,-85,-49,-131,-156,-131r-186,0","w":648,"k":{"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":179,".":179}},"Q":{"d":"708,2r-48,60r-109,-88v-47,28,-104,43,-171,43v-227,0,-342,-180,-342,-374v0,-194,115,-374,342,-374v350,0,437,451,234,658xm400,-146r50,-58r95,76v147,-157,104,-522,-165,-523v-174,0,-247,150,-247,294v0,144,73,294,247,294v37,0,69,-7,99,-20","w":760},"R":{"d":"173,-385v158,-10,361,47,363,-127v0,-70,-38,-122,-127,-122r-236,0r0,249xm78,0r0,-714r336,0v134,0,217,73,217,187v0,88,-42,155,-124,183v164,28,69,240,149,344r-106,0v-27,-29,-14,-106,-26,-176v-12,-70,-27,-129,-119,-129r-232,0r0,305r-95,0","w":685,"k":{"T":18,"V":18,"W":18,"Y":37,"\u00dd":37,"\u0178":37}},"S":{"d":"581,-505r-90,0v-11,-102,-79,-146,-177,-146v-80,0,-160,29,-160,122v0,88,114,101,229,127v114,26,228,66,228,206v0,152,-149,213,-279,213v-160,0,-295,-78,-295,-252r90,0v0,120,100,172,208,172v86,0,181,-27,181,-130v0,-98,-114,-115,-228,-140v-114,-25,-229,-58,-229,-189v0,-145,129,-209,259,-209v146,0,257,69,263,226","w":648},"T":{"d":"240,0r0,-634r-238,0r0,-80r571,0r0,80r-238,0r0,634r-95,0","w":574,"k":{"\u00fc":92,"\u0161":111,"\u00f2":111,"\u00f6":111,"\u00e8":111,"\u00eb":111,"\u00ea":111,"\u00e3":111,"\u00e5":111,"\u00e0":111,"\u00e4":111,"\u00e2":111,"w":111,"y":111,"\u00fd":111,"\u00ff":111,"A":92,"\u00c6":92,"\u00c1":92,"\u00c2":92,"\u00c4":92,"\u00c0":92,"\u00c5":92,"\u00c3":92,",":111,".":111,"c":111,"\u00e7":111,"e":111,"\u00e9":111,"o":111,"\u00f8":111,"\u0153":111,"\u00f3":111,"\u00f4":111,"\u00f5":111,"-":129,"a":111,"\u00e6":111,"\u00e1":111,"r":92,"s":111,"u":92,"\u00fa":92,"\u00fb":92,"\u00f9":92,":":111,";":111}},"U":{"d":"648,-714r0,456v0,180,-103,275,-281,275v-184,0,-293,-85,-293,-275r0,-456r95,0r0,456v0,126,72,195,198,195v120,0,186,-69,186,-195r0,-456r95,0","w":722},"V":{"d":"613,-714r-254,714r-108,0r-252,-714r103,0r204,619r2,0r206,-619r99,0","w":611,"k":{"\u00f6":55,"\u00f4":55,"\u00ee":18,"\u00e8":55,"\u00eb":55,"\u00ea":55,"\u00e3":55,"\u00e5":55,"\u00e0":55,"\u00e4":55,"\u00e2":55,"y":18,"\u00fd":18,"\u00ff":18,"A":46,"\u00c6":46,"\u00c1":46,"\u00c2":46,"\u00c4":46,"\u00c0":46,"\u00c5":46,"\u00c3":46,",":129,".":129,"e":55,"\u00e9":55,"o":55,"\u00f8":55,"\u0153":55,"\u00f3":55,"\u00f2":55,"\u00f5":55,"-":55,"a":55,"\u00e6":55,"\u00e1":55,"r":37,"u":37,"\u00fa":37,"\u00fb":37,"\u00fc":37,"\u00f9":37,":":46,";":46,"i":18,"\u00ed":18,"\u00ef":18,"\u00ec":18}},"W":{"d":"914,-714r-191,714r-99,0r-163,-600r-2,0r-165,600r-99,0r-183,-714r97,0r140,594r2,0r160,-594r103,0r158,594r2,0r145,-594r95,0","w":926,"k":{"\u00fc":18,"\u00f6":57,"\u00ea":57,"\u00e4":37,"A":24,"\u00c6":24,"\u00c1":24,"\u00c2":24,"\u00c4":24,"\u00c0":24,"\u00c5":24,"\u00c3":24,",":74,".":74,"e":57,"\u00e9":57,"\u00eb":57,"\u00e8":57,"o":57,"\u00f8":57,"\u0153":57,"\u00f3":57,"\u00f4":57,"\u00f2":57,"\u00f5":57,"a":37,"\u00e6":37,"\u00e1":37,"\u00e2":37,"\u00e0":37,"\u00e5":37,"\u00e3":37,"r":18,"u":18,"\u00fa":18,"\u00fb":18,"\u00f9":18,":":18,";":18}},"X":{"d":"-1,0r249,-367r-235,-347r114,0r178,279r186,-279r106,0r-236,347r251,367r-116,0r-192,-295r-198,295r-107,0","w":611},"Y":{"d":"646,-714r-273,422r0,292r-95,0r0,-292r-276,-422r113,0r214,340r209,-340r108,0","w":648,"k":{"\u00fc":74,"\u00f6":111,"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":139,".":139,"e":111,"\u00e9":111,"\u00ea":111,"\u00eb":111,"\u00e8":111,"o":111,"\u00f8":111,"\u0153":111,"\u00f3":111,"\u00f4":111,"\u00f2":111,"\u00f5":111,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"u":74,"\u00fa":74,"\u00fb":74,"\u00f9":74,":":92,";":92,"i":37,"\u00ed":37,"\u00ee":37,"\u00ef":37,"\u00ec":37,"p":74}},"Z":{"d":"22,0r0,-80r444,-554r-410,0r0,-80r522,0r0,80r-443,554r455,0r0,80r-568,0","w":611},"[":{"d":"259,-731r0,68r-107,0r0,793r107,0r0,68r-187,0r0,-929r187,0","w":259},"\\":{"d":"55,-731r296,748r-73,0r-296,-748r73,0","w":333},"]":{"d":"0,198r0,-68r107,0r0,-793r-107,0r0,-68r187,0r0,929r-187,0","w":259},"^":{"d":"50,-264r218,-433r64,0r218,433r-70,0r-181,-360r-178,360r-71,0","w":600},"_":{"d":"0,125r0,-50r500,0r0,50r-500,0","w":500},"\u2018":{"d":"195,-567r0,111r-111,0v-4,-123,-7,-257,110,-258r0,50v-45,0,-58,51,-58,97r59,0","w":278,"k":{"\u2018":95}},"a":{"d":"384,-267v-70,49,-258,4,-258,128v0,56,48,76,100,76v112,0,158,-70,158,-117r0,-87xm522,-68r0,66v-51,28,-141,18,-133,-66v-47,55,-110,80,-182,80v-94,0,-171,-42,-171,-145v0,-117,87,-142,175,-159v94,-18,174,-12,174,-76v0,-74,-61,-86,-115,-86v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v93,0,194,21,194,142r0,266v-4,55,13,67,53,53","w":537},"b":{"d":"467,-263v0,-97,-50,-191,-160,-191v-114,0,-158,98,-158,195v0,102,46,196,163,196v116,0,155,-99,155,-200xm67,0r0,-714r85,0r0,267r2,0v28,-57,94,-82,165,-82v159,0,238,126,238,272v0,146,-78,269,-236,269v-55,0,-128,-24,-169,-81r0,69r-85,0","w":593},"c":{"d":"503,-351r-88,0v-13,-65,-58,-103,-128,-103v-123,0,-161,97,-161,202v0,95,43,189,151,189v82,0,131,-48,142,-126r87,0v-19,126,-98,201,-228,201v-158,0,-242,-110,-242,-264v0,-155,80,-277,244,-277v117,0,211,55,223,178","w":537},"d":{"d":"526,-714r0,714r-85,0v-2,-22,4,-52,-2,-70v-28,57,-94,82,-165,82v-159,0,-238,-126,-238,-272v0,-146,78,-269,236,-269v53,0,129,20,167,81r2,0r0,-266r85,0xm126,-254v0,97,50,191,160,191v114,0,158,-98,158,-195v0,-102,-46,-196,-163,-196v-116,0,-155,99,-155,200","w":593},"e":{"d":"421,-164r84,0v-23,114,-105,176,-221,176v-165,0,-248,-114,-248,-272v0,-155,102,-269,243,-269v183,0,239,171,234,296r-387,0v-3,90,48,170,161,170v70,0,119,-34,134,-101xm126,-308r297,0v-4,-81,-65,-146,-149,-146v-89,0,-142,67,-148,146","w":537},"f":{"d":"99,0r0,-442r-87,0r0,-75r87,0r0,-77v-7,-103,94,-147,198,-120r0,74v-51,-14,-113,-14,-113,51r0,72r100,0r0,75r-100,0r0,442r-85,0","w":296,"k":{"\u2019":-18,"f":18,"\u00df":18,"\ufb01":18,"\ufb02":18}},"g":{"d":"272,-69v112,0,154,-103,154,-200v0,-91,-40,-185,-147,-185v-109,0,-153,89,-153,188v0,93,34,197,146,197xm510,-517r0,473v0,168,-77,254,-240,254v-97,0,-209,-39,-214,-152r85,0v4,62,78,84,134,84v121,0,166,-103,153,-230v-28,63,-96,94,-161,94v-154,0,-231,-121,-231,-262v0,-122,60,-273,239,-273v66,-1,122,31,155,86r0,-74r80,0","w":574},"h":{"d":"64,0r0,-714r85,0r0,273r2,0v28,-62,101,-88,163,-88v136,0,178,78,178,189r0,340r-85,0r0,-350v0,-63,-40,-104,-105,-104v-103,0,-153,69,-153,162r0,292r-85,0"},"i":{"d":"69,0r0,-517r85,0r0,517r-85,0xm154,-714r0,104r-85,0r0,-104r85,0","w":222},"j":{"d":"154,-517r0,583v0,109,-70,145,-168,128r0,-73v55,7,83,2,83,-69r0,-569r85,0xm154,-714r0,104r-85,0r0,-104r85,0","w":222},"k":{"d":"69,0r0,-714r85,0r0,423r236,-226r114,0r-205,188r220,329r-108,0r-177,-270r-80,74r0,196r-85,0","w":519},"l":{"d":"69,0r0,-714r85,0r0,714r-85,0","w":222},"m":{"d":"64,0r0,-517r80,0v2,24,-4,56,2,76v38,-58,96,-88,167,-88v63,0,121,25,142,88v67,-127,334,-125,334,62r0,379r-85,0r0,-339v0,-64,-17,-115,-100,-115v-82,0,-135,52,-135,132r0,322r-85,0r0,-339v0,-67,-21,-115,-97,-115v-101,0,-138,93,-138,132r0,322r-85,0","w":853},"n":{"d":"64,0r0,-517r80,0v2,26,-4,60,2,82v36,-64,94,-94,168,-94v136,0,178,78,178,189r0,340r-85,0r0,-350v0,-63,-40,-104,-105,-104v-103,0,-153,69,-153,162r0,292r-85,0"},"o":{"d":"36,-258v0,-152,87,-271,251,-271v164,0,251,119,251,271v0,151,-87,270,-251,270v-164,0,-251,-119,-251,-270xm126,-258v0,125,72,195,161,195v89,0,161,-70,161,-195v0,-126,-72,-196,-161,-196v-89,0,-161,70,-161,196","w":574},"p":{"d":"467,-263v0,-97,-50,-191,-160,-191v-114,0,-158,98,-158,195v0,102,46,196,163,196v116,0,155,-99,155,-200xm67,198r0,-715r85,0v2,22,-4,52,2,70v28,-57,94,-82,165,-82v159,0,238,126,238,272v0,146,-78,269,-236,269v-55,0,-128,-24,-169,-81r0,267r-85,0","w":593},"q":{"d":"526,-517r0,715r-85,0r0,-268r-2,0v-28,57,-94,82,-165,82v-159,0,-238,-126,-238,-272v0,-146,78,-269,236,-269v53,0,129,20,167,81r2,0r0,-69r85,0xm126,-254v0,97,50,191,160,191v114,0,158,-98,158,-195v0,-102,-46,-196,-163,-196v-116,0,-155,99,-155,200","w":593},"r":{"d":"61,0r0,-517r80,0v2,35,-4,78,2,109v41,-83,98,-124,190,-121r0,90v-137,0,-187,78,-187,209r0,230r-85,0","w":333,"k":{",":92,".":92,"c":18,"\u00e7":18,"d":18,"\u0131":18,"e":18,"\u00e9":18,"\u00ea":18,"\u00eb":18,"\u00e8":18,"g":28,"n":-18,"\u00f1":-18,"o":18,"\u00f8":18,"\u0153":18,"\u00f3":18,"\u00f4":18,"\u00f6":18,"\u00f2":18,"\u00f5":18,"q":18,"-":55}},"s":{"d":"31,-163r85,0v3,76,69,100,139,100v53,0,125,-12,125,-77v0,-66,-84,-77,-169,-96v-84,-19,-169,-47,-169,-146v0,-104,103,-147,193,-147v114,0,205,36,212,162r-85,0v-6,-66,-64,-87,-120,-87v-51,0,-110,14,-110,66v0,61,90,72,169,90v85,20,169,47,169,147v0,123,-115,163,-220,163v-116,0,-214,-47,-219,-175","w":500},"t":{"d":"182,-672r0,155r103,0r0,75r-103,0r0,321v-10,53,50,46,103,46r0,75v-109,1,-188,10,-188,-113r0,-329r-88,0r0,-75r88,0r0,-155r85,0","w":315},"u":{"d":"492,-517r0,517r-80,0v-2,-26,4,-60,-2,-82v-36,64,-94,94,-168,94v-136,0,-178,-78,-178,-189r0,-340r85,0r0,350v0,63,40,104,105,104v103,0,153,-69,153,-162r0,-292r85,0"},"v":{"d":"486,-517r-189,517r-91,0r-192,-517r95,0r145,431r2,0r141,-431r89,0","w":500,"k":{",":74,".":74}},"w":{"d":"741,-517r-165,517r-90,0r-105,-411r-2,0r-104,411r-92,0r-166,-517r94,0r117,423r2,0r104,-423r93,0r108,423r2,0r116,-423r88,0","w":758,"k":{",":55,".":55}},"x":{"d":"9,0r194,-272r-180,-245r109,0r124,181r129,-181r102,0r-178,238r200,279r-109,0r-144,-214r-144,214r-103,0","w":518},"y":{"d":"492,-517r-225,586v-46,123,-95,156,-209,129r0,-78v48,21,101,13,120,-35r35,-88r-205,-514r96,0r151,423r2,0r145,-423r90,0","w":500,"k":{",":74,".":74}},"z":{"d":"22,0r0,-65r312,-377r-292,0r0,-75r402,0r0,58r-316,384r330,0r0,75r-436,0","w":480},"{":{"d":"309,-731r0,60r-50,0v-39,0,-48,44,-48,86r0,183v0,94,-64,128,-87,134r0,3v23,3,87,32,87,134r0,183v0,41,9,86,48,86r50,0r0,60v-108,9,-178,-14,-178,-131r0,-182v0,-73,-41,-117,-83,-117r0,-68v42,0,83,-48,83,-118r0,-182v9,-116,70,-141,178,-131","w":333},"|":{"d":"77,214r0,-1000r68,0r0,1000r-68,0","w":222},"}":{"d":"24,198r0,-60r50,0v39,0,48,-44,48,-86r0,-183v0,-94,64,-128,87,-134r0,-3v-23,-3,-87,-32,-87,-134r0,-181v0,-43,-9,-88,-48,-88r-50,0r0,-60v108,-9,178,14,178,131r0,182v0,73,41,118,83,118r0,68v-42,0,-83,47,-83,117r0,182v-9,116,-70,141,-178,131","w":333},"~":{"d":"194,-315v67,0,157,72,213,72v40,0,65,-41,88,-76r36,50v-30,42,-63,86,-125,86v-82,0,-131,-72,-217,-72v-44,0,-68,41,-84,76r-36,-50v22,-42,58,-86,125,-86","w":600},"\u00a1":{"d":"82,198r0,-220r25,-311r45,0r25,311r0,220r-95,0xm185,-517r0,111r-111,0r0,-111r111,0","w":259},"\u00a2":{"d":"272,-63r0,-391v-103,13,-136,103,-136,202v0,91,39,182,136,189xm272,116r0,-104v-146,-6,-226,-116,-226,-264v0,-144,76,-267,226,-277r0,-95r39,0r0,95v109,3,189,57,203,178r-87,0v-13,-61,-52,-101,-116,-103r0,391v72,-7,110,-58,121,-126r85,0v-10,99,-81,201,-206,201r0,104r-39,0"},"\u00a3":{"d":"398,17v-69,0,-132,-41,-204,-41v-39,0,-69,14,-100,35r-47,-66v84,-50,145,-167,85,-272r-85,0r0,-45r58,0v-27,-45,-47,-94,-47,-156v0,-127,106,-203,228,-203v147,0,239,87,236,258r-85,0v-3,-115,-50,-183,-146,-183v-80,0,-143,57,-143,131v0,66,29,104,52,153r159,0r0,45r-137,0v11,21,20,47,20,75v0,90,-56,139,-116,188r2,2v34,-15,65,-29,105,-29v74,-1,113,33,179,33v32,0,65,-18,89,-37r43,66v-40,34,-94,46,-146,46"},"\u2044":{"d":"-166,28r435,-753r64,0r-435,753r-64,0","w":167},"\u00a5":{"d":"232,0r0,-174r-141,0r0,-60r141,0r0,-42r-18,-33r-123,0r0,-60r92,0r-183,-345r109,0r174,353r169,-353r104,0r-178,345r93,0r0,60r-126,0r-18,33r0,42r144,0r0,60r-144,0r0,174r-95,0"},"\u0192":{"d":"492,-714r-14,75v-61,-2,-109,-3,-121,52r-31,138r102,0r-14,68r-100,0r-55,311v-42,212,-83,243,-256,235r14,-75v105,0,122,-10,139,-102r70,-369r-92,0r13,-68r91,0r26,-136v15,-76,69,-129,154,-129r74,0"},"\u00a7":{"d":"190,-420v-38,13,-73,49,-73,91v0,63,55,94,105,122v47,26,95,52,140,83v39,-16,72,-48,72,-93v0,-57,-63,-92,-106,-118v-46,-28,-93,-55,-138,-85xm465,-546r-85,0v-1,-61,-38,-110,-101,-110v-53,0,-95,28,-95,86v0,64,82,102,166,147v83,45,166,97,166,194v0,65,-44,124,-104,147v40,30,59,70,59,120v0,97,-79,160,-187,160v-118,0,-193,-72,-201,-189r85,0v1,68,46,114,114,114v49,0,101,-27,101,-83v0,-70,-86,-108,-174,-156v-85,-46,-174,-101,-174,-209v0,-62,48,-122,107,-138v-29,-30,-46,-63,-46,-106v0,-104,92,-162,185,-162v109,0,181,60,184,185"},"\u00a4":{"d":"25,-555r49,-49r56,56v75,-65,220,-68,294,-1r57,-56r50,51r-55,55v69,77,69,219,0,298r55,56r-47,47r-56,-58v-76,73,-223,73,-300,1r-55,58r-48,-48r55,-58v-68,-78,-71,-219,2,-294xm281,-524v-95,0,-168,77,-168,175v0,97,74,173,167,173v93,0,163,-79,163,-175v0,-94,-70,-173,-162,-173"},"'":{"d":"105,-456r0,-258r68,0r0,258r-68,0","w":278},"\u201c":{"d":"360,-567r0,111r-111,0v-4,-123,-7,-257,110,-258r0,50v-45,0,-58,51,-58,97r59,0xm177,-567r0,111r-111,0v-4,-123,-7,-257,110,-258r0,50v-45,0,-58,51,-58,97r59,0","w":426},"\u00ab":{"d":"398,-114r-152,-120r0,-88r152,-120r0,86r-100,78r100,78r0,86xm202,-114r-152,-120r0,-88r152,-120r0,86r-100,78r100,78r0,86","w":463},"\u2039":{"d":"202,-114r-152,-120r0,-88r152,-120r0,86r-100,78r100,78r0,86","w":259},"\u203a":{"d":"57,-442r152,120r0,88r-152,120r0,-86r100,-78r-100,-78r0,-86","w":259},"\u2013":{"d":"0,-238r0,-80r500,0r0,80r-500,0","w":500},"\u2020":{"d":"236,155r0,-597r-194,0r0,-75r194,0r0,-197r80,0r0,197r198,0r0,75r-198,0r0,597r-80,0"},"\u2021":{"d":"236,155r0,-197r-194,0r0,-75r194,0r0,-325r-194,0r0,-75r194,0r0,-197r80,0r0,197r198,0r0,75r-198,0r0,325r198,0r0,75r-198,0r0,197r-80,0"},"\u00b7":{"d":"75,-311v0,-35,30,-64,64,-64v34,0,64,30,64,64v0,32,-30,64,-63,64v-35,0,-65,-30,-65,-64","w":278},"\u00b6":{"d":"515,-714r0,869r-68,0r0,-814r-122,0r0,814r-68,0r0,-477v-122,0,-209,-82,-209,-186v0,-130,88,-206,228,-206r239,0","w":600},"\u2022":{"d":"72,-357v0,-99,79,-178,178,-178v99,0,179,79,179,178v0,99,-80,179,-179,179v-99,0,-178,-80,-178,-179","w":500},"\u201a":{"d":"83,0r0,-111r111,0v4,123,7,257,-110,258r0,-50v45,0,58,-51,58,-97r-59,0","w":278},"\u201e":{"d":"66,0r0,-111r111,0v4,123,7,257,-110,258r0,-50v45,0,58,-51,58,-97r-59,0xm249,0r0,-111r111,0v4,123,7,257,-110,258r0,-50v45,0,58,-51,58,-97r-59,0","w":426},"\u201d":{"d":"249,-603r0,-111r111,0v4,123,7,257,-110,258r0,-50v45,0,58,-51,58,-97r-59,0xm66,-603r0,-111r111,0v4,123,7,257,-110,258r0,-50v45,0,58,-51,58,-97r-59,0","w":426},"\u00bb":{"d":"65,-442r152,120r0,88r-152,120r0,-86r100,-78r-100,-78r0,-86xm261,-442r152,120r0,88r-152,120r0,-86r100,-78r-100,-78r0,-86","w":463},"\u2026":{"d":"111,0r0,-111r111,0r0,111r-111,0xm444,0r0,-111r111,0r0,111r-111,0xm777,0r0,-111r111,0r0,111r-111,0","w":1000},"\u2030":{"d":"72,-535v0,-97,40,-174,148,-174v158,0,181,195,106,296v-25,34,-67,45,-108,45v-104,0,-146,-71,-146,-167xm147,-538v0,52,10,115,72,115v59,0,72,-65,72,-114v0,-43,-11,-117,-68,-117v-61,0,-76,65,-76,116xm437,-155v0,-97,40,-174,148,-174v158,0,181,195,106,296v-25,34,-67,45,-108,45v-104,0,-146,-71,-146,-167xm512,-158v0,52,10,115,72,115v59,0,72,-65,72,-114v0,-43,-11,-117,-68,-117v-61,0,-76,65,-76,116xm782,-155v0,-97,40,-174,148,-174v158,0,181,195,106,296v-25,34,-67,45,-108,45v-104,0,-146,-71,-146,-167xm857,-158v0,52,10,115,72,115v59,0,72,-65,72,-114v0,-43,-11,-117,-68,-117v-61,0,-76,65,-76,116xm90,22r538,-744r85,0r-537,744r-86,0","w":1148},"\u00bf":{"d":"68,10v0,-170,188,-173,173,-349r85,0v10,105,-42,160,-90,206v-40,38,-83,77,-78,151v5,73,50,117,121,117v95,0,140,-73,139,-165r85,0v2,137,-84,240,-224,240v-123,0,-211,-73,-211,-200xm337,-517r0,111r-111,0r0,-111r111,0"},"`":{"d":"108,-588r-131,-143r109,0r86,143r-64,0","w":222},"\u00b4":{"d":"50,-588r86,-143r109,0r-131,143r-64,0","w":222},"\u02c6":{"d":"-39,-588r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":222},"\u02dc":{"d":"47,-707v49,-1,95,39,140,40v26,0,40,-24,46,-44r52,0v-9,56,-35,114,-110,111v-35,-1,-97,-39,-133,-39v-22,0,-50,17,-48,42r-52,0v16,-59,42,-110,105,-110","w":222},"\u00af":{"d":"-59,-620r0,-55r341,0r0,55r-341,0","w":222},"\u02d8":{"d":"-45,-731r50,0v16,56,55,83,112,83v58,0,79,-32,99,-83r51,0v-14,85,-70,138,-158,138v-91,0,-141,-49,-154,-138","w":222},"\u02d9":{"d":"156,-696r0,104r-90,0r0,-104r90,0","w":222},"\u00a8":{"d":"160,-592r0,-104r90,0r0,104r-90,0xm-28,-592r0,-104r90,0r0,104r-90,0","w":222},"\u02da":{"d":"43,-662v0,38,30,68,68,68v38,0,68,-30,68,-68v0,-38,-30,-68,-68,-68v-38,0,-68,30,-68,68xm-2,-662v0,-62,51,-113,113,-113v62,0,113,51,113,113v0,62,-51,113,-113,113v-62,0,-113,-51,-113,-113","w":222},"\u00b8":{"d":"5,191r18,-44v33,16,123,25,123,-18v0,-32,-58,-38,-88,-23r-17,-28r61,-78r42,0r-42,53v41,-13,123,2,119,68v-7,101,-137,103,-216,70","w":222},"\u02dd":{"d":"-48,-588r81,-143r105,0r-120,143r-66,0xm108,-588r81,-143r105,0r-120,143r-66,0","w":222},"\u02db":{"d":"190,-3r40,0v-70,58,-86,93,-86,121v0,21,11,42,38,42v31,0,47,-11,62,-30r32,18v-27,46,-67,62,-108,62v-24,0,-92,-3,-92,-82v0,-18,12,-70,114,-131","w":222},"\u02c7":{"d":"261,-731r-103,143r-93,0r-104,-143r84,0r69,96r70,-96r77,0","w":222},"\u2014":{"d":"130,-238r0,-80r740,0r0,80r-740,0","w":1000},"\u00c6":{"d":"-7,0r361,-714r535,0r0,80r-351,0r0,227r330,0r0,80r-330,0r0,247r357,0r0,80r-447,0r0,-208r-246,0r-102,208r-107,0xm448,-634r-41,0r-168,346r209,0r0,-346","w":926},"\u00aa":{"d":"317,-622r0,160v-1,37,11,34,36,30r0,44v-37,16,-96,9,-94,-38v-55,67,-233,68,-234,-43v0,-70,62,-93,120,-99v93,-10,115,-16,115,-42v0,-44,-39,-49,-77,-49v-48,0,-81,13,-83,56r-60,0v4,-77,70,-106,147,-106v60,0,130,15,130,87xm257,-546v-40,29,-165,5,-164,75v0,34,29,41,62,41v98,0,107,-40,102,-116","w":378},"\u0141":{"d":"78,0r0,-240r-79,56r0,-76r79,-56r0,-398r95,0r0,330r228,-160r0,76r-228,160r0,228r378,0r0,80r-473,0","k":{"T":92,"V":92,"W":55,"y":37,"\u00fd":37,"\u00ff":37,"Y":111,"\u00dd":111,"\u0178":111,"\u2019":74}},"\u00d8":{"d":"183,-170r369,-407v-39,-45,-96,-74,-172,-74v-246,2,-302,302,-197,481xm579,-541r-369,407v39,44,95,71,170,71v245,-2,301,-298,199,-478xm691,-730r32,30r-82,90v169,225,70,627,-261,627v-97,0,-174,-33,-229,-86r-81,89r-33,-30r84,-92v-171,-223,-73,-629,259,-629v99,0,176,34,232,88","w":760},"\u0152":{"d":"551,-505v2,-96,-77,-146,-177,-146v-162,0,-241,139,-241,300v0,159,82,288,239,288v100,0,179,-59,179,-141r0,-301xm551,0v-2,-22,4,-51,-2,-69v-35,55,-111,86,-184,86v-211,0,-327,-163,-327,-372v0,-215,123,-376,335,-376v68,0,146,28,178,76r0,-59r477,0r0,80r-382,0r0,227r350,0r0,80r-350,0r0,247r388,0r0,80r-483,0","w":1074},"\u00ba":{"d":"192,-709v109,0,167,72,167,166v0,96,-57,163,-166,163v-106,0,-168,-68,-168,-163v0,-93,58,-166,167,-166xm192,-659v-54,0,-99,36,-99,116v0,60,28,113,100,113v71,0,98,-52,98,-112v0,-97,-56,-117,-99,-117","w":384},"\u00e6":{"d":"756,-308v-2,-81,-53,-146,-143,-146v-91,0,-146,63,-146,146r289,0xm387,-267v-74,46,-262,4,-261,128v0,56,48,76,100,76v112,0,161,-70,161,-117r0,-87xm846,-233r-379,0v0,92,50,170,155,170v63,0,115,-33,131,-100r87,0v-25,112,-109,175,-224,175v-79,0,-153,-24,-187,-101v-39,71,-122,101,-218,101v-98,0,-175,-42,-175,-145v0,-117,87,-142,175,-159v125,-24,176,-20,176,-82v0,-70,-60,-80,-117,-80v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v60,0,142,10,173,75v35,-54,102,-75,165,-75v180,0,233,135,233,296","w":870},"\u0131":{"d":"154,0r-85,0r0,-517r85,0r0,517","w":222},"\u0142":{"d":"69,0r0,-321r-70,55r0,-71r70,-56r0,-321r85,0r0,254r70,-56r0,71r-70,56r0,389r-85,0","w":222},"\u00f8":{"d":"156,-137r242,-270v-105,-102,-272,-32,-272,149v0,48,11,88,30,121xm420,-377r-242,269v106,99,270,28,270,-150v0,-49,-11,-89,-28,-119xm540,-509r-60,66v38,47,58,112,58,185v0,151,-87,270,-251,270v-69,0,-124,-21,-165,-57r-60,67r-28,-24r62,-69v-40,-49,-60,-114,-60,-187v0,-152,87,-271,251,-271v70,0,126,21,168,59r58,-65","w":574},"\u0153":{"d":"864,-233r-375,0v-3,90,48,170,149,170v67,0,120,-37,140,-101r81,0v-34,117,-106,176,-221,176v-87,0,-156,-27,-188,-102v-32,75,-102,102,-179,102v-163,0,-235,-124,-235,-271v0,-151,75,-270,235,-270v79,0,145,19,179,91v36,-61,110,-91,183,-91v172,0,240,133,231,296xm489,-308r285,0v0,-87,-52,-146,-141,-146v-89,0,-145,61,-144,146xm415,-260v0,-95,-27,-194,-144,-194v-109,0,-145,105,-145,206v0,88,40,185,145,185v114,0,144,-104,144,-197","w":891},"\u00df":{"d":"64,0r0,-531v0,-119,79,-200,198,-200v98,0,207,59,207,175v0,76,-34,129,-104,157v90,11,136,92,136,179v0,150,-114,252,-274,228r0,-74v107,18,184,-46,184,-143v0,-130,-72,-150,-184,-150r0,-68v81,5,152,-23,152,-116v0,-63,-48,-113,-111,-113v-70,0,-119,29,-119,140r0,516r-85,0"},"\u00ac":{"d":"552,-388r0,279r-68,0r0,-211r-436,0r0,-68r504,0","w":600},"\u00b5":{"d":"492,-517r0,517r-80,0v-2,-26,4,-60,-2,-82v-36,64,-94,94,-168,94v-43,0,-74,-11,-91,-38r-2,0r0,224r-85,0r0,-715r85,0r0,350v0,63,40,104,105,104v103,0,153,-69,153,-162r0,-292r85,0"},"\u2122":{"d":"390,-714r0,55r-129,0r0,357r-68,0r0,-357r-129,0r0,-55r326,0xm927,-714r0,412r-68,0r0,-344r-2,0r-135,344r-43,0r-135,-344r-2,0r0,344r-68,0r0,-412r105,0r123,313r121,-313r104,0","w":990},"\u00d0":{"d":"78,0r0,-339r-70,0r0,-68r70,0r0,-307r245,0v220,0,343,110,343,340v0,239,-105,374,-343,374r-245,0xm173,-634r0,227r214,0r0,68r-214,0r0,259r158,0v65,0,240,-18,240,-281v0,-170,-63,-273,-238,-273r-160,0","w":704},"\u00bd":{"d":"135,28r435,-753r64,0r-435,753r-64,0xm219,-709r0,429r-68,0r0,-308r-115,0r0,-50v63,0,121,-15,133,-71r50,0xm803,-303v0,138,-209,157,-241,248r235,0r0,55r-303,0v5,-101,60,-137,125,-174v78,-45,116,-71,116,-130v0,-42,-37,-70,-83,-70v-60,0,-82,47,-82,99r-68,0v0,-92,60,-154,153,-154v81,0,148,46,148,126","w":834},"\u00b1":{"d":"48,-271r0,-68r218,0r0,-166r68,0r0,166r218,0r0,68r-218,0r0,167r-68,0r0,-167r-218,0xm48,0r0,-68r504,0r0,68r-504,0","w":600},"\u00de":{"d":"173,0r-95,0r0,-714r95,0r0,114r218,0v142,0,219,79,219,212v0,132,-77,210,-219,210r-218,0r0,178xm173,-520r0,262r186,0v107,1,156,-45,156,-130v0,-86,-49,-132,-156,-132r-186,0","w":648},"\u00bc":{"d":"723,-429r0,278r60,0r0,55r-60,0r0,96r-62,0r0,-96r-199,0r0,-58r210,-275r51,0xm659,-344r-148,193r150,0r0,-193r-2,0xm150,28r435,-753r64,0r-435,753r-64,0xm219,-709r0,429r-68,0r0,-308r-115,0r0,-50v63,0,121,-15,133,-71r50,0","w":834},"\u00f7":{"d":"48,-219r0,-68r504,0r0,68r-504,0xm236,-459v0,-35,30,-64,64,-64v34,0,64,30,64,64v0,32,-30,64,-63,64v-35,0,-65,-30,-65,-64xm236,-47v0,-35,30,-64,64,-64v34,0,64,30,64,64v0,32,-30,64,-63,64v-35,0,-65,-30,-65,-64","w":600},"\u00a6":{"d":"77,89r0,-250r68,0r0,250r-68,0xm145,-661r0,250r-68,0r0,-250r68,0","w":222},"\u00b0":{"d":"56,-565v0,-80,64,-144,144,-144v80,0,144,64,144,144v0,80,-64,144,-144,144v-80,0,-144,-64,-144,-144xm106,-565v0,47,35,94,94,94v123,0,124,-188,0,-188v-59,0,-94,47,-94,94","w":400},"\u00fe":{"d":"467,-263v0,-97,-50,-191,-160,-191v-114,0,-158,98,-158,195v0,102,46,196,163,196v116,0,155,-99,155,-200xm152,-714r0,267r2,0v28,-57,94,-82,165,-82v159,0,238,126,238,272v0,146,-78,269,-236,269v-55,0,-128,-24,-169,-81r0,267r-85,0r0,-912r85,0","w":593},"\u00be":{"d":"740,-429r0,278r60,0r0,55r-60,0r0,96r-62,0r0,-96r-199,0r0,-58r210,-275r51,0xm677,-340r-148,193r150,0r0,-193r-2,0xm215,28r435,-753r64,0r-435,753r-64,0xm177,-477r0,-50v57,3,106,-9,107,-65v0,-40,-33,-62,-77,-62v-54,0,-80,37,-80,86r-68,0v5,-86,59,-141,148,-141v82,0,145,39,145,117v0,40,-35,68,-68,87v58,11,87,51,87,102v0,88,-79,130,-162,130v-89,0,-160,-48,-160,-141r68,0v1,55,34,86,92,86v50,0,94,-27,94,-72v0,-68,-59,-81,-126,-77","w":834},"\u00ae":{"d":"24,-359v0,-211,172,-372,377,-372v204,0,375,161,375,372v0,215,-171,376,-375,376v-205,0,-377,-161,-377,-376xm92,-359v0,183,134,321,309,321v173,0,307,-138,307,-321v0,-179,-134,-317,-307,-317v-175,0,-309,138,-309,317xm255,-139r0,-437r169,0v104,0,153,41,153,125v0,79,-50,112,-115,119r125,193r-73,0r-119,-188r-72,0r0,188r-68,0xm323,-521r0,139v84,-4,186,22,186,-72v0,-90,-104,-62,-186,-67","w":800},"\u2212":{"d":"48,-219r0,-68r504,0r0,68r-504,0","w":600},"\u00f0":{"d":"272,-506v55,0,102,18,141,56v-31,-69,-91,-129,-122,-151r-115,54r-38,-40r109,-51v-21,-18,-51,-37,-70,-48r70,-45v18,13,51,35,77,56r120,-56r35,37r-115,53v119,98,174,230,174,383v0,151,-87,270,-251,270v-164,0,-251,-119,-251,-270v0,-130,84,-248,236,-248xm282,-431v-102,0,-156,71,-156,173v0,125,72,195,161,195v89,0,161,-70,161,-195v0,-91,-49,-173,-166,-173","w":574},"\u00d7":{"d":"119,-24r-48,-48r182,-182r-181,-180r48,-48r181,180r180,-180r48,48r-180,180r181,182r-48,48r-181,-182","w":600},"\u00a9":{"d":"24,-359v0,-211,172,-372,377,-372v204,0,375,161,375,372v0,215,-171,376,-375,376v-205,0,-377,-161,-377,-376xm92,-359v0,183,134,321,309,321v173,0,307,-138,307,-321v0,-178,-134,-317,-307,-317v-175,0,-309,139,-309,317xm535,-285r64,0v-21,97,-94,157,-186,157v-135,0,-222,-98,-222,-229v0,-133,82,-228,219,-228v95,0,173,52,188,152r-63,0v-11,-56,-58,-97,-124,-97v-99,0,-152,75,-152,171v0,94,59,176,154,176v66,0,113,-43,122,-102","w":800},"\u00c1":{"d":"-7,0r279,-714r105,0r279,714r-105,0r-78,-215r-302,0r-80,215r-98,0xm322,-628r-121,333r242,0r-119,-333r-2,0xm263,-765r86,-143r109,0r-131,143r-64,0","w":648},"\u00c2":{"d":"-7,0r279,-714r105,0r279,714r-105,0r-78,-215r-302,0r-80,215r-98,0xm322,-628r-121,333r242,0r-119,-333r-2,0xm174,-765r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":648},"\u00c4":{"d":"-7,0r279,-714r105,0r279,714r-105,0r-78,-215r-302,0r-80,215r-98,0xm322,-628r-121,333r242,0r-119,-333r-2,0xm373,-769r0,-104r90,0r0,104r-90,0xm185,-769r0,-104r90,0r0,104r-90,0","w":648},"\u00c0":{"d":"-7,0r279,-714r105,0r279,714r-105,0r-78,-215r-302,0r-80,215r-98,0xm322,-628r-121,333r242,0r-119,-333r-2,0xm321,-765r-131,-143r109,0r86,143r-64,0","w":648},"\u00c5":{"d":"-7,0r279,-714r105,0r279,714r-105,0r-78,-215r-302,0r-80,215r-98,0xm322,-628r-121,333r242,0r-119,-333r-2,0xm256,-839v0,38,30,68,68,68v38,0,68,-30,68,-68v0,-38,-30,-68,-68,-68v-38,0,-68,30,-68,68xm211,-839v0,-62,51,-113,113,-113v62,0,113,51,113,113v0,62,-51,113,-113,113v-62,0,-113,-51,-113,-113","w":648},"\u00c3":{"d":"-7,0r279,-714r105,0r279,714r-105,0r-78,-215r-302,0r-80,215r-98,0xm322,-628r-121,333r242,0r-119,-333r-2,0xm260,-884v49,-1,95,39,140,40v26,0,40,-24,46,-44r52,0v-9,56,-35,114,-110,111v-35,-1,-97,-39,-133,-39v-22,0,-50,17,-48,42r-52,0v16,-59,42,-110,105,-110","w":648},"\u00c7":{"d":"675,-499r-95,0v-20,-101,-101,-152,-200,-152v-170,0,-242,140,-242,289v0,163,72,299,243,299v125,0,197,-88,206,-206r95,0v-18,175,-126,283,-301,285r-29,37v41,-13,123,2,119,68v-7,101,-137,103,-216,70r18,-44v33,16,123,25,123,-18v0,-32,-58,-38,-88,-23r-17,-28r49,-63v-200,-15,-297,-172,-297,-370v0,-207,118,-376,338,-376v149,0,272,80,294,232","w":722},"\u00c9":{"d":"78,0r0,-714r493,0r0,80r-398,0r0,227r371,0r0,80r-371,0r0,247r401,0r0,80r-496,0xm245,-765r86,-143r109,0r-131,143r-64,0","w":611},"\u00ca":{"d":"78,0r0,-714r493,0r0,80r-398,0r0,227r371,0r0,80r-371,0r0,247r401,0r0,80r-496,0xm156,-765r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":611},"\u00cb":{"d":"78,0r0,-714r493,0r0,80r-398,0r0,227r371,0r0,80r-371,0r0,247r401,0r0,80r-496,0xm355,-769r0,-104r90,0r0,104r-90,0xm167,-769r0,-104r90,0r0,104r-90,0","w":611},"\u00c8":{"d":"78,0r0,-714r493,0r0,80r-398,0r0,227r371,0r0,80r-371,0r0,247r401,0r0,80r-496,0xm303,-765r-131,-143r109,0r86,143r-64,0","w":611},"\u00cd":{"d":"82,0r0,-714r95,0r0,714r-95,0xm69,-765r86,-143r109,0r-131,143r-64,0","w":259},"\u00ce":{"d":"82,0r0,-714r95,0r0,714r-95,0xm-20,-765r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":259},"\u00cf":{"d":"82,0r0,-714r95,0r0,714r-95,0xm179,-769r0,-104r90,0r0,104r-90,0xm-9,-769r0,-104r90,0r0,104r-90,0","w":259},"\u00cc":{"d":"82,0r0,-714r95,0r0,714r-95,0xm127,-765r-131,-143r109,0r86,143r-64,0","w":259},"\u00d1":{"d":"77,0r0,-714r101,0r376,579r2,0r0,-579r90,0r0,714r-104,0r-373,-573r-2,0r0,573r-90,0xm297,-884v49,-1,95,39,140,40v26,0,40,-24,46,-44r52,0v-9,56,-35,114,-110,111v-35,-1,-97,-39,-133,-39v-22,0,-50,17,-48,42r-52,0v16,-59,42,-110,105,-110","w":722},"\u00d3":{"d":"38,-357v0,-194,115,-374,342,-374v227,0,342,180,342,374v0,194,-115,374,-342,374v-227,0,-342,-180,-342,-374xm133,-357v0,144,73,294,247,294v174,0,247,-150,247,-294v0,-144,-73,-294,-247,-294v-174,0,-247,150,-247,294xm319,-765r86,-143r109,0r-131,143r-64,0","w":760},"\u00d4":{"d":"38,-357v0,-194,115,-374,342,-374v227,0,342,180,342,374v0,194,-115,374,-342,374v-227,0,-342,-180,-342,-374xm133,-357v0,144,73,294,247,294v174,0,247,-150,247,-294v0,-144,-73,-294,-247,-294v-174,0,-247,150,-247,294xm230,-765r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":760},"\u00d6":{"d":"38,-357v0,-194,115,-374,342,-374v227,0,342,180,342,374v0,194,-115,374,-342,374v-227,0,-342,-180,-342,-374xm133,-357v0,144,73,294,247,294v174,0,247,-150,247,-294v0,-144,-73,-294,-247,-294v-174,0,-247,150,-247,294xm429,-769r0,-104r90,0r0,104r-90,0xm241,-769r0,-104r90,0r0,104r-90,0","w":760},"\u00d2":{"d":"38,-357v0,-194,115,-374,342,-374v227,0,342,180,342,374v0,194,-115,374,-342,374v-227,0,-342,-180,-342,-374xm133,-357v0,144,73,294,247,294v174,0,247,-150,247,-294v0,-144,-73,-294,-247,-294v-174,0,-247,150,-247,294xm377,-765r-131,-143r109,0r86,143r-64,0","w":760},"\u00d5":{"d":"38,-357v0,-194,115,-374,342,-374v227,0,342,180,342,374v0,194,-115,374,-342,374v-227,0,-342,-180,-342,-374xm133,-357v0,144,73,294,247,294v174,0,247,-150,247,-294v0,-144,-73,-294,-247,-294v-174,0,-247,150,-247,294xm316,-884v49,-1,95,39,140,40v26,0,40,-24,46,-44r52,0v-9,56,-35,114,-110,111v-35,-1,-97,-39,-133,-39v-22,0,-50,17,-48,42r-52,0v16,-59,42,-110,105,-110","w":760},"\u0160":{"d":"581,-505r-90,0v-11,-102,-79,-146,-177,-146v-80,0,-160,29,-160,122v0,88,114,101,229,127v114,26,228,66,228,206v0,152,-149,213,-279,213v-160,0,-295,-78,-295,-252r90,0v0,120,100,172,208,172v86,0,181,-27,181,-130v0,-98,-114,-115,-228,-140v-114,-25,-229,-58,-229,-189v0,-145,129,-209,259,-209v146,0,257,69,263,226xm474,-908r-103,143r-93,0r-104,-143r84,0r69,96r70,-96r77,0","w":648},"\u00da":{"d":"648,-714r0,456v0,180,-103,275,-281,275v-184,0,-293,-85,-293,-275r0,-456r95,0r0,456v0,126,72,195,198,195v120,0,186,-69,186,-195r0,-456r95,0xm300,-765r86,-143r109,0r-131,143r-64,0","w":722},"\u00db":{"d":"648,-714r0,456v0,180,-103,275,-281,275v-184,0,-293,-85,-293,-275r0,-456r95,0r0,456v0,126,72,195,198,195v120,0,186,-69,186,-195r0,-456r95,0xm211,-765r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":722},"\u00dc":{"d":"648,-714r0,456v0,180,-103,275,-281,275v-184,0,-293,-85,-293,-275r0,-456r95,0r0,456v0,126,72,195,198,195v120,0,186,-69,186,-195r0,-456r95,0xm410,-769r0,-104r90,0r0,104r-90,0xm222,-769r0,-104r90,0r0,104r-90,0","w":722},"\u00d9":{"d":"648,-714r0,456v0,180,-103,275,-281,275v-184,0,-293,-85,-293,-275r0,-456r95,0r0,456v0,126,72,195,198,195v120,0,186,-69,186,-195r0,-456r95,0xm358,-765r-131,-143r109,0r86,143r-64,0","w":722},"\u00dd":{"d":"646,-714r-273,422r0,292r-95,0r0,-292r-276,-422r113,0r214,340r209,-340r108,0xm263,-765r86,-143r109,0r-131,143r-64,0","w":648,"k":{"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":139,".":139,"e":111,"\u00e9":111,"\u00ea":111,"\u00eb":111,"\u00e8":111,"o":111,"\u00f8":111,"\u0153":111,"\u00f3":111,"\u00f4":111,"\u00f6":111,"\u00f2":111,"\u00f5":111,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"u":74,"\u00fa":74,"\u00fb":74,"\u00fc":74,"\u00f9":74,":":92,";":92,"i":37,"\u00ed":37,"\u00ee":37,"\u00ef":37,"\u00ec":37,"p":74}},"\u0178":{"d":"646,-714r-273,422r0,292r-95,0r0,-292r-276,-422r113,0r214,340r209,-340r108,0xm373,-769r0,-104r90,0r0,104r-90,0xm185,-769r0,-104r90,0r0,104r-90,0","w":648,"k":{"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":139,".":139,"e":111,"\u00e9":111,"\u00ea":111,"\u00eb":111,"\u00e8":111,"o":111,"\u00f8":111,"\u0153":111,"\u00f3":111,"\u00f4":111,"\u00f6":111,"\u00f2":111,"\u00f5":111,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"u":74,"\u00fa":74,"\u00fb":74,"\u00fc":74,"\u00f9":74,":":92,";":92,"i":37,"\u00ed":37,"\u00ee":37,"\u00ef":37,"\u00ec":37,"p":74}},"\u017d":{"d":"22,0r0,-80r444,-554r-410,0r0,-80r522,0r0,80r-443,554r455,0r0,80r-568,0xm456,-908r-103,143r-93,0r-104,-143r84,0r69,96r70,-96r77,0","w":611},"\u00e1":{"d":"384,-267v-70,49,-258,4,-258,128v0,56,48,76,100,76v112,0,158,-70,158,-117r0,-87xm522,-68r0,66v-51,28,-141,18,-133,-66v-47,55,-110,80,-182,80v-94,0,-171,-42,-171,-145v0,-117,87,-142,175,-159v94,-18,174,-12,174,-76v0,-74,-61,-86,-115,-86v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v93,0,194,21,194,142r0,266v-4,55,13,67,53,53xm208,-588r86,-143r109,0r-131,143r-64,0","w":537},"\u00e2":{"d":"384,-267v-70,49,-258,4,-258,128v0,56,48,76,100,76v112,0,158,-70,158,-117r0,-87xm522,-68r0,66v-51,28,-141,18,-133,-66v-47,55,-110,80,-182,80v-94,0,-171,-42,-171,-145v0,-117,87,-142,175,-159v94,-18,174,-12,174,-76v0,-74,-61,-86,-115,-86v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v93,0,194,21,194,142r0,266v-4,55,13,67,53,53xm119,-588r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":537},"\u00e4":{"d":"384,-267v-70,49,-258,4,-258,128v0,56,48,76,100,76v112,0,158,-70,158,-117r0,-87xm522,-68r0,66v-51,28,-141,18,-133,-66v-47,55,-110,80,-182,80v-94,0,-171,-42,-171,-145v0,-117,87,-142,175,-159v94,-18,174,-12,174,-76v0,-74,-61,-86,-115,-86v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v93,0,194,21,194,142r0,266v-4,55,13,67,53,53xm318,-592r0,-104r90,0r0,104r-90,0xm130,-592r0,-104r90,0r0,104r-90,0","w":537},"\u00e0":{"d":"384,-267v-70,49,-258,4,-258,128v0,56,48,76,100,76v112,0,158,-70,158,-117r0,-87xm522,-68r0,66v-51,28,-141,18,-133,-66v-47,55,-110,80,-182,80v-94,0,-171,-42,-171,-145v0,-117,87,-142,175,-159v94,-18,174,-12,174,-76v0,-74,-61,-86,-115,-86v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v93,0,194,21,194,142r0,266v-4,55,13,67,53,53xm266,-588r-131,-143r109,0r86,143r-64,0","w":537},"\u00e5":{"d":"384,-267v-70,49,-258,4,-258,128v0,56,48,76,100,76v112,0,158,-70,158,-117r0,-87xm522,-68r0,66v-51,28,-141,18,-133,-66v-47,55,-110,80,-182,80v-94,0,-171,-42,-171,-145v0,-117,87,-142,175,-159v94,-18,174,-12,174,-76v0,-74,-61,-86,-115,-86v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v93,0,194,21,194,142r0,266v-4,55,13,67,53,53xm201,-662v0,38,30,68,68,68v38,0,68,-30,68,-68v0,-38,-30,-68,-68,-68v-38,0,-68,30,-68,68xm156,-662v0,-62,51,-113,113,-113v62,0,113,51,113,113v0,62,-51,113,-113,113v-62,0,-113,-51,-113,-113","w":537},"\u00e3":{"d":"384,-267v-70,49,-258,4,-258,128v0,56,48,76,100,76v112,0,158,-70,158,-117r0,-87xm522,-68r0,66v-51,28,-141,18,-133,-66v-47,55,-110,80,-182,80v-94,0,-171,-42,-171,-145v0,-117,87,-142,175,-159v94,-18,174,-12,174,-76v0,-74,-61,-86,-115,-86v-72,0,-125,22,-129,98r-85,0v5,-128,104,-173,219,-173v93,0,194,21,194,142r0,266v-4,55,13,67,53,53xm205,-707v49,-1,95,39,140,40v26,0,40,-24,46,-44r52,0v-9,56,-35,114,-110,111v-35,-1,-97,-39,-133,-39v-22,0,-50,17,-48,42r-52,0v16,-59,42,-110,105,-110","w":537},"\u00e7":{"d":"503,-351r-88,0v-13,-65,-58,-103,-128,-103v-123,0,-161,97,-161,202v0,95,43,189,151,189v82,0,131,-48,142,-126r87,0v-18,121,-92,195,-213,200r-33,42v41,-13,123,2,119,68v-7,101,-137,103,-216,70r18,-44v33,16,123,25,123,-18v0,-32,-58,-38,-88,-23r-17,-28r53,-68v-141,-11,-216,-117,-216,-262v0,-155,80,-277,244,-277v117,0,211,55,223,178","w":537},"\u00e9":{"d":"421,-164r84,0v-23,114,-105,176,-221,176v-165,0,-248,-114,-248,-272v0,-155,102,-269,243,-269v183,0,239,171,234,296r-387,0v-3,90,48,170,161,170v70,0,119,-34,134,-101xm126,-308r297,0v-4,-81,-65,-146,-149,-146v-89,0,-142,67,-148,146xm208,-588r86,-143r109,0r-131,143r-64,0","w":537},"\u00ea":{"d":"421,-164r84,0v-23,114,-105,176,-221,176v-165,0,-248,-114,-248,-272v0,-155,102,-269,243,-269v183,0,239,171,234,296r-387,0v-3,90,48,170,161,170v70,0,119,-34,134,-101xm126,-308r297,0v-4,-81,-65,-146,-149,-146v-89,0,-142,67,-148,146xm119,-588r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":537},"\u00eb":{"d":"421,-164r84,0v-23,114,-105,176,-221,176v-165,0,-248,-114,-248,-272v0,-155,102,-269,243,-269v183,0,239,171,234,296r-387,0v-3,90,48,170,161,170v70,0,119,-34,134,-101xm126,-308r297,0v-4,-81,-65,-146,-149,-146v-89,0,-142,67,-148,146xm318,-592r0,-104r90,0r0,104r-90,0xm130,-592r0,-104r90,0r0,104r-90,0","w":537},"\u00e8":{"d":"421,-164r84,0v-23,114,-105,176,-221,176v-165,0,-248,-114,-248,-272v0,-155,102,-269,243,-269v183,0,239,171,234,296r-387,0v-3,90,48,170,161,170v70,0,119,-34,134,-101xm126,-308r297,0v-4,-81,-65,-146,-149,-146v-89,0,-142,67,-148,146xm266,-588r-131,-143r109,0r86,143r-64,0","w":537},"\u00ed":{"d":"154,0r-85,0r0,-517r85,0r0,517xm50,-588r86,-143r109,0r-131,143r-64,0","w":222},"\u00ee":{"d":"154,0r-85,0r0,-517r85,0r0,517xm-39,-588r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":222},"\u00ef":{"d":"154,0r-85,0r0,-517r85,0r0,517xm160,-592r0,-104r90,0r0,104r-90,0xm-28,-592r0,-104r90,0r0,104r-90,0","w":222},"\u00ec":{"d":"154,0r-85,0r0,-517r85,0r0,517xm108,-588r-131,-143r109,0r86,143r-64,0","w":222},"\u00f1":{"d":"64,0r0,-517r80,0v2,26,-4,60,2,82v36,-64,94,-94,168,-94v136,0,178,78,178,189r0,340r-85,0r0,-350v0,-63,-40,-104,-105,-104v-103,0,-153,69,-153,162r0,292r-85,0xm214,-707v49,-1,95,39,140,40v26,0,40,-24,46,-44r52,0v-9,56,-35,114,-110,111v-35,-1,-97,-39,-133,-39v-22,0,-50,17,-48,42r-52,0v16,-59,42,-110,105,-110"},"\u00f3":{"d":"36,-258v0,-152,87,-271,251,-271v164,0,251,119,251,271v0,151,-87,270,-251,270v-164,0,-251,-119,-251,-270xm126,-258v0,125,72,195,161,195v89,0,161,-70,161,-195v0,-126,-72,-196,-161,-196v-89,0,-161,70,-161,196xm226,-588r86,-143r109,0r-131,143r-64,0","w":574},"\u00f4":{"d":"36,-258v0,-152,87,-271,251,-271v164,0,251,119,251,271v0,151,-87,270,-251,270v-164,0,-251,-119,-251,-270xm126,-258v0,125,72,195,161,195v89,0,161,-70,161,-195v0,-126,-72,-196,-161,-196v-89,0,-161,70,-161,196xm137,-588r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0","w":574},"\u00f6":{"d":"36,-258v0,-152,87,-271,251,-271v164,0,251,119,251,271v0,151,-87,270,-251,270v-164,0,-251,-119,-251,-270xm126,-258v0,125,72,195,161,195v89,0,161,-70,161,-195v0,-126,-72,-196,-161,-196v-89,0,-161,70,-161,196xm336,-592r0,-104r90,0r0,104r-90,0xm148,-592r0,-104r90,0r0,104r-90,0","w":574},"\u00f2":{"d":"36,-258v0,-152,87,-271,251,-271v164,0,251,119,251,271v0,151,-87,270,-251,270v-164,0,-251,-119,-251,-270xm126,-258v0,125,72,195,161,195v89,0,161,-70,161,-195v0,-126,-72,-196,-161,-196v-89,0,-161,70,-161,196xm284,-588r-131,-143r109,0r86,143r-64,0","w":574},"\u00f5":{"d":"36,-258v0,-152,87,-271,251,-271v164,0,251,119,251,271v0,151,-87,270,-251,270v-164,0,-251,-119,-251,-270xm126,-258v0,125,72,195,161,195v89,0,161,-70,161,-195v0,-126,-72,-196,-161,-196v-89,0,-161,70,-161,196xm223,-707v49,-1,95,39,140,40v26,0,40,-24,46,-44r52,0v-9,56,-35,114,-110,111v-35,-1,-97,-39,-133,-39v-22,0,-50,17,-48,42r-52,0v16,-59,42,-110,105,-110","w":574},"\u0161":{"d":"31,-163r85,0v3,76,69,100,139,100v53,0,125,-12,125,-77v0,-66,-84,-77,-169,-96v-84,-19,-169,-47,-169,-146v0,-104,103,-147,193,-147v114,0,205,36,212,162r-85,0v-6,-66,-64,-87,-120,-87v-51,0,-110,14,-110,66v0,61,90,72,169,90v85,20,169,47,169,147v0,123,-115,163,-220,163v-116,0,-214,-47,-219,-175xm400,-731r-103,143r-93,0r-104,-143r84,0r69,96r70,-96r77,0","w":500},"\u00fa":{"d":"492,-517r0,517r-80,0v-2,-26,4,-60,-2,-82v-36,64,-94,94,-168,94v-136,0,-178,-78,-178,-189r0,-340r85,0r0,350v0,63,40,104,105,104v103,0,153,-69,153,-162r0,-292r85,0xm217,-588r86,-143r109,0r-131,143r-64,0"},"\u00fb":{"d":"492,-517r0,517r-80,0v-2,-26,4,-60,-2,-82v-36,64,-94,94,-168,94v-136,0,-178,-78,-178,-189r0,-340r85,0r0,350v0,63,40,104,105,104v103,0,153,-69,153,-162r0,-292r85,0xm128,-588r103,-143r93,0r104,143r-84,0r-69,-96r-70,96r-77,0"},"\u00fc":{"d":"492,-517r0,517r-80,0v-2,-26,4,-60,-2,-82v-36,64,-94,94,-168,94v-136,0,-178,-78,-178,-189r0,-340r85,0r0,350v0,63,40,104,105,104v103,0,153,-69,153,-162r0,-292r85,0xm327,-592r0,-104r90,0r0,104r-90,0xm139,-592r0,-104r90,0r0,104r-90,0"},"\u00f9":{"d":"492,-517r0,517r-80,0v-2,-26,4,-60,-2,-82v-36,64,-94,94,-168,94v-136,0,-178,-78,-178,-189r0,-340r85,0r0,350v0,63,40,104,105,104v103,0,153,-69,153,-162r0,-292r85,0xm275,-588r-131,-143r109,0r86,143r-64,0"},"\u00fd":{"d":"492,-517r-225,586v-46,123,-95,156,-209,129r0,-78v48,21,101,13,120,-35r35,-88r-205,-514r96,0r151,423r2,0r145,-423r90,0xm189,-588r86,-143r109,0r-131,143r-64,0","w":500,"k":{",":74,".":74}},"\u00ff":{"d":"492,-517r-225,586v-46,123,-95,156,-209,129r0,-78v48,21,101,13,120,-35r35,-88r-205,-514r96,0r151,423r2,0r145,-423r90,0xm299,-592r0,-104r90,0r0,104r-90,0xm111,-592r0,-104r90,0r0,104r-90,0","w":500,"k":{",":74,".":74}},"\u017e":{"d":"22,0r0,-65r312,-377r-292,0r0,-75r402,0r0,58r-316,384r330,0r0,75r-436,0xm390,-731r-103,143r-93,0r-104,-143r84,0r69,96r70,-96r77,0","w":480},"\u00b9":{"d":"249,-709r0,429r-68,0r0,-308r-115,0r0,-50v63,-1,121,-15,133,-71r50,0","w":333},"\u00b2":{"d":"323,-583v0,138,-209,157,-241,248r235,0r0,55r-303,0v5,-101,60,-137,125,-174v78,-45,116,-71,116,-130v0,-42,-37,-70,-83,-70v-60,0,-82,47,-82,99r-68,0v0,-92,60,-154,153,-154v81,0,148,46,148,126","w":333},"\u00b3":{"d":"132,-477r0,-50v57,3,106,-9,107,-65v0,-40,-33,-62,-77,-62v-54,0,-80,37,-80,86r-68,0v5,-86,59,-141,148,-141v82,0,145,39,145,117v0,40,-35,68,-68,87v58,11,87,51,87,102v0,88,-79,130,-162,130v-89,0,-160,-48,-160,-141r68,0v1,55,34,86,92,86v50,0,94,-27,94,-72v0,-68,-59,-81,-126,-77","w":333},"\u2206":{"d":"31,0r0,-53r248,-660r93,0r245,659r0,54r-586,0xm112,-65r419,0r-142,-372v-17,-54,-52,-139,-65,-185r-4,0v-9,37,-39,116,-59,169","w":647},"\u2126":{"d":"48,-66v48,-2,104,4,148,-2v-70,-62,-134,-168,-134,-307v0,-193,129,-331,300,-331v181,0,294,157,294,327v0,145,-72,252,-136,313r149,0r0,66r-242,0r0,-48v75,-49,148,-157,148,-316v0,-130,-73,-275,-214,-275v-133,0,-218,123,-218,278v0,148,73,266,147,313r0,48r-242,0r0,-66","w":719},"\u03bc":{"d":"492,-517r0,517r-80,0v-2,-26,4,-60,-2,-82v-36,64,-94,94,-168,94v-43,0,-74,-11,-91,-38r-2,0r0,224r-85,0r0,-715r85,0r0,350v0,63,40,104,105,104v103,0,153,-69,153,-162r0,-292r85,0"},"\u03c0":{"d":"547,-449r-80,0v3,134,-10,351,14,449r-77,0v-31,-85,-12,-321,-17,-449r-177,0v-6,127,-35,354,-72,449r-79,0v38,-108,67,-318,72,-449v-61,0,-89,5,-110,12r-12,-53v25,-16,68,-27,145,-27r400,0","w":573},"\u20ac":{"d":"93,-450v28,-152,119,-274,284,-275v61,0,121,21,181,63r-46,83v-82,-85,-194,-97,-267,-5v-27,35,-45,79,-56,134r283,0r-27,61r-263,0v-1,18,-2,36,0,52r249,0r-26,61r-218,0v22,143,85,215,190,215v76,0,105,-28,154,-72r0,105v-45,29,-97,44,-154,44v-163,0,-259,-97,-287,-292r-77,0r27,-61r46,0v-2,-17,-1,-37,0,-52r-73,0r26,-61r54,0"},"\u2113":{"d":"427,-153r37,34v-45,85,-110,126,-188,126v-119,0,-166,-87,-167,-194r-57,46r-24,-46v29,-24,56,-45,80,-68r0,-270v0,-181,79,-241,156,-241v86,0,126,73,126,163v0,125,-82,243,-206,364v-7,109,42,183,108,183v62,0,110,-52,135,-97xm184,-531r0,214v85,-93,151,-190,151,-285v0,-62,-19,-104,-70,-104v-36,0,-81,42,-81,175","w":487},"\u212e":{"d":"835,-338r-649,0v-3,0,-5,1,-5,4r0,196v0,9,2,17,8,23v62,66,150,106,246,106v103,0,195,-47,258,-119r57,0v-71,84,-187,139,-316,139v-221,0,-400,-161,-400,-359v0,-199,179,-361,400,-361v224,0,406,162,401,371xm687,-364r0,-197v0,-9,-3,-17,-9,-24v-63,-62,-148,-102,-243,-102v-95,0,-182,41,-245,105v-6,7,-9,15,-9,25r0,193v0,2,2,6,5,6r497,0v3,0,4,-4,4,-6","w":869},"\u2202":{"d":"102,-666r-27,-58v32,-26,85,-56,168,-56v132,0,255,122,255,377v0,233,-91,411,-263,411v-131,0,-193,-119,-193,-231v0,-155,100,-254,212,-254v93,0,147,66,165,94v14,-181,-72,-331,-184,-331v-62,0,-106,26,-133,48xm121,-223v0,95,48,164,122,164v93,0,153,-120,167,-247v-15,-42,-66,-106,-142,-106v-81,0,-147,86,-147,189","w":548},"\u220f":{"d":"674,-622r-109,0r0,718r-78,0r0,-718r-274,0r0,718r-79,0r0,-718r-108,0r0,-72r648,0r0,72","w":701},"\u2211":{"d":"531,96r-508,0r0,-53r266,-341r-255,-341r0,-55r481,0r0,68r-371,0r0,4r236,312r-255,328r0,3r406,0r0,75","w":554},"\u2219":{"d":"75,-311v0,-35,30,-64,64,-64v34,0,64,30,64,64v0,32,-30,64,-63,64v-35,0,-65,-30,-65,-64","w":278},"\u221a":{"d":"574,-826r-222,974r-70,0r-158,-459r-75,28r-15,-48r139,-54r125,366v8,24,13,55,19,73v3,-19,9,-49,14,-74r184,-806r59,0","w":573},"\u221e":{"d":"578,-443v87,0,146,63,146,155v0,98,-74,160,-153,160v-62,0,-113,-38,-183,-118v-52,61,-107,118,-188,118v-81,0,-150,-67,-150,-156v0,-92,67,-159,157,-159v75,0,132,55,185,118v50,-56,101,-118,186,-118xm102,-283v0,58,43,106,106,106v63,0,112,-61,153,-105v-44,-55,-89,-114,-160,-114v-63,0,-99,53,-99,113xm672,-284v0,-71,-42,-113,-101,-112v-66,0,-119,73,-153,109v65,73,103,110,156,110v63,0,98,-56,98,-107","w":774},"\u222b":{"d":"343,-836r-13,55v-27,-15,-65,-13,-86,14v-24,30,-34,83,-34,176v0,157,14,335,14,498v0,106,-21,165,-42,201v-32,55,-115,65,-168,37r13,-58v23,13,72,19,91,-10v18,-27,34,-70,34,-170v0,-165,-14,-347,-14,-505v0,-110,16,-170,54,-212v36,-40,104,-51,151,-26","w":361},"\u2248":{"d":"183,-432v90,0,128,73,205,75v41,0,67,-26,95,-74r32,28v-28,54,-69,96,-129,96v-67,0,-139,-75,-210,-76v-46,0,-73,36,-98,74r-33,-27v30,-59,80,-96,138,-96xm183,-261v91,0,126,75,205,75v41,0,66,-25,95,-73r32,28v-28,53,-68,95,-129,95v-67,0,-139,-75,-209,-76v-48,0,-74,36,-99,75r-33,-28v30,-58,80,-96,138,-96","w":561},"\u2260":{"d":"399,-503r-44,97r155,0r0,51r-175,0r-67,145r242,0r0,51r-262,0r-54,117r-40,-18r45,-99r-148,0r0,-51r169,0r66,-145r-235,0r0,-51r255,0r53,-115","w":561},"\u2264":{"d":"507,-96r-451,-224r0,-54r451,-224r0,61r-397,188r0,2r397,191r0,60xm510,-3r-459,0r0,-52r459,0r0,52","w":561},"\u2265":{"d":"57,-598r451,224r0,54r-451,224r0,-60r396,-190r0,-2r-396,-189r0,-61xm509,-3r-457,0r0,-52r457,0r0,52","w":561},"\u25ca":{"d":"519,-347r-202,393r-64,0r-199,-393r201,-393r65,0xm446,-345v-52,-110,-116,-209,-159,-328r-2,0v-43,116,-108,216,-159,324v51,110,117,211,159,329r3,0v42,-118,107,-216,158,-325","w":573},"\u00a0":{"w":278},"\u00ad":{"d":"50,-238r0,-80r289,0r0,80r-289,0","w":389},"\u02c9":{"d":"-59,-620r0,-55r341,0r0,55r-341,0","w":222},"\u2215":{"d":"-166,28r435,-753r64,0r-435,753r-64,0","w":167},"\u03a9":{"d":"48,-66v48,-2,104,4,148,-2v-70,-62,-134,-168,-134,-307v0,-193,129,-331,300,-331v181,0,294,157,294,327v0,145,-72,252,-136,313r149,0r0,66r-242,0r0,-48v75,-49,148,-157,148,-316v0,-130,-73,-275,-214,-275v-133,0,-218,123,-218,278v0,148,73,266,147,313r0,48r-242,0r0,-66","w":719}}});
;
/*!
 * The following copyright notice may not be removed under any circumstances.
 * 
 * Copyright:
 * Copyright ?1988, 1990, 1993, 2002 Adobe Systems Incorporated.  All Rights
 * Reserved. ?1981, 2002 Heidelberger Druckmaschinen AG. All rights reserved.
 * 
 * Trademark:
 * Helvetica is a trademark of Heidelberger Druckmaschinen AG, exclusively
 * licensed through Linotype Library GmbH, and may be registered in certain
 * jurisdictions.
 * 
 * Full name:
 * HelveticaNeueLTStd-Lt
 * 
 * Designer:
 * Linotype Staff
 * 
 * Vendor URL:
 * http://www.adobe.com/type
 * 
 * License information:
 * http://www.adobe.com/type/legal.html
 */
Cufon.registerFont({"w":556,"face":{"font-family":"Helvetica Neue LT Std 45 Light","font-weight":300,"font-stretch":"normal","units-per-em":"1000","panose-1":"2 11 4 3 2 2 2 2 2 4","ascent":"714","descent":"-286","x-height":"15","bbox":"-166 -967 1050 214","underline-thickness":"50","underline-position":"-50","stemh":"53","stemv":"63","unicode-range":"U+0020-U+FB02"},"glyphs":{" ":{"w":278},"\ufb01":{"d":"320,0r0,-516r63,0r0,516r-63,0xm94,0r0,-463r-88,0r0,-53r88,0v-4,-112,5,-199,125,-198v18,0,33,1,53,4r0,54v-17,-3,-31,-5,-46,-5v-87,-1,-65,72,-69,145r102,0r0,53r-102,0r0,463r-63,0xm320,-613r0,-101r63,0r0,101r-63,0","w":444,"k":{"\u2019":-18,"f":18,"\ufb01":18,"\ufb02":18,"\u00df":18}},"\ufb02":{"d":"94,0r0,-463r-88,0r0,-53r88,0v-4,-112,5,-199,125,-198v18,0,33,1,53,4r0,54v-17,-3,-31,-5,-46,-5v-87,-1,-65,72,-69,145r102,0r0,53r-102,0r0,463r-63,0xm320,0r0,-714r63,0r0,714r-63,0","w":444,"k":{"\u2019":-18,"f":18,"\ufb01":18,"\ufb02":18,"\u00df":18}},"!":{"d":"102,-178r-15,-323r0,-213r68,0r0,213r-15,323r-38,0xm163,0r-84,0r0,-106r84,0r0,106","w":241},"\"":{"d":"87,-471r0,-243r63,0r0,243r-63,0xm220,-471r0,-243r63,0r0,243r-63,0","w":370},"#":{"d":"497,-268r0,45r-111,0r-31,223r-50,0r31,-223r-141,0r-31,223r-50,0r31,-223r-110,0r0,-45r116,0r22,-156r-110,0r0,-45r116,0r31,-222r50,0r-31,222r141,0r31,-222r50,0r-31,222r105,0r0,45r-111,0r-22,156r105,0xm364,-424r-141,0r-22,156r141,0"},"$":{"d":"296,-326r0,288v85,0,168,-42,168,-146v0,-95,-90,-123,-168,-142xm257,-397r0,-279v-80,0,-147,44,-147,149v0,93,72,112,147,130xm29,-228r63,0v1,115,60,178,165,190r0,-296v-113,-25,-210,-60,-210,-200v0,-120,94,-195,210,-195r0,-78r39,0r0,78v116,0,215,86,212,217r-63,0v-1,-100,-62,-164,-149,-164r0,288v114,24,231,63,231,203v0,131,-114,201,-231,200r0,85r-39,0r0,-86v-156,-15,-222,-90,-228,-242"},"%":{"d":"228,-661v-84,0,-106,81,-106,145v0,63,22,143,106,143v84,0,106,-80,106,-143v0,-64,-22,-145,-106,-145xm228,-706v113,0,159,82,159,189v0,107,-46,189,-159,189v-113,0,-159,-82,-159,-189v0,-107,46,-189,159,-189xm667,-363v113,0,159,82,159,189v0,107,-46,189,-159,189v-113,0,-159,-82,-159,-189v0,-107,46,-189,159,-189xm667,-318v-84,0,-106,81,-106,145v0,63,22,143,106,143v84,0,106,-80,106,-143v0,-64,-22,-145,-106,-145xm190,34r453,-759r47,0r-452,759r-48,0","w":889},"&":{"d":"426,-140r-178,-216v-68,37,-146,91,-146,177v0,84,74,141,155,141v71,0,129,-45,169,-102xm541,0r-76,-93v-52,65,-126,108,-210,108v-123,0,-216,-71,-216,-200v0,-102,95,-169,175,-212v-39,-51,-88,-103,-88,-172v0,-87,71,-145,158,-145v87,0,158,58,158,145v0,89,-69,140,-140,185r156,188v17,-35,21,-66,21,-120r63,0v0,40,-9,110,-42,171r120,145r-79,0xm284,-661v-52,0,-95,33,-95,92v0,50,49,106,80,144v50,-31,110,-75,110,-144v0,-59,-43,-92,-95,-92","w":611},"\u2019":{"d":"99,-714r84,0v3,115,6,213,-89,243r0,-43v34,-12,49,-61,47,-94r-42,0r0,-106","w":278,"k":{"\u2019":119,"s":111,"\u0161":111,"t":18}},"(":{"d":"242,191r-50,0v-89,-147,-141,-283,-141,-457v0,-170,54,-328,141,-463r50,0v-169,271,-170,653,0,920","w":241},")":{"d":"-1,-729r50,0v89,147,141,283,141,457v0,170,-54,328,-141,463r-50,0v169,-271,170,-653,0,-920","w":241},"*":{"d":"142,-556r-115,-39r14,-37r115,41r0,-123r39,0r0,123r115,-41r15,37r-118,39r72,98r-32,23r-73,-100r-73,100r-32,-23","w":352},"+":{"d":"274,-278r0,-225r53,0r0,225r225,0r0,53r-225,0r0,225r-53,0r0,-225r-225,0r0,-53r225,0","w":600},",":{"d":"97,-106r84,0v3,114,6,213,-88,243r0,-43v33,-12,48,-61,46,-94r-42,0r0,-106","w":278},"-":{"d":"310,-246r-250,0r0,-58r250,0r0,58","w":370},".":{"d":"181,0r-84,0r0,-106r84,0r0,106","w":278},"\/":{"d":"41,15r-54,0r308,-744r51,0","w":333},"0":{"d":"278,-653v-160,0,-181,186,-181,307v0,122,21,308,181,308v160,0,181,-186,181,-308v0,-121,-21,-307,-181,-307xm278,-706v204,0,244,197,244,360v0,164,-40,361,-244,361v-204,0,-244,-197,-244,-360v0,-164,40,-361,244,-361"},"1":{"d":"97,-520r0,-45v110,-4,178,-6,197,-135r51,0r0,700r-63,0r0,-520r-185,0"},"2":{"d":"497,-58r0,58r-461,0v5,-208,218,-259,337,-378v38,-39,58,-76,58,-132v0,-91,-72,-143,-157,-143v-115,0,-160,92,-158,195r-63,0v-3,-144,72,-248,224,-248v122,0,217,67,217,197v0,117,-86,189,-176,249v-69,45,-193,108,-210,202r389,0"},"3":{"d":"33,-223r63,0v-4,111,62,185,175,185v92,0,179,-55,179,-155v0,-118,-96,-158,-221,-148r0,-53v104,8,195,-18,195,-127v0,-92,-69,-132,-153,-132v-105,0,-161,73,-159,177r-63,0v1,-132,85,-230,221,-230v110,0,217,54,217,178v0,78,-47,134,-118,157v91,15,144,81,144,173v0,137,-112,213,-241,213v-145,0,-249,-88,-239,-238"},"4":{"d":"30,-174r0,-61r324,-465r57,0r0,473r105,0r0,53r-105,0r0,174r-58,0r0,-174r-323,0xm87,-227r266,0r0,-381r-2,0"},"5":{"d":"36,-200r63,0v3,97,76,162,172,162v108,0,175,-89,175,-191v0,-103,-75,-182,-179,-182v-62,0,-122,30,-156,83r-54,0r68,-363r350,0r0,58r-306,0r-45,232r2,2v35,-41,94,-65,148,-65v144,0,235,101,235,242v0,137,-109,237,-243,237v-130,0,-227,-82,-230,-215"},"6":{"d":"290,-403v-110,0,-174,82,-174,185v0,99,53,180,175,180v100,0,168,-83,168,-180v0,-102,-62,-185,-169,-185xm505,-523r-63,0v-11,-77,-65,-130,-146,-130v-161,-2,-198,191,-191,313v33,-69,107,-116,186,-116v140,0,231,97,231,234v0,136,-99,237,-237,237v-170,0,-245,-99,-245,-372v0,-83,21,-349,250,-349v123,0,200,61,215,183"},"7":{"d":"51,-633r0,-58r450,0r0,58v-87,91,-267,307,-283,633r-68,0v17,-237,94,-412,288,-633r-387,0"},"8":{"d":"278,-38v98,0,178,-51,178,-158v0,-101,-84,-152,-178,-152v-96,0,-178,49,-178,152v0,105,80,158,178,158xm490,-531v0,78,-47,130,-114,156v90,18,143,85,143,179v0,141,-110,211,-241,211v-131,0,-241,-70,-241,-211v0,-91,56,-163,142,-179r0,-2v-73,-22,-114,-77,-114,-154v0,-120,109,-175,212,-175v104,0,213,55,213,175xm278,-401v77,0,149,-40,149,-130v0,-80,-65,-122,-149,-122v-77,0,-150,42,-150,122v0,94,76,130,150,130"},"9":{"d":"266,-288v110,0,174,-82,174,-185v0,-99,-53,-180,-175,-180v-100,0,-168,83,-168,180v0,102,62,185,169,185xm51,-168r63,0v11,77,65,130,146,130v161,2,198,-191,191,-313v-33,69,-107,116,-186,116v-140,0,-231,-97,-231,-234v0,-136,99,-237,237,-237v170,0,245,99,245,372v0,83,-21,349,-250,349v-123,0,-200,-61,-215,-183"},":":{"d":"181,-394r-84,0r0,-106r84,0r0,106xm181,0r-84,0r0,-106r84,0r0,106","w":278},";":{"d":"97,-106r84,0v3,114,6,213,-88,243r0,-43v33,-12,48,-61,46,-94r-42,0r0,-106xm181,-394r-84,0r0,-106r84,0r0,106","w":278},"<":{"d":"554,-47r0,55r-508,-234r0,-54r508,-234r0,55r-446,206","w":600},"=":{"d":"552,-378r0,53r-503,0r0,-53r503,0xm552,-181r0,53r-503,0r0,-53r503,0","w":600},">":{"d":"46,-47r446,-206r-446,-206r0,-55r508,234r0,54r-508,234r0,-55","w":600},"?":{"d":"117,-494r-63,0v1,-142,82,-235,227,-235v111,0,203,69,203,185v0,74,-37,121,-87,170v-86,84,-90,73,-93,195r-63,0v-3,-123,31,-155,115,-233v37,-35,65,-77,65,-129v0,-80,-65,-135,-143,-135v-109,0,-163,79,-161,182xm231,0r0,-106r84,0r0,106r-84,0","w":537},"@":{"d":"426,-510v-102,0,-170,137,-170,231v0,60,35,97,83,97v93,0,166,-147,166,-235v0,-47,-40,-93,-79,-93xm614,-544r-96,288v-14,43,-20,83,13,83v89,0,169,-136,169,-248v0,-165,-130,-263,-284,-263v-187,0,-316,147,-316,331v0,182,136,323,318,323v98,0,200,-50,258,-133r51,0v-60,110,-184,178,-309,178v-215,0,-371,-164,-371,-375v0,-208,163,-369,367,-369v193,0,339,127,339,311v0,161,-120,290,-232,290v-37,0,-60,-28,-69,-67v-30,32,-76,66,-127,66v-82,0,-132,-66,-132,-145v0,-138,95,-289,237,-289v44,0,84,24,106,84r25,-65r53,0","w":800},"A":{"d":"-7,0r288,-714r76,0r280,714r-73,0r-87,-222r-325,0r-86,222r-73,0xm316,-646r-143,366r281,0","w":630},"B":{"d":"140,-343r0,285v184,-9,422,51,422,-147v0,-91,-69,-138,-188,-138r-234,0xm72,0r0,-714r302,0v120,0,229,42,229,175v0,77,-54,141,-124,162v92,12,151,81,151,174v0,68,-24,203,-256,203r-302,0xm140,-656r0,255r234,0v109,0,161,-54,161,-121v0,-89,-55,-134,-161,-134r-234,0","w":667},"C":{"d":"662,-498r-68,0v-24,-110,-115,-173,-219,-173v-189,0,-269,158,-269,314v0,156,80,314,269,314v132,0,216,-102,229,-229r68,0v-19,175,-130,287,-297,287v-225,0,-337,-177,-337,-372v0,-195,112,-372,337,-372v135,0,269,81,287,231","w":704},"D":{"d":"140,-656r0,598r145,0v205,1,294,-86,294,-299v0,-213,-89,-300,-294,-299r-145,0xm72,0r0,-714r247,0v215,5,328,123,328,357v0,234,-113,357,-328,357r-247,0","w":685},"E":{"d":"72,0r0,-714r493,0r0,58r-425,0r0,258r398,0r0,58r-398,0r0,282r430,0r0,58r-498,0","w":593},"F":{"d":"72,0r0,-714r453,0r0,58r-385,0r0,258r342,0r0,58r-342,0r0,340r-68,0","w":537,"k":{"A":55,"\u00c6":55,"\u00c1":55,"\u00c2":55,"\u00c4":55,"\u00c0":55,"\u00c5":55,"\u00c3":55,",":129,".":129}},"G":{"d":"682,-363r0,363r-49,0v-5,-42,-2,-92,-11,-130v-48,102,-144,145,-247,145v-225,0,-337,-177,-337,-372v0,-195,112,-372,337,-372v150,0,272,81,298,237r-68,0v-9,-82,-94,-179,-230,-179v-189,0,-269,158,-269,314v0,156,80,314,269,314v158,0,250,-111,247,-262r-243,0r0,-58r303,0","w":741},"H":{"d":"72,0r0,-714r68,0r0,310r424,0r0,-310r68,0r0,714r-68,0r0,-346r-424,0r0,346r-68,0","w":704},"I":{"d":"77,0r0,-714r68,0r0,714r-68,0","w":222},"J":{"d":"360,-228r0,-486r68,0r0,515v0,145,-54,214,-208,214v-164,0,-200,-120,-199,-244r68,0v2,61,-4,186,137,186v106,0,134,-54,134,-185","w":500},"K":{"d":"72,0r0,-714r68,0r0,384r415,-384r92,0r-319,295r334,419r-88,0r-297,-372r-137,127r0,245r-68,0","w":648},"L":{"d":"72,0r0,-714r68,0r0,656r400,0r0,58r-468,0","w":537,"k":{"T":92,"V":92,"W":55,"y":37,"\u00fd":37,"\u00ff":37,"Y":111,"\u00dd":111,"\u0178":111,"\u2019":98}},"M":{"d":"70,0r0,-714r100,0r247,626r246,-626r100,0r0,714r-68,0r0,-617r-2,0r-244,617r-64,0r-245,-617r-2,0r0,617r-68,0","w":833},"N":{"d":"71,0r0,-714r76,0r416,604r2,0r0,-604r68,0r0,714r-76,0r-416,-604r-2,0r0,604r-68,0","w":704},"O":{"d":"34,-357v0,-195,112,-372,337,-372v224,0,336,177,336,372v0,195,-112,372,-336,372v-225,0,-337,-177,-337,-372xm102,-357v0,156,80,314,269,314v188,0,268,-158,268,-314v0,-156,-80,-314,-268,-314v-189,0,-269,158,-269,314","w":741},"P":{"d":"72,0r0,-714r317,0v128,0,212,75,212,204v0,129,-84,204,-212,204r-249,0r0,306r-68,0xm140,-656r0,292r234,0v94,0,159,-48,159,-146v0,-98,-65,-146,-159,-146r-234,0","w":630,"k":{"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":153,".":153}},"Q":{"d":"444,-192r104,79v164,-167,113,-558,-177,-558v-189,0,-269,158,-269,314v0,156,80,314,269,314v52,0,96,-12,132,-35r-94,-71xm701,4r-34,42r-111,-84v-49,34,-111,53,-185,53v-225,0,-337,-177,-337,-372v0,-195,112,-372,337,-372v346,0,430,453,228,655","w":741},"R":{"d":"72,0r0,-714r327,0v118,0,217,57,217,186v0,93,-51,161,-139,184v93,12,120,76,125,161v3,49,3,147,33,183r-75,0v-17,-28,-17,-81,-19,-111v-6,-100,-14,-206,-143,-206r-258,0r0,317r-68,0xm140,-656r0,281r254,0v83,0,154,-52,154,-139v0,-87,-52,-142,-154,-142r-254,0","w":667,"k":{"T":-6,"V":-6,"W":-6,"y":-24,"\u00fd":-24,"\u00ff":-24,"Y":13,"\u00dd":13,"\u0178":13}},"S":{"d":"36,-235r68,0v-3,147,103,192,233,192v75,0,189,-41,189,-145v0,-81,-82,-110,-147,-129v-137,-39,-320,-33,-320,-211v0,-70,45,-201,246,-201v141,0,265,76,264,222r-68,0v-4,-109,-94,-164,-195,-164v-93,0,-179,36,-179,142v0,67,51,93,108,109v149,42,358,46,359,231v0,49,-20,204,-273,204v-169,0,-293,-76,-285,-250","w":630},"T":{"d":"-5,-656r0,-58r566,0r0,58r-249,0r0,656r-68,0r0,-656r-249,0","k":{"\u00fc":92,"\u0161":111,"\u00f2":111,"\u00f6":111,"\u00e8":111,"\u00eb":111,"\u00ea":111,"\u00e3":111,"\u00e5":111,"\u00e0":111,"\u00e4":111,"\u00e2":111,"w":111,"y":111,"\u00fd":111,"\u00ff":111,"A":68,"\u00c6":68,"\u00c1":68,"\u00c2":68,"\u00c4":68,"\u00c0":68,"\u00c5":68,"\u00c3":68,",":111,".":111,"c":111,"\u00e7":111,"e":111,"\u00e9":111,"o":111,"\u00f8":111,"\u0153":111,"\u00f3":111,"\u00f4":111,"\u00f5":111,"-":129,"a":111,"\u00e6":111,"\u00e1":111,"i":-24,"\u00ed":-24,"\u00ee":-24,"\u00ef":-24,"\u00ec":-24,"r":92,"s":111,"u":92,"\u00fa":92,"\u00fb":92,"\u00f9":92,":":111,";":111}},"U":{"d":"65,-257r0,-457r68,0r0,442v-1,166,78,229,209,229v132,0,210,-63,210,-229r0,-442r68,0r0,457v0,147,-79,272,-278,272v-198,0,-277,-125,-277,-272","w":685},"V":{"d":"258,0r-265,-714r73,0r232,639r2,0r230,-639r72,0r-265,714r-79,0","w":593,"k":{"\u00f6":55,"\u00f4":55,"\u00e8":55,"\u00eb":55,"\u00ea":55,"\u00e3":55,"\u00e5":55,"\u00e0":55,"\u00e4":55,"\u00e2":55,"y":18,"\u00fd":18,"\u00ff":18,"A":55,"\u00c6":55,"\u00c1":55,"\u00c2":55,"\u00c4":55,"\u00c0":55,"\u00c5":55,"\u00c3":55,",":129,".":129,"e":55,"\u00e9":55,"o":55,"\u00f8":55,"\u0153":55,"\u00f3":55,"\u00f2":55,"\u00f5":55,"-":55,"a":55,"\u00e6":55,"\u00e1":55,"i":-6,"\u00ed":-6,"\u00ee":-6,"\u00ef":-6,"\u00ec":-6,"r":37,"u":37,"\u00fa":37,"\u00fb":37,"\u00fc":37,"\u00f9":37,":":74,";":74}},"W":{"d":"198,0r-198,-714r73,0r162,625r2,0r174,-625r85,0r175,625r2,0r163,-625r68,0r-197,714r-72,0r-181,-639r-2,0r-180,639r-74,0","w":907,"k":{"\u00fc":18,"\u00f6":18,"\u00ea":18,"\u00e4":37,"A":18,"\u00c6":18,"\u00c1":18,"\u00c2":18,"\u00c4":18,"\u00c0":18,"\u00c5":18,"\u00c3":18,",":74,".":74,"e":18,"\u00e9":18,"\u00eb":18,"\u00e8":18,"o":18,"\u00f8":18,"\u0153":18,"\u00f3":18,"\u00f4":18,"\u00f2":18,"\u00f5":18,"a":37,"\u00e6":37,"\u00e1":37,"\u00e2":37,"\u00e0":37,"\u00e5":37,"\u00e3":37,"i":-24,"\u00ed":-24,"\u00ee":-24,"\u00ef":-24,"\u00ec":-24,"r":18,"u":18,"\u00fa":18,"\u00fb":18,"\u00f9":18,":":18,";":18}},"X":{"d":"245,-368r-242,-346r80,0r202,299r209,-299r76,0r-244,348r258,366r-82,0r-217,-315r-220,315r-77,0","w":574},"Y":{"d":"271,0r0,-294r-281,-420r81,0r235,362r234,-362r81,0r-282,420r0,294r-68,0","w":611,"k":{"\u00fc":74,"\u00f6":92,"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":123,".":99,"e":92,"\u00e9":92,"\u00ea":92,"\u00eb":92,"\u00e8":92,"o":92,"\u00f8":92,"\u0153":92,"\u00f3":92,"\u00f4":92,"\u00f2":92,"\u00f5":92,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"i":7,"\u00ed":7,"\u00ee":7,"\u00ef":7,"\u00ec":7,"u":74,"\u00fa":74,"\u00fb":74,"\u00f9":74,":":92,";":92,"p":74}},"Z":{"d":"37,-656r0,-58r514,0r0,60r-470,596r480,0r0,58r-556,0r0,-60r470,-596r-438,0","w":574},"[":{"d":"237,191r-161,0r0,-920r161,0r0,53r-98,0r0,814r98,0r0,53","w":241},"\\":{"d":"346,15r-54,0r-305,-744r51,0","w":333},"]":{"d":"4,-729r161,0r0,920r-161,0r0,-53r98,0r0,-814r-98,0r0,-53","w":241},"^":{"d":"102,-238r-58,0r230,-453r52,0r230,453r-57,0r-200,-390","w":600},"_":{"d":"500,125r-500,0r0,-50r500,0r0,50","w":500},"\u2018":{"d":"179,-471r-84,0v-3,-115,-6,-213,89,-243r0,43v-34,12,-49,61,-47,94r42,0r0,106","w":278,"k":{"\u2018":119}},"a":{"d":"390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm117,-358r-63,0v6,-121,91,-173,209,-173v91,0,190,28,190,166r0,274v-3,32,29,45,57,34r0,53v-14,3,-24,4,-41,4v-67,2,-70,-41,-76,-90v-44,67,-89,105,-188,105v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120","w":519},"b":{"d":"473,-258v0,-107,-49,-220,-169,-220v-134,0,-181,113,-181,220v0,107,47,220,181,220v120,0,169,-113,169,-220xm60,0r0,-714r63,0r0,296r2,0v27,-74,102,-113,179,-113v157,0,232,127,232,273v0,146,-75,273,-232,273v-85,0,-153,-47,-186,-113r0,98r-58,0","w":574},"c":{"d":"487,-354r-63,0v-17,-77,-64,-124,-146,-124v-242,0,-241,440,0,440v78,0,142,-61,150,-147r63,0v-17,124,-98,200,-213,200v-158,0,-244,-125,-244,-273v0,-148,86,-273,244,-273v110,0,195,59,209,177","w":519},"d":{"d":"101,-258v0,107,49,220,169,220v134,0,181,-113,181,-220v0,-107,-47,-220,-181,-220v-120,0,-169,113,-169,220xm514,-714r0,714r-58,0v-2,-31,4,-71,-2,-98v-27,67,-109,113,-184,113v-157,0,-232,-127,-232,-273v0,-146,75,-273,232,-273v79,0,150,43,181,113r0,-296r63,0","w":574},"e":{"d":"494,-241r-397,0v1,92,49,203,170,203v92,0,142,-54,162,-132r63,0v-27,117,-95,185,-225,185v-164,0,-233,-126,-233,-273v0,-136,69,-273,233,-273v166,0,232,145,227,290xm97,-294r334,0v-3,-95,-62,-184,-164,-184v-103,0,-160,90,-170,184","w":519},"f":{"d":"259,-516r0,53r-102,0r0,463r-63,0r0,-463r-88,0r0,-53r88,0v-4,-112,5,-199,125,-198v18,0,33,1,53,4r0,54v-17,-3,-31,-5,-46,-5v-87,-1,-65,72,-69,145r102,0","w":259,"k":{"\u2019":-18,"f":18,"\ufb01":18,"\ufb02":18,"\u00df":18}},"g":{"d":"267,-59v109,0,166,-100,166,-205v0,-101,-47,-214,-166,-214v-120,0,-170,107,-170,214v0,104,54,205,170,205xm496,-516r0,474v-1,154,-60,248,-229,248v-103,0,-205,-46,-214,-156r63,0v14,76,81,103,151,103v141,0,176,-110,166,-265r-2,0v-29,63,-90,106,-164,106v-165,0,-233,-118,-233,-265v0,-142,84,-260,233,-260v77,0,137,51,166,104r0,-89r63,0"},"h":{"d":"59,0r0,-714r63,0r0,287r2,0v23,-62,92,-104,165,-104v145,0,189,76,189,199r0,332r-63,0r0,-322v0,-89,-29,-156,-131,-156v-100,0,-162,76,-162,177r0,301r-63,0","w":537},"i":{"d":"61,0r0,-516r63,0r0,516r-63,0xm61,-613r0,-101r63,0r0,101r-63,0","w":185},"j":{"d":"61,63r0,-579r63,0r0,564v0,77,-21,143,-110,143v-15,0,-29,-2,-44,-2r0,-52v10,1,24,1,34,1v49,0,57,-32,57,-75xm61,-613r0,-101r63,0r0,101r-63,0","w":185},"k":{"d":"60,0r0,-714r63,0r0,448r286,-250r84,0r-220,191r235,325r-79,0r-204,-281r-102,85r0,196r-63,0","w":500},"l":{"d":"61,0r0,-714r63,0r0,714r-63,0","w":185},"m":{"d":"60,0r0,-516r58,0r0,87r3,0v33,-62,87,-102,169,-102v68,0,129,33,148,101v31,-68,96,-101,164,-101v113,0,171,59,171,181r0,350r-63,0r0,-347v0,-85,-32,-131,-122,-131v-109,0,-140,90,-140,186r0,292r-63,0r0,-350v1,-70,-28,-128,-108,-128v-109,0,-154,82,-154,190r0,288r-63,0","w":833},"n":{"d":"59,0r0,-516r63,0v2,28,-4,65,2,89v23,-62,92,-104,165,-104v145,0,189,76,189,199r0,332r-63,0r0,-322v0,-89,-29,-156,-131,-156v-100,0,-162,76,-162,177r0,301r-63,0","w":537},"o":{"d":"278,-531v158,0,244,125,244,273v0,148,-86,273,-244,273v-158,0,-244,-125,-244,-273v0,-148,86,-273,244,-273xm278,-478v-242,0,-241,440,0,440v242,0,241,-440,0,-440"},"p":{"d":"473,-258v0,-107,-49,-220,-169,-220v-139,0,-181,103,-181,220v0,107,47,220,181,220v120,0,169,-113,169,-220xm60,191r0,-707r58,0v2,31,-4,71,2,98v29,-70,101,-113,184,-113v157,0,232,127,232,273v0,146,-75,273,-232,273v-79,0,-150,-43,-181,-113r0,289r-63,0","w":574},"q":{"d":"101,-258v0,107,49,220,169,220v134,0,181,-113,181,-220v0,-107,-47,-220,-181,-220v-120,0,-169,113,-169,220xm514,-516r0,707r-63,0r0,-289r-2,0v-27,74,-102,113,-179,113v-157,0,-232,-127,-232,-273v0,-146,75,-273,232,-273v77,0,155,50,186,113r0,-98r58,0","w":574},"r":{"d":"60,0r0,-516r58,0v2,39,-4,86,2,121v32,-83,102,-132,195,-128r0,63v-114,-6,-192,78,-192,185r0,275r-63,0","w":315,"k":{",":92,".":92,"c":18,"\u00e7":18,"d":18,"\u0131":18,"e":18,"\u00e9":18,"\u00ea":18,"\u00eb":18,"\u00e8":18,"n":-18,"\u00f1":-18,"o":18,"\u00f8":18,"\u0153":18,"\u00f3":18,"\u00f4":18,"\u00f6":18,"\u00f2":18,"\u00f5":18,"q":18,"-":55}},"s":{"d":"432,-365r-63,0v-3,-78,-63,-113,-135,-113v-56,0,-122,22,-122,89v0,87,116,89,191,106v71,16,147,53,147,143v0,112,-111,155,-207,155v-120,0,-202,-56,-212,-182r63,0v5,85,68,129,152,129v59,0,141,-26,141,-98v0,-88,-117,-91,-194,-112v-82,-22,-144,-50,-144,-138v0,-105,103,-145,194,-145v103,0,185,54,189,166","w":481},"t":{"d":"160,-671r0,155r105,0r0,53r-105,0r0,348v0,41,6,68,51,68v18,0,36,-1,54,-3r0,54v-102,8,-168,-1,-168,-114r0,-353r-90,0r0,-53r90,0r0,-155r63,0","w":296},"u":{"d":"478,-516r0,516r-58,0v-2,-30,4,-67,-2,-93v-33,69,-101,108,-178,108v-130,0,-181,-76,-181,-196r0,-335r63,0r0,336v3,93,38,142,138,142v108,0,155,-102,155,-207r0,-271r63,0","w":537},"v":{"d":"201,0r-199,-516r70,0r162,453r2,0r160,-453r65,0r-193,516r-67,0","w":463,"k":{",":74,".":74}},"w":{"d":"174,0r-166,-516r67,0r132,442r2,0r126,-442r71,0r126,442r2,0r132,-442r67,0r-166,516r-69,0r-127,-434r-2,0r-126,434r-69,0","w":741,"k":{",":55,".":55}},"x":{"d":"1,0r200,-268r-185,-248r79,0r147,196r144,-196r79,0r-186,247r201,269r-80,0r-162,-217r-159,217r-78,0","w":481},"y":{"d":"208,-2r-206,-514r67,0r170,440r159,-440r63,0r-224,595v-35,105,-85,122,-184,107r0,-53v76,18,113,-13,133,-76","w":463,"k":{",":92,".":92}},"z":{"d":"432,-471r-341,418r355,0r0,53r-431,0r0,-51r337,-412r-314,0r0,-53r394,0r0,45","w":463},"{":{"d":"293,191v-95,10,-135,-26,-138,-125v-3,-113,24,-300,-72,-308r0,-53v96,-7,68,-197,72,-309v3,-98,43,-136,138,-125r0,53v-106,-13,-75,144,-75,244v0,112,-59,152,-70,164v13,7,70,53,70,162v0,94,-35,254,75,244r0,53","w":333},"|":{"d":"85,214r0,-1000r53,0r0,1000r-53,0","w":222},"}":{"d":"40,-729v95,-10,135,26,138,125v3,113,-24,300,72,308r0,53v-96,7,-68,197,-72,309v-3,98,-43,136,-138,125r0,-53v106,13,75,-144,75,-244v0,-112,59,-152,70,-164v-13,-7,-70,-53,-70,-162v0,-94,35,-254,-75,-244r0,-53","w":333},"~":{"d":"194,-311v68,0,156,65,213,65v40,0,65,-34,88,-69r36,36v-30,42,-63,86,-125,86v-78,1,-140,-65,-217,-65v-44,0,-68,34,-84,69r-36,-36v22,-42,58,-86,125,-86","w":600},"\u00a1":{"d":"140,-338r15,323r0,206r-68,0r0,-206r15,-323r38,0xm79,-516r84,0r0,106r-84,0r0,-106","w":241},"\u00a2":{"d":"270,-39r0,-437v-205,31,-204,405,0,437xm507,-354r-63,0v-16,-74,-60,-121,-135,-124r0,440v73,-4,131,-65,139,-147r63,0v-16,120,-93,196,-202,200r0,101r-39,0r0,-102v-140,-14,-216,-133,-216,-272v0,-139,76,-258,216,-271r0,-89r39,0r0,87v104,3,184,63,198,177"},"\u00a3":{"d":"396,15v-86,0,-125,-40,-207,-41v-50,0,-71,18,-110,41r-36,-54v90,-62,148,-176,84,-291r-81,0r0,-32r63,0v-30,-49,-55,-102,-55,-161v0,-143,113,-206,228,-206v147,0,235,90,232,237r-63,0v0,-107,-57,-184,-169,-184v-85,0,-165,50,-165,157v0,68,34,99,59,157r160,0r0,32r-142,0v60,110,-4,221,-86,279r2,2v29,-15,62,-30,97,-30v82,-2,116,41,188,41v42,0,87,-25,115,-56r32,48v-44,39,-87,61,-146,61"},"\u2044":{"d":"-166,34r453,-759r47,0r-452,759r-48,0","w":167},"\u00a5":{"d":"110,-304r0,-45r106,0r-210,-365r72,0r199,365r201,-365r69,0r-211,365r109,0r0,45r-131,0v-12,17,-3,56,-6,83r137,0r0,45r-137,0r0,176r-63,0r0,-176r-135,0r0,-45r135,0v-2,-27,6,-66,-6,-83r-129,0"},"\u0192":{"d":"7,168r9,-54v17,4,35,6,52,6v59,-2,82,-64,92,-112r75,-394r-100,0r9,-53r101,0v29,-126,26,-274,183,-275v21,0,41,2,62,5r-12,52v-15,-2,-31,-4,-46,-4v-110,0,-98,134,-124,222r113,0r-9,53r-113,0r-61,323v-23,120,-33,239,-186,236v-15,0,-30,-2,-45,-5"},"\u00a7":{"d":"448,-230v0,-106,-173,-158,-258,-217v-48,20,-82,53,-82,108v0,101,171,158,256,217v48,-20,84,-53,84,-108xm511,-231v0,66,-50,117,-109,140v33,29,50,66,50,109v0,92,-86,143,-169,143v-112,0,-185,-63,-185,-177r63,0v-3,68,46,124,116,124v56,0,112,-26,112,-89v0,-151,-344,-159,-344,-356v0,-66,50,-117,109,-140v-33,-29,-50,-66,-50,-109v0,-92,86,-143,169,-143v112,0,185,63,185,177r-63,0v3,-68,-46,-124,-116,-124v-56,0,-112,26,-112,89v0,151,344,159,344,356"},"\u00a4":{"d":"29,-136r57,-57v-73,-82,-72,-224,-1,-307r-56,-56r38,-38r56,56v80,-71,228,-75,309,-1r55,-55r38,38r-54,54v73,82,74,230,0,312r54,54r-38,38r-55,-55v-80,73,-229,72,-309,-1r-56,56xm94,-346v0,107,83,190,185,190v102,0,185,-83,185,-190v0,-107,-83,-190,-185,-190v-102,0,-185,83,-185,190"},"'":{"d":"108,-471r0,-243r63,0r0,243r-63,0","w":278},"\u201c":{"d":"302,-471r-84,0v-3,-115,-6,-213,89,-243r0,43v-34,12,-49,61,-47,94r42,0r0,106xm148,-471r-84,0v-3,-115,-6,-213,89,-243r0,43v-34,12,-49,61,-47,94r42,0r0,106","w":370},"\u00ab":{"d":"354,-119r-133,-125r0,-62r133,-125r0,67r-94,89r94,89r0,67xm193,-119r-133,-125r0,-62r133,-125r0,67r-94,89r94,89r0,67","w":426},"\u2039":{"d":"186,-119r-133,-125r0,-62r133,-125r0,67r-94,89r94,89r0,67","w":259},"\u203a":{"d":"73,-119r0,-67r94,-89r-94,-89r0,-67r133,125r0,62","w":259},"\u2013":{"d":"500,-246r-500,0r0,-58r500,0r0,58","w":500},"\u2020":{"d":"246,-440r-198,0r0,-53r198,0r0,-221r63,0r0,221r199,0r0,53r-199,0r0,591r-63,0r0,-591"},"\u2021":{"d":"246,-47r-198,0r0,-53r198,0r0,-363r-198,0r0,-53r198,0r0,-198r63,0r0,198r199,0r0,53r-199,0r0,363r199,0r0,53r-199,0r0,198r-63,0r0,-198"},"\u00b7":{"d":"139,-375v32,0,59,28,59,60v0,31,-28,58,-59,58v-32,0,-59,-28,-59,-60v0,-30,29,-58,59,-58","w":278},"\u00b6":{"d":"273,177r0,-505v-117,0,-207,-87,-207,-190v0,-126,83,-196,223,-196r230,0r0,891r-63,0r0,-838r-120,0r0,838r-63,0","w":600},"\u2022":{"d":"72,-357v0,-99,79,-178,178,-178v99,0,179,79,179,178v0,99,-80,179,-179,179v-99,0,-178,-80,-178,-179","w":500},"\u201a":{"d":"99,-106r84,0v3,115,6,213,-89,243r0,-43v34,-12,49,-61,47,-94r-42,0r0,-106","w":278},"\u201e":{"d":"222,-106r84,0v3,115,6,213,-89,243r0,-43v34,-12,49,-61,47,-94r-42,0r0,-106xm68,-106r84,0v3,115,6,213,-89,243r0,-43v34,-12,49,-61,47,-94r-42,0r0,-106","w":370},"\u201d":{"d":"222,-714r84,0v3,115,6,213,-89,243r0,-43v34,-12,49,-61,47,-94r-42,0r0,-106xm68,-714r84,0v3,115,6,213,-89,243r0,-43v34,-12,49,-61,47,-94r-42,0r0,-106","w":370},"\u00bb":{"d":"233,-119r0,-67r94,-89r-94,-89r0,-67r133,125r0,62xm72,-119r0,-67r94,-89r-94,-89r0,-67r133,125r0,62","w":426},"\u2026":{"d":"209,0r-84,0r0,-106r84,0r0,106xm542,0r-84,0r0,-106r84,0r0,106xm875,0r-84,0r0,-106r84,0r0,106","w":1000},"\u2030":{"d":"547,-323v107,0,149,72,149,169v0,97,-42,169,-149,169v-107,0,-149,-72,-149,-169v0,-97,42,-169,149,-169xm547,-278v-80,0,-96,65,-96,124v0,59,16,124,96,124v80,0,96,-65,96,-124v0,-59,-16,-124,-96,-124xm901,-323v107,0,149,72,149,169v0,97,-42,169,-149,169v-107,0,-149,-72,-149,-169v0,-97,42,-169,149,-169xm901,-278v-80,0,-96,65,-96,124v0,59,16,124,96,124v80,0,96,-65,96,-124v0,-59,-16,-124,-96,-124xm214,-706v107,0,149,72,149,169v0,97,-42,169,-149,169v-107,0,-149,-72,-149,-169v0,-97,42,-169,149,-169xm214,-661v-80,0,-96,65,-96,124v0,59,16,124,96,124v80,0,96,-65,96,-124v0,-59,-16,-124,-96,-124xm129,34r453,-759r47,0r-452,759r-48,0","w":1111},"\u00bf":{"d":"421,-29r63,0v-1,142,-82,235,-227,235v-111,0,-203,-69,-203,-185v0,-74,37,-121,87,-170v86,-84,90,-73,93,-195r63,0v3,123,-31,155,-115,233v-37,35,-65,77,-65,129v0,80,65,135,143,135v109,0,163,-79,161,-182xm307,-516r0,106r-84,0r0,-106r84,0","w":537},"`":{"d":"101,-588r-132,-141r78,0r105,141r-51,0","w":185},"\u00b4":{"d":"216,-729r-132,141r-51,0r105,-141r78,0","w":185},"\u02c6":{"d":"127,-729r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":185},"\u02dc":{"d":"24,-704v52,0,93,44,135,44v31,0,44,-22,52,-48r38,0v-7,46,-35,95,-87,95v-49,0,-96,-46,-135,-46v-30,0,-53,18,-53,51r-39,0v6,-47,36,-96,89,-96","w":185},"\u00af":{"d":"257,-634r-329,0r0,-45r329,0r0,45","w":185},"\u02d8":{"d":"-51,-722r39,0v8,111,198,107,209,0r39,0v-10,78,-66,127,-145,127v-81,0,-136,-46,-142,-127","w":185},"\u02d9":{"d":"127,-605r-68,0r0,-101r68,0r0,101","w":185},"\u00a8":{"d":"215,-605r-68,0r0,-101r68,0r0,101xm38,-605r-68,0r0,-101r68,0r0,101","w":185},"\u02da":{"d":"-14,-677v0,-59,48,-107,107,-107v59,0,107,48,107,107v0,59,-48,107,-107,107v-59,0,-107,-48,-107,-107xm25,-677v0,38,30,68,68,68v38,0,68,-30,68,-68v0,-38,-30,-68,-68,-68v-38,0,-68,30,-68,68","w":185},"\u00b8":{"d":"91,0r35,0r-44,58v47,-9,116,0,116,62v0,97,-130,102,-204,65r14,-32v36,14,132,33,132,-29v0,-44,-65,-44,-94,-30r-16,-15","w":185},"\u02dd":{"d":"18,-588r-51,0r101,-141r76,0xm174,-588r-51,0r101,-141r76,0","w":185},"\u02db":{"d":"151,-2r38,0v-73,58,-89,93,-89,127v0,32,21,47,44,47v25,0,45,-23,55,-41r29,13v-29,49,-60,67,-95,67v-71,0,-96,-34,-96,-82v0,-18,12,-70,114,-131","w":185},"\u02c7":{"d":"58,-588r-114,-141r65,0r86,102r88,-102r58,0r-114,141r-69,0","w":185},"\u2014":{"d":"870,-246r-740,0r0,-58r740,0r0,58","w":1000},"\u00c6":{"d":"513,-340r0,282r371,0r0,58r-439,0r0,-226r-259,0r-116,226r-77,0r374,-714r513,0r0,58r-367,0r0,258r344,0r0,58r-344,0xm216,-284r229,0r0,-372r-40,0","w":907},"\u00aa":{"d":"228,-502v-1,-15,4,-36,-2,-47v-4,9,-23,12,-34,14v-81,13,-150,13,-150,69v0,36,38,59,69,59v62,0,119,-36,117,-95xm54,-597r-47,0v4,-77,59,-109,140,-109v65,0,128,17,128,100r0,172v0,20,20,28,38,20r0,34v-43,10,-83,-5,-75,-53r-2,0v-42,90,-242,94,-241,-31v0,-93,92,-96,180,-105v34,-4,53,-8,53,-31v0,-54,-40,-67,-88,-67v-46,0,-85,22,-86,70","w":311},"\u0141":{"d":"72,-306r0,-408r68,0r0,361r243,-168r0,56r-243,167r0,240r400,0r0,58r-468,0r0,-251r-82,57r0,-56","w":537,"k":{"T":92,"V":92,"W":55,"y":37,"\u00fd":37,"\u00ff":37,"Y":111,"\u00dd":111,"\u0178":111,"\u2019":98}},"\u00d8":{"d":"580,-565r-397,443v44,49,106,79,188,79v270,0,330,-332,209,-522xm160,-150r398,-443v-43,-47,-105,-78,-187,-78v-271,0,-330,330,-211,521xm672,-719r26,23r-73,81v168,221,76,630,-254,630v-99,0,-176,-34,-231,-89r-76,86r-27,-25r79,-88v-168,-222,-75,-628,255,-628v98,0,175,34,230,88","w":741},"\u0152":{"d":"569,-509v16,-129,-105,-162,-194,-162v-174,0,-270,145,-270,314v0,169,96,314,270,314v73,0,129,-20,176,-79v13,-16,18,-35,18,-55r0,-332xm637,-656r0,258r368,0r0,58r-368,0r0,282r409,0r0,58r-477,0r0,-70v-47,56,-125,85,-194,85v-213,0,-338,-173,-338,-372v0,-199,125,-372,338,-372v75,0,148,22,194,74r0,-59r474,0r0,58r-406,0","w":1074},"\u00ba":{"d":"162,-368v-103,0,-159,-75,-159,-169v0,-94,56,-169,159,-169v113,0,169,75,169,169v0,94,-56,169,-169,169xm167,-407v155,-1,156,-259,0,-260v-155,1,-156,259,0,260","w":334},"\u00e6":{"d":"453,-294r325,0v0,-95,-63,-184,-163,-184v-105,0,-164,87,-162,184xm390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm841,-241r-388,0v-7,110,47,203,168,203v85,0,135,-48,153,-129r63,0v-26,115,-106,182,-225,182v-93,0,-156,-48,-191,-130v-15,86,-126,130,-216,130v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120r-63,0v6,-121,91,-173,209,-173v70,0,160,19,172,107r2,0v28,-72,106,-107,181,-107v170,0,223,140,223,290","w":870},"\u0131":{"d":"124,0r-63,0r0,-516r63,0r0,516","w":185},"\u0142":{"d":"61,-395r0,-319r63,0r0,267r67,-55r0,55r-67,55r0,392r-63,0r0,-340r-67,55r0,-55","w":185},"\u00f8":{"d":"133,-123r273,-295v-29,-36,-72,-60,-128,-60v-171,1,-225,222,-145,355xm424,-392r-273,295v28,35,71,59,127,59v171,0,224,-221,146,-354xm500,-519r22,20r-54,59v116,166,41,455,-190,455v-71,0,-128,-25,-168,-68r-59,64r-22,-20r61,-67v-119,-162,-43,-455,188,-455v72,0,130,26,170,68"},"\u0153":{"d":"265,-478v-131,0,-166,124,-166,232v0,103,46,208,165,208v128,0,152,-119,152,-224v0,-104,-27,-216,-151,-216xm799,-166r63,0v-28,115,-110,181,-228,181v-86,0,-160,-50,-186,-131v-25,88,-96,131,-185,131v-162,0,-227,-128,-227,-273v0,-143,73,-273,230,-273v90,0,155,45,186,127v28,-75,99,-127,190,-127v152,0,221,101,221,290r-384,0v0,101,49,203,161,203v87,0,136,-48,159,-128xm479,-294r321,0v0,-97,-56,-184,-160,-184v-106,0,-161,86,-161,184","w":889},"\u00df":{"d":"61,0r0,-535v0,-132,80,-194,199,-194v107,0,196,62,196,176v0,74,-45,132,-113,152v97,8,149,94,149,185v0,165,-113,237,-272,222r0,-55v113,18,209,-23,209,-156v0,-139,-86,-163,-209,-163r0,-53v88,0,173,-27,173,-130v0,-74,-61,-125,-132,-125v-99,0,-137,52,-137,146r0,530r-63,0","w":537},"\u00b9":{"d":"35,-584r0,-39v71,-2,115,-5,128,-74r40,0r0,423r-47,0r0,-310r-121,0","w":333},"\u00ac":{"d":"499,-111r0,-214r-450,0r0,-53r503,0r0,267r-53,0","w":600},"\u00b5":{"d":"478,-516r0,516r-58,0v-2,-30,4,-67,-2,-93v-46,105,-195,142,-295,73r0,211r-64,0r0,-707r63,0r0,336v3,93,38,142,138,142v108,0,155,-102,155,-207r0,-271r63,0","w":537},"\u2122":{"d":"269,-302r-53,0r0,-367r-134,0r0,-45r321,0r0,45r-134,0r0,367xm529,-302r-53,0r0,-412r85,0r133,334r129,-334r85,0r0,412r-53,0r0,-364r-2,0r-146,364r-30,0r-146,-364r-2,0r0,364","w":990},"\u00d0":{"d":"140,-656r0,263r236,0r0,45r-236,0r0,290r145,0v205,1,294,-86,294,-299v0,-213,-89,-300,-294,-299r-145,0xm72,-393r0,-321r247,0v215,5,328,123,328,357v0,234,-113,357,-328,357r-247,0r0,-348r-72,0r0,-45r72,0","w":685},"\u00bd":{"d":"133,34r453,-759r47,0r-452,759r-48,0xm48,-584r0,-39v71,-2,115,-5,128,-74r40,0r0,423r-47,0r0,-310r-121,0xm536,-279r-47,0v-2,-91,46,-153,150,-153v84,0,146,40,146,125v0,70,-56,113,-115,149v-44,27,-119,58,-136,119r248,0r0,39r-302,0v3,-129,142,-160,219,-231v25,-24,39,-46,39,-80v0,-51,-48,-82,-103,-82v-71,0,-100,55,-99,114","w":834},"\u00b1":{"d":"274,-323r0,-180r53,0r0,180r225,0r0,53r-225,0r0,180r-53,0r0,-180r-224,0r0,-53r224,0xm49,0r0,-53r503,0r0,53r-503,0","w":600},"\u00de":{"d":"72,0r0,-714r68,0r0,117r249,0v128,0,212,75,212,204v0,129,-84,204,-212,204r-249,0r0,189r-68,0xm140,-539r0,292r234,0v94,0,159,-48,159,-146v0,-98,-65,-146,-159,-146r-234,0","w":630},"\u00bc":{"d":"642,0r0,-100r-210,0r0,-39r213,-284r44,0r0,284r68,0r0,39r-68,0r0,100r-47,0xm640,-355r-162,216r164,0r0,-216r-2,0xm133,34r453,-759r47,0r-452,759r-48,0xm48,-584r0,-39v71,-2,115,-5,128,-74r40,0r0,423r-47,0r0,-310r-121,0","w":834},"\u00f7":{"d":"552,-225r-503,0r0,-53r503,0r0,53xm244,-453v0,-31,26,-56,56,-56v30,0,56,26,56,56v0,28,-26,56,-55,56v-31,0,-57,-26,-57,-56xm244,-50v0,-31,26,-56,56,-56v30,0,56,26,56,56v0,28,-26,56,-55,56v-31,0,-57,-26,-57,-56","w":600},"\u00a6":{"d":"85,89r0,-250r53,0r0,250r-53,0xm85,-411r0,-250r53,0r0,250r-53,0","w":222},"\u00b0":{"d":"56,-562v0,-80,64,-144,144,-144v80,0,144,64,144,144v0,80,-64,144,-144,144v-80,0,-144,-64,-144,-144xm95,-562v0,58,47,105,105,105v58,0,105,-47,105,-105v0,-58,-47,-105,-105,-105v-58,0,-105,47,-105,105","w":400},"\u00fe":{"d":"473,-258v0,-107,-49,-220,-169,-220v-134,0,-181,113,-181,220v0,107,47,220,181,220v120,0,169,-113,169,-220xm60,191r0,-905r63,0r0,296r2,0v27,-74,102,-113,179,-113v157,0,232,127,232,273v0,146,-75,273,-232,273v-79,0,-150,-43,-181,-113r0,289r-63,0","w":574},"\u00be":{"d":"642,0r0,-100r-210,0r0,-39r213,-284r44,0r0,284r68,0r0,39r-68,0r0,100r-47,0xm640,-355r-162,216r164,0r0,-216r-2,0xm135,-478r0,-39v65,5,128,-11,128,-74v0,-52,-46,-76,-98,-76v-65,0,-101,44,-100,103r-47,0v1,-83,54,-142,147,-142v75,0,145,33,145,111v0,48,-32,79,-77,95v59,8,94,48,94,103v0,86,-73,132,-161,132v-98,0,-165,-53,-158,-147r47,0v-3,64,40,108,110,108v57,0,115,-33,115,-90v-1,-66,-68,-90,-145,-84xm173,34r453,-759r47,0r-452,759r-48,0","w":834},"\u00b2":{"d":"71,-553r-47,0v-2,-91,46,-153,150,-153v84,0,146,40,146,125v0,70,-56,113,-115,149v-44,27,-119,58,-136,119r248,0r0,39r-302,0v3,-129,142,-160,219,-231v25,-24,39,-46,39,-80v0,-51,-48,-82,-103,-82v-71,0,-100,55,-99,114","w":333},"\u00ae":{"d":"400,-729v210,0,372,163,372,372v0,209,-162,372,-372,372v-210,0,-372,-163,-372,-372v0,-209,162,-372,372,-372xm400,-676v-180,0,-314,141,-314,319v0,178,134,319,314,319v180,0,314,-141,314,-319v0,-178,-134,-319,-314,-319xm318,-333r0,191r-53,0r0,-430r165,0v97,0,140,42,140,119v0,76,-50,110,-108,120r128,191r-62,0r-121,-191r-89,0xm318,-378v89,-4,199,22,199,-75v0,-94,-110,-71,-199,-74r0,149","w":800},"\u2212":{"d":"552,-278r0,53r-503,0r0,-53r503,0","w":600},"\u00f0":{"d":"276,-453v-129,0,-179,95,-179,205v0,109,52,210,177,210v118,0,179,-88,179,-212v0,-100,-50,-203,-177,-203xm135,-596r100,-56v-30,-20,-61,-39,-93,-55r36,-35v35,18,71,39,104,64r114,-64r33,32r-109,61v111,91,196,218,196,391v0,150,-78,273,-245,273v-161,0,-237,-115,-237,-264v0,-139,85,-257,237,-257v51,0,103,12,135,40v-33,-62,-79,-113,-133,-157r-105,59"},"\u00d7":{"d":"262,-251r-188,-189r37,-37r189,189r189,-189r37,37r-189,189r189,189r-37,37r-189,-188r-189,188r-37,-37","w":600},"\u00b3":{"d":"135,-478r0,-39v65,5,128,-11,128,-74v0,-52,-46,-76,-98,-76v-65,0,-101,44,-100,103r-47,0v1,-83,54,-142,147,-142v75,0,145,33,145,111v0,48,-32,79,-77,95v59,8,94,48,94,103v0,86,-73,132,-161,132v-98,0,-165,-53,-158,-147r47,0v-3,64,40,108,110,108v57,0,115,-33,115,-90v-1,-66,-68,-90,-145,-84","w":333},"\u00a9":{"d":"772,-357v0,209,-162,372,-372,372v-210,0,-372,-163,-372,-372v0,-209,162,-372,372,-372v210,0,372,163,372,372xm714,-357v0,-178,-134,-319,-314,-319v-180,0,-314,141,-314,319v0,178,134,319,314,319v180,0,314,-141,314,-319xm594,-435r-51,0v-12,-59,-60,-106,-133,-106v-108,0,-165,80,-165,184v0,104,57,184,165,184v73,0,121,-46,133,-106r51,0v-13,97,-91,151,-185,151v-136,0,-217,-97,-217,-229v0,-132,81,-229,217,-229v94,0,172,54,185,151","w":800},"\u00c1":{"d":"-7,0r288,-714r76,0r280,714r-73,0r-87,-222r-325,0r-86,222r-73,0xm316,-646r-143,366r281,0xm439,-912r-132,141r-51,0r105,-141r78,0","w":630},"\u00c2":{"d":"-7,0r288,-714r76,0r280,714r-73,0r-87,-222r-325,0r-86,222r-73,0xm316,-646r-143,366r281,0xm350,-912r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":630},"\u00c4":{"d":"-7,0r288,-714r76,0r280,714r-73,0r-87,-222r-325,0r-86,222r-73,0xm316,-646r-143,366r281,0xm438,-788r-68,0r0,-101r68,0r0,101xm261,-788r-68,0r0,-101r68,0r0,101","w":630},"\u00c0":{"d":"-7,0r288,-714r76,0r280,714r-73,0r-87,-222r-325,0r-86,222r-73,0xm316,-646r-143,366r281,0xm324,-771r-132,-141r78,0r105,141r-51,0","w":630},"\u00c5":{"d":"-7,0r288,-714r76,0r280,714r-73,0r-87,-222r-325,0r-86,222r-73,0xm316,-646r-143,366r281,0xm209,-860v0,-59,48,-107,107,-107v59,0,107,48,107,107v0,59,-48,107,-107,107v-59,0,-107,-48,-107,-107xm248,-860v0,38,30,68,68,68v38,0,68,-30,68,-68v0,-38,-30,-68,-68,-68v-38,0,-68,30,-68,68","w":630},"\u00c3":{"d":"-7,0r288,-714r76,0r280,714r-73,0r-87,-222r-325,0r-86,222r-73,0xm316,-646r-143,366r281,0xm247,-887v52,0,93,44,135,44v31,0,44,-22,52,-48r38,0v-7,46,-35,95,-87,95v-49,0,-96,-46,-135,-46v-30,0,-53,18,-53,51r-39,0v6,-47,36,-96,89,-96","w":630},"\u00c7":{"d":"604,-272r68,0v-19,175,-130,287,-298,287r-33,43v47,-9,116,0,116,62v0,97,-130,102,-204,65r14,-32v36,14,132,33,132,-29v0,-44,-65,-44,-94,-30r-16,-15r51,-66v-202,-17,-302,-185,-302,-370v0,-195,112,-372,337,-372v135,0,269,81,287,231r-68,0v-24,-110,-115,-173,-219,-173v-189,0,-269,158,-269,314v0,156,80,314,269,314v132,0,216,-102,229,-229","w":704},"\u00c9":{"d":"72,0r0,-714r493,0r0,58r-425,0r0,258r398,0r0,58r-398,0r0,282r430,0r0,58r-498,0xm420,-912r-132,141r-51,0r105,-141r78,0","w":593},"\u00ca":{"d":"72,0r0,-714r493,0r0,58r-425,0r0,258r398,0r0,58r-398,0r0,282r430,0r0,58r-498,0xm331,-912r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":593},"\u00cb":{"d":"72,0r0,-714r493,0r0,58r-425,0r0,258r398,0r0,58r-398,0r0,282r430,0r0,58r-498,0xm419,-788r-68,0r0,-101r68,0r0,101xm242,-788r-68,0r0,-101r68,0r0,101","w":593},"\u00c8":{"d":"72,0r0,-714r493,0r0,58r-425,0r0,258r398,0r0,58r-398,0r0,282r430,0r0,58r-498,0xm305,-771r-132,-141r78,0r105,141r-51,0","w":593},"\u00cd":{"d":"77,0r0,-714r68,0r0,714r-68,0xm235,-912r-132,141r-51,0r105,-141r78,0","w":222},"\u00ce":{"d":"77,0r0,-714r68,0r0,714r-68,0xm146,-912r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":222},"\u00cf":{"d":"77,0r0,-714r68,0r0,714r-68,0xm234,-788r-68,0r0,-101r68,0r0,101xm57,-788r-68,0r0,-101r68,0r0,101","w":222},"\u00cc":{"d":"77,0r0,-714r68,0r0,714r-68,0xm120,-771r-132,-141r78,0r105,141r-51,0","w":222},"\u00d1":{"d":"71,0r0,-714r76,0r416,604r2,0r0,-604r68,0r0,714r-76,0r-416,-604r-2,0r0,604r-68,0xm284,-887v52,0,93,44,135,44v31,0,44,-22,52,-48r38,0v-7,46,-35,95,-87,95v-49,0,-96,-46,-135,-46v-30,0,-53,18,-53,51r-39,0v6,-47,36,-96,89,-96","w":704},"\u00d3":{"d":"34,-357v0,-195,112,-372,337,-372v224,0,336,177,336,372v0,195,-112,372,-336,372v-225,0,-337,-177,-337,-372xm102,-357v0,156,80,314,269,314v188,0,268,-158,268,-314v0,-156,-80,-314,-268,-314v-189,0,-269,158,-269,314xm494,-912r-132,141r-51,0r105,-141r78,0","w":741},"\u00d4":{"d":"34,-357v0,-195,112,-372,337,-372v224,0,336,177,336,372v0,195,-112,372,-336,372v-225,0,-337,-177,-337,-372xm102,-357v0,156,80,314,269,314v188,0,268,-158,268,-314v0,-156,-80,-314,-268,-314v-189,0,-269,158,-269,314xm405,-912r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":741},"\u00d6":{"d":"34,-357v0,-195,112,-372,337,-372v224,0,336,177,336,372v0,195,-112,372,-336,372v-225,0,-337,-177,-337,-372xm102,-357v0,156,80,314,269,314v188,0,268,-158,268,-314v0,-156,-80,-314,-268,-314v-189,0,-269,158,-269,314xm493,-788r-68,0r0,-101r68,0r0,101xm316,-788r-68,0r0,-101r68,0r0,101","w":741},"\u00d2":{"d":"34,-357v0,-195,112,-372,337,-372v224,0,336,177,336,372v0,195,-112,372,-336,372v-225,0,-337,-177,-337,-372xm102,-357v0,156,80,314,269,314v188,0,268,-158,268,-314v0,-156,-80,-314,-268,-314v-189,0,-269,158,-269,314xm379,-771r-132,-141r78,0r105,141r-51,0","w":741},"\u00d5":{"d":"34,-357v0,-195,112,-372,337,-372v224,0,336,177,336,372v0,195,-112,372,-336,372v-225,0,-337,-177,-337,-372xm102,-357v0,156,80,314,269,314v188,0,268,-158,268,-314v0,-156,-80,-314,-268,-314v-189,0,-269,158,-269,314xm302,-887v52,0,93,44,135,44v31,0,44,-22,52,-48r38,0v-7,46,-35,95,-87,95v-49,0,-96,-46,-135,-46v-30,0,-53,18,-53,51r-39,0v6,-47,36,-96,89,-96","w":741},"\u0160":{"d":"36,-235r68,0v-3,147,103,192,233,192v75,0,189,-41,189,-145v0,-81,-82,-110,-147,-129v-137,-39,-320,-33,-320,-211v0,-70,45,-201,246,-201v141,0,265,76,264,222r-68,0v-4,-109,-94,-164,-195,-164v-93,0,-179,36,-179,142v0,67,51,93,108,109v149,42,358,46,359,231v0,49,-20,204,-273,204v-169,0,-293,-76,-285,-250xm281,-771r-114,-141r65,0r86,102r88,-102r58,0r-114,141r-69,0","w":630},"\u00da":{"d":"65,-257r0,-457r68,0r0,442v-1,166,78,229,209,229v132,0,210,-63,210,-229r0,-442r68,0r0,457v0,147,-79,272,-278,272v-198,0,-277,-125,-277,-272xm466,-912r-132,141r-51,0r105,-141r78,0","w":685},"\u00db":{"d":"65,-257r0,-457r68,0r0,442v-1,166,78,229,209,229v132,0,210,-63,210,-229r0,-442r68,0r0,457v0,147,-79,272,-278,272v-198,0,-277,-125,-277,-272xm377,-912r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":685},"\u00dc":{"d":"65,-257r0,-457r68,0r0,442v-1,166,78,229,209,229v132,0,210,-63,210,-229r0,-442r68,0r0,457v0,147,-79,272,-278,272v-198,0,-277,-125,-277,-272xm465,-788r-68,0r0,-101r68,0r0,101xm288,-788r-68,0r0,-101r68,0r0,101","w":685},"\u00d9":{"d":"65,-257r0,-457r68,0r0,442v-1,166,78,229,209,229v132,0,210,-63,210,-229r0,-442r68,0r0,457v0,147,-79,272,-278,272v-198,0,-277,-125,-277,-272xm351,-771r-132,-141r78,0r105,141r-51,0","w":685},"\u00dd":{"d":"271,0r0,-294r-281,-420r81,0r235,362r234,-362r81,0r-282,420r0,294r-68,0xm429,-912r-132,141r-51,0r105,-141r78,0","w":611,"k":{"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":123,".":99,"e":92,"\u00e9":92,"\u00ea":92,"\u00eb":92,"\u00e8":92,"o":92,"\u00f8":92,"\u0153":92,"\u00f3":92,"\u00f4":92,"\u00f6":92,"\u00f2":92,"\u00f5":92,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"i":7,"\u00ed":7,"\u00ee":7,"\u00ef":7,"\u00ec":7,"u":74,"\u00fa":74,"\u00fb":74,"\u00fc":74,"\u00f9":74,":":92,";":92,"p":74}},"\u0178":{"d":"271,0r0,-294r-281,-420r81,0r235,362r234,-362r81,0r-282,420r0,294r-68,0xm428,-788r-68,0r0,-101r68,0r0,101xm251,-788r-68,0r0,-101r68,0r0,101","w":611,"k":{"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":123,".":99,"e":92,"\u00e9":92,"\u00ea":92,"\u00eb":92,"\u00e8":92,"o":92,"\u00f8":92,"\u0153":92,"\u00f3":92,"\u00f4":92,"\u00f6":92,"\u00f2":92,"\u00f5":92,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"i":7,"\u00ed":7,"\u00ee":7,"\u00ef":7,"\u00ec":7,"u":74,"\u00fa":74,"\u00fb":74,"\u00fc":74,"\u00f9":74,":":92,";":92,"p":74}},"\u017d":{"d":"37,-656r0,-58r514,0r0,60r-470,596r480,0r0,58r-556,0r0,-60r470,-596r-438,0xm253,-771r-114,-141r65,0r86,102r88,-102r58,0r-114,141r-69,0","w":574},"\u00e1":{"d":"390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm117,-358r-63,0v6,-121,91,-173,209,-173v91,0,190,28,190,166r0,274v-3,32,29,45,57,34r0,53v-14,3,-24,4,-41,4v-67,2,-70,-41,-76,-90v-44,67,-89,105,-188,105v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120xm383,-729r-132,141r-51,0r105,-141r78,0","w":519},"\u00e2":{"d":"390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm117,-358r-63,0v6,-121,91,-173,209,-173v91,0,190,28,190,166r0,274v-3,32,29,45,57,34r0,53v-14,3,-24,4,-41,4v-67,2,-70,-41,-76,-90v-44,67,-89,105,-188,105v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120xm294,-729r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":519},"\u00e4":{"d":"390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm117,-358r-63,0v6,-121,91,-173,209,-173v91,0,190,28,190,166r0,274v-3,32,29,45,57,34r0,53v-14,3,-24,4,-41,4v-67,2,-70,-41,-76,-90v-44,67,-89,105,-188,105v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120xm382,-605r-68,0r0,-101r68,0r0,101xm205,-605r-68,0r0,-101r68,0r0,101","w":519},"\u00e0":{"d":"390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm117,-358r-63,0v6,-121,91,-173,209,-173v91,0,190,28,190,166r0,274v-3,32,29,45,57,34r0,53v-14,3,-24,4,-41,4v-67,2,-70,-41,-76,-90v-44,67,-89,105,-188,105v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120xm268,-588r-132,-141r78,0r105,141r-51,0","w":519},"\u00e5":{"d":"390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm117,-358r-63,0v6,-121,91,-173,209,-173v91,0,190,28,190,166r0,274v-3,32,29,45,57,34r0,53v-14,3,-24,4,-41,4v-67,2,-70,-41,-76,-90v-44,67,-89,105,-188,105v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120xm153,-677v0,-59,48,-107,107,-107v59,0,107,48,107,107v0,59,-48,107,-107,107v-59,0,-107,-48,-107,-107xm192,-677v0,38,30,68,68,68v38,0,68,-30,68,-68v0,-38,-30,-68,-68,-68v-38,0,-68,30,-68,68","w":519},"\u00e3":{"d":"390,-200v-1,-26,4,-60,-2,-82v-8,15,-36,20,-53,23v-107,19,-240,18,-240,119v0,63,56,102,115,102v96,0,182,-61,180,-162xm117,-358r-63,0v6,-121,91,-173,209,-173v91,0,190,28,190,166r0,274v-3,32,29,45,57,34r0,53v-14,3,-24,4,-41,4v-67,2,-70,-41,-76,-90v-44,67,-89,105,-188,105v-95,0,-173,-47,-173,-151v0,-145,141,-150,277,-166v52,-6,81,-13,81,-70v0,-85,-61,-106,-135,-106v-78,0,-136,36,-138,120xm191,-704v52,0,93,44,135,44v31,0,44,-22,52,-48r38,0v-7,46,-35,95,-87,95v-49,0,-96,-46,-135,-46v-30,0,-53,18,-53,51r-39,0v6,-47,36,-96,89,-96","w":519},"\u00e7":{"d":"428,-185r63,0v-17,123,-96,198,-209,200r-33,43v47,-9,116,0,116,62v0,97,-130,102,-204,65r14,-32v36,14,132,33,132,-29v0,-44,-65,-44,-94,-30r-16,-15r51,-66v-139,-15,-214,-133,-214,-271v0,-148,86,-273,244,-273v110,0,195,59,209,177r-63,0v-17,-77,-64,-124,-146,-124v-242,0,-241,440,0,440v78,0,142,-61,150,-147","w":519},"\u00e9":{"d":"494,-241r-397,0v1,92,49,203,170,203v92,0,142,-54,162,-132r63,0v-27,117,-95,185,-225,185v-164,0,-233,-126,-233,-273v0,-136,69,-273,233,-273v166,0,232,145,227,290xm97,-294r334,0v-3,-95,-62,-184,-164,-184v-103,0,-160,90,-170,184xm383,-729r-132,141r-51,0r105,-141r78,0","w":519},"\u00ea":{"d":"494,-241r-397,0v1,92,49,203,170,203v92,0,142,-54,162,-132r63,0v-27,117,-95,185,-225,185v-164,0,-233,-126,-233,-273v0,-136,69,-273,233,-273v166,0,232,145,227,290xm97,-294r334,0v-3,-95,-62,-184,-164,-184v-103,0,-160,90,-170,184xm294,-729r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":519},"\u00eb":{"d":"494,-241r-397,0v1,92,49,203,170,203v92,0,142,-54,162,-132r63,0v-27,117,-95,185,-225,185v-164,0,-233,-126,-233,-273v0,-136,69,-273,233,-273v166,0,232,145,227,290xm97,-294r334,0v-3,-95,-62,-184,-164,-184v-103,0,-160,90,-170,184xm382,-605r-68,0r0,-101r68,0r0,101xm205,-605r-68,0r0,-101r68,0r0,101","w":519},"\u00e8":{"d":"494,-241r-397,0v1,92,49,203,170,203v92,0,142,-54,162,-132r63,0v-27,117,-95,185,-225,185v-164,0,-233,-126,-233,-273v0,-136,69,-273,233,-273v166,0,232,145,227,290xm97,-294r334,0v-3,-95,-62,-184,-164,-184v-103,0,-160,90,-170,184xm268,-588r-132,-141r78,0r105,141r-51,0","w":519},"\u00ed":{"d":"124,0r-63,0r0,-516r63,0r0,516xm216,-729r-132,141r-51,0r105,-141r78,0","w":185},"\u00ee":{"d":"124,0r-63,0r0,-516r63,0r0,516xm127,-729r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":185},"\u00ef":{"d":"124,0r-63,0r0,-516r63,0r0,516xm215,-605r-68,0r0,-101r68,0r0,101xm38,-605r-68,0r0,-101r68,0r0,101","w":185},"\u00ec":{"d":"124,0r-63,0r0,-516r63,0r0,516xm101,-588r-132,-141r78,0r105,141r-51,0","w":185},"\u00f1":{"d":"59,0r0,-516r63,0v2,28,-4,65,2,89v23,-62,92,-104,165,-104v145,0,189,76,189,199r0,332r-63,0r0,-322v0,-89,-29,-156,-131,-156v-100,0,-162,76,-162,177r0,301r-63,0xm200,-704v52,0,93,44,135,44v31,0,44,-22,52,-48r38,0v-7,46,-35,95,-87,95v-49,0,-96,-46,-135,-46v-30,0,-53,18,-53,51r-39,0v6,-47,36,-96,89,-96","w":537},"\u00f3":{"d":"278,-531v158,0,244,125,244,273v0,148,-86,273,-244,273v-158,0,-244,-125,-244,-273v0,-148,86,-273,244,-273xm278,-478v-242,0,-241,440,0,440v242,0,241,-440,0,-440xm402,-729r-132,141r-51,0r105,-141r78,0"},"\u00f4":{"d":"278,-531v158,0,244,125,244,273v0,148,-86,273,-244,273v-158,0,-244,-125,-244,-273v0,-148,86,-273,244,-273xm278,-478v-242,0,-241,440,0,440v242,0,241,-440,0,-440xm313,-729r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0"},"\u00f6":{"d":"278,-531v158,0,244,125,244,273v0,148,-86,273,-244,273v-158,0,-244,-125,-244,-273v0,-148,86,-273,244,-273xm278,-478v-242,0,-241,440,0,440v242,0,241,-440,0,-440xm401,-605r-68,0r0,-101r68,0r0,101xm224,-605r-68,0r0,-101r68,0r0,101"},"\u00f2":{"d":"278,-531v158,0,244,125,244,273v0,148,-86,273,-244,273v-158,0,-244,-125,-244,-273v0,-148,86,-273,244,-273xm278,-478v-242,0,-241,440,0,440v242,0,241,-440,0,-440xm287,-588r-132,-141r78,0r105,141r-51,0"},"\u00f5":{"d":"278,-531v158,0,244,125,244,273v0,148,-86,273,-244,273v-158,0,-244,-125,-244,-273v0,-148,86,-273,244,-273xm278,-478v-242,0,-241,440,0,440v242,0,241,-440,0,-440xm210,-704v52,0,93,44,135,44v31,0,44,-22,52,-48r38,0v-7,46,-35,95,-87,95v-49,0,-96,-46,-135,-46v-30,0,-53,18,-53,51r-39,0v6,-47,36,-96,89,-96"},"\u0161":{"d":"432,-365r-63,0v-3,-78,-63,-113,-135,-113v-56,0,-122,22,-122,89v0,87,116,89,191,106v71,16,147,53,147,143v0,112,-111,155,-207,155v-120,0,-202,-56,-212,-182r63,0v5,85,68,129,152,129v59,0,141,-26,141,-98v0,-88,-117,-91,-194,-112v-82,-22,-144,-50,-144,-138v0,-105,103,-145,194,-145v103,0,185,54,189,166xm206,-588r-114,-141r65,0r86,102r88,-102r58,0r-114,141r-69,0","w":481},"\u00fa":{"d":"478,-516r0,516r-58,0v-2,-30,4,-67,-2,-93v-33,69,-101,108,-178,108v-130,0,-181,-76,-181,-196r0,-335r63,0r0,336v3,93,38,142,138,142v108,0,155,-102,155,-207r0,-271r63,0xm392,-729r-132,141r-51,0r105,-141r78,0","w":537},"\u00fb":{"d":"478,-516r0,516r-58,0v-2,-30,4,-67,-2,-93v-33,69,-101,108,-178,108v-130,0,-181,-76,-181,-196r0,-335r63,0r0,336v3,93,38,142,138,142v108,0,155,-102,155,-207r0,-271r63,0xm303,-729r114,141r-65,0r-86,-102r-88,102r-58,0r114,-141r69,0","w":537},"\u00fc":{"d":"478,-516r0,516r-58,0v-2,-30,4,-67,-2,-93v-33,69,-101,108,-178,108v-130,0,-181,-76,-181,-196r0,-335r63,0r0,336v3,93,38,142,138,142v108,0,155,-102,155,-207r0,-271r63,0xm391,-605r-68,0r0,-101r68,0r0,101xm214,-605r-68,0r0,-101r68,0r0,101","w":537},"\u00f9":{"d":"478,-516r0,516r-58,0v-2,-30,4,-67,-2,-93v-33,69,-101,108,-178,108v-130,0,-181,-76,-181,-196r0,-335r63,0r0,336v3,93,38,142,138,142v108,0,155,-102,155,-207r0,-271r63,0xm277,-588r-132,-141r78,0r105,141r-51,0","w":537},"\u00fd":{"d":"208,-2r-206,-514r67,0r170,440r159,-440r63,0r-224,595v-35,105,-85,122,-184,107r0,-53v76,18,113,-13,133,-76xm355,-729r-132,141r-51,0r105,-141r78,0","w":463,"k":{",":92,".":92}},"\u00ff":{"d":"208,-2r-206,-514r67,0r170,440r159,-440r63,0r-224,595v-35,105,-85,122,-184,107r0,-53v76,18,113,-13,133,-76xm354,-605r-68,0r0,-101r68,0r0,101xm177,-605r-68,0r0,-101r68,0r0,101","w":463,"k":{",":92,".":92}},"\u017e":{"d":"432,-471r-341,418r355,0r0,53r-431,0r0,-51r337,-412r-314,0r0,-53r394,0r0,45xm197,-588r-114,-141r65,0r86,102r88,-102r58,0r-114,141r-69,0","w":463},"\u2206":{"d":"33,0r0,-43r259,-671r64,0r257,671r0,43r-580,0xm90,-47r461,0r-228,-599r-4,0v-10,37,-43,119,-63,171","w":645},"\u2126":{"d":"208,-47r0,-3v-75,-65,-141,-175,-141,-319v0,-203,132,-337,299,-337v175,0,292,153,292,333v0,151,-72,266,-142,323r0,3r156,0r0,47r-233,0r0,-35v82,-53,162,-169,162,-329v0,-133,-80,-294,-236,-294v-147,0,-241,133,-241,296v0,151,81,276,161,327r0,35r-232,0r0,-47r155,0","w":726},"\u03bc":{"d":"478,-516r0,516r-58,0v-2,-30,4,-67,-2,-93v-46,105,-195,142,-295,73r0,211r-64,0r0,-707r63,0r0,336v3,93,38,142,138,142v108,0,155,-102,155,-207r0,-271r63,0","w":537},"\u03c0":{"d":"532,-466r-84,0v4,137,-11,369,15,466r-55,0v-9,-15,-16,-53,-16,-139r0,-327r-197,0v-7,136,-39,375,-77,466r-56,0v39,-104,70,-324,76,-466v-68,0,-96,5,-116,13r-11,-37v25,-15,63,-25,136,-25r392,0","w":560},"\u20ac":{"d":"84,-434v26,-152,122,-277,287,-278v83,0,145,26,186,77r-27,59v-76,-100,-230,-106,-310,-12v-34,39,-57,90,-69,154r333,0r-18,45r-321,0v-2,24,-4,38,-2,59r304,0r-19,45r-282,0v15,133,87,244,221,244v86,0,122,-35,174,-86r0,77v-51,43,-109,65,-176,65v-170,0,-268,-133,-284,-300r-66,0r19,-45r44,0v-2,-19,0,-35,1,-59r-64,0r18,-45r51,0"},"\u2113":{"d":"420,-134r30,23v-41,78,-102,121,-179,121v-117,0,-162,-93,-161,-204v-18,14,-39,32,-59,49r-20,-36v28,-23,55,-45,79,-68r0,-280v0,-183,78,-234,145,-234v80,0,118,69,118,156v0,128,-84,250,-210,372v-8,126,49,200,118,200v66,0,115,-54,139,-99xm163,-536r0,243v94,-101,168,-203,168,-311v0,-68,-21,-115,-78,-115v-40,0,-90,46,-90,183","w":475},"\u212e":{"d":"835,-335r-649,0v-3,0,-5,1,-5,4r0,196v0,9,2,17,8,23v62,66,150,106,246,106v103,0,195,-47,258,-119r57,0v-71,84,-187,139,-316,139v-221,0,-400,-161,-400,-359v0,-199,179,-361,400,-361v224,0,406,162,401,371xm687,-361r0,-197v0,-9,-3,-17,-9,-24v-63,-62,-148,-102,-243,-102v-95,0,-182,41,-245,105v-6,7,-9,15,-9,25r0,193v0,2,2,6,5,6r497,0v3,0,4,-4,4,-6","w":869},"\u2202":{"d":"101,-679r-22,-41v33,-26,84,-54,161,-54v124,0,245,118,245,375v0,227,-84,410,-254,410v-127,0,-187,-113,-187,-228v0,-154,101,-251,211,-251v98,0,155,71,174,102v16,-201,-79,-361,-194,-361v-62,0,-106,25,-134,48xm101,-219v0,106,52,183,135,183v103,0,170,-125,184,-268v-16,-48,-74,-118,-157,-118v-88,0,-162,90,-162,203","w":536},"\u220f":{"d":"664,-643r-112,0r0,739r-55,0r0,-739r-303,0r0,739r-55,0r0,-739r-113,0r0,-51r638,0r0,51","w":692},"\u2211":{"d":"528,96r-505,0r0,-39r278,-355r-265,-357r0,-39r477,0r0,49r-397,0r0,4r251,336r-270,344r0,3r431,0r0,54","w":552},"\u2219":{"d":"139,-375v32,0,59,28,59,60v0,31,-28,58,-59,58v-32,0,-59,-28,-59,-60v0,-30,29,-58,59,-58","w":278},"\u221a":{"d":"561,-823r-221,975r-53,0r-164,-471r-75,28r-13,-36r124,-48r136,390v7,22,11,51,18,67r12,-65r192,-840r44,0","w":563},"\u221e":{"d":"581,-439v87,0,142,62,142,151v0,93,-72,153,-150,153v-64,0,-113,-44,-184,-121v-54,60,-109,121,-191,121v-80,0,-146,-66,-146,-150v0,-88,65,-154,153,-154v75,0,134,61,187,123v55,-61,107,-123,189,-123xm94,-284v0,61,44,110,111,110v67,0,119,-67,161,-110v-45,-54,-95,-117,-169,-117v-66,0,-103,56,-103,117xm681,-286v0,-74,-45,-115,-106,-115v-69,0,-126,78,-162,113v69,73,109,114,164,114v66,0,104,-56,104,-112","w":775},"\u222b":{"d":"329,-831r-12,40v-27,-17,-65,-15,-88,12v-26,32,-37,84,-37,182v0,153,14,351,14,508v0,111,-24,166,-46,201v-29,46,-106,55,-149,27r12,-41v22,14,74,18,93,-11v18,-28,37,-71,37,-178v0,-157,-13,-359,-13,-512v0,-106,14,-166,51,-206v35,-38,95,-45,138,-22","w":343},"\u2248":{"d":"184,-423v90,0,130,77,211,77v42,0,69,-25,98,-76r25,22v-25,50,-63,92,-126,92v-67,0,-142,-76,-215,-77v-48,0,-76,35,-102,75r-27,-21v31,-58,79,-92,136,-92xm183,-259v93,0,129,77,213,77v42,0,68,-25,97,-75r25,22v-25,49,-63,91,-126,91v-66,0,-143,-76,-215,-77v-48,0,-77,36,-102,76r-27,-21v31,-59,79,-93,135,-93","w":568},"\u2260":{"d":"398,-499r-47,102r164,0r0,39r-181,0r-70,151r251,0r0,39r-268,0r-56,120r-32,-16r49,-104r-156,0r0,-39r173,0r70,-151r-243,0r0,-39r260,0r54,-118","w":568},"\u2264":{"d":"513,-94r-455,-226r0,-41r455,-225r0,48r-410,196r0,2r410,198r0,48xm515,-11r-463,0r0,-41r463,0r0,41","w":568},"\u2265":{"d":"58,-586r455,225r0,41r-454,226r0,-48r409,-197r0,-2r-410,-197r0,-48xm515,-11r-462,0r0,-41r462,0r0,41","w":568},"\u25ca":{"d":"508,-347r-204,391r-46,0r-202,-391r204,-391r46,0xm455,-345r-154,-295v-9,-14,-12,-40,-21,-49v-47,121,-116,227,-171,341r154,296v9,20,15,37,18,48r3,0v46,-122,116,-227,171,-341","w":565},"\u00a0":{"w":278},"\u00ad":{"d":"310,-246r-250,0r0,-58r250,0r0,58","w":370},"\u02c9":{"d":"257,-634r-329,0r0,-45r329,0r0,45","w":185},"\u03a9":{"d":"208,-47r0,-3v-75,-65,-141,-175,-141,-319v0,-203,132,-337,299,-337v175,0,292,153,292,333v0,151,-72,266,-142,323r0,3r156,0r0,47r-233,0r0,-35v82,-53,162,-169,162,-329v0,-133,-80,-294,-236,-294v-147,0,-241,133,-241,296v0,151,81,276,161,327r0,35r-232,0r0,-47r155,0","w":726},"\u2215":{"d":"-166,34r453,-759r47,0r-452,759r-48,0","w":167}}});
;
/*!
 * The following copyright notice may not be removed under any circumstances.
 * 
 * Copyright:
 * Copyright ?1988, 1990, 1993, 2002 Adobe Systems Incorporated.  All Rights
 * Reserved. ?1981, 2002 Heidelberger Druckmaschinen AG. All rights reserved.
 * 
 * Trademark:
 * Helvetica is a trademark of Heidelberger Druckmaschinen AG, exclusively
 * licensed through Linotype Library GmbH, and may be registered in certain
 * jurisdictions.
 * 
 * Full name:
 * HelveticaNeueLTStd-Bd
 * 
 * Designer:
 * Linotype Staff
 * 
 * Vendor URL:
 * http://www.adobe.com/type
 * 
 * License information:
 * http://www.adobe.com/type/legal.html
 */
Cufon.registerFont({"w":556,"face":{"font-family":"Helvetica Neue LT Std 75 Bold","font-weight":700,"font-stretch":"normal","units-per-em":"1000","panose-1":"2 11 8 4 2 2 2 2 2 4","ascent":"714","descent":"-286","x-height":"14","bbox":"-166 -975 1078 218.225","underline-thickness":"50","underline-position":"-50","stemh":"107","stemv":"142","unicode-range":"U+0020-U+FB02"},"glyphs":{" ":{"w":278},"\ufb01":{"d":"553,-714r0,117r-142,0r0,-117r142,0xm411,0r0,-517r142,0r0,517r-142,0xm85,0r0,-422r-85,0r0,-95r85,0v-9,-122,41,-193,173,-197v25,0,51,3,75,4r0,106v-54,-6,-112,-10,-106,56r0,31r98,0r0,95r-98,0r0,422r-142,0","w":611,"k":{"\u2019":-18,"f":18,"\ufb01":18,"\ufb02":18,"\u00df":18}},"\ufb02":{"d":"411,0r0,-714r142,0r0,714r-142,0xm85,0r0,-422r-85,0r0,-95r85,0v-9,-122,41,-193,173,-197v25,0,51,3,75,4r0,106v-54,-6,-112,-10,-106,56r0,31r98,0r0,95r-98,0r0,422r-142,0","w":611,"k":{"\u2019":-18,"f":18,"\ufb01":18,"\ufb02":18,"\u00df":18}},"!":{"d":"213,-714r0,192r-39,308r-71,0r-38,-308r0,-192r148,0xm61,0r0,-154r157,0r0,154r-157,0","w":278},"\"":{"d":"271,-393r0,-321r107,0r0,321r-107,0xm85,-393r0,-321r107,0r0,321r-107,0","w":463},"#":{"d":"88,0r28,-198r-83,0r0,-90r95,0r18,-124r-83,0r0,-90r95,0r28,-198r97,0r-28,198r97,0r27,-198r97,0r-27,198r74,0r0,90r-87,0r-17,124r74,0r0,90r-87,0r-27,198r-97,0r27,-198r-96,0r-28,198r-97,0xm339,-412r-96,0r-18,124r97,0"},"$":{"d":"248,-451r0,-163v-45,0,-90,28,-90,83v0,44,26,65,90,80xm307,-292r0,192v49,-5,108,-34,108,-98v0,-53,-29,-72,-108,-94xm248,-309v-111,-30,-232,-61,-232,-208v0,-137,112,-207,232,-214r0,-77r59,0r0,77v119,13,217,75,225,213r-142,0v-1,-53,-38,-96,-83,-96r0,177v16,4,33,8,51,13v195,54,199,165,199,232v0,57,-51,196,-250,209r0,85r-59,0r0,-85v-148,-11,-234,-92,-249,-249r142,0v-1,74,46,120,107,132r0,-209"},"%":{"d":"75,-516v0,-109,53,-198,170,-198v124,0,164,92,164,202v0,109,-52,188,-169,188v-123,0,-165,-82,-165,-192xm177,-514v0,43,0,115,64,115v62,0,66,-72,66,-114v0,-39,-3,-126,-62,-126v-64,0,-68,79,-68,125xm591,-183v0,-109,53,-193,170,-193v124,0,164,87,164,197v0,109,-52,193,-169,193v-123,0,-165,-87,-165,-197xm693,-182v0,43,0,121,64,121v62,0,66,-78,66,-120v0,-39,-3,-120,-62,-120v-64,0,-68,73,-68,119xm254,22r414,-744r85,0r-412,744r-87,0","w":1000},"&":{"d":"382,-559v0,-37,-30,-68,-65,-68v-34,0,-64,26,-64,63v0,45,29,74,56,107v40,-26,73,-50,73,-102xm389,-166r-122,-149v-50,22,-100,60,-100,118v0,52,48,94,102,94v51,0,87,-26,120,-63xm522,0r-58,-71v-57,56,-120,85,-201,85v-129,0,-231,-79,-231,-214v0,-100,76,-168,160,-207v-38,-47,-68,-90,-68,-152v0,-99,91,-163,186,-163v108,0,196,58,196,171v0,83,-55,140,-127,178r92,113v14,-26,22,-56,26,-85r123,0v-7,64,-29,126,-69,176r145,169r-174,0","w":685},"\u2019":{"d":"68,-560r0,-154r142,0r0,153v3,89,-59,157,-142,168r0,-66v51,-17,66,-52,66,-101r-66,0","w":278,"k":{"\u2019":65,"s":74,"\u0161":74,"t":18}},"(":{"d":"184,-731r119,0v-147,284,-145,626,-1,913r-118,0v-172,-273,-177,-638,0,-913","w":296},")":{"d":"112,182r-119,0v147,-284,145,-626,1,-913r118,0v172,274,177,638,0,913","w":296},"*":{"d":"242,-714r0,125r116,-44r27,77r-118,39r74,98r-66,49r-74,-103r-70,103r-67,-49r74,-98r-116,-39r27,-77r113,44r0,-125r80,0","w":407},"+":{"d":"354,-505r0,199r199,0r0,107r-199,0r0,199r-107,0r0,-199r-199,0r0,-107r199,0r0,-199r107,0","w":600},",":{"d":"61,0r0,-154r157,0r0,154v0,92,-70,153,-156,166r0,-72v42,-7,73,-50,71,-94r-72,0","w":278},"-":{"d":"53,-212r0,-122r301,0r0,122r-301,0","w":407},".":{"d":"60,0r0,-154r157,0r0,154r-157,0","w":278},"\/":{"d":"-11,17r279,-748r114,0r-279,748r-114,0","w":371},"0":{"d":"21,-353v0,-273,118,-361,256,-361v140,0,258,88,258,361v0,279,-118,367,-258,367v-138,0,-256,-88,-256,-367xm163,-353v0,79,0,250,114,250v116,0,116,-171,116,-250v0,-73,0,-244,-116,-244v-114,0,-114,171,-114,244"},"1":{"d":"392,-700r0,700r-142,0r0,-454r-176,0r0,-107v99,2,190,-32,205,-139r113,0"},"2":{"d":"531,-495v0,219,-234,234,-327,373r331,0r0,122r-514,0v1,-162,100,-231,221,-313v62,-42,146,-85,147,-171v0,-66,-44,-108,-105,-108v-84,0,-111,87,-111,161r-136,0v-5,-160,86,-283,254,-283v128,0,240,82,240,219"},"3":{"d":"228,-317r0,-100v61,5,149,-7,149,-87v0,-59,-47,-93,-99,-93v-71,0,-109,53,-108,125r-135,0v5,-143,98,-242,242,-242v112,0,235,69,235,194v0,68,-37,123,-98,145v77,17,124,81,124,159v0,147,-123,230,-260,230v-157,0,-263,-94,-260,-255r135,0v3,76,41,138,123,138v64,0,114,-44,114,-110v0,-106,-93,-104,-162,-104"},"4":{"d":"312,0r0,-162r-296,0r0,-130r304,-408r127,0r0,421r93,0r0,117r-93,0r0,162r-135,0xm312,-279r0,-243r-3,0r-181,243r184,0"},"5":{"d":"500,-700r0,117r-289,0r-28,159r2,2v38,-39,82,-54,137,-54v138,0,216,107,216,238v0,144,-119,252,-260,252v-136,0,-258,-75,-260,-222r142,0v7,62,53,105,115,105v74,0,121,-65,121,-135v0,-73,-45,-131,-121,-131v-51,0,-79,18,-107,56r-128,0r69,-387r391,0"},"6":{"d":"523,-520r-135,0v-9,-47,-45,-87,-94,-87v-102,-1,-135,140,-132,215v70,-109,241,-99,319,-6v129,154,17,412,-191,412v-212,0,-269,-185,-269,-362v0,-171,76,-366,275,-366v121,0,211,71,227,194xm286,-363v-76,0,-113,61,-113,130v0,65,42,130,113,130v140,-1,144,-260,0,-260"},"7":{"d":"517,-700r0,122v-147,128,-227,391,-229,578r-152,0v16,-207,102,-408,235,-568r-332,0r0,-132r478,0"},"8":{"d":"156,-212v0,72,56,119,124,119v68,0,120,-48,120,-118v0,-67,-54,-112,-120,-112v-69,0,-124,39,-124,111xm43,-523v0,-126,122,-191,236,-191v174,0,234,120,234,188v0,71,-40,123,-102,149v83,19,131,83,131,171v0,293,-528,296,-528,1v0,-91,54,-149,132,-174v-68,-19,-103,-75,-103,-144xm172,-510v0,61,50,92,107,92v58,0,105,-32,105,-92v0,-36,-17,-97,-105,-97v-57,0,-107,35,-107,97"},"9":{"d":"33,-180r135,0v9,47,45,87,94,87v102,1,135,-140,132,-215v-70,109,-241,99,-319,6v-129,-154,-17,-412,191,-412v212,0,269,185,269,362v0,171,-76,366,-275,366v-121,0,-211,-71,-227,-194xm270,-337v76,0,113,-61,113,-130v0,-65,-42,-130,-113,-130v-140,1,-144,260,0,260"},":":{"d":"61,0r0,-154r157,0r0,154r-157,0xm218,-508r0,154r-157,0r0,-154r157,0","w":278},";":{"d":"61,0r0,-154r157,0r0,154v-1,92,-70,153,-156,166r0,-72v42,-7,73,-50,71,-94r-72,0xm218,-508r0,154r-157,0r0,-154r157,0","w":278},"<":{"d":"554,-514r0,115r-367,146r367,146r0,115r-508,-202r0,-118","w":600},"=":{"d":"553,-413r0,107r-505,0r0,-107r505,0xm553,-199r0,107r-505,0r0,-107r505,0","w":600},">":{"d":"46,8r0,-115r367,-146r-367,-146r0,-115r508,202r0,118","w":600},"?":{"d":"524,-538v0,179,-151,158,-182,286r0,38r-135,0v-8,-106,40,-168,87,-202v40,-29,73,-54,73,-108v0,-61,-32,-90,-85,-90v-72,0,-102,59,-103,130r-147,0v3,-143,97,-247,243,-247v189,0,249,116,249,193xm193,0r0,-154r157,0r0,154r-157,0"},"@":{"d":"291,-319v0,53,32,89,78,89v72,0,123,-82,123,-151v0,-49,-34,-85,-77,-85v-72,0,-124,81,-124,147xm622,-545r-63,258v-8,27,-16,66,9,66v57,0,118,-77,118,-184v0,-159,-119,-246,-271,-246v-172,0,-283,127,-283,296v0,175,123,292,295,292v84,0,163,-30,215,-82r84,0v-68,103,-180,162,-305,162v-213,0,-385,-160,-385,-374v0,-211,172,-374,380,-374v182,0,348,121,348,305v0,206,-171,290,-235,290v-45,0,-68,-28,-75,-57v-80,123,-265,31,-265,-113v0,-126,86,-255,218,-255v43,0,94,18,117,70r16,-54r82,0","w":800},"A":{"d":"-7,0r271,-714r161,0r267,714r-163,0r-54,-159r-267,0r-56,159r-159,0xm342,-538r-93,262r185,0r-90,-262r-2,0","w":685},"B":{"d":"226,-318r0,196v121,-9,281,39,284,-94v3,-137,-160,-96,-284,-102xm69,0r0,-714r336,0v136,0,228,44,228,177v0,70,-35,119,-97,149v87,25,131,92,131,181v0,145,-123,207,-252,207r-346,0xm226,-592r0,167v108,-6,257,33,255,-85v-2,-119,-148,-73,-255,-82","w":704},"C":{"d":"697,-474r-152,0v-10,-71,-78,-125,-156,-125v-141,0,-194,120,-194,245v0,119,53,239,194,239v96,0,150,-66,162,-160r152,0v-16,178,-139,292,-314,292v-221,0,-351,-165,-351,-371v0,-212,130,-377,351,-377v157,0,289,92,308,257","w":741},"D":{"d":"69,0r0,-714r308,0v185,0,324,116,324,353v0,207,-106,361,-324,361r-308,0xm226,-582r0,450v189,7,318,9,318,-214v0,-144,-50,-236,-206,-236r-112,0","w":741},"E":{"d":"69,0r0,-714r534,0r0,132r-377,0r0,153r346,0r0,122r-346,0r0,175r385,0r0,132r-542,0","w":648},"F":{"d":"69,0r0,-714r502,0r0,132r-345,0r0,165r299,0r0,122r-299,0r0,295r-157,0","w":593,"k":{"A":55,"\u00c6":55,"\u00c1":55,"\u00c2":55,"\u00c4":55,"\u00c0":55,"\u00c5":55,"\u00c3":55,",":129,".":129}},"G":{"d":"601,0r-16,-81v-55,71,-127,98,-196,98v-221,0,-351,-165,-351,-371v0,-212,130,-377,351,-377v146,0,284,90,301,252r-150,0v-18,-79,-77,-120,-151,-120v-141,0,-194,120,-194,245v0,119,53,239,194,239v102,0,161,-55,170,-154r-158,0r0,-117r300,0r0,386r-100,0","w":759},"H":{"d":"69,0r0,-714r157,0r0,274r289,0r0,-274r157,0r0,714r-157,0r0,-308r-289,0r0,308r-157,0","w":741},"I":{"d":"69,0r0,-714r157,0r0,714r-157,0","w":295},"J":{"d":"487,-714r0,489v0,92,-25,242,-238,242v-119,0,-235,-65,-235,-221r0,-62r142,0v-2,83,5,152,91,151v83,0,83,-67,83,-117r0,-482r157,0"},"K":{"d":"69,0r0,-714r157,0r0,296r279,-296r196,0r-279,282r306,432r-197,0r-215,-321r-90,91r0,230r-157,0","w":722},"L":{"d":"69,0r0,-714r157,0r0,582r348,0r0,132r-505,0","w":593,"k":{"T":111,"V":92,"W":55,"y":37,"\u00fd":37,"\u00ff":37,"Y":111,"\u00dd":111,"\u0178":111,"\u2019":74}},"M":{"d":"69,0r0,-714r221,0r167,491r2,0r158,-491r221,0r0,714r-147,0r0,-506r-2,0r-175,506r-121,0r-175,-501r-2,0r0,501r-147,0","w":907},"N":{"d":"69,0r0,-714r156,0r298,479r2,0r0,-479r147,0r0,714r-157,0r-297,-478r-2,0r0,478r-147,0","w":741},"O":{"d":"38,-354v0,-212,130,-377,351,-377v221,0,351,165,351,377v0,206,-130,371,-351,371v-221,0,-351,-165,-351,-371xm195,-354v0,119,53,239,194,239v141,0,194,-120,194,-239v0,-125,-53,-245,-194,-245v-141,0,-194,120,-194,245","w":778},"P":{"d":"69,0r0,-714r322,0v178,0,247,113,247,229v0,116,-69,229,-247,229r-165,0r0,256r-157,0xm226,-592r0,214v122,-5,260,30,260,-107v0,-137,-138,-102,-260,-107","w":667,"k":{"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":129,".":129}},"Q":{"d":"736,-10r-73,77r-101,-92v-47,27,-106,42,-173,42v-221,0,-351,-165,-351,-371v0,-212,130,-377,351,-377v221,0,351,165,351,377v0,103,-33,195,-93,263xm385,-186r73,-78r81,74v26,-36,44,-90,44,-164v0,-125,-53,-245,-194,-245v-141,0,-194,120,-194,245v0,119,53,239,194,239v23,0,52,0,63,-10","w":778},"R":{"d":"69,0r0,-714r385,0v128,0,209,89,209,197v0,86,-38,145,-113,181v77,20,99,95,104,167v3,45,2,129,30,169r-157,0v-19,-45,-17,-114,-25,-171v-11,-75,-40,-108,-119,-108r-157,0r0,279r-157,0xm226,-592r0,201v122,-6,280,36,280,-102v0,-69,-38,-99,-108,-99r-172,0","w":722,"k":{"T":18,"V":-6,"W":-6,"Y":37,"\u00dd":37,"\u0178":37}},"S":{"d":"24,-237r152,0v0,95,74,132,158,132v55,0,139,-16,139,-89v0,-77,-107,-90,-212,-118v-106,-28,-213,-69,-213,-203v0,-146,138,-216,267,-216v149,0,286,65,286,230r-152,0v-5,-86,-66,-108,-141,-108v-50,0,-108,21,-108,81v0,55,34,62,213,108v52,13,212,46,212,208v0,131,-103,229,-297,229v-158,0,-306,-78,-304,-254","w":649},"T":{"d":"227,0r0,-582r-214,0r0,-132r585,0r0,132r-214,0r0,582r-157,0","w":611,"k":{"\u00fc":92,"\u0161":111,"\u00f2":111,"\u00f6":111,"\u00e8":111,"\u00eb":111,"\u00ea":111,"\u00e3":111,"\u00e5":111,"\u00e0":111,"\u00e4":111,"\u00e2":111,"w":111,"y":92,"\u00fd":92,"\u00ff":92,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":111,".":111,"c":111,"\u00e7":111,"e":111,"\u00e9":111,"o":111,"\u00f8":111,"\u0153":111,"\u00f3":111,"\u00f4":111,"\u00f5":111,"-":129,"a":111,"\u00e6":111,"\u00e1":111,"r":92,"s":111,"u":92,"\u00fa":92,"\u00fb":92,"\u00f9":92,":":87,";":87}},"U":{"d":"675,-714r0,444v0,193,-114,287,-305,287v-192,0,-304,-93,-304,-287r0,-444r157,0r0,444v0,78,20,155,147,155v112,0,148,-49,148,-155r0,-444r157,0","w":741},"V":{"d":"638,-714r-237,714r-177,0r-232,-714r162,0r159,502r2,0r161,-502r162,0","w":630,"k":{"\u00f6":55,"\u00f4":55,"\u00ee":18,"\u00e8":55,"\u00eb":55,"\u00ea":55,"\u00e3":55,"\u00e5":55,"\u00e0":55,"\u00e4":55,"\u00e2":55,"y":18,"\u00fd":18,"\u00ff":18,"A":46,"\u00c6":46,"\u00c1":46,"\u00c2":46,"\u00c4":46,"\u00c0":46,"\u00c5":46,"\u00c3":46,",":129,".":129,"e":55,"\u00e9":55,"o":55,"\u00f8":55,"\u0153":55,"\u00f3":55,"\u00f2":55,"\u00f5":55,"-":55,"a":55,"\u00e6":55,"\u00e1":55,"r":37,"u":37,"\u00fa":37,"\u00fb":37,"\u00fc":37,"\u00f9":37,":":50,";":50,"i":18,"\u00ed":18,"\u00ef":18,"\u00ec":18}},"W":{"d":"941,-714r-192,714r-156,0r-121,-486r-2,0r-119,486r-159,0r-189,-714r157,0r113,486r2,0r124,-486r147,0r122,492r2,0r117,-492r154,0","w":944,"k":{"\u00fc":18,"\u00f6":37,"\u00ea":37,"\u00e4":37,"A":18,"\u00c6":18,"\u00c1":18,"\u00c2":18,"\u00c4":18,"\u00c0":18,"\u00c5":18,"\u00c3":18,",":74,".":74,"e":37,"\u00e9":37,"\u00eb":37,"\u00e8":37,"o":37,"\u00f8":37,"\u0153":37,"\u00f3":37,"\u00f4":37,"\u00f2":37,"\u00f5":37,"a":37,"\u00e6":37,"\u00e1":37,"\u00e2":37,"\u00e0":37,"\u00e5":37,"\u00e3":37,"r":18,"u":18,"\u00fa":18,"\u00fb":18,"\u00f9":18,":":18,";":18}},"X":{"d":"-6,0r250,-374r-230,-340r182,0r140,228r145,-228r172,0r-229,341r249,373r-187,0r-156,-247r-159,247r-177,0","w":667},"Y":{"d":"254,0r0,-278r-263,-436r177,0r167,282r166,-282r175,0r-265,440r0,274r-157,0","w":667,"k":{"\u00fc":74,"\u00f6":92,"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":111,".":111,"e":92,"\u00e9":92,"\u00ea":92,"\u00eb":92,"\u00e8":92,"o":92,"\u00f8":92,"\u0153":92,"\u00f3":92,"\u00f4":92,"\u00f2":92,"\u00f5":92,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"u":74,"\u00fa":74,"\u00fb":74,"\u00f9":74,":":80,";":68,"i":13,"\u00ed":13,"\u00ee":13,"\u00ef":13,"\u00ec":13,"p":74}},"Z":{"d":"23,0r0,-124r383,-458r-353,0r0,-132r562,0r0,124r-383,458r393,0r0,132r-602,0","w":648},"[":{"d":"68,182r0,-913r264,0r0,112r-122,0r0,689r122,0r0,112r-264,0","w":333},"\\":{"d":"103,-731r279,748r-114,0r-279,-748r114,0","w":371},"]":{"d":"265,-731r0,913r-264,0r0,-112r122,0r0,-689r-122,0r0,-112r264,0","w":333},"^":{"d":"61,-315r172,-385r134,0r172,385r-117,0r-122,-275r-122,275r-117,0","w":600},"_":{"d":"0,125r0,-50r500,0r0,50r-500,0","w":500},"\u2018":{"d":"210,-547r0,154r-142,0r0,-153v-3,-89,59,-157,142,-168r0,66v-51,17,-66,52,-66,101r66,0","w":278,"k":{"\u2018":65}},"a":{"d":"376,-253v-53,42,-204,6,-202,106v0,53,41,66,87,66v111,0,115,-88,115,-119r0,-53xm190,-358r-142,0v8,-133,127,-173,243,-173v103,0,227,23,227,147r0,269v0,47,5,94,18,115r-144,0v-5,-16,-9,-33,-10,-50v-102,104,-350,90,-350,-91v0,-117,88,-144,176,-157v87,-12,168,-10,168,-68v0,-61,-42,-70,-92,-70v-54,0,-89,22,-94,78","w":574},"b":{"d":"54,0r0,-714r142,0r0,260r2,0v35,-53,99,-77,163,-77v103,0,214,83,214,272v0,190,-111,273,-214,273v-78,0,-137,-26,-172,-80r0,66r-135,0xm433,-258v0,-87,-37,-166,-121,-166v-84,0,-121,79,-121,166v0,86,37,165,121,165v84,0,121,-79,121,-165","w":611},"c":{"d":"545,-335r-139,0v-9,-58,-46,-89,-105,-89v-91,0,-121,92,-121,168v0,74,29,163,118,163v66,0,104,-42,113,-105r137,0v-18,137,-113,212,-249,212v-156,0,-261,-110,-261,-265v0,-161,96,-280,264,-280v122,0,234,64,243,196","w":574},"d":{"d":"423,0v-2,-21,4,-49,-2,-66v-33,56,-91,80,-157,80v-156,0,-232,-134,-232,-277v0,-139,77,-268,229,-268v63,0,119,29,155,77r0,-260r142,0r0,714r-135,0xm421,-260v0,-84,-30,-164,-122,-164v-93,0,-125,80,-125,165v0,81,37,166,125,166v94,0,122,-82,122,-167","w":611},"e":{"d":"544,-224r-373,0v4,90,48,131,127,131v57,0,103,-35,112,-67r125,0v-40,122,-125,174,-242,174v-163,0,-264,-112,-264,-272v0,-155,107,-273,264,-273v176,0,261,148,251,307xm171,-314r231,0v-13,-72,-44,-110,-113,-110v-90,0,-116,70,-118,110","w":574},"f":{"d":"85,0r0,-422r-85,0r0,-95r85,0v-9,-122,41,-193,173,-197v25,0,51,3,75,4r0,106v-54,-6,-112,-10,-106,56r0,31r98,0r0,95r-98,0r0,422r-142,0","w":333,"k":{"\u2019":-18,"f":18,"\ufb01":18,"\ufb02":18,"\u00df":18}},"g":{"d":"551,-517r0,484v0,85,-30,229,-268,229v-102,0,-221,-48,-228,-166r141,0v13,53,56,71,106,71v104,0,126,-93,112,-196v-31,54,-93,80,-155,80v-155,0,-221,-118,-221,-260v0,-134,77,-256,222,-256v70,0,118,27,156,83r0,-69r135,0xm295,-122v81,0,121,-69,121,-143v0,-84,-29,-159,-121,-159v-80,0,-115,70,-115,147v0,74,28,155,115,155","w":611},"h":{"d":"54,0r0,-714r142,0r0,269r3,0v36,-58,98,-86,151,-86v151,0,189,85,189,213r0,318r-142,0r0,-292v0,-85,-25,-127,-91,-127v-77,0,-110,43,-110,148r0,271r-142,0","w":593},"i":{"d":"58,0r0,-517r142,0r0,517r-142,0xm200,-714r0,117r-142,0r0,-117r142,0","w":258},"j":{"d":"-19,178r0,-117v38,8,87,9,86,-42r0,-536r142,0r0,543v0,106,-39,156,-182,156v-15,0,-31,-2,-46,-4xm209,-714r0,117r-142,0r0,-117r142,0","w":278},"k":{"d":"67,0r0,-714r142,0r0,383r179,-186r168,0r-195,190r217,327r-172,0r-142,-231r-55,53r0,178r-142,0","w":574},"l":{"d":"58,0r0,-714r142,0r0,714r-142,0","w":258},"m":{"d":"58,0r0,-517r134,0v2,22,-4,52,2,70v63,-105,253,-120,309,2v30,-45,84,-86,156,-86v110,0,189,51,189,185r0,346r-142,0r0,-293v0,-69,-6,-126,-87,-126v-80,0,-95,66,-95,131r0,288r-142,0r0,-290v0,-60,4,-129,-86,-129v-28,0,-96,18,-96,119r0,300r-142,0","w":906},"n":{"d":"54,0r0,-517r135,0r0,72r3,0v36,-58,98,-86,158,-86v151,0,189,85,189,213r0,318r-142,0r0,-292v0,-85,-25,-127,-91,-127v-77,0,-110,43,-110,148r0,271r-142,0","w":593},"o":{"d":"38,-258v0,-165,105,-273,267,-273v163,0,268,108,268,273v0,164,-105,272,-268,272v-162,0,-267,-108,-267,-272xm180,-258v0,82,29,165,125,165v97,0,126,-83,126,-165v0,-83,-29,-166,-126,-166v-96,0,-125,83,-125,166","w":611},"p":{"d":"54,182r0,-699r135,0v2,21,-4,49,2,66v34,-55,90,-80,154,-80v162,0,235,131,235,278v0,138,-76,267,-227,267v-64,0,-120,-30,-157,-78r0,246r-142,0xm315,-93v93,0,123,-80,123,-164v0,-82,-33,-167,-124,-167v-93,0,-123,83,-123,167v0,84,32,164,124,164","w":611},"q":{"d":"558,-517r0,699r-142,0r0,-247r-2,0v-33,56,-101,79,-163,79v-95,0,-219,-69,-219,-269v0,-142,75,-276,232,-276v66,0,123,27,159,80r0,-66r135,0xm174,-258v0,83,30,165,123,165v92,0,124,-78,124,-163v0,-81,-32,-168,-121,-168v-92,0,-126,82,-126,166","w":611},"r":{"d":"54,0r0,-517r135,0v2,31,-4,69,2,96v32,-72,110,-125,196,-105r0,132v-13,-3,-34,-5,-51,-5v-189,1,-131,225,-140,399r-142,0","w":389,"k":{",":92,".":92,"c":18,"\u00e7":18,"d":18,"\u0131":18,"e":18,"\u00e9":18,"\u00ea":18,"\u00eb":18,"\u00e8":18,"n":-18,"\u00f1":-18,"o":18,"\u00f8":18,"\u0153":18,"\u00f3":18,"\u00f4":18,"\u00f6":18,"\u00f2":18,"\u00f5":18,"q":18,"-":55}},"s":{"d":"29,-168r135,0v1,62,53,87,109,87v41,0,93,-16,93,-66v0,-43,-59,-58,-161,-80v-82,-18,-164,-47,-164,-138v0,-132,114,-166,225,-166v113,0,217,38,228,165r-135,0v-4,-55,-46,-70,-97,-70v-32,0,-79,6,-79,48v0,51,80,58,161,77v83,19,164,49,164,145v0,136,-118,180,-236,180v-120,0,-237,-45,-243,-182","w":537},"t":{"d":"234,-672r0,155r104,0r0,95r-104,0r0,256v-9,67,50,64,104,56r0,111v-115,12,-246,16,-246,-118r0,-305r-86,0r0,-95r86,0r0,-155r142,0","w":352},"u":{"d":"539,-517r0,517r-135,0r0,-72r-3,0v-36,58,-98,86,-158,86v-151,0,-189,-85,-189,-213r0,-318r142,0r0,292v0,85,25,127,91,127v77,0,110,-43,110,-148r0,-271r142,0","w":593},"v":{"d":"515,-517r-175,517r-158,0r-177,-517r149,0r109,353r2,0r109,-353r141,0","w":520,"k":{",":55,".":55}},"w":{"d":"808,-517r-163,517r-146,0r-92,-347r-2,0r-88,347r-147,0r-164,-517r150,0r95,351r2,0r86,-351r138,0r88,350r2,0r95,-350r146,0","w":814,"k":{",":55,".":55}},"x":{"d":"0,0r186,-272r-170,-245r162,0r91,135r90,-135r157,0r-170,242r191,275r-162,0r-108,-163r-108,163r-159,0","w":537},"y":{"d":"525,-517r-216,582v-33,111,-133,128,-265,112r0,-117v27,2,55,7,83,5v49,-4,65,-56,49,-97r-182,-485r152,0r117,354r2,0r113,-354r147,0","w":519,"k":{",":55,".":55}},"z":{"d":"22,0r0,-107r269,-303r-249,0r0,-107r436,0r0,107r-269,303r288,0r0,107r-475,0","w":519},"{":{"d":"333,-731r0,112r-54,0v-48,0,-54,41,-54,73r0,152v0,98,-69,114,-99,118r0,3v30,3,99,26,99,110r0,160v0,33,6,73,54,73r54,0r0,112r-127,0v-45,0,-123,-44,-123,-135r0,-195v0,-61,-60,-83,-92,-83r0,-87v32,0,92,-23,92,-90r0,-188v0,-91,78,-135,123,-135r127,0","w":333},"|":{"d":"58,214r0,-1000r107,0r0,1000r-107,0","w":223},"}":{"d":"0,182r0,-112r54,0v48,0,54,-40,54,-73r0,-160v0,-84,69,-107,99,-110r0,-3v-30,-4,-99,-20,-99,-118r0,-152v0,-32,-6,-73,-54,-73r-54,0r0,-112r127,0v45,0,123,44,123,135r0,188v0,67,60,90,92,90r0,87v-32,0,-92,22,-92,83r0,195v0,91,-78,135,-123,135r-127,0","w":333},"~":{"d":"192,-340v66,0,155,68,213,68v40,0,65,-37,88,-72r40,93v-30,42,-63,86,-125,86v-80,0,-130,-68,-217,-68v-44,0,-68,37,-84,72r-40,-93v22,-42,58,-86,125,-86","w":600},"\u00a1":{"d":"218,-531r0,154r-157,0r0,-154r157,0xm65,183r0,-192r39,-308r71,0r38,308r0,192r-148,0","w":278},"\u00a2":{"d":"260,-93r0,-330v-71,19,-97,90,-97,159v0,71,22,148,97,171xm260,115r0,-101v-143,-9,-239,-122,-239,-264v0,-152,84,-268,239,-281r0,-85r59,0r0,85v110,1,220,77,224,197r-139,0v-4,-48,-40,-86,-85,-90r0,331v49,-9,86,-59,87,-110r139,0v-11,127,-104,207,-226,217r0,101r-59,0"},"\u00a3":{"d":"411,17v-98,0,-118,-37,-208,-38v-36,0,-76,8,-115,38r-62,-93v73,-45,148,-135,94,-233r-99,0r0,-85r64,0v-20,-25,-45,-79,-45,-138v0,-106,88,-199,256,-199v151,0,244,88,243,250r-135,0v-3,-39,-7,-133,-109,-133v-58,0,-98,36,-98,92v0,31,1,56,41,128r133,0r0,85r-105,0v27,91,-14,159,-74,202r2,2v104,-77,224,68,313,-43r60,101v-48,46,-105,64,-156,64"},"\u2044":{"d":"-166,22r414,-744r85,0r-412,744r-87,0","w":167},"\u00a5":{"d":"359,0r-152,0r0,-138r-157,0r0,-95r157,0r0,-46r-14,-29r-143,0r0,-95r95,0r-154,-311r163,0r132,311r129,-311r162,0r-155,311r95,0r0,95r-143,0r-15,29r0,46r158,0r0,95r-158,0r0,138"},"\u0192":{"d":"-5,167r19,-107v65,12,100,-7,112,-68r63,-329r-92,0r18,-95r91,0r24,-127v24,-126,75,-172,206,-172v25,0,50,2,75,7r-18,107v-57,-19,-105,-1,-116,55r-25,130r98,0r-16,95r-100,0r-56,310v-25,134,-61,197,-209,197v-25,0,-46,-1,-74,-3"},"\u00a7":{"d":"377,-287r-165,-93v-36,-30,-80,18,-78,57v0,19,10,37,32,49r180,98v35,22,76,-13,76,-48v0,-30,-21,-50,-45,-63xm494,-538r-124,0v0,-49,-32,-86,-85,-86v-36,0,-76,24,-76,64v0,56,87,86,170,126v84,40,167,89,167,184v0,66,-42,124,-104,146v38,29,55,70,55,118v0,113,-112,168,-213,168v-127,0,-226,-58,-225,-194r124,0v3,58,38,87,98,87v41,0,74,-22,74,-66v0,-59,-82,-89,-170,-130v-87,-41,-175,-92,-175,-190v0,-72,40,-122,107,-146v-28,-30,-43,-59,-43,-101v0,-113,107,-173,209,-173v118,0,211,69,211,193"},"\u00a4":{"d":"20,-149r57,-58v-63,-69,-65,-218,0,-286r-57,-58r57,-57r57,56v64,-58,220,-62,283,-1r58,-57r61,61r-56,55v60,68,60,219,-1,289r55,56r-55,55r-57,-58v-66,66,-223,66,-290,1r-56,58xm280,-491v-84,0,-127,68,-127,142v0,73,44,140,126,140v82,0,122,-70,122,-142v0,-70,-40,-140,-121,-140"},"'":{"d":"86,-393r0,-321r107,0r0,321r-107,0","w":278},"\u201c":{"d":"409,-547r0,154r-142,0r0,-153v-3,-89,59,-157,142,-168r0,66v-51,17,-66,52,-66,101r66,0xm196,-547r0,154r-142,0r0,-153v-3,-89,59,-157,142,-168r0,66v-51,17,-66,52,-66,101r66,0","w":463},"\u00ab":{"d":"402,-80r-157,-125r0,-135r157,-123r0,120r-90,71r90,72r0,120xm199,-80r-157,-125r0,-135r157,-123r0,120r-90,71r90,72r0,120","w":444},"\u2039":{"d":"199,-80r-157,-125r0,-135r157,-123r0,120r-90,71r90,72r0,120","w":241},"\u203a":{"d":"42,-463r157,123r0,135r-157,125r0,-120r90,-72r-90,-71r0,-120","w":241},"\u2013":{"d":"0,-212r0,-122r500,0r0,122r-500,0","w":500},"\u2020":{"d":"210,166r0,-566r-186,0r0,-117r186,0r0,-197r135,0r0,197r187,0r0,117r-187,0r0,566r-135,0"},"\u2021":{"d":"210,166r0,-193r-186,0r0,-117r186,0r0,-256r-186,0r0,-117r186,0r0,-197r135,0r0,197r187,0r0,117r-187,0r0,256r187,0r0,117r-187,0r0,193r-135,0"},"\u00b7":{"d":"55,-287v0,-47,39,-85,85,-85v45,0,84,39,84,85v0,42,-39,84,-83,84v-47,0,-86,-38,-86,-84","w":278},"\u00b6":{"d":"221,166r0,-523v-139,0,-219,-69,-219,-175v0,-243,306,-171,533,-182r0,880r-102,0r0,-800r-110,0r0,800r-102,0","w":620},"\u2022":{"d":"72,-357v0,-99,79,-178,178,-178v99,0,179,79,179,178v0,99,-80,179,-179,179v-99,0,-178,-80,-178,-179","w":501},"\u201a":{"d":"68,0r0,-154r142,0r0,153v3,89,-59,157,-142,168r0,-66v51,-17,66,-52,66,-101r-66,0","w":278},"\u201e":{"d":"267,0r0,-154r142,0r0,153v3,89,-59,157,-142,168r0,-66v51,-17,66,-52,66,-101r-66,0xm54,0r0,-154r142,0r0,153v3,89,-59,157,-142,168r0,-66v51,-17,66,-52,66,-101r-66,0","w":463},"\u201d":{"d":"267,-560r0,-154r142,0r0,153v3,89,-59,157,-142,168r0,-66v51,-17,66,-52,66,-101r-66,0xm54,-560r0,-154r142,0r0,153v3,89,-59,157,-142,168r0,-66v51,-17,66,-52,66,-101r-66,0","w":463},"\u00bb":{"d":"245,-463r157,123r0,135r-157,125r0,-120r90,-72r-90,-71r0,-120xm42,-463r157,123r0,135r-157,125r0,-120r90,-72r-90,-71r0,-120","w":444},"\u2026":{"d":"88,0r0,-154r157,0r0,154r-157,0xm421,0r0,-154r157,0r0,154r-157,0xm754,0r0,-154r157,0r0,154r-157,0","w":1000},"\u2030":{"d":"530,-172v0,57,3,121,56,121v57,0,56,-59,56,-107v0,-52,-1,-107,-54,-107v-56,0,-58,55,-58,93xm440,-155v0,-94,43,-175,147,-175v109,0,145,77,145,174v0,97,-43,170,-146,170v-93,0,-146,-69,-146,-169xm786,-155v0,-94,43,-175,147,-175v109,0,145,77,145,174v0,97,-43,170,-146,170v-93,0,-146,-69,-146,-169xm876,-172v0,57,3,121,56,121v57,0,56,-59,56,-107v0,-52,-1,-107,-54,-107v-56,0,-58,55,-58,93xm69,-539v0,-94,43,-175,147,-175v109,0,145,77,145,174v0,97,-43,170,-146,170v-93,0,-146,-69,-146,-169xm159,-556v0,57,3,121,56,121v57,0,56,-59,56,-107v0,-52,-1,-107,-54,-107v-56,0,-58,55,-58,93xm150,22r414,-744r85,0r-412,744r-87,0","w":1147},"\u00bf":{"d":"32,3v0,-179,151,-158,182,-286r0,-38r135,0v8,106,-40,168,-87,202v-40,29,-73,54,-73,108v0,61,32,90,85,90v70,0,100,-59,101,-130r147,0v-3,143,-97,247,-241,247v-189,0,-249,-116,-249,-193xm363,-531r0,154r-157,0r0,-154r157,0"},"`":{"d":"106,-581r-154,-142r157,0r96,142r-99,0","w":259},"\u00b4":{"d":"307,-723r-154,142r-99,0r96,-142r157,0","w":259},"\u02c6":{"d":"-46,-581r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":259},"\u02dc":{"d":"72,-714v46,0,84,35,130,35v30,0,43,-16,42,-35r69,0v-16,52,-37,114,-112,114v-56,0,-102,-33,-152,-34v-27,0,-36,17,-42,42r-61,0v13,-58,38,-122,126,-122","w":259},"\u00af":{"d":"-57,-616r0,-75r373,0r0,75r-373,0","w":259},"\u02d8":{"d":"-39,-727r65,0v19,104,194,91,209,0r63,0v-13,102,-80,146,-178,146v-96,0,-151,-50,-159,-146","w":259},"\u02d9":{"d":"62,-597r0,-117r135,0r0,117r-135,0","w":259},"\u00a8":{"d":"165,-597r0,-117r135,0r0,117r-135,0xm-41,-597r0,-117r135,0r0,117r-135,0","w":259},"\u02da":{"d":"66,-656v0,39,25,70,63,70v38,0,64,-31,64,-70v0,-39,-26,-70,-64,-70v-38,0,-63,31,-63,70xm7,-656v0,-67,55,-122,122,-122v68,0,123,55,123,122v0,67,-55,122,-123,122v-67,0,-122,-55,-122,-122","w":259},"\u00b8":{"d":"12,196r21,-48v29,13,122,41,124,-16v1,-41,-52,-34,-80,-25r-21,-24r59,-85r49,0r-39,55v52,-15,128,4,128,65v0,129,-165,107,-241,78","w":259},"\u02dd":{"d":"192,-723r-149,142r-99,0r91,-142r157,0xm399,-723r-149,142r-99,0r91,-142r157,0","w":259},"\u02db":{"d":"202,128r48,22v-40,51,-89,68,-134,68v-25,0,-107,-6,-107,-90v0,-18,2,-64,114,-131r65,0v-64,58,-83,93,-83,127v0,47,72,45,97,4","w":259},"\u02c7":{"d":"305,-723r-110,142r-130,0r-111,-142r112,0r64,81r65,-81r110,0","w":259},"\u2014":{"d":"130,-212r0,-122r740,0r0,122r-740,0","w":1000},"\u00c6":{"d":"418,-582r-138,306r172,0r0,-306r-34,0xm-11,0r341,-714r606,0r0,132r-332,0r0,153r312,0r0,122r-312,0r0,175r340,0r0,132r-492,0r0,-159r-224,0r-72,159r-167,0","w":981},"\u00aa":{"d":"230,-535v-33,25,-117,8,-118,58v0,29,26,37,45,37v75,0,73,-46,73,-95xm20,-605v10,-81,72,-108,156,-109v70,0,150,14,150,89v0,77,-10,186,12,241r-101,0v-4,-11,-6,-21,-7,-31v-68,66,-220,58,-220,-60v0,-73,57,-90,114,-97v60,-7,107,-6,107,-38v0,-34,-27,-39,-60,-39v-23,0,-46,13,-49,44r-102,0","w":344},"\u0141":{"d":"69,0r0,-214r-80,56r0,-112r80,-56r0,-388r157,0r0,278r211,-148r0,112r-211,148r0,192r348,0r0,132r-505,0","w":593,"k":{"T":111,"V":92,"W":55,"y":37,"\u00fd":37,"\u00ff":37,"Y":111,"\u00dd":111,"\u0178":111,"\u2019":74}},"\u00d8":{"d":"34,-10r86,-95v-54,-66,-82,-153,-82,-249v0,-212,130,-377,351,-377v92,0,168,28,225,77r80,-89r50,44r-84,93v52,66,80,154,80,252v0,206,-130,371,-351,371v-92,0,-168,-29,-223,-76r-83,92xm223,-219r296,-329v-30,-31,-73,-51,-130,-51v-190,0,-230,227,-166,380xm556,-490r-295,326v30,30,72,49,128,49v191,-1,228,-222,167,-375","w":778},"\u0152":{"d":"544,0v-2,-17,4,-41,-2,-54v-42,46,-110,71,-172,71v-211,0,-332,-172,-332,-370v0,-198,124,-378,335,-378v62,0,130,18,169,68r2,0r0,-51r500,0r0,132r-354,0r0,153r324,0r0,122r-324,0r0,175r361,0r0,132r-507,0xm538,-278r0,-170v0,-90,-65,-151,-154,-151v-139,0,-189,125,-189,244v0,117,52,240,186,240v96,0,157,-66,157,-163","w":1093},"\u00ba":{"d":"12,-544v0,-105,68,-170,171,-170v104,0,172,65,172,170v0,105,-68,169,-172,169v-103,0,-171,-64,-171,-169xm183,-639v-86,2,-87,186,0,189v89,-3,87,-187,0,-189","w":367},"\u00e6":{"d":"385,-252v-55,45,-205,5,-205,104v0,52,45,67,88,67v52,0,117,-31,117,-107r0,-64xm878,-224r-369,0v-2,68,44,143,123,143v62,0,94,-30,103,-81r138,0v-39,186,-316,240,-431,88v-48,63,-134,88,-211,88v-98,0,-193,-40,-193,-154v0,-118,87,-145,173,-158v121,-18,174,-19,174,-72v0,-54,-46,-66,-92,-66v-54,0,-97,20,-99,80r-142,0v5,-130,123,-175,244,-175v60,0,135,16,171,70v40,-47,91,-70,173,-70v173,0,238,146,238,307xm509,-314r227,0v1,-63,-40,-122,-111,-122v-73,0,-115,59,-116,122","w":907},"\u0131":{"d":"58,0r0,-517r142,0r0,517r-142,0","w":258},"\u0142":{"d":"58,0r0,-286r-67,53r0,-102r67,-53r0,-326r142,0r0,214r67,-53r0,102r-67,53r0,398r-142,0","w":258},"\u00f8":{"d":"195,-172r193,-220v-19,-20,-46,-32,-83,-32v-127,0,-144,149,-110,252xm415,-349r-195,222v19,21,47,34,85,34v130,-1,145,-152,110,-256xm44,-1r60,-68v-43,-47,-66,-112,-66,-189v0,-165,105,-273,267,-273v64,0,119,17,163,48r53,-61r37,32r-54,61v44,47,69,114,69,193v0,164,-105,272,-268,272v-65,0,-120,-17,-165,-50r-59,67","w":611},"\u0153":{"d":"528,-314r227,0v-3,-66,-40,-110,-115,-110v-71,0,-109,49,-112,110xm290,-93v95,0,114,-92,114,-167v0,-69,-16,-164,-111,-164v-87,0,-113,89,-113,168v0,71,25,163,110,163xm897,-224r-369,0v0,74,40,131,123,131v52,0,85,-24,104,-71r138,0v-33,112,-125,178,-242,178v-72,0,-147,-24,-187,-88v-36,63,-114,88,-183,88v-160,0,-243,-128,-243,-275v0,-152,100,-270,254,-270v67,0,141,22,178,82v36,-53,111,-82,173,-82v183,0,254,142,254,307","w":926},"\u00df":{"d":"56,0r0,-491v-1,-112,36,-240,235,-240v119,0,227,60,227,192v0,64,-41,120,-93,141v87,19,130,93,130,179v0,124,-90,233,-217,233v-27,0,-55,-2,-82,-6r0,-116v87,24,157,-35,157,-116v0,-83,-59,-135,-157,-120r0,-94v66,9,127,-18,127,-87v0,-84,-66,-89,-96,-89v-57,0,-89,39,-89,98r0,516r-142,0","w":611},"\u00b9":{"d":"289,-705r0,432r-102,0r0,-274r-110,0r0,-75v64,2,119,-19,128,-83r84,0","w":392},"\u00ac":{"d":"552,-406r0,307r-107,0r0,-200r-397,0r0,-107r504,0","w":600},"\u00b5":{"d":"539,-517r0,517r-135,0r0,-72r-3,0v-36,58,-98,86,-130,86v-40,0,-63,-11,-75,-24r0,192r-142,0r0,-699r142,0r0,292v0,85,25,127,91,127v77,0,110,-43,110,-148r0,-271r142,0","w":593},"\u2122":{"d":"928,-714r0,412r-96,0r0,-292r-2,0r-106,292r-69,0r-106,-292r-2,0r0,292r-96,0r0,-412r136,0r103,271r103,-271r135,0xm398,-714r0,80r-117,0r0,332r-102,0r0,-332r-117,0r0,-80r336,0","w":1000},"\u00d0":{"d":"69,0r0,-311r-66,0r0,-116r66,0r0,-287r308,0v185,0,324,116,324,353v0,207,-106,361,-324,361r-308,0xm226,-582r0,155r160,0r0,117r-160,0r0,178v189,7,318,9,318,-214v0,-144,-50,-236,-206,-236r-112,0","w":741},"\u00bd":{"d":"152,22r414,-744r85,0r-412,744r-87,0xm236,-705r0,432r-102,0r0,-274r-110,0r0,-75v64,2,119,-19,128,-83r84,0xm846,-306v0,134,-153,140,-213,226r209,0r0,80r-334,0v1,-106,62,-147,140,-196v40,-26,95,-51,96,-103v0,-35,-24,-62,-61,-62v-56,0,-74,54,-72,94r-102,0v-1,-103,58,-174,181,-174v83,0,156,46,156,135","w":889},"\u00b1":{"d":"354,-506r0,131r199,0r0,107r-199,0r0,131r-107,0r0,-131r-199,0r0,-107r199,0r0,-131r107,0xm48,0r0,-107r505,0r0,107r-505,0","w":600},"\u00de":{"d":"69,0r0,-714r157,0r0,97r165,0v178,0,247,113,247,229v0,116,-69,229,-247,229r-165,0r0,159r-157,0xm226,-495r0,214v122,-5,260,30,260,-107v0,-137,-138,-102,-260,-107","w":667},"\u00bc":{"d":"689,0r0,-89r-192,0r0,-93r188,-238r100,0r0,251r57,0r0,80r-57,0r0,89r-96,0xm689,-169v-2,-46,4,-102,-2,-144r-114,144r116,0xm152,22r414,-744r85,0r-412,744r-87,0xm236,-705r0,432r-102,0r0,-274r-110,0r0,-75v64,2,119,-19,128,-83r84,0","w":892},"\u00f7":{"d":"48,-199r0,-107r505,0r0,107r-505,0xm216,-460v0,-47,39,-85,85,-85v45,0,84,39,84,85v0,42,-39,84,-83,84v-47,0,-86,-38,-86,-84xm216,-45v0,-47,39,-84,85,-84v45,0,84,38,84,84v0,42,-39,85,-83,85v-47,0,-86,-39,-86,-85","w":600},"\u00a6":{"d":"58,89r0,-250r107,0r0,250r-107,0xm165,-661r0,250r-107,0r0,-250r107,0","w":223},"\u00b0":{"d":"52,-566v0,-82,66,-148,148,-148v82,0,148,66,148,148v0,82,-66,148,-148,148v-82,0,-148,-66,-148,-148xm117,-566v0,48,40,88,83,88v43,0,83,-40,83,-88v0,-48,-40,-88,-83,-88v-43,0,-83,40,-83,88","w":400},"\u00fe":{"d":"54,182r0,-896r142,0r0,263r2,0v34,-55,90,-80,147,-80v162,0,235,131,235,278v0,138,-76,267,-227,267v-64,0,-120,-30,-157,-78r0,246r-142,0xm438,-257v0,-82,-33,-167,-124,-167v-93,0,-123,83,-123,167v0,84,32,164,124,164v93,0,123,-80,123,-164","w":611},"\u00be":{"d":"709,0r0,-89r-192,0r0,-93r188,-238r100,0r0,251r57,0r0,80r-57,0r0,89r-96,0xm709,-169v-2,-46,4,-102,-2,-144r-114,144r116,0xm239,22r414,-744r85,0r-412,744r-87,0xm181,-464r0,-70v38,0,95,0,95,-47v0,-33,-30,-53,-62,-53v-43,0,-68,32,-67,72r-96,0v3,-93,63,-152,162,-152v79,0,159,41,159,123v0,42,-26,73,-66,88v50,9,81,48,81,94v0,96,-80,145,-173,145v-108,0,-177,-56,-175,-160r96,0v2,43,27,80,77,80v40,0,73,-26,73,-63v0,-57,-61,-57,-104,-57","w":889},"\u00b2":{"d":"365,-579v0,134,-153,140,-213,226r209,0r0,80r-334,0v1,-106,62,-147,140,-196v40,-26,95,-51,96,-103v0,-35,-24,-62,-61,-62v-56,0,-74,54,-72,94r-102,0v-1,-103,58,-174,181,-174v83,0,156,46,156,135","w":392},"\u00ae":{"d":"16,-358v0,-227,181,-373,384,-373v203,0,384,146,384,373v0,228,-181,375,-384,375v-203,0,-384,-147,-384,-375xm118,-358v0,171,125,295,282,295v155,0,282,-124,282,-295v0,-170,-127,-293,-282,-293v-157,0,-282,123,-282,293xm251,-149r0,-419r158,0v105,0,164,33,164,124v0,77,-48,107,-110,112r112,183r-87,0r-102,-179r-52,0r0,179r-83,0xm334,-503r0,110v69,-4,160,19,156,-58v-3,-68,-90,-49,-156,-52","w":800},"\u2212":{"d":"48,-199r0,-107r505,0r0,107r-505,0","w":600},"\u00f0":{"d":"305,-93v97,0,126,-83,126,-165v0,-74,-35,-153,-127,-153v-93,0,-124,80,-124,153v0,82,29,165,125,165xm133,-593r107,-54v-28,-20,-56,-37,-84,-50r92,-69v36,20,69,42,99,66r123,-62r52,59r-114,57v106,102,165,219,165,373v0,164,-105,287,-268,287v-162,0,-267,-108,-267,-261v0,-123,70,-259,230,-259v39,0,89,11,137,40v-24,-56,-69,-98,-102,-127r-112,56","w":611},"\u00d7":{"d":"140,-17r-76,-76r161,-160r-159,-160r76,-76r159,160r160,-160r76,76r-161,160r161,160r-76,76r-160,-161","w":600},"\u00b3":{"d":"164,-464r0,-70v38,0,95,0,95,-47v0,-33,-30,-53,-62,-53v-43,0,-68,32,-67,72r-96,0v3,-93,63,-152,162,-152v79,0,159,41,159,123v0,42,-26,73,-66,88v50,9,81,48,81,94v0,96,-80,145,-173,145v-108,0,-177,-56,-175,-160r96,0v2,43,27,80,77,80v40,0,73,-26,73,-63v0,-57,-61,-57,-104,-57","w":392},"\u00a9":{"d":"16,-357v0,-228,181,-374,384,-374v203,0,384,146,384,374v0,227,-181,374,-384,374v-203,0,-384,-147,-384,-374xm118,-357v0,170,125,294,282,294v155,0,282,-124,282,-294v0,-171,-127,-294,-282,-294v-157,0,-282,123,-282,294xm515,-295r81,0v-16,100,-93,158,-182,158v-130,0,-215,-97,-215,-222v0,-129,80,-224,213,-224v92,0,167,54,181,155r-78,0v-10,-47,-51,-75,-103,-75v-76,0,-117,60,-117,142v0,78,46,144,119,144v55,0,95,-30,101,-78","w":800},"\u00c1":{"d":"-7,0r271,-714r161,0r267,714r-163,0r-54,-159r-267,0r-56,159r-159,0xm342,-538r-93,262r185,0r-90,-262r-2,0xm520,-920r-154,142r-99,0r96,-142r157,0","w":685},"\u00c2":{"d":"-7,0r271,-714r161,0r267,714r-163,0r-54,-159r-267,0r-56,159r-159,0xm342,-538r-93,262r185,0r-90,-262r-2,0xm167,-778r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":685},"\u00c4":{"d":"-7,0r271,-714r161,0r267,714r-163,0r-54,-159r-267,0r-56,159r-159,0xm342,-538r-93,262r185,0r-90,-262r-2,0xm378,-794r0,-117r135,0r0,117r-135,0xm172,-794r0,-117r135,0r0,117r-135,0","w":685},"\u00c0":{"d":"-7,0r271,-714r161,0r267,714r-163,0r-54,-159r-267,0r-56,159r-159,0xm342,-538r-93,262r185,0r-90,-262r-2,0xm319,-778r-154,-142r157,0r96,142r-99,0","w":685},"\u00c5":{"d":"-7,0r271,-714r161,0r267,714r-163,0r-54,-159r-267,0r-56,159r-159,0xm342,-538r-93,262r185,0r-90,-262r-2,0xm279,-853v0,39,25,70,63,70v38,0,64,-31,64,-70v0,-39,-26,-70,-64,-70v-38,0,-63,31,-63,70xm220,-853v0,-67,55,-122,122,-122v68,0,123,55,123,122v0,67,-55,122,-123,122v-67,0,-122,-55,-122,-122","w":685},"\u00c3":{"d":"-7,0r271,-714r161,0r267,714r-163,0r-54,-159r-267,0r-56,159r-159,0xm342,-538r-93,262r185,0r-90,-262r-2,0xm285,-911v46,0,84,35,130,35v30,0,43,-16,42,-35r69,0v-16,52,-37,114,-112,114v-56,0,-102,-33,-152,-34v-27,0,-36,17,-42,42r-61,0v13,-58,38,-122,126,-122","w":685},"\u00c7":{"d":"697,-474r-152,0v-10,-71,-78,-125,-156,-125v-141,0,-194,120,-194,245v0,119,53,239,194,239v96,0,150,-66,162,-160r152,0v-16,177,-137,290,-311,292r-26,36v52,-15,128,4,128,65v0,129,-165,107,-241,78r21,-48v29,13,122,41,124,-16v1,-41,-52,-34,-80,-25r-21,-24r48,-69v-194,-20,-307,-176,-307,-368v0,-212,130,-377,351,-377v157,0,289,92,308,257","w":741},"\u00c9":{"d":"69,0r0,-714r534,0r0,132r-377,0r0,153r346,0r0,122r-346,0r0,175r385,0r0,132r-542,0xm502,-920r-154,142r-99,0r96,-142r157,0","w":648},"\u00ca":{"d":"69,0r0,-714r534,0r0,132r-377,0r0,153r346,0r0,122r-346,0r0,175r385,0r0,132r-542,0xm149,-778r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":648},"\u00cb":{"d":"69,0r0,-714r534,0r0,132r-377,0r0,153r346,0r0,122r-346,0r0,175r385,0r0,132r-542,0xm360,-794r0,-117r135,0r0,117r-135,0xm154,-794r0,-117r135,0r0,117r-135,0","w":648},"\u00c8":{"d":"69,0r0,-714r534,0r0,132r-377,0r0,153r346,0r0,122r-346,0r0,175r385,0r0,132r-542,0xm301,-778r-154,-142r157,0r96,142r-99,0","w":648},"\u00cd":{"d":"69,0r0,-714r157,0r0,714r-157,0xm325,-920r-154,142r-99,0r96,-142r157,0","w":295},"\u00ce":{"d":"69,0r0,-714r157,0r0,714r-157,0xm-28,-778r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":295},"\u00cf":{"d":"69,0r0,-714r157,0r0,714r-157,0xm183,-794r0,-117r135,0r0,117r-135,0xm-23,-794r0,-117r135,0r0,117r-135,0","w":295},"\u00cc":{"d":"69,0r0,-714r157,0r0,714r-157,0xm124,-778r-154,-142r157,0r96,142r-99,0","w":295},"\u00d1":{"d":"69,0r0,-714r156,0r298,479r2,0r0,-479r147,0r0,714r-157,0r-297,-478r-2,0r0,478r-147,0xm313,-911v46,0,84,35,130,35v30,0,43,-16,42,-35r69,0v-16,52,-37,114,-112,114v-56,0,-102,-33,-152,-34v-27,0,-36,17,-42,42r-61,0v13,-58,38,-122,126,-122","w":741},"\u00d3":{"d":"38,-354v0,-212,130,-377,351,-377v221,0,351,165,351,377v0,206,-130,371,-351,371v-221,0,-351,-165,-351,-371xm195,-354v0,119,53,239,194,239v141,0,194,-120,194,-239v0,-125,-53,-245,-194,-245v-141,0,-194,120,-194,245xm567,-920r-154,142r-99,0r96,-142r157,0","w":778},"\u00d4":{"d":"38,-354v0,-212,130,-377,351,-377v221,0,351,165,351,377v0,206,-130,371,-351,371v-221,0,-351,-165,-351,-371xm195,-354v0,119,53,239,194,239v141,0,194,-120,194,-239v0,-125,-53,-245,-194,-245v-141,0,-194,120,-194,245xm214,-778r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":778},"\u00d6":{"d":"38,-354v0,-212,130,-377,351,-377v221,0,351,165,351,377v0,206,-130,371,-351,371v-221,0,-351,-165,-351,-371xm195,-354v0,119,53,239,194,239v141,0,194,-120,194,-239v0,-125,-53,-245,-194,-245v-141,0,-194,120,-194,245xm425,-794r0,-117r135,0r0,117r-135,0xm219,-794r0,-117r135,0r0,117r-135,0","w":778},"\u00d2":{"d":"38,-354v0,-212,130,-377,351,-377v221,0,351,165,351,377v0,206,-130,371,-351,371v-221,0,-351,-165,-351,-371xm195,-354v0,119,53,239,194,239v141,0,194,-120,194,-239v0,-125,-53,-245,-194,-245v-141,0,-194,120,-194,245xm366,-778r-154,-142r157,0r96,142r-99,0","w":778},"\u00d5":{"d":"38,-354v0,-212,130,-377,351,-377v221,0,351,165,351,377v0,206,-130,371,-351,371v-221,0,-351,-165,-351,-371xm195,-354v0,119,53,239,194,239v141,0,194,-120,194,-239v0,-125,-53,-245,-194,-245v-141,0,-194,120,-194,245xm332,-911v46,0,84,35,130,35v30,0,43,-16,42,-35r69,0v-16,52,-37,114,-112,114v-56,0,-102,-33,-152,-34v-27,0,-36,17,-42,42r-61,0v13,-58,38,-122,126,-122","w":778},"\u0160":{"d":"24,-237r152,0v0,95,74,132,158,132v55,0,139,-16,139,-89v0,-77,-107,-90,-212,-118v-106,-28,-213,-69,-213,-203v0,-146,138,-216,267,-216v149,0,286,65,286,230r-152,0v-5,-86,-66,-108,-141,-108v-50,0,-108,21,-108,81v0,55,34,62,213,108v52,13,212,46,212,208v0,131,-103,229,-297,229v-158,0,-306,-78,-304,-254xm500,-920r-110,142r-130,0r-111,-142r112,0r64,81r65,-81r110,0","w":649},"\u00da":{"d":"675,-714r0,444v0,193,-114,287,-305,287v-192,0,-304,-93,-304,-287r0,-444r157,0r0,444v0,78,20,155,147,155v112,0,148,-49,148,-155r0,-444r157,0xm548,-920r-154,142r-99,0r96,-142r157,0","w":741},"\u00db":{"d":"675,-714r0,444v0,193,-114,287,-305,287v-192,0,-304,-93,-304,-287r0,-444r157,0r0,444v0,78,20,155,147,155v112,0,148,-49,148,-155r0,-444r157,0xm195,-778r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":741},"\u00dc":{"d":"675,-714r0,444v0,193,-114,287,-305,287v-192,0,-304,-93,-304,-287r0,-444r157,0r0,444v0,78,20,155,147,155v112,0,148,-49,148,-155r0,-444r157,0xm406,-794r0,-117r135,0r0,117r-135,0xm200,-794r0,-117r135,0r0,117r-135,0","w":741},"\u00d9":{"d":"675,-714r0,444v0,193,-114,287,-305,287v-192,0,-304,-93,-304,-287r0,-444r157,0r0,444v0,78,20,155,147,155v112,0,148,-49,148,-155r0,-444r157,0xm347,-778r-154,-142r157,0r96,142r-99,0","w":741},"\u00dd":{"d":"254,0r0,-278r-263,-436r177,0r167,282r166,-282r175,0r-265,440r0,274r-157,0xm511,-920r-154,142r-99,0r96,-142r157,0","w":667,"k":{"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":111,".":111,"e":92,"\u00e9":92,"\u00ea":92,"\u00eb":92,"\u00e8":92,"o":92,"\u00f8":92,"\u0153":92,"\u00f3":92,"\u00f4":92,"\u00f6":92,"\u00f2":92,"\u00f5":92,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"u":74,"\u00fa":74,"\u00fb":74,"\u00fc":74,"\u00f9":74,":":80,";":68,"i":13,"\u00ed":13,"\u00ee":13,"\u00ef":13,"\u00ec":13,"p":74}},"\u0178":{"d":"254,0r0,-278r-263,-436r177,0r167,282r166,-282r175,0r-265,440r0,274r-157,0xm369,-794r0,-117r135,0r0,117r-135,0xm163,-794r0,-117r135,0r0,117r-135,0","w":667,"k":{"v":55,"A":74,"\u00c6":74,"\u00c1":74,"\u00c2":74,"\u00c4":74,"\u00c0":74,"\u00c5":74,"\u00c3":74,",":111,".":111,"e":92,"\u00e9":92,"\u00ea":92,"\u00eb":92,"\u00e8":92,"o":92,"\u00f8":92,"\u0153":92,"\u00f3":92,"\u00f4":92,"\u00f6":92,"\u00f2":92,"\u00f5":92,"q":92,"-":111,"a":92,"\u00e6":92,"\u00e1":92,"\u00e2":92,"\u00e4":92,"\u00e0":92,"\u00e5":92,"\u00e3":92,"u":74,"\u00fa":74,"\u00fb":74,"\u00fc":74,"\u00f9":74,":":80,";":68,"i":13,"\u00ed":13,"\u00ee":13,"\u00ef":13,"\u00ec":13,"p":74}},"\u017d":{"d":"23,0r0,-124r383,-458r-353,0r0,-132r562,0r0,124r-383,458r393,0r0,132r-602,0xm500,-920r-110,142r-130,0r-111,-142r112,0r64,81r65,-81r110,0","w":648},"\u00e1":{"d":"376,-253v-53,42,-204,6,-202,106v0,53,41,66,87,66v111,0,115,-88,115,-119r0,-53xm190,-358r-142,0v8,-133,127,-173,243,-173v103,0,227,23,227,147r0,269v0,47,5,94,18,115r-144,0v-5,-16,-9,-33,-10,-50v-102,104,-350,90,-350,-91v0,-117,88,-144,176,-157v87,-12,168,-10,168,-68v0,-61,-42,-70,-92,-70v-54,0,-89,22,-94,78xm465,-723r-154,142r-99,0r96,-142r157,0","w":574},"\u00e2":{"d":"376,-253v-53,42,-204,6,-202,106v0,53,41,66,87,66v111,0,115,-88,115,-119r0,-53xm190,-358r-142,0v8,-133,127,-173,243,-173v103,0,227,23,227,147r0,269v0,47,5,94,18,115r-144,0v-5,-16,-9,-33,-10,-50v-102,104,-350,90,-350,-91v0,-117,88,-144,176,-157v87,-12,168,-10,168,-68v0,-61,-42,-70,-92,-70v-54,0,-89,22,-94,78xm112,-581r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":574},"\u00e4":{"d":"376,-253v-53,42,-204,6,-202,106v0,53,41,66,87,66v111,0,115,-88,115,-119r0,-53xm190,-358r-142,0v8,-133,127,-173,243,-173v103,0,227,23,227,147r0,269v0,47,5,94,18,115r-144,0v-5,-16,-9,-33,-10,-50v-102,104,-350,90,-350,-91v0,-117,88,-144,176,-157v87,-12,168,-10,168,-68v0,-61,-42,-70,-92,-70v-54,0,-89,22,-94,78xm323,-597r0,-117r135,0r0,117r-135,0xm117,-597r0,-117r135,0r0,117r-135,0","w":574},"\u00e0":{"d":"376,-253v-53,42,-204,6,-202,106v0,53,41,66,87,66v111,0,115,-88,115,-119r0,-53xm190,-358r-142,0v8,-133,127,-173,243,-173v103,0,227,23,227,147r0,269v0,47,5,94,18,115r-144,0v-5,-16,-9,-33,-10,-50v-102,104,-350,90,-350,-91v0,-117,88,-144,176,-157v87,-12,168,-10,168,-68v0,-61,-42,-70,-92,-70v-54,0,-89,22,-94,78xm264,-581r-154,-142r157,0r96,142r-99,0","w":574},"\u00e5":{"d":"376,-253v-53,42,-204,6,-202,106v0,53,41,66,87,66v111,0,115,-88,115,-119r0,-53xm190,-358r-142,0v8,-133,127,-173,243,-173v103,0,227,23,227,147r0,269v0,47,5,94,18,115r-144,0v-5,-16,-9,-33,-10,-50v-102,104,-350,90,-350,-91v0,-117,88,-144,176,-157v87,-12,168,-10,168,-68v0,-61,-42,-70,-92,-70v-54,0,-89,22,-94,78xm224,-656v0,39,25,70,63,70v38,0,64,-31,64,-70v0,-39,-26,-70,-64,-70v-38,0,-63,31,-63,70xm165,-656v0,-67,55,-122,122,-122v68,0,123,55,123,122v0,67,-55,122,-123,122v-67,0,-122,-55,-122,-122","w":574},"\u00e3":{"d":"376,-253v-53,42,-204,6,-202,106v0,53,41,66,87,66v111,0,115,-88,115,-119r0,-53xm190,-358r-142,0v8,-133,127,-173,243,-173v103,0,227,23,227,147r0,269v0,47,5,94,18,115r-144,0v-5,-16,-9,-33,-10,-50v-102,104,-350,90,-350,-91v0,-117,88,-144,176,-157v87,-12,168,-10,168,-68v0,-61,-42,-70,-92,-70v-54,0,-89,22,-94,78xm230,-714v46,0,84,35,130,35v30,0,43,-16,42,-35r69,0v-16,52,-37,114,-112,114v-56,0,-102,-33,-152,-34v-27,0,-36,17,-42,42r-61,0v13,-58,38,-122,126,-122","w":574},"\u00e7":{"d":"545,-335r-139,0v-9,-58,-46,-89,-105,-89v-91,0,-121,92,-121,168v0,74,29,163,118,163v66,0,104,-42,113,-105r137,0v-17,133,-107,207,-237,211r-28,40v52,-15,128,4,128,65v0,129,-165,107,-241,78r21,-48v29,13,122,41,124,-16v1,-41,-52,-34,-80,-25r-21,-24r50,-72v-136,-14,-226,-119,-226,-262v0,-161,96,-280,264,-280v122,0,234,64,243,196","w":574},"\u00e9":{"d":"544,-224r-373,0v4,90,48,131,127,131v57,0,103,-35,112,-67r125,0v-40,122,-125,174,-242,174v-163,0,-264,-112,-264,-272v0,-155,107,-273,264,-273v176,0,261,148,251,307xm171,-314r231,0v-13,-72,-44,-110,-113,-110v-90,0,-116,70,-118,110xm465,-723r-154,142r-99,0r96,-142r157,0","w":574},"\u00ea":{"d":"544,-224r-373,0v4,90,48,131,127,131v57,0,103,-35,112,-67r125,0v-40,122,-125,174,-242,174v-163,0,-264,-112,-264,-272v0,-155,107,-273,264,-273v176,0,261,148,251,307xm171,-314r231,0v-13,-72,-44,-110,-113,-110v-90,0,-116,70,-118,110xm112,-581r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":574},"\u00eb":{"d":"544,-224r-373,0v4,90,48,131,127,131v57,0,103,-35,112,-67r125,0v-40,122,-125,174,-242,174v-163,0,-264,-112,-264,-272v0,-155,107,-273,264,-273v176,0,261,148,251,307xm171,-314r231,0v-13,-72,-44,-110,-113,-110v-90,0,-116,70,-118,110xm323,-597r0,-117r135,0r0,117r-135,0xm117,-597r0,-117r135,0r0,117r-135,0","w":574},"\u00e8":{"d":"544,-224r-373,0v4,90,48,131,127,131v57,0,103,-35,112,-67r125,0v-40,122,-125,174,-242,174v-163,0,-264,-112,-264,-272v0,-155,107,-273,264,-273v176,0,261,148,251,307xm171,-314r231,0v-13,-72,-44,-110,-113,-110v-90,0,-116,70,-118,110xm264,-581r-154,-142r157,0r96,142r-99,0","w":574},"\u00ed":{"d":"58,0r0,-517r142,0r0,517r-142,0xm307,-723r-154,142r-99,0r96,-142r157,0","w":258},"\u00ee":{"d":"58,0r0,-517r142,0r0,517r-142,0xm-46,-581r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":258},"\u00ef":{"d":"58,0r0,-517r142,0r0,517r-142,0xm165,-597r0,-117r135,0r0,117r-135,0xm-41,-597r0,-117r135,0r0,117r-135,0","w":258},"\u00ec":{"d":"58,0r0,-517r142,0r0,517r-142,0xm106,-581r-154,-142r157,0r96,142r-99,0","w":258},"\u00f1":{"d":"54,0r0,-517r135,0r0,72r3,0v36,-58,98,-86,158,-86v151,0,189,85,189,213r0,318r-142,0r0,-292v0,-85,-25,-127,-91,-127v-77,0,-110,43,-110,148r0,271r-142,0xm239,-714v46,0,84,35,130,35v30,0,43,-16,42,-35r69,0v-16,52,-37,114,-112,114v-56,0,-102,-33,-152,-34v-27,0,-36,17,-42,42r-61,0v13,-58,38,-122,126,-122","w":593},"\u00f3":{"d":"38,-258v0,-165,105,-273,267,-273v163,0,268,108,268,273v0,164,-105,272,-268,272v-162,0,-267,-108,-267,-272xm180,-258v0,82,29,165,125,165v97,0,126,-83,126,-165v0,-83,-29,-166,-126,-166v-96,0,-125,83,-125,166xm483,-723r-154,142r-99,0r96,-142r157,0","w":611},"\u00f4":{"d":"38,-258v0,-165,105,-273,267,-273v163,0,268,108,268,273v0,164,-105,272,-268,272v-162,0,-267,-108,-267,-272xm180,-258v0,82,29,165,125,165v97,0,126,-83,126,-165v0,-83,-29,-166,-126,-166v-96,0,-125,83,-125,166xm130,-581r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":611},"\u00f6":{"d":"38,-258v0,-165,105,-273,267,-273v163,0,268,108,268,273v0,164,-105,272,-268,272v-162,0,-267,-108,-267,-272xm180,-258v0,82,29,165,125,165v97,0,126,-83,126,-165v0,-83,-29,-166,-126,-166v-96,0,-125,83,-125,166xm341,-597r0,-117r135,0r0,117r-135,0xm135,-597r0,-117r135,0r0,117r-135,0","w":611},"\u00f2":{"d":"38,-258v0,-165,105,-273,267,-273v163,0,268,108,268,273v0,164,-105,272,-268,272v-162,0,-267,-108,-267,-272xm180,-258v0,82,29,165,125,165v97,0,126,-83,126,-165v0,-83,-29,-166,-126,-166v-96,0,-125,83,-125,166xm282,-581r-154,-142r157,0r96,142r-99,0","w":611},"\u00f5":{"d":"38,-258v0,-165,105,-273,267,-273v163,0,268,108,268,273v0,164,-105,272,-268,272v-162,0,-267,-108,-267,-272xm180,-258v0,82,29,165,125,165v97,0,126,-83,126,-165v0,-83,-29,-166,-126,-166v-96,0,-125,83,-125,166xm248,-714v46,0,84,35,130,35v30,0,43,-16,42,-35r69,0v-16,52,-37,114,-112,114v-56,0,-102,-33,-152,-34v-27,0,-36,17,-42,42r-61,0v13,-58,38,-122,126,-122","w":611},"\u0161":{"d":"29,-168r135,0v1,62,53,87,109,87v41,0,93,-16,93,-66v0,-43,-59,-58,-161,-80v-82,-18,-164,-47,-164,-138v0,-132,114,-166,225,-166v113,0,217,38,228,165r-135,0v-4,-55,-46,-70,-97,-70v-32,0,-79,6,-79,48v0,51,80,58,161,77v83,19,164,49,164,145v0,136,-118,180,-236,180v-120,0,-237,-45,-243,-182xm444,-723r-110,142r-130,0r-111,-142r112,0r64,81r65,-81r110,0","w":537},"\u00fa":{"d":"539,-517r0,517r-135,0r0,-72r-3,0v-36,58,-98,86,-158,86v-151,0,-189,-85,-189,-213r0,-318r142,0r0,292v0,85,25,127,91,127v77,0,110,-43,110,-148r0,-271r142,0xm474,-723r-154,142r-99,0r96,-142r157,0","w":593},"\u00fb":{"d":"539,-517r0,517r-135,0r0,-72r-3,0v-36,58,-98,86,-158,86v-151,0,-189,-85,-189,-213r0,-318r142,0r0,292v0,85,25,127,91,127v77,0,110,-43,110,-148r0,-271r142,0xm121,-581r111,-142r130,0r110,142r-117,0r-62,-81r-64,81r-108,0","w":593},"\u00fc":{"d":"539,-517r0,517r-135,0r0,-72r-3,0v-36,58,-98,86,-158,86v-151,0,-189,-85,-189,-213r0,-318r142,0r0,292v0,85,25,127,91,127v77,0,110,-43,110,-148r0,-271r142,0xm332,-597r0,-117r135,0r0,117r-135,0xm126,-597r0,-117r135,0r0,117r-135,0","w":593},"\u00f9":{"d":"539,-517r0,517r-135,0r0,-72r-3,0v-36,58,-98,86,-158,86v-151,0,-189,-85,-189,-213r0,-318r142,0r0,292v0,85,25,127,91,127v77,0,110,-43,110,-148r0,-271r142,0xm273,-581r-154,-142r157,0r96,142r-99,0","w":593},"\u00fd":{"d":"525,-517r-216,582v-33,111,-133,128,-265,112r0,-117v27,2,55,7,83,5v49,-4,65,-56,49,-97r-182,-485r152,0r117,354r2,0r113,-354r147,0xm437,-723r-154,142r-99,0r96,-142r157,0","w":519,"k":{",":55,".":55}},"\u00ff":{"d":"525,-517r-216,582v-33,111,-133,128,-265,112r0,-117v27,2,55,7,83,5v49,-4,65,-56,49,-97r-182,-485r152,0r117,354r2,0r113,-354r147,0xm295,-597r0,-117r135,0r0,117r-135,0xm89,-597r0,-117r135,0r0,117r-135,0","w":519,"k":{",":55,".":55}},"\u017e":{"d":"22,0r0,-107r269,-303r-249,0r0,-107r436,0r0,107r-269,303r288,0r0,107r-475,0xm435,-723r-110,142r-130,0r-111,-142r112,0r64,81r65,-81r110,0","w":519},"\u2206":{"d":"27,0r0,-73r229,-644r148,0r222,642r0,75r-599,0xm155,-101r337,0r-109,-298v-16,-54,-46,-137,-57,-181r-3,0v-42,151,-114,331,-168,479","w":654},"\u2126":{"d":"173,-101r0,-3v-61,-58,-122,-156,-122,-284v0,-178,123,-324,303,-324v193,0,296,167,296,317v0,131,-61,234,-123,291r0,3r137,0r0,101r-261,0r0,-75v62,-41,121,-133,121,-293v0,-125,-59,-240,-170,-240v-107,0,-176,104,-176,245v0,145,58,249,122,288r0,75r-263,0r0,-101r136,0","w":704},"\u03bc":{"d":"539,-517r0,517r-135,0r0,-72r-3,0v-36,58,-98,86,-130,86v-40,0,-63,-11,-75,-24r0,192r-142,0r0,-699r142,0r0,292v0,85,25,127,91,127v77,0,110,-43,110,-148r0,-271r142,0","w":593},"\u03c0":{"d":"574,-420r-70,0v2,129,-8,322,13,420r-121,0v-31,-86,-12,-295,-17,-420r-139,0v-5,109,-29,315,-64,420r-122,0v34,-117,61,-309,63,-420v-48,0,-77,3,-98,10r-14,-84v25,-19,77,-33,162,-33r417,0","w":598},"\u20ac":{"d":"97,-455v28,-150,132,-266,297,-267v57,0,118,17,182,50r-61,125v-116,-77,-238,-59,-271,92r244,0r-31,72r-224,0r-1,42r213,0r-31,72r-174,0v30,183,180,196,302,95r-1,151v-46,27,-97,40,-154,40v-175,0,-270,-124,-294,-286r-77,0r31,-72r40,0v0,-21,0,-35,1,-42r-73,0r31,-72r51,0"},"\u2113":{"d":"441,-181r51,52v-52,102,-127,139,-206,139v-124,-1,-173,-78,-180,-177r-51,41r-32,-67v30,-25,57,-45,82,-69r0,-254v0,-178,81,-254,178,-254v96,0,138,79,138,177v0,119,-76,232,-196,349v-6,85,26,155,90,155v54,0,99,-47,126,-92xm225,-518r0,159v69,-79,118,-166,118,-238v0,-52,-14,-84,-55,-84v-30,0,-63,36,-63,163","w":510},"\u212e":{"d":"843,-340r-655,0v-3,0,-5,1,-5,4r0,199v0,8,3,16,8,22v64,66,151,108,248,108v104,0,197,-47,261,-121r58,0v-72,85,-190,141,-320,141v-223,0,-404,-162,-404,-363v0,-201,181,-364,404,-364v227,0,410,163,405,374xm694,-366r0,-199v0,-9,-3,-17,-10,-24v-63,-62,-149,-103,-245,-103v-96,0,-183,42,-247,107v-6,6,-9,15,-9,24r0,195v0,2,2,7,5,7r502,0v3,0,4,-5,4,-7","w":878},"\u2202":{"d":"103,-639r-35,-94v31,-23,89,-57,182,-57v145,0,273,127,273,382v0,250,-102,417,-279,417v-140,0,-206,-128,-206,-237v0,-162,97,-263,215,-263v83,0,131,55,148,80v11,-143,-57,-275,-167,-276v-61,0,-105,26,-131,48xm161,-224v0,72,37,130,96,129v75,0,122,-112,132,-211v-9,-32,-49,-85,-111,-85v-69,0,-117,79,-117,167","w":572},"\u220f":{"d":"692,-588r-101,0r0,685r-123,0r0,-685r-219,0r0,685r-123,0r0,-685r-101,0r0,-113r667,0r0,113","w":718},"\u2211":{"d":"536,97r-513,0r0,-80r242,-320r-233,-311r0,-87r488,0r0,105r-321,0r0,4r206,269r-229,301r0,3r360,0r0,116"},"\u2219":{"d":"55,-287v0,-47,39,-85,85,-85v45,0,84,39,84,85v0,42,-39,84,-83,84v-47,0,-86,-38,-86,-84","w":278},"\u221a":{"d":"598,-841r-224,983r-99,0r-149,-442r-74,29r-20,-70r168,-68r104,321v9,32,12,68,22,88v3,-22,7,-58,15,-94r171,-747r86,0","w":590},"\u221e":{"d":"575,-455v87,0,154,65,154,166v0,108,-78,172,-160,172v-58,0,-112,-26,-179,-113v-52,63,-104,113,-185,113v-83,0,-157,-68,-157,-167v0,-101,71,-171,165,-171v75,0,129,45,180,111v43,-48,93,-112,182,-111xm118,-284v0,54,41,99,97,99v56,0,99,-51,137,-96v-41,-56,-77,-108,-142,-108v-57,0,-92,46,-92,105xm659,-284v0,-61,-37,-105,-92,-105v-61,0,-107,65,-139,101v61,75,93,103,143,103v57,0,88,-55,88,-99","w":778},"\u222b":{"d":"323,-778v-79,0,-77,80,-77,195v0,163,14,304,14,483v0,96,-16,161,-36,201v-39,75,-133,86,-204,56r16,-89v22,11,69,17,87,-10v16,-25,27,-66,27,-157v0,-182,-15,-325,-15,-496v0,-117,19,-180,60,-226v39,-44,120,-57,177,-34r-14,85v-9,-4,-21,-8,-35,-8","w":394},"\u2248":{"d":"182,-453v87,0,125,73,193,73v38,0,63,-29,90,-72r44,40v-31,60,-78,103,-135,103v-66,0,-131,-72,-198,-73v-44,0,-69,36,-93,72r-45,-41v29,-58,83,-102,144,-102xm182,-267v90,0,121,73,193,73v39,0,62,-29,90,-72r44,40v-31,59,-77,103,-135,103v-66,0,-132,-73,-198,-74v-45,0,-69,36,-93,73r-45,-41v29,-58,83,-102,144,-102","w":549},"\u2260":{"d":"403,-514r-39,89r136,0r0,72r-163,0r-60,136r223,0r0,72r-250,0r-50,114r-57,-23r40,-91r-135,0r0,-72r163,0r59,-136r-222,0r0,-72r249,0r49,-112","w":549},"\u2264":{"d":"498,-100r-445,-223r0,-78r445,-223r0,84r-373,176r0,3r373,177r0,84xm502,12r-453,0r0,-74r453,0r0,74","w":549},"\u2265":{"d":"55,-624r445,223r0,78r-445,223r0,-84r372,-177r0,-2r-372,-177r0,-84xm500,12r-450,0r0,-74r450,0r0,74","w":549},"\u25ca":{"d":"540,-350r-198,399r-100,0r-192,-399r198,-400r99,0xm430,-347v-43,-101,-100,-190,-133,-301r-3,0v-31,109,-91,197,-134,295v43,101,100,191,132,302r4,0v34,-111,90,-196,134,-296","w":590},"\u00a0":{"w":278},"\u00ad":{"d":"53,-212r0,-122r301,0r0,122r-301,0","w":407},"\u02c9":{"d":"-57,-616r0,-75r373,0r0,75r-373,0","w":259},"\u03a9":{"d":"173,-101r0,-3v-61,-58,-122,-156,-122,-284v0,-178,123,-324,303,-324v193,0,296,167,296,317v0,131,-61,234,-123,291r0,3r137,0r0,101r-261,0r0,-75v62,-41,121,-133,121,-293v0,-125,-59,-240,-170,-240v-107,0,-176,104,-176,245v0,145,58,249,122,288r0,75r-263,0r0,-101r136,0","w":704},"\u2215":{"d":"-166,22r414,-744r85,0r-412,744r-87,0","w":167}}});
;
// $Id: jquery.bgiframe.min.js,v 1.1.2.1 2010/07/03 03:00:39 sociotech Exp $

/* Copyright (c) 2006 Brandon Aaron (http://brandonaaron.net)
 * Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) 
 * and GPL (http://www.opensource.org/licenses/gpl-license.php) licenses.
 *
 * $LastChangedDate: 2007-06-19 20:25:28 -0500 (Tue, 19 Jun 2007) $
 * $Rev: 2111 $
 *
 * Version 2.1
 */
;(function($){$.fn.bgIframe=$.fn.bgiframe=function(s){if($.browser.msie&&parseInt($.browser.version)<=6){s=$.extend({top:'auto',left:'auto',width:'auto',height:'auto',opacity:true,src:'javascript:false;'},s||{});var prop=function(n){return n&&n.constructor==Number?n+'px':n;},html='<iframe class="bgiframe"frameborder="0"tabindex="-1"src="'+s.src+'"'+'style="display:block;position:absolute;z-index:-1;'+(s.opacity!==false?'filter:Alpha(Opacity=\'0\');':'')+'top:'+(s.top=='auto'?'expression(((parseInt(this.parentNode.currentStyle.borderTopWidth)||0)*-1)+\'px\')':prop(s.top))+';'+'left:'+(s.left=='auto'?'expression(((parseInt(this.parentNode.currentStyle.borderLeftWidth)||0)*-1)+\'px\')':prop(s.left))+';'+'width:'+(s.width=='auto'?'expression(this.parentNode.offsetWidth+\'px\')':prop(s.width))+';'+'height:'+(s.height=='auto'?'expression(this.parentNode.offsetHeight+\'px\')':prop(s.height))+';'+'"/>';return this.each(function(){if($('> iframe.bgiframe',this).length==0)this.insertBefore(document.createElement(html),this.firstChild);});}return this;};if(!$.browser.version)$.browser.version=navigator.userAgent.toLowerCase().match(/.+(?:rv|it|ra|ie)[\/: ]([\d.]+)/)[1];})(jQuery);;
// $Id: hoverIntent.js,v 1.1.2.1 2010/07/03 03:00:39 sociotech Exp $

;(function($){
	/* hoverIntent by Brian Cherne */
	$.fn.hoverIntent = function(f,g) {
		// default configuration options
		var cfg = {
			sensitivity: 7,
			interval: 100,
			timeout: 0
		};
		// override configuration options with user supplied object
		cfg = $.extend(cfg, g ? { over: f, out: g } : f );

		// instantiate variables
		// cX, cY = current X and Y position of mouse, updated by mousemove event
		// pX, pY = previous X and Y position of mouse, set by mouseover and polling interval
		var cX, cY, pX, pY;

		// A private function for getting mouse position
		var track = function(ev) {
			cX = ev.pageX;
			cY = ev.pageY;
		};

		// A private function for comparing current and previous mouse position
		var compare = function(ev,ob) {
			ob.hoverIntent_t = clearTimeout(ob.hoverIntent_t);
			// compare mouse positions to see if they've crossed the threshold
			if ( ( Math.abs(pX-cX) + Math.abs(pY-cY) ) < cfg.sensitivity ) {
				$(ob).unbind("mousemove",track);
				// set hoverIntent state to true (so mouseOut can be called)
				ob.hoverIntent_s = 1;
				return cfg.over.apply(ob,[ev]);
			} else {
				// set previous coordinates for next time
				pX = cX; pY = cY;
				// use self-calling timeout, guarantees intervals are spaced out properly (avoids JavaScript timer bugs)
				ob.hoverIntent_t = setTimeout( function(){compare(ev, ob);} , cfg.interval );
			}
		};

		// A private function for delaying the mouseOut function
		var delay = function(ev,ob) {
			ob.hoverIntent_t = clearTimeout(ob.hoverIntent_t);
			ob.hoverIntent_s = 0;
			return cfg.out.apply(ob,[ev]);
		};

		// A private function for handling mouse 'hovering'
		var handleHover = function(e) {
			// next three lines copied from jQuery.hover, ignore children onMouseOver/onMouseOut
			var p = (e.type == "mouseover" ? e.fromElement : e.toElement) || e.relatedTarget;
			while ( p && p != this ) { try { p = p.parentNode; } catch(e) { p = this; } }
			if ( p == this ) { return false; }

			// copy objects to be passed into t (required for event object to be passed in IE)
			var ev = jQuery.extend({},e);
			var ob = this;

			// cancel hoverIntent timer if it exists
			if (ob.hoverIntent_t) { ob.hoverIntent_t = clearTimeout(ob.hoverIntent_t); }

			// else e.type == "onmouseover"
			if (e.type == "mouseover") {
				// set "previous" X and Y position based on initial entry point
				pX = ev.pageX; pY = ev.pageY;
				// update "current" X and Y position based on mousemove
				$(ob).bind("mousemove",track);
				// start polling interval (self-calling timeout) to compare mouse coordinates over time
				if (ob.hoverIntent_s != 1) { ob.hoverIntent_t = setTimeout( function(){compare(ev,ob);} , cfg.interval );}

			// else e.type == "onmouseout"
			} else {
				// unbind expensive mousemove event
				$(ob).unbind("mousemove",track);
				// if hoverIntent state is true, then call the mouseOut function after the specified delay
				if (ob.hoverIntent_s == 1) { ob.hoverIntent_t = setTimeout( function(){delay(ev,ob);} , cfg.timeout );}
			}
		};

		// bind the function to the two event listeners
		return this.mouseover(handleHover).mouseout(handleHover);
	};
	
})(jQuery);;
// $Id: supposition.js,v 1.1 2009/08/19 04:28:07 sociotech Exp $

/*
 * Supposition v0.2 - an optional enhancer for Superfish jQuery menu widget
 *
 * Copyright (c) 2008 Joel Birch - based mostly on work by Jesse Klaasse and credit goes largely to him.
 * Special thanks to Karl Swedberg for valuable input.
 *
 * Dual licensed under the MIT and GPL licenses:
 * 	http://www.opensource.org/licenses/mit-license.php
 * 	http://www.gnu.org/licenses/gpl.html
 */

(function($){

	$.fn.supposition = function(){
		var $w = $(window), /*do this once instead of every onBeforeShow call*/
			_offset = function(dir) {
				return window[dir == 'y' ? 'pageYOffset' : 'pageXOffset']
				|| document.documentElement && document.documentElement[dir=='y' ? 'scrollTop' : 'scrollLeft']
			    || document.body[dir=='y' ? 'scrollTop' : 'scrollLeft'];
			},
			onInit = function(){
				/* I haven't touched this bit - needs work as there are still z-index issues */
				$topNav = $('li',this);
				var cZ=parseInt($topNav.css('z-index')) + $topNav.length;
				$topNav.each(function() {
					$(this).css({zIndex:--cZ});
				});
			},
			onHide = function(){
				this.css({marginTop:'',marginLeft:''});
			},
			onBeforeShow = function(){
				this.each(function(){
					var $u = $(this);
					$u.css('display','block');
					var menuWidth = $u.width(),
						parentWidth = $u.parents('ul').width(),
						totalRight = $w.width() + _offset('x'),
						menuRight = $u.offset().left + menuWidth;
					if (menuRight > totalRight) {
						$u.css('margin-left', ($u.parents('ul').length == 1 ? totalRight - menuRight : -(menuWidth + parentWidth)) + 'px');
					}

					var windowHeight = $w.height(),
						offsetTop = $u.offset().top,
						menuHeight = $u.height(),
						baseline = windowHeight + _offset('y');
					var expandUp = (offsetTop + menuHeight > baseline);
					if (expandUp) {
						$u.css('margin-top',baseline - (menuHeight + offsetTop));
					}
					$u.css('display','none');
				});
			};
		
		return this.each(function() {
			var o = $.fn.superfish.o[this.serial]; /* get this menu's options */
			
			/* if callbacks already set, store them */
			var _onInit = o.onInit,
				_onBeforeShow = o.onBeforeShow,
				_onHide = o.onHide;
				
			$.extend($.fn.superfish.o[this.serial],{
				onInit		: function() {
					onInit.call(this); /* fire our Supposition callback */
					_onInit.call(this); /* fire stored callbacks */
				},
				onBeforeShow: function() {
					onBeforeShow.call(this); /* fire our Supposition callback */
					_onBeforeShow.call(this); /* fire stored callbacks */
				},
				onHide		: function() {
					onHide.call(this); /* fire our Supposition callback */
					_onHide.call(this); /* fire stored callbacks */
				}
			});
		});
	};

})(jQuery);;
// $Id: supersubs.js,v 1.1 2009/08/19 04:28:07 sociotech Exp $

/*
 * Supersubs v0.2b - jQuery plugin
 * Copyright (c) 2008 Joel Birch
 *
 * Dual licensed under the MIT and GPL licenses:
 * 	http://www.opensource.org/licenses/mit-license.php
 * 	http://www.gnu.org/licenses/gpl.html
 *
 *
 * This plugin automatically adjusts submenu widths of suckerfish-style menus to that of
 * their longest list item children. If you use this, please expect bugs and report them
 * to the jQuery Google Group with the word 'Superfish' in the subject line.
 *
 */

;(function($){ // $ will refer to jQuery within this closure

	$.fn.supersubs = function(options){
		var opts = $.extend({}, $.fn.supersubs.defaults, options);
		// return original object to support chaining
		return this.each(function() {
			// cache selections
			var $$ = $(this);
			// support metadata
			var o = $.meta ? $.extend({}, opts, $$.data()) : opts;
			// get the font size of menu.
			// .css('fontSize') returns various results cross-browser, so measure an em dash instead
			var fontsize = $('<li id="menu-fontsize">&#8212;</li>').css({
				'padding' : 0,
				'position' : 'absolute',
				'top' : '-999em',
				'width' : 'auto'
			}).appendTo($$).width(); //clientWidth is faster, but was incorrect here
			// remove em dash
			$('#menu-fontsize').remove();
			// cache all ul elements
			$ULs = $$.find('ul');
			// loop through each ul in menu
			$ULs.each(function(i) {	
				// cache this ul
				var $ul = $ULs.eq(i);
				// get all (li) children of this ul
				var $LIs = $ul.children();
				// get all anchor grand-children
				var $As = $LIs.children('a');
				// force content to one line and save current float property
				var liFloat = $LIs.css('white-space','nowrap').css('float');
				// remove width restrictions and floats so elements remain vertically stacked
				var emWidth = $ul.add($LIs).add($As).css({
					'float' : 'none',
					'width'	: 'auto'
				})
				// this ul will now be shrink-wrapped to longest li due to position:absolute
				// so save its width as ems. Clientwidth is 2 times faster than .width() - thanks Dan Switzer
				.end().end()[0].clientWidth / fontsize;
				// add more width to ensure lines don't turn over at certain sizes in various browsers
				emWidth += o.extraWidth;
				// restrict to at least minWidth and at most maxWidth
				if (emWidth > o.maxWidth)		{ emWidth = o.maxWidth; }
				else if (emWidth < o.minWidth)	{ emWidth = o.minWidth; }
				emWidth += 'em';
				// set ul to width in ems
				$ul.css('width',emWidth);
				// restore li floats to avoid IE bugs
				// set li width to full width of this ul
				// revert white-space to normal
				$LIs.css({
					'float' : liFloat,
					'width' : '100%',
					'white-space' : 'normal'
				})
				// update offset position of descendant ul to reflect new width of parent
				.each(function(){
					var $childUl = $('>ul',this);
					var offsetDirection = $childUl.css('left')!==undefined ? 'left' : 'right';
					$childUl.css(offsetDirection,emWidth);
				});
			});
			
		});
	};
	// expose defaults
	$.fn.supersubs.defaults = {
		minWidth		: 9,		// requires em unit.
		maxWidth		: 25,		// requires em unit.
		extraWidth		: 0			// extra width can ensure lines don't sometimes turn over due to slight browser differences in how they round-off values
	};
	
})(jQuery); // plugin code ends
;
// $Id: superfish.js,v 1.1 2009/08/19 04:28:07 sociotech Exp $

/*
 * Superfish v1.4.8 - jQuery menu widget
 * Copyright (c) 2008 Joel Birch
 *
 * Dual licensed under the MIT and GPL licenses:
 * 	http://www.opensource.org/licenses/mit-license.php
 * 	http://www.gnu.org/licenses/gpl.html
 *
 * CHANGELOG: http://users.tpg.com.au/j_birch/plugins/superfish/changelog.txt
 */

;(function($){
	$.fn.superfish = function(op){

		var sf = $.fn.superfish,
			c = sf.c,
			$arrow = $(['<span class="',c.arrowClass,'"> &#187;</span>'].join('')),
			over = function(){
				var $$ = $(this), menu = getMenu($$);
				clearTimeout(menu.sfTimer);
				$$.showSuperfishUl().siblings().hideSuperfishUl();
			},
			out = function(){
				var $$ = $(this), menu = getMenu($$), o = sf.op;
				clearTimeout(menu.sfTimer);
				menu.sfTimer=setTimeout(function(){
					o.retainPath=($.inArray($$[0],o.$path)>-1);
					$$.hideSuperfishUl();
					if (o.$path.length && $$.parents(['li.',o.hoverClass].join('')).length<1){over.call(o.$path);}
				},o.delay);	
			},
			getMenu = function($menu){
				var menu = $menu.parents(['ul.',c.menuClass,':first'].join(''))[0];
				sf.op = sf.o[menu.serial];
				return menu;
			},
			addArrow = function($a){ $a.addClass(c.anchorClass).append($arrow.clone()); };
			
		return this.each(function() {
			var s = this.serial = sf.o.length;
			var o = $.extend({},sf.defaults,op);
			o.$path = $('li.'+o.pathClass,this).slice(0,o.pathLevels).each(function(){
				$(this).addClass([o.hoverClass,c.bcClass].join(' '))
					.filter('li:has(ul)').removeClass(o.pathClass);
			});
			sf.o[s] = sf.op = o;
			
			$('li:has(ul)',this)[($.fn.hoverIntent && !o.disableHI) ? 'hoverIntent' : 'hover'](over,out).each(function() {
				if (o.autoArrows) addArrow( $('>a:first-child',this) );
			})
			.not('.'+c.bcClass)
				.hideSuperfishUl();
			
			var $a = $('a',this);
			$a.each(function(i){
				var $li = $a.eq(i).parents('li');
				$a.eq(i).focus(function(){over.call($li);}).blur(function(){out.call($li);});
			});
			o.onInit.call(this);
			
		}).each(function() {
			menuClasses = [c.menuClass];
			if (sf.op.dropShadows  && !($.browser.msie && $.browser.version < 7)) menuClasses.push(c.shadowClass);
			$(this).addClass(menuClasses.join(' '));
		});
	};

	var sf = $.fn.superfish;
	sf.o = [];
	sf.op = {};
	sf.IE7fix = function(){
		var o = sf.op;
		if ($.browser.msie && $.browser.version > 6 && o.dropShadows && o.animation.opacity!=undefined)
			this.toggleClass(sf.c.shadowClass+'-off');
		};
	sf.c = {
		bcClass     : 'sf-breadcrumb',
		menuClass   : 'sf-js-enabled',
		anchorClass : 'sf-with-ul',
		arrowClass  : 'sf-sub-indicator',
		shadowClass : 'sf-shadow'
	};
	sf.defaults = {
		hoverClass	: 'sfHover',
		pathClass	: 'overideThisToUse',
		pathLevels	: 1,
		delay		: 800,
		animation	: {opacity:'show'},
		speed		: 'normal',
		autoArrows	: true,
		dropShadows : true,
		disableHI	: false,		// true disables hoverIntent detection
		onInit		: function(){}, // callback functions
		onBeforeShow: function(){},
		onShow		: function(){},
		onHide		: function(){}
	};
	$.fn.extend({
		hideSuperfishUl : function(){
			var o = sf.op,
				not = (o.retainPath===true) ? o.$path : '';
			o.retainPath = false;
			var $ul = $(['li.',o.hoverClass].join(''),this).add(this).not(not).removeClass(o.hoverClass)
					.find('>ul').hide().css('visibility','hidden');
			o.onHide.call($ul);
			return this;
		},
		showSuperfishUl : function(){
			var o = sf.op,
				sh = sf.c.shadowClass+'-off',
				$ul = this.addClass(o.hoverClass)
					.find('>ul:hidden').css('visibility','visible');
			sf.IE7fix.call($ul);
			o.onBeforeShow.call($ul);
			$ul.animate(o.animation,o.speed,function(){ sf.IE7fix.call($ul); o.onShow.call($ul); });
			return this;
		}
	});

})(jQuery);
;
var timeout = 1000;
var closetimer = ddmenuitem = 0;

jQuery(document).ready(function(){
	var pageTitle_0 = jQuery.url.segment(0);
	var pageTitle = jQuery.url.segment(1);
	
	delayShow();
	jQuery.cookie('visited', 'yes');	
	jQuery.cookie('filter-success', null);
	
	jQuery('.page-news-press #block-menu-menu-categories ul.menu li a').each(function () {
		var cate_id = "pid-news-press-"+jQuery(this).text().toLowerCase().replace(' ', '-');
		jQuery('#'+cate_id+' h1.title').css({'cursor':'pointer'});
		jQuery('#'+cate_id+' h1.title').click(function (){
			window.location = '/news-press/all';
		});
	});
	
	jQuery('.node-type-news h1.title').css({'cursor':'pointer'});
	jQuery('.node-type-news h1.title').click(function (){
		window.location = '/news-press/all';
	});
	
	if (jQuery.url.segment(2) == 'dealer-area') {
		jQuery('#collapser').jqcollapse({
		   slide: true,
		   speed: 1000,
		   easing: 'easeinout'
		});
	}
	
	// browser ie6 message
	var sBrowser = navigator.userAgent;	
	if ((sBrowser.toLowerCase().indexOf('msie 6') > -1) ) {
		
		var contentHeight = jQuery('#unsupport-browser-cover').height();
		var winHeight = jQuery(window).height(); 
		var winWidth = jQuery(window).width();
		jQuery('#unsupport-browser-cover').css({'margin-top':'-'+parseInt(contentHeight/2)+'px'});
		jQuery('#unsupport-browser-cover-bg').css({'width':winWidth+'px','height':winHeight+'px'});
		
	} else {
		jQuery('#unsupport-browser-cover, #unsupport-browser-cover-bg').hide();
	}
	
	/* Search box */
	jQuery('#edit-search-theme-form-header').attr('title', 'Search');
	jQuery('#edit-search-theme-form-header').hint();
	
	
	/* Tooltip 
	jQuery('#header-group-wrapper #block-menu-menu-product-menu ul li').hover( function () {
		var index = jQuery('#header-group-wrapper #block-menu-menu-product-menu ul li').index(this);
		
		switch (index) {
			case 0:
				var div = 'tool-tip.edifier-image';
				break;
			case 1:
				var div = 'tool-tip.edifier-multimedia';
				break;
			case 2:
				var div = 'tool-tip.edifier-studio';
				break;
		}
		
		jQuery('.'+div).fadeIn(300);
	}, function () {
		jQuery('.tool-tip').fadeOut(50);
	});
	*/
	
		
	/* set free_region Div width */
	var win_width = jQuery(window).width();
	jQuery('#free_region').css({'width':win_width+'px', 'margin-left':'-'+parseFloat(win_width/2)+'px'});
	jQuery('.page-home #block-views-product_list-block_1 .view-product-list .view-content').css({'width':win_width+'px', 'margin-left':'-'+parseInt((win_width)/2)+'px'});
	
	// IE 7 Homepage slider
	var sBrowser = navigator.userAgent;	
	if ((sBrowser.toLowerCase().indexOf('msie 7.0') > -1) ) {
		jQuery("html").css("overflow-x", "hidden");
		jQuery('#free_region').css({'width':win_width+'px', 'overflow':'hidden'});
	}
	
	/* Homepage Slider */
	jQuery('.view-home-slider .view-content').before('<a href="#" id="prev"></a><a href="#" id="next"></a><div id="nav">').cycle({
		fx: 'scrollHorz',
		timeout: 7500,
		speed: 1000,
		easeIn:  'easeout', 
		easeOut: 'easein',
		pager: '#nav',
		next: '#next',
		prev: '#prev'
	});
	
	jQuery('#free_region #block-views-home_slider-block_1').hover( function() {
		jQuery('#prev, #next').fadeIn();
	}, function () {
		jQuery('#prev, #next').fadeOut();
	});
	
	
	jQuery(window).resize(function() {
		/* set free_region Div width */
		var win_width = jQuery(window).width();

		jQuery('#free_region').css({'width':win_width+'px', 'margin-left':'-'+parseFloat(win_width/2)+'px'});

		jQuery('.page-home #block-views-product_list-block_1 .view-product-list .view-content').css({'width':win_width+'px', 'margin-left':'-'+parseFloat((win_width)/2)+'px'});	
	});
	
	
	/* Make Bottom Block be clickable */
	jQuery('.bottom-block').click(function () {
		window.location = jQuery(this).attr('title');
	});
	
	/* Product Detail page */	
	if ((pageTitle == 'edifier-image') || (pageTitle == 'edifier-multimedia') || (pageTitle == 'edifier-studio')) {
		productDetail_overview();
		
		/* Set pre-loading img */
		if (jQuery.cookie('visited') != 'yes') {
			jQuery('.node-type-product ul.quicktabs_tabs').addClass('not-overview');
			jQuery('.node-type-product #main').css({'background':'transparent url(/sites/all/themes/edifier/images/109.gif) no-repeat center 300px', 'width':jQuery(window).width()+'px'});
			jQuery('.view-product-details .view-content .views-row-1 img').load( function () {
				jQuery('.node-type-product ul.quicktabs_tabs').removeClass('not-overview');
				var img_path = jQuery('.view-product-details .view-content .views-row img.imagefield').attr('src');
				jQuery('.node-type-product #main').css({'background':'transparent url('+img_path+') no-repeat center 26px', 'width':jQuery(window).width()+'px'});
			});
		}
		
		/* show the category indicator */
		switch(pageTitle) {
			case 'edifier-image':
				jQuery('#block-menu-menu-product-menu ul.menu li.leaf.first a').addClass('active');
				jQuery('.node-type-product #main #main-inner').css({'background':'transparent url(/sites/all/themes/edifier/images/icon_indicator.png) no-repeat 591px -1px'});
				break;
			case 'edifier-multimedia':
				jQuery('#block-menu-menu-product-menu ul.menu li.leaf a').addClass('active');
				jQuery('#block-menu-menu-product-menu ul.menu li.leaf.first a,#block-menu-menu-product-menu ul.menu li.leaf.last a').removeClass('active');
				jQuery('.node-type-product #main #main-inner').css({'background':'transparent url(/sites/all/themes/edifier/images/icon_indicator.png) no-repeat 750px -1px'});
				break;
			case 'edifier-studio':
				jQuery('#block-menu-menu-product-menu ul.menu li.leaf.last a').addClass('active');
				jQuery('.node-type-product #main #main-inner').css({'background':'transparent url(/sites/all/themes/edifier/images/icon_indicator.png) no-repeat 908px -1px'});
				break;
		}
		
		
		/* Overview Tab */
		jQuery('.node-type-product #quicktabs-tab-1-0').click( function() {
			productDetail_overview();
		});
		
		/* Gallery Tab */
		jQuery('.node-type-product #quicktabs-tab-1-1, .node-type-product #quicktabs-tab-1-2, .node-type-product #quicktabs-tab-1-3, .node-type-product #quicktabs-tab-1-4, .node-type-product #quicktabs-tab-1-5').click( function() {
			productDetail_other();
		});
	}
	
	
	
	/* Static pages */
	if ((pageTitle_0 != '') || (pageTitle_0 != 'products')) {
		jQuery('.page-about #main #main-inner, #pid-about-our-passion-for-sound #main #main-inner, #pid-about-overview #main #main-inner, #pid-terms-of-use #main #main-inner, #pid-contact #main #main-inner, #pid-find-a-region #main-inner, .page-user #main-inner, .page-news-press #main-inner, .node-type-news #main-inner').css({'width':'960px', 'margin':'0 0 0 -470px', 'position':'relative', 'left':'50%', 'min-height':'500px'});
		
		jQuery('.page-customerror #main-inner').css({'width':'960px', 'margin':'0 0 0 -470px', 'position':'relative', 'left':'50%', 'min-height':'350px'});
		
		jQuery('#sidebar-last .leftDiv a.email').after('<a class="feed-icon-right" href="/feed/news-press/all" target="_blank">RSS</a>');	
	}
	
	jQuery('.page-user #content-tabs #content-tabs-inner ul').before("<h2 class='title block-title'>Dealer's area</h2>");
	
	/* Award page - get the title of the image and stick it the the href attribute of the outter <A> tag */
	jQuery('.imagefield-field_award_logo').each(function(){
		var link = jQuery(this).attr('title');
		if(link.length > 0){
			jQuery(this).wrap('<a target="_blank" href="' + link + '" />');
		}
	});
	
	
	/* search page, move pager and search form */
	var pager = jQuery('.page-search .box .item-list').html();
	jQuery('.page-search .box .item-list').remove();
	if(pager == null) pager = ''; 
	jQuery('.page-search #main').append('<div class="item-list pager-container">' + pager + '</div>');
	jQuery('.page-search .pager').css('display','block');
	
	var searchForm = jQuery('.page-search #search-form').html();
	jQuery('.page-search #search-form').remove();
	jQuery('.page-search #main').append('<div class="search-form-container"><form class="search-form" id="search-form" method="post" accept-charset="UTF-8" action="/search/apachesolr_search">' + searchForm + '</form></div><div style="clear:both;"></div>');
	jQuery('.page-search #search-form').css('display','block');
	
	
	
	var pane = jQuery('.scroll-pane-split');
	var settings = {
			showArrows: true,
			horizontalGutter: 0,
			enableKeyboardNavigation: true
		};
	pane.jScrollPane(settings);				
	var api = pane.data('jsp');
	
	
	
	
	/* Tooltip */
	jQuery('.tool-tip').css({'cursor':'pointer'});
	jQuery('.tool-tip.edifier-image').click(function () {
		window.location = '/products/edifier-image';
	});
	jQuery('.tool-tip.edifier-multimedia').click(function () {
		window.location = '/products/edifier-multimedia';
	});
	jQuery('.tool-tip.edifier-studio').click(function () {
		window.location = '/products/edifier-studio';
	});
	
	jQuery('#header-group-wrapper #block-menu-menu-product-menu ul li').mouseenter( function () {
		jQuery('.tool-tip').fadeOut(300);
		var index = jQuery('#header-group-wrapper #block-menu-menu-product-menu ul li').index(this);
		
		switch (index) {
			case 0:
				var div = 'tool-tip.edifier-image';
				break;
			case 1:
				var div = 'tool-tip.edifier-multimedia';
				break;
			case 2:
				var div = 'tool-tip.edifier-studio';
				break;
		}
		
		jQuery('.'+div).fadeIn(300);
		
		// cancel close timer
		mcancelclosetime();
	});
	
	jQuery('#header-group-wrapper #block-menu-menu-product-menu ul li').mouseleave(function () {
		closetimer = window.setTimeout(mclose, timeout);
	});
	
	jQuery('.tool-tip').mouseenter(function () {
		mcancelclosetime();
	});
	
	jQuery('.tool-tip').mouseleave(function () {
		jQuery('.tool-tip').fadeOut(300);
		mclosetime();
	});
	
});


// close showed layer
function mclose() {
	jQuery('.tool-tip').fadeOut(300);
}

// go close timer
function mclosetime() {
	closetimer = window.setTimeout(mclose, timeout);
}

// cancel close timer
function mcancelclosetime() {
	if(closetimer) {
		window.clearTimeout(closetimer);
		closetimer = null;
	}
}



function delayShow() {
   	var secs = 400;
	setTimeout('jQuery("#page, #free_region").css("visibility","visible");jQuery("#page, #free_region").fadeIn()', secs);
}

function productDetail_overview() {	
	jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1').removeClass('other-tabs');
	jQuery('.node-type-product ul.quicktabs_tabs, .node-type-product #content-group-inner .content-inner .content-inner-inner').removeClass('not-overview');
	var img_path = jQuery('.view-product-details .view-content .views-row img.imagefield').attr('src'); // get the image path
	
	var img_height = jQuery('.views-field-field-main-image-fid img.imagefield').attr('height'); // get the image height
	jQuery('.views-field-field-main-image-fid').hide();
	
	var layout = jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-field-overview-layout-value .field-content').text();
	
	jQuery(window).resize(function() {
		jQuery('.node-type-product #main').css({'width':jQuery(window).width()+'px'});
	});

	jQuery('.node-type-product #main #main-inner').css({'width':'960px', 'margin':'0 0 0 -470px', 'position':'relative', 'left':'50%'});
	
	//Create Awards logo link function
	jQuery('.feature-awards').each(function () {
		var awards_url = jQuery(this).attr('title');
		
		if (awards_url != '') {
			jQuery(this).css({'cursor':'pointer'});
			jQuery(this).click(function () {
				window.open(awards_url);
			});
		}
	});
	
	if (layout == 'Option B') {	
		var sBrowser = navigator.userAgent;	
		if ((sBrowser.toLowerCase().indexOf('firefox') != -1) || (sBrowser.toLowerCase().indexOf('msie') > -1) ) {
			var p_width = '211px';
		} else {
			var p_width = '239px';
		}
		
		if (sBrowser.toLowerCase().indexOf('msie') > -1) {
			jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 #quicktabs_tabpage_1_0 .view-display-id-panel_pane_1 .view-content .views-row').css('width','960px');
		}
		
		jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-body .field-content p').css({'display':'table','float': 'left','width':p_width,'padding-right':'27px','margin':'0px'});
		
		jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-tid, .node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-tid-1').css({'margin-top':'20px'});	
		
		var headline_height = jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-field-headline-value').height();
		var body_height = jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-body .field-content p').height();
		var body_p = parseInt(jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-body .field-content p').length*240);
		
		//set Feature/Awards icons position
		jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-tid,.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 .view-content .views-field-tid-1').css({'margin-left':body_p+'px'});
		
		body_height = (body_height > 200) ? body_height : 200;
		
		var new_body_height = parseInt(49+img_height+headline_height+body_height);		
		jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1 #quicktabs_tabpage_1_0 .view-content').css({'margin-top':parseInt(img_height-46)+'px', 'width':'479px'});	
		jQuery('.node-type-product #main').css({'background':'transparent url('+img_path+') no-repeat center 49px', 'width':jQuery(window).width()+'px'});
		jQuery('.node-type-product #main #main-inner .content-group-inner').css({'height':parseInt(new_body_height)+'px', 'padding-bottom':'30px', 'position':'relative'});
	} else if (layout == 'Option C') {			
		jQuery('.node-type-product #main').css({'background':'transparent url('+img_path+') no-repeat center 49px', 'width':jQuery(window).width()+'px', 'height':parseInt(img_height+167)+'px'});
		jQuery('.node-type-product #main #main-inner .content-group-inner').css({'height':parseInt(img_height)+'px', 'padding-bottom':'81px', 'position':'relative'});		
	} else{
		jQuery('.node-type-product #main').css({'background':'transparent url('+img_path+') no-repeat center 30px', 'width':jQuery(window).width()+'px'});
		jQuery('.node-type-product #main #main-inner .content-group-inner').css({'height':parseInt(img_height)+'px', 'padding-bottom':'108px', 'position':'relative'}); 
	}
}

function productDetail_other() {	
	jQuery('.node-type-product #main-wrapper').css({'background':'transparent url(/sites/all/themes/edifier/images/bg_product-detail.jpg) repeat-x center top'});
	jQuery('.node-type-product #main').css({'background':'transparent none', 'width':jQuery(window).width()-60+'px', 'height':'100%', 'padding-bottom':'10px'});
	jQuery('.node-type-product #main #main-inner .content-group-inner').css({'height':'auto', 'padding-bottom':'67px'});
	
	jQuery('.node-type-product #content-group-inner .content-inner .content-inner-inner #quicktabs_container_1').addClass('other-tabs');
	jQuery('.node-type-product ul.quicktabs_tabs, .node-type-product #content-group-inner .content-inner .content-inner-inner').addClass('not-overview');
}



/* facebook share link */
function fbs_click() {
u=location.href;
t=document.title;
window.open('http://www.facebook.com/sharer.php?u='+encodeURIComponent(u)+'&t='+encodeURIComponent(t),'sharer','toolbar=0,status=0,width=626,height=436');
return false;
};
/*
CSS Browser Selector v0.4.0 (Nov 02, 2010)
Rafael Lima (http://rafael.adm.br)
http://rafael.adm.br/css_browser_selector
License: http://creativecommons.org/licenses/by/2.5/
Contributors: http://rafael.adm.br/css_browser_selector#contributors
*/
function css_browser_selector(u){var ua=u.toLowerCase(),is=function(t){return ua.indexOf(t)>-1},g='gecko',w='webkit',s='safari',o='opera',m='mobile',h=document.documentElement,b=[(!(/opera|webtv/i.test(ua))&&/msie\s(\d)/.test(ua))?('ie ie'+RegExp.$1):is('firefox/2')?g+' ff2':is('firefox/3.5')?g+' ff3 ff3_5':is('firefox/3.6')?g+' ff3 ff3_6':is('firefox/3')?g+' ff3':is('gecko/')?g:is('opera')?o+(/version\/(\d+)/.test(ua)?' '+o+RegExp.$1:(/opera(\s|\/)(\d+)/.test(ua)?' '+o+RegExp.$2:'')):is('konqueror')?'konqueror':is('blackberry')?m+' blackberry':is('android')?m+' android':is('chrome')?w+' chrome':is('iron')?w+' iron':is('applewebkit/')?w+' '+s+(/version\/(\d+)/.test(ua)?' '+s+RegExp.$1:''):is('mozilla/')?g:'',is('j2me')?m+' j2me':is('iphone')?m+' iphone':is('ipod')?m+' ipod':is('ipad')?m+' ipad':is('mac')?'mac':is('darwin')?'mac':is('webtv')?'webtv':is('win')?'win'+(is('windows nt 6.0')?' vista':''):is('freebsd')?'freebsd':(is('x11')||is('linux'))?'linux':'','js']; c = b.join(' '); h.className += ' '+c; return c;}; css_browser_selector(navigator.userAgent);
;
/*
 * jQuery Cycle Plugin (with Transition Definitions)
 * Examples and documentation at: http://jquery.malsup.com/cycle/
 * Copyright (c) 2007-2010 M. Alsup
 * Version: 2.94 (20-DEC-2010)
 * Dual licensed under the MIT and GPL licenses.
 * http://jquery.malsup.com/license.html
 * Requires: jQuery v1.2.6 or later
 */
(function($){var ver="2.94";if($.support==undefined){$.support={opacity:!($.browser.msie)};}function debug(s){if($.fn.cycle.debug){log(s);}}function log(){if(window.console&&window.console.log){window.console.log("[cycle] "+Array.prototype.join.call(arguments," "));}}$.fn.cycle=function(options,arg2){var o={s:this.selector,c:this.context};if(this.length===0&&options!="stop"){if(!$.isReady&&o.s){log("DOM not ready, queuing slideshow");$(function(){$(o.s,o.c).cycle(options,arg2);});return this;}log("terminating; zero elements found by selector"+($.isReady?"":" (DOM not ready)"));return this;}return this.each(function(){var opts=handleArguments(this,options,arg2);if(opts===false){return;}opts.updateActivePagerLink=opts.updateActivePagerLink||$.fn.cycle.updateActivePagerLink;if(this.cycleTimeout){clearTimeout(this.cycleTimeout);}this.cycleTimeout=this.cyclePause=0;var $cont=$(this);var $slides=opts.slideExpr?$(opts.slideExpr,this):$cont.children();var els=$slides.get();if(els.length<2){log("terminating; too few slides: "+els.length);return;}var opts2=buildOptions($cont,$slides,els,opts,o);if(opts2===false){return;}var startTime=opts2.continuous?10:getTimeout(els[opts2.currSlide],els[opts2.nextSlide],opts2,!opts2.backwards);if(startTime){startTime+=(opts2.delay||0);if(startTime<10){startTime=10;}debug("first timeout: "+startTime);this.cycleTimeout=setTimeout(function(){go(els,opts2,0,!opts.backwards);},startTime);}});};function handleArguments(cont,options,arg2){if(cont.cycleStop==undefined){cont.cycleStop=0;}if(options===undefined||options===null){options={};}if(options.constructor==String){switch(options){case"destroy":case"stop":var opts=$(cont).data("cycle.opts");if(!opts){return false;}cont.cycleStop++;if(cont.cycleTimeout){clearTimeout(cont.cycleTimeout);}cont.cycleTimeout=0;$(cont).removeData("cycle.opts");if(options=="destroy"){destroy(opts);}return false;case"toggle":cont.cyclePause=(cont.cyclePause===1)?0:1;checkInstantResume(cont.cyclePause,arg2,cont);return false;case"pause":cont.cyclePause=1;return false;case"resume":cont.cyclePause=0;checkInstantResume(false,arg2,cont);return false;case"prev":case"next":var opts=$(cont).data("cycle.opts");if(!opts){log('options not found, "prev/next" ignored');return false;}$.fn.cycle[options](opts);return false;default:options={fx:options};}return options;}else{if(options.constructor==Number){var num=options;options=$(cont).data("cycle.opts");if(!options){log("options not found, can not advance slide");return false;}if(num<0||num>=options.elements.length){log("invalid slide index: "+num);return false;}options.nextSlide=num;if(cont.cycleTimeout){clearTimeout(cont.cycleTimeout);cont.cycleTimeout=0;}if(typeof arg2=="string"){options.oneTimeFx=arg2;}go(options.elements,options,1,num>=options.currSlide);return false;}}return options;function checkInstantResume(isPaused,arg2,cont){if(!isPaused&&arg2===true){var options=$(cont).data("cycle.opts");if(!options){log("options not found, can not resume");return false;}if(cont.cycleTimeout){clearTimeout(cont.cycleTimeout);cont.cycleTimeout=0;}go(options.elements,options,1,!options.backwards);}}}function removeFilter(el,opts){if(!$.support.opacity&&opts.cleartype&&el.style.filter){try{el.style.removeAttribute("filter");}catch(smother){}}}function destroy(opts){if(opts.next){$(opts.next).unbind(opts.prevNextEvent);}if(opts.prev){$(opts.prev).unbind(opts.prevNextEvent);}if(opts.pager||opts.pagerAnchorBuilder){$.each(opts.pagerAnchors||[],function(){this.unbind().remove();});}opts.pagerAnchors=null;if(opts.destroy){opts.destroy(opts);}}function buildOptions($cont,$slides,els,options,o){var opts=$.extend({},$.fn.cycle.defaults,options||{},$.metadata?$cont.metadata():$.meta?$cont.data():{});if(opts.autostop){opts.countdown=opts.autostopCount||els.length;}var cont=$cont[0];$cont.data("cycle.opts",opts);opts.$cont=$cont;opts.stopCount=cont.cycleStop;opts.elements=els;opts.before=opts.before?[opts.before]:[];opts.after=opts.after?[opts.after]:[];opts.after.unshift(function(){opts.busy=0;});if(!$.support.opacity&&opts.cleartype){opts.after.push(function(){removeFilter(this,opts);});}if(opts.continuous){opts.after.push(function(){go(els,opts,0,!opts.backwards);});}saveOriginalOpts(opts);if(!$.support.opacity&&opts.cleartype&&!opts.cleartypeNoBg){clearTypeFix($slides);}if($cont.css("position")=="static"){$cont.css("position","relative");}if(opts.width){$cont.width(opts.width);}if(opts.height&&opts.height!="auto"){$cont.height(opts.height);}if(opts.startingSlide){opts.startingSlide=parseInt(opts.startingSlide);}else{if(opts.backwards){opts.startingSlide=els.length-1;}}if(opts.random){opts.randomMap=[];for(var i=0;i<els.length;i++){opts.randomMap.push(i);}opts.randomMap.sort(function(a,b){return Math.random()-0.5;});opts.randomIndex=1;opts.startingSlide=opts.randomMap[1];}else{if(opts.startingSlide>=els.length){opts.startingSlide=0;}}opts.currSlide=opts.startingSlide||0;var first=opts.startingSlide;$slides.css({position:"absolute",top:0,left:0}).hide().each(function(i){var z;if(opts.backwards){z=first?i<=first?els.length+(i-first):first-i:els.length-i;}else{z=first?i>=first?els.length-(i-first):first-i:els.length-i;}$(this).css("z-index",z);});$(els[first]).css("opacity",1).show();removeFilter(els[first],opts);if(opts.fit&&opts.width){$slides.width(opts.width);}if(opts.fit&&opts.height&&opts.height!="auto"){$slides.height(opts.height);}var reshape=opts.containerResize&&!$cont.innerHeight();if(reshape){var maxw=0,maxh=0;for(var j=0;j<els.length;j++){var $e=$(els[j]),e=$e[0],w=$e.outerWidth(),h=$e.outerHeight();if(!w){w=e.offsetWidth||e.width||$e.attr("width");}if(!h){h=e.offsetHeight||e.height||$e.attr("height");}maxw=w>maxw?w:maxw;maxh=h>maxh?h:maxh;}if(maxw>0&&maxh>0){$cont.css({width:maxw+"px",height:maxh+"px"});}}if(opts.pause){$cont.hover(function(){this.cyclePause++;},function(){this.cyclePause--;});}if(supportMultiTransitions(opts)===false){return false;}var requeue=false;options.requeueAttempts=options.requeueAttempts||0;$slides.each(function(){var $el=$(this);this.cycleH=(opts.fit&&opts.height)?opts.height:($el.height()||this.offsetHeight||this.height||$el.attr("height")||0);this.cycleW=(opts.fit&&opts.width)?opts.width:($el.width()||this.offsetWidth||this.width||$el.attr("width")||0);if($el.is("img")){var loadingIE=($.browser.msie&&this.cycleW==28&&this.cycleH==30&&!this.complete);var loadingFF=($.browser.mozilla&&this.cycleW==34&&this.cycleH==19&&!this.complete);var loadingOp=($.browser.opera&&((this.cycleW==42&&this.cycleH==19)||(this.cycleW==37&&this.cycleH==17))&&!this.complete);var loadingOther=(this.cycleH==0&&this.cycleW==0&&!this.complete);if(loadingIE||loadingFF||loadingOp||loadingOther){if(o.s&&opts.requeueOnImageNotLoaded&&++options.requeueAttempts<100){log(options.requeueAttempts," - img slide not loaded, requeuing slideshow: ",this.src,this.cycleW,this.cycleH);setTimeout(function(){$(o.s,o.c).cycle(options);},opts.requeueTimeout);requeue=true;return false;}else{log("could not determine size of image: "+this.src,this.cycleW,this.cycleH);}}}return true;});if(requeue){return false;}opts.cssBefore=opts.cssBefore||{};opts.animIn=opts.animIn||{};opts.animOut=opts.animOut||{};$slides.not(":eq("+first+")").css(opts.cssBefore);if(opts.cssFirst){$($slides[first]).css(opts.cssFirst);}if(opts.timeout){opts.timeout=parseInt(opts.timeout);if(opts.speed.constructor==String){opts.speed=$.fx.speeds[opts.speed]||parseInt(opts.speed);}if(!opts.sync){opts.speed=opts.speed/2;}var buffer=opts.fx=="shuffle"?500:250;while((opts.timeout-opts.speed)<buffer){opts.timeout+=opts.speed;}}if(opts.easing){opts.easeIn=opts.easeOut=opts.easing;}if(!opts.speedIn){opts.speedIn=opts.speed;}if(!opts.speedOut){opts.speedOut=opts.speed;}opts.slideCount=els.length;opts.currSlide=opts.lastSlide=first;if(opts.random){if(++opts.randomIndex==els.length){opts.randomIndex=0;}opts.nextSlide=opts.randomMap[opts.randomIndex];}else{if(opts.backwards){opts.nextSlide=opts.startingSlide==0?(els.length-1):opts.startingSlide-1;}else{opts.nextSlide=opts.startingSlide>=(els.length-1)?0:opts.startingSlide+1;}}if(!opts.multiFx){var init=$.fn.cycle.transitions[opts.fx];if($.isFunction(init)){init($cont,$slides,opts);}else{if(opts.fx!="custom"&&!opts.multiFx){log("unknown transition: "+opts.fx,"; slideshow terminating");return false;}}}var e0=$slides[first];if(opts.before.length){opts.before[0].apply(e0,[e0,e0,opts,true]);}if(opts.after.length>1){opts.after[1].apply(e0,[e0,e0,opts,true]);}if(opts.next){$(opts.next).bind(opts.prevNextEvent,function(){return advance(opts,1);});}if(opts.prev){$(opts.prev).bind(opts.prevNextEvent,function(){return advance(opts,0);});}if(opts.pager||opts.pagerAnchorBuilder){buildPager(els,opts);}exposeAddSlide(opts,els);return opts;}function saveOriginalOpts(opts){opts.original={before:[],after:[]};opts.original.cssBefore=$.extend({},opts.cssBefore);opts.original.cssAfter=$.extend({},opts.cssAfter);opts.original.animIn=$.extend({},opts.animIn);opts.original.animOut=$.extend({},opts.animOut);$.each(opts.before,function(){opts.original.before.push(this);});$.each(opts.after,function(){opts.original.after.push(this);});}function supportMultiTransitions(opts){var i,tx,txs=$.fn.cycle.transitions;if(opts.fx.indexOf(",")>0){opts.multiFx=true;opts.fxs=opts.fx.replace(/\s*/g,"").split(",");for(i=0;i<opts.fxs.length;i++){var fx=opts.fxs[i];tx=txs[fx];if(!tx||!txs.hasOwnProperty(fx)||!$.isFunction(tx)){log("discarding unknown transition: ",fx);opts.fxs.splice(i,1);i--;}}if(!opts.fxs.length){log("No valid transitions named; slideshow terminating.");return false;}}else{if(opts.fx=="all"){opts.multiFx=true;opts.fxs=[];for(p in txs){tx=txs[p];if(txs.hasOwnProperty(p)&&$.isFunction(tx)){opts.fxs.push(p);}}}}if(opts.multiFx&&opts.randomizeEffects){var r1=Math.floor(Math.random()*20)+30;for(i=0;i<r1;i++){var r2=Math.floor(Math.random()*opts.fxs.length);opts.fxs.push(opts.fxs.splice(r2,1)[0]);}debug("randomized fx sequence: ",opts.fxs);}return true;}function exposeAddSlide(opts,els){opts.addSlide=function(newSlide,prepend){var $s=$(newSlide),s=$s[0];if(!opts.autostopCount){opts.countdown++;}els[prepend?"unshift":"push"](s);if(opts.els){opts.els[prepend?"unshift":"push"](s);}opts.slideCount=els.length;$s.css("position","absolute");$s[prepend?"prependTo":"appendTo"](opts.$cont);if(prepend){opts.currSlide++;opts.nextSlide++;}if(!$.support.opacity&&opts.cleartype&&!opts.cleartypeNoBg){clearTypeFix($s);}if(opts.fit&&opts.width){$s.width(opts.width);}if(opts.fit&&opts.height&&opts.height!="auto"){$s.height(opts.height);}s.cycleH=(opts.fit&&opts.height)?opts.height:$s.height();s.cycleW=(opts.fit&&opts.width)?opts.width:$s.width();$s.css(opts.cssBefore);if(opts.pager||opts.pagerAnchorBuilder){$.fn.cycle.createPagerAnchor(els.length-1,s,$(opts.pager),els,opts);}if($.isFunction(opts.onAddSlide)){opts.onAddSlide($s);}else{$s.hide();}};}$.fn.cycle.resetState=function(opts,fx){fx=fx||opts.fx;opts.before=[];opts.after=[];opts.cssBefore=$.extend({},opts.original.cssBefore);opts.cssAfter=$.extend({},opts.original.cssAfter);opts.animIn=$.extend({},opts.original.animIn);opts.animOut=$.extend({},opts.original.animOut);opts.fxFn=null;$.each(opts.original.before,function(){opts.before.push(this);});$.each(opts.original.after,function(){opts.after.push(this);});var init=$.fn.cycle.transitions[fx];if($.isFunction(init)){init(opts.$cont,$(opts.elements),opts);}};function go(els,opts,manual,fwd){if(manual&&opts.busy&&opts.manualTrump){debug("manualTrump in go(), stopping active transition");$(els).stop(true,true);opts.busy=false;}if(opts.busy){debug("transition active, ignoring new tx request");return;}var p=opts.$cont[0],curr=els[opts.currSlide],next=els[opts.nextSlide];if(p.cycleStop!=opts.stopCount||p.cycleTimeout===0&&!manual){return;}if(!manual&&!p.cyclePause&&!opts.bounce&&((opts.autostop&&(--opts.countdown<=0))||(opts.nowrap&&!opts.random&&opts.nextSlide<opts.currSlide))){if(opts.end){opts.end(opts);}return;}var changed=false;if((manual||!p.cyclePause)&&(opts.nextSlide!=opts.currSlide)){changed=true;var fx=opts.fx;curr.cycleH=curr.cycleH||$(curr).height();curr.cycleW=curr.cycleW||$(curr).width();next.cycleH=next.cycleH||$(next).height();next.cycleW=next.cycleW||$(next).width();if(opts.multiFx){if(opts.lastFx==undefined||++opts.lastFx>=opts.fxs.length){opts.lastFx=0;}fx=opts.fxs[opts.lastFx];opts.currFx=fx;}if(opts.oneTimeFx){fx=opts.oneTimeFx;opts.oneTimeFx=null;}$.fn.cycle.resetState(opts,fx);if(opts.before.length){$.each(opts.before,function(i,o){if(p.cycleStop!=opts.stopCount){return;}o.apply(next,[curr,next,opts,fwd]);});}var after=function(){$.each(opts.after,function(i,o){if(p.cycleStop!=opts.stopCount){return;}o.apply(next,[curr,next,opts,fwd]);});};debug("tx firing; currSlide: "+opts.currSlide+"; nextSlide: "+opts.nextSlide);opts.busy=1;if(opts.fxFn){opts.fxFn(curr,next,opts,after,fwd,manual&&opts.fastOnEvent);}else{if($.isFunction($.fn.cycle[opts.fx])){$.fn.cycle[opts.fx](curr,next,opts,after,fwd,manual&&opts.fastOnEvent);}else{$.fn.cycle.custom(curr,next,opts,after,fwd,manual&&opts.fastOnEvent);}}}if(changed||opts.nextSlide==opts.currSlide){opts.lastSlide=opts.currSlide;if(opts.random){opts.currSlide=opts.nextSlide;if(++opts.randomIndex==els.length){opts.randomIndex=0;}opts.nextSlide=opts.randomMap[opts.randomIndex];if(opts.nextSlide==opts.currSlide){opts.nextSlide=(opts.currSlide==opts.slideCount-1)?0:opts.currSlide+1;}}else{if(opts.backwards){var roll=(opts.nextSlide-1)<0;if(roll&&opts.bounce){opts.backwards=!opts.backwards;opts.nextSlide=1;opts.currSlide=0;}else{opts.nextSlide=roll?(els.length-1):opts.nextSlide-1;opts.currSlide=roll?0:opts.nextSlide+1;}}else{var roll=(opts.nextSlide+1)==els.length;if(roll&&opts.bounce){opts.backwards=!opts.backwards;opts.nextSlide=els.length-2;opts.currSlide=els.length-1;}else{opts.nextSlide=roll?0:opts.nextSlide+1;opts.currSlide=roll?els.length-1:opts.nextSlide-1;}}}}if(changed&&opts.pager){opts.updateActivePagerLink(opts.pager,opts.currSlide,opts.activePagerClass);}var ms=0;if(opts.timeout&&!opts.continuous){ms=getTimeout(els[opts.currSlide],els[opts.nextSlide],opts,fwd);}else{if(opts.continuous&&p.cyclePause){ms=10;}}if(ms>0){p.cycleTimeout=setTimeout(function(){go(els,opts,0,!opts.backwards);},ms);}}$.fn.cycle.updateActivePagerLink=function(pager,currSlide,clsName){$(pager).each(function(){$(this).children().removeClass(clsName).eq(currSlide).addClass(clsName);});};function getTimeout(curr,next,opts,fwd){if(opts.timeoutFn){var t=opts.timeoutFn.call(curr,curr,next,opts,fwd);while((t-opts.speed)<250){t+=opts.speed;}debug("calculated timeout: "+t+"; speed: "+opts.speed);if(t!==false){return t;}}return opts.timeout;}$.fn.cycle.next=function(opts){advance(opts,1);};$.fn.cycle.prev=function(opts){advance(opts,0);};function advance(opts,moveForward){var val=moveForward?1:-1;var els=opts.elements;var p=opts.$cont[0],timeout=p.cycleTimeout;if(timeout){clearTimeout(timeout);p.cycleTimeout=0;}if(opts.random&&val<0){opts.randomIndex--;if(--opts.randomIndex==-2){opts.randomIndex=els.length-2;}else{if(opts.randomIndex==-1){opts.randomIndex=els.length-1;}}opts.nextSlide=opts.randomMap[opts.randomIndex];}else{if(opts.random){opts.nextSlide=opts.randomMap[opts.randomIndex];}else{opts.nextSlide=opts.currSlide+val;if(opts.nextSlide<0){if(opts.nowrap){return false;}opts.nextSlide=els.length-1;}else{if(opts.nextSlide>=els.length){if(opts.nowrap){return false;}opts.nextSlide=0;}}}}var cb=opts.onPrevNextEvent||opts.prevNextClick;if($.isFunction(cb)){cb(val>0,opts.nextSlide,els[opts.nextSlide]);}go(els,opts,1,moveForward);return false;}function buildPager(els,opts){var $p=$(opts.pager);$.each(els,function(i,o){$.fn.cycle.createPagerAnchor(i,o,$p,els,opts);});opts.updateActivePagerLink(opts.pager,opts.startingSlide,opts.activePagerClass);}$.fn.cycle.createPagerAnchor=function(i,el,$p,els,opts){var a;if($.isFunction(opts.pagerAnchorBuilder)){a=opts.pagerAnchorBuilder(i,el);debug("pagerAnchorBuilder("+i+", el) returned: "+a);}else{a='<a href="#">'+(i+1)+"</a>";}if(!a){return;}var $a=$(a);if($a.parents("body").length===0){var arr=[];if($p.length>1){$p.each(function(){var $clone=$a.clone(true);$(this).append($clone);arr.push($clone[0]);});$a=$(arr);}else{$a.appendTo($p);}}opts.pagerAnchors=opts.pagerAnchors||[];opts.pagerAnchors.push($a);$a.bind(opts.pagerEvent,function(e){e.preventDefault();opts.nextSlide=i;var p=opts.$cont[0],timeout=p.cycleTimeout;if(timeout){clearTimeout(timeout);p.cycleTimeout=0;}var cb=opts.onPagerEvent||opts.pagerClick;if($.isFunction(cb)){cb(opts.nextSlide,els[opts.nextSlide]);}go(els,opts,1,opts.currSlide<i);});if(!/^click/.test(opts.pagerEvent)&&!opts.allowPagerClickBubble){$a.bind("click.cycle",function(){return false;});}if(opts.pauseOnPagerHover){$a.hover(function(){opts.$cont[0].cyclePause++;},function(){opts.$cont[0].cyclePause--;});}};$.fn.cycle.hopsFromLast=function(opts,fwd){var hops,l=opts.lastSlide,c=opts.currSlide;if(fwd){hops=c>l?c-l:opts.slideCount-l;}else{hops=c<l?l-c:l+opts.slideCount-c;}return hops;};function clearTypeFix($slides){debug("applying clearType background-color hack");function hex(s){s=parseInt(s).toString(16);return s.length<2?"0"+s:s;}function getBg(e){for(;e&&e.nodeName.toLowerCase()!="html";e=e.parentNode){var v=$.css(e,"background-color");if(v.indexOf("rgb")>=0){var rgb=v.match(/\d+/g);return"#"+hex(rgb[0])+hex(rgb[1])+hex(rgb[2]);}if(v&&v!="transparent"){return v;}}return"#ffffff";}$slides.each(function(){$(this).css("background-color",getBg(this));});}$.fn.cycle.commonReset=function(curr,next,opts,w,h,rev){$(opts.elements).not(curr).hide();opts.cssBefore.opacity=1;opts.cssBefore.display="block";if(opts.slideResize&&w!==false&&next.cycleW>0){opts.cssBefore.width=next.cycleW;}if(opts.slideResize&&h!==false&&next.cycleH>0){opts.cssBefore.height=next.cycleH;}opts.cssAfter=opts.cssAfter||{};opts.cssAfter.display="none";$(curr).css("zIndex",opts.slideCount+(rev===true?1:0));$(next).css("zIndex",opts.slideCount+(rev===true?0:1));};$.fn.cycle.custom=function(curr,next,opts,cb,fwd,speedOverride){var $l=$(curr),$n=$(next);var speedIn=opts.speedIn,speedOut=opts.speedOut,easeIn=opts.easeIn,easeOut=opts.easeOut;$n.css(opts.cssBefore);if(speedOverride){if(typeof speedOverride=="number"){speedIn=speedOut=speedOverride;}else{speedIn=speedOut=1;}easeIn=easeOut=null;}var fn=function(){$n.animate(opts.animIn,speedIn,easeIn,cb);};$l.animate(opts.animOut,speedOut,easeOut,function(){if(opts.cssAfter){$l.css(opts.cssAfter);}if(!opts.sync){fn();}});if(opts.sync){fn();}};$.fn.cycle.transitions={fade:function($cont,$slides,opts){$slides.not(":eq("+opts.currSlide+")").css("opacity",0);opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts);opts.cssBefore.opacity=0;});opts.animIn={opacity:1};opts.animOut={opacity:0};opts.cssBefore={top:0,left:0};}};$.fn.cycle.ver=function(){return ver;};$.fn.cycle.defaults={fx:"fade",timeout:4000,timeoutFn:null,continuous:0,speed:1000,speedIn:null,speedOut:null,next:null,prev:null,onPrevNextEvent:null,prevNextEvent:"click.cycle",pager:null,onPagerEvent:null,pagerEvent:"click.cycle",allowPagerClickBubble:false,pagerAnchorBuilder:null,before:null,after:null,end:null,easing:null,easeIn:null,easeOut:null,shuffle:null,animIn:null,animOut:null,cssBefore:null,cssAfter:null,fxFn:null,height:"auto",startingSlide:0,sync:1,random:0,fit:0,containerResize:1,slideResize:1,pause:0,pauseOnPagerHover:0,autostop:0,autostopCount:0,delay:0,slideExpr:null,cleartype:!$.support.opacity,cleartypeNoBg:false,nowrap:0,fastOnEvent:0,randomizeEffects:1,rev:0,manualTrump:true,requeueOnImageNotLoaded:true,requeueTimeout:250,activePagerClass:"activeSlide",updateActivePagerLink:null,backwards:false};})(jQuery);
/*
 * jQuery Cycle Plugin Transition Definitions
 * This script is a plugin for the jQuery Cycle Plugin
 * Examples and documentation at: http://malsup.com/jquery/cycle/
 * Copyright (c) 2007-2010 M. Alsup
 * Version:	 2.73
 * Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 */
(function($){$.fn.cycle.transitions.none=function($cont,$slides,opts){opts.fxFn=function(curr,next,opts,after){$(next).show();$(curr).hide();after();};};$.fn.cycle.transitions.fadeout=function($cont,$slides,opts){$slides.not(":eq("+opts.currSlide+")").css({display:"block",opacity:1});opts.before.push(function(curr,next,opts,w,h,rev){$(curr).css("zIndex",opts.slideCount+(!rev===true?1:0));$(next).css("zIndex",opts.slideCount+(!rev===true?0:1));});opts.animIn={opacity:1};opts.animOut={opacity:0};opts.cssBefore={opacity:1,display:"block"};opts.cssAfter={zIndex:0};};$.fn.cycle.transitions.scrollUp=function($cont,$slides,opts){$cont.css("overflow","hidden");opts.before.push($.fn.cycle.commonReset);var h=$cont.height();opts.cssBefore={top:h,left:0};opts.cssFirst={top:0};opts.animIn={top:0};opts.animOut={top:-h};};$.fn.cycle.transitions.scrollDown=function($cont,$slides,opts){$cont.css("overflow","hidden");opts.before.push($.fn.cycle.commonReset);var h=$cont.height();opts.cssFirst={top:0};opts.cssBefore={top:-h,left:0};opts.animIn={top:0};opts.animOut={top:h};};$.fn.cycle.transitions.scrollLeft=function($cont,$slides,opts){$cont.css("overflow","hidden");opts.before.push($.fn.cycle.commonReset);var w=$cont.width();opts.cssFirst={left:0};opts.cssBefore={left:w,top:0};opts.animIn={left:0};opts.animOut={left:0-w};};$.fn.cycle.transitions.scrollRight=function($cont,$slides,opts){$cont.css("overflow","hidden");opts.before.push($.fn.cycle.commonReset);var w=$cont.width();opts.cssFirst={left:0};opts.cssBefore={left:-w,top:0};opts.animIn={left:0};opts.animOut={left:w};};$.fn.cycle.transitions.scrollHorz=function($cont,$slides,opts){$cont.css("overflow","hidden").width();opts.before.push(function(curr,next,opts,fwd){if(opts.rev){fwd=!fwd;}$.fn.cycle.commonReset(curr,next,opts);opts.cssBefore.left=fwd?(next.cycleW-1):(1-next.cycleW);opts.animOut.left=fwd?-curr.cycleW:curr.cycleW;});opts.cssFirst={left:0};opts.cssBefore={top:0};opts.animIn={left:0};opts.animOut={top:0};};$.fn.cycle.transitions.scrollVert=function($cont,$slides,opts){$cont.css("overflow","hidden");opts.before.push(function(curr,next,opts,fwd){if(opts.rev){fwd=!fwd;}$.fn.cycle.commonReset(curr,next,opts);opts.cssBefore.top=fwd?(1-next.cycleH):(next.cycleH-1);opts.animOut.top=fwd?curr.cycleH:-curr.cycleH;});opts.cssFirst={top:0};opts.cssBefore={left:0};opts.animIn={top:0};opts.animOut={left:0};};$.fn.cycle.transitions.slideX=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$(opts.elements).not(curr).hide();$.fn.cycle.commonReset(curr,next,opts,false,true);opts.animIn.width=next.cycleW;});opts.cssBefore={left:0,top:0,width:0};opts.animIn={width:"show"};opts.animOut={width:0};};$.fn.cycle.transitions.slideY=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$(opts.elements).not(curr).hide();$.fn.cycle.commonReset(curr,next,opts,true,false);opts.animIn.height=next.cycleH;});opts.cssBefore={left:0,top:0,height:0};opts.animIn={height:"show"};opts.animOut={height:0};};$.fn.cycle.transitions.shuffle=function($cont,$slides,opts){var i,w=$cont.css("overflow","visible").width();$slides.css({left:0,top:0});opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,true,true,true);});if(!opts.speedAdjusted){opts.speed=opts.speed/2;opts.speedAdjusted=true;}opts.random=0;opts.shuffle=opts.shuffle||{left:-w,top:15};opts.els=[];for(i=0;i<$slides.length;i++){opts.els.push($slides[i]);}for(i=0;i<opts.currSlide;i++){opts.els.push(opts.els.shift());}opts.fxFn=function(curr,next,opts,cb,fwd){if(opts.rev){fwd=!fwd;}var $el=fwd?$(curr):$(next);$(next).css(opts.cssBefore);var count=opts.slideCount;$el.animate(opts.shuffle,opts.speedIn,opts.easeIn,function(){var hops=$.fn.cycle.hopsFromLast(opts,fwd);for(var k=0;k<hops;k++){fwd?opts.els.push(opts.els.shift()):opts.els.unshift(opts.els.pop());}if(fwd){for(var i=0,len=opts.els.length;i<len;i++){$(opts.els[i]).css("z-index",len-i+count);}}else{var z=$(curr).css("z-index");$el.css("z-index",parseInt(z)+1+count);}$el.animate({left:0,top:0},opts.speedOut,opts.easeOut,function(){$(fwd?this:curr).hide();if(cb){cb();}});});};opts.cssBefore={display:"block",opacity:1,top:0,left:0};};$.fn.cycle.transitions.turnUp=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,true,false);opts.cssBefore.top=next.cycleH;opts.animIn.height=next.cycleH;opts.animOut.width=next.cycleW;});opts.cssFirst={top:0};opts.cssBefore={left:0,height:0};opts.animIn={top:0};opts.animOut={height:0};};$.fn.cycle.transitions.turnDown=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,true,false);opts.animIn.height=next.cycleH;opts.animOut.top=curr.cycleH;});opts.cssFirst={top:0};opts.cssBefore={left:0,top:0,height:0};opts.animOut={height:0};};$.fn.cycle.transitions.turnLeft=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,false,true);opts.cssBefore.left=next.cycleW;opts.animIn.width=next.cycleW;});opts.cssBefore={top:0,width:0};opts.animIn={left:0};opts.animOut={width:0};};$.fn.cycle.transitions.turnRight=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,false,true);opts.animIn.width=next.cycleW;opts.animOut.left=curr.cycleW;});opts.cssBefore={top:0,left:0,width:0};opts.animIn={left:0};opts.animOut={width:0};};$.fn.cycle.transitions.zoom=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,false,false,true);opts.cssBefore.top=next.cycleH/2;opts.cssBefore.left=next.cycleW/2;opts.animIn={top:0,left:0,width:next.cycleW,height:next.cycleH};opts.animOut={width:0,height:0,top:curr.cycleH/2,left:curr.cycleW/2};});opts.cssFirst={top:0,left:0};opts.cssBefore={width:0,height:0};};$.fn.cycle.transitions.fadeZoom=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,false,false);opts.cssBefore.left=next.cycleW/2;opts.cssBefore.top=next.cycleH/2;opts.animIn={top:0,left:0,width:next.cycleW,height:next.cycleH};});opts.cssBefore={width:0,height:0};opts.animOut={opacity:0};};$.fn.cycle.transitions.blindX=function($cont,$slides,opts){var w=$cont.css("overflow","hidden").width();opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts);opts.animIn.width=next.cycleW;opts.animOut.left=curr.cycleW;});opts.cssBefore={left:w,top:0};opts.animIn={left:0};opts.animOut={left:w};};$.fn.cycle.transitions.blindY=function($cont,$slides,opts){var h=$cont.css("overflow","hidden").height();opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts);opts.animIn.height=next.cycleH;opts.animOut.top=curr.cycleH;});opts.cssBefore={top:h,left:0};opts.animIn={top:0};opts.animOut={top:h};};$.fn.cycle.transitions.blindZ=function($cont,$slides,opts){var h=$cont.css("overflow","hidden").height();var w=$cont.width();opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts);opts.animIn.height=next.cycleH;opts.animOut.top=curr.cycleH;});opts.cssBefore={top:h,left:w};opts.animIn={top:0,left:0};opts.animOut={top:h,left:w};};$.fn.cycle.transitions.growX=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,false,true);opts.cssBefore.left=this.cycleW/2;opts.animIn={left:0,width:this.cycleW};opts.animOut={left:0};});opts.cssBefore={width:0,top:0};};$.fn.cycle.transitions.growY=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,true,false);opts.cssBefore.top=this.cycleH/2;opts.animIn={top:0,height:this.cycleH};opts.animOut={top:0};});opts.cssBefore={height:0,left:0};};$.fn.cycle.transitions.curtainX=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,false,true,true);opts.cssBefore.left=next.cycleW/2;opts.animIn={left:0,width:this.cycleW};opts.animOut={left:curr.cycleW/2,width:0};});opts.cssBefore={top:0,width:0};};$.fn.cycle.transitions.curtainY=function($cont,$slides,opts){opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,true,false,true);opts.cssBefore.top=next.cycleH/2;opts.animIn={top:0,height:next.cycleH};opts.animOut={top:curr.cycleH/2,height:0};});opts.cssBefore={left:0,height:0};};$.fn.cycle.transitions.cover=function($cont,$slides,opts){var d=opts.direction||"left";var w=$cont.css("overflow","hidden").width();var h=$cont.height();opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts);if(d=="right"){opts.cssBefore.left=-w;}else{if(d=="up"){opts.cssBefore.top=h;}else{if(d=="down"){opts.cssBefore.top=-h;}else{opts.cssBefore.left=w;}}}});opts.animIn={left:0,top:0};opts.animOut={opacity:1};opts.cssBefore={top:0,left:0};};$.fn.cycle.transitions.uncover=function($cont,$slides,opts){var d=opts.direction||"left";var w=$cont.css("overflow","hidden").width();var h=$cont.height();opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,true,true,true);if(d=="right"){opts.animOut.left=w;}else{if(d=="up"){opts.animOut.top=-h;}else{if(d=="down"){opts.animOut.top=h;}else{opts.animOut.left=-w;}}}});opts.animIn={left:0,top:0};opts.animOut={opacity:1};opts.cssBefore={top:0,left:0};};$.fn.cycle.transitions.toss=function($cont,$slides,opts){var w=$cont.css("overflow","visible").width();var h=$cont.height();opts.before.push(function(curr,next,opts){$.fn.cycle.commonReset(curr,next,opts,true,true,true);if(!opts.animOut.left&&!opts.animOut.top){opts.animOut={left:w*2,top:-h/2,opacity:0};}else{opts.animOut.opacity=0;}});opts.cssBefore={left:0,top:0};opts.animIn={left:0};};$.fn.cycle.transitions.wipe=function($cont,$slides,opts){var w=$cont.css("overflow","hidden").width();var h=$cont.height();opts.cssBefore=opts.cssBefore||{};var clip;if(opts.clip){if(/l2r/.test(opts.clip)){clip="rect(0px 0px "+h+"px 0px)";}else{if(/r2l/.test(opts.clip)){clip="rect(0px "+w+"px "+h+"px "+w+"px)";}else{if(/t2b/.test(opts.clip)){clip="rect(0px "+w+"px 0px 0px)";}else{if(/b2t/.test(opts.clip)){clip="rect("+h+"px "+w+"px "+h+"px 0px)";}else{if(/zoom/.test(opts.clip)){var top=parseInt(h/2);var left=parseInt(w/2);clip="rect("+top+"px "+left+"px "+top+"px "+left+"px)";}}}}}}opts.cssBefore.clip=opts.cssBefore.clip||clip||"rect(0px 0px 0px 0px)";var d=opts.cssBefore.clip.match(/(\d+)/g);var t=parseInt(d[0]),r=parseInt(d[1]),b=parseInt(d[2]),l=parseInt(d[3]);opts.before.push(function(curr,next,opts){if(curr==next){return;}var $curr=$(curr),$next=$(next);$.fn.cycle.commonReset(curr,next,opts,true,true,false);opts.cssAfter.display="block";var step=1,count=parseInt((opts.speedIn/13))-1;(function f(){var tt=t?t-parseInt(step*(t/count)):0;var ll=l?l-parseInt(step*(l/count)):0;var bb=b<h?b+parseInt(step*((h-b)/count||1)):h;var rr=r<w?r+parseInt(step*((w-r)/count||1)):w;$next.css({clip:"rect("+tt+"px "+rr+"px "+bb+"px "+ll+"px)"});(step++<=count)?setTimeout(f,13):$curr.css("display","none");})();});opts.cssBefore={display:"block",opacity:1,top:0,left:0};opts.animIn={left:0};opts.animOut={left:0};};})(jQuery);;
/*
 * jQuery Easing v1.1.1 - http://gsgd.co.uk/sandbox/jquery.easing.php
 *
 * Uses the built in easing capabilities added in jQuery 1.1
 * to offer multiple easing options
 *
 * Copyright (c) 2007 George Smith
 * Licensed under the MIT License:
 *   http://www.opensource.org/licenses/mit-license.php
 */

jQuery.extend({
	easing: {
		easein: function(x, t, b, c, d) {
			return c*(t/=d)*t + b; // in
		},
		easeinout: function(x, t, b, c, d) {
			if (t < d/2) return 2*c*t*t/(d*d) + b;
			var ts = t - d/2;
			return -2*c*ts*ts/(d*d) + 2*c*ts/d + c/2 + b;		
		},
		easeout: function(x, t, b, c, d) {
			return -c*t*t/(d*d) + 2*c*t/d + b;
		},
		expoin: function(x, t, b, c, d) {
			var flip = 1;
			if (c < 0) {
				flip *= -1;
				c *= -1;
			}
			return flip * (Math.exp(Math.log(c)/d * t)) + b;		
		},
		expoout: function(x, t, b, c, d) {
			var flip = 1;
			if (c < 0) {
				flip *= -1;
				c *= -1;
			}
			return flip * (-Math.exp(-Math.log(c)/d * (t-d)) + c + 1) + b;
		},
		expoinout: function(x, t, b, c, d) {
			var flip = 1;
			if (c < 0) {
				flip *= -1;
				c *= -1;
			}
			if (t < d/2) return flip * (Math.exp(Math.log(c/2)/(d/2) * t)) + b;
			return flip * (-Math.exp(-2*Math.log(c/2)/d * (t-d)) + c + 1) + b;
		},
		bouncein: function(x, t, b, c, d) {
			return c - jQuery.easing['bounceout'](x, d-t, 0, c, d) + b;
		},
		bounceout: function(x, t, b, c, d) {
			if ((t/=d) < (1/2.75)) {
				return c*(7.5625*t*t) + b;
			} else if (t < (2/2.75)) {
				return c*(7.5625*(t-=(1.5/2.75))*t + .75) + b;
			} else if (t < (2.5/2.75)) {
				return c*(7.5625*(t-=(2.25/2.75))*t + .9375) + b;
			} else {
				return c*(7.5625*(t-=(2.625/2.75))*t + .984375) + b;
			}
		},
		bounceinout: function(x, t, b, c, d) {
			if (t < d/2) return jQuery.easing['bouncein'] (x, t*2, 0, c, d) * .5 + b;
			return jQuery.easing['bounceout'] (x, t*2-d,0, c, d) * .5 + c*.5 + b;
		},
		elasin: function(x, t, b, c, d) {
			var s=1.70158;var p=0;var a=c;
			if (t==0) return b;  if ((t/=d)==1) return b+c;  if (!p) p=d*.3;
			if (a < Math.abs(c)) { a=c; var s=p/4; }
			else var s = p/(2*Math.PI) * Math.asin (c/a);
			return -(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )) + b;
		},
		elasout: function(x, t, b, c, d) {
			var s=1.70158;var p=0;var a=c;
			if (t==0) return b;  if ((t/=d)==1) return b+c;  if (!p) p=d*.3;
			if (a < Math.abs(c)) { a=c; var s=p/4; }
			else var s = p/(2*Math.PI) * Math.asin (c/a);
			return a*Math.pow(2,-10*t) * Math.sin( (t*d-s)*(2*Math.PI)/p ) + c + b;
		},
		elasinout: function(x, t, b, c, d) {
			var s=1.70158;var p=0;var a=c;
			if (t==0) return b;  if ((t/=d/2)==2) return b+c;  if (!p) p=d*(.3*1.5);
			if (a < Math.abs(c)) { a=c; var s=p/4; }
			else var s = p/(2*Math.PI) * Math.asin (c/a);
			if (t < 1) return -.5*(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )) + b;
			return a*Math.pow(2,-10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )*.5 + c + b;
		},
		backin: function(x, t, b, c, d) {
			var s=1.70158;
			return c*(t/=d)*t*((s+1)*t - s) + b;
		},
		backout: function(x, t, b, c, d) {
			var s=1.70158;
			return c*((t=t/d-1)*t*((s+1)*t + s) + 1) + b;
		},
		backinout: function(x, t, b, c, d) {
			var s=1.70158;
			if ((t/=d/2) < 1) return c/2*(t*t*(((s*=(1.525))+1)*t - s)) + b;
			return c/2*((t-=2)*t*(((s*=(1.525))+1)*t + s) + 2) + b;
		},
		linear: function(x, t, b, c, d) {
			return c*t/d + b; //linear
		}
	}
});;
/*!
 * jScrollPane - v2.0.0beta6 - 2010-12-06
 * http://jscrollpane.kelvinluck.com/
 *
 * Copyright (c) 2010 Kelvin Luck
 * Dual licensed under the MIT and GPL licenses.
 */

// Script: jScrollPane - cross browser customisable scrollbars
//
// *Version: 2.0.0beta6, Last updated: 2010-12-06*
//
// Project Home - http://jscrollpane.kelvinluck.com/
// GitHub       - http://github.com/vitch/jScrollPane
// Source       - http://github.com/vitch/jScrollPane/raw/master/script/jquery.jscrollpane.js
// (Minified)   - http://github.com/vitch/jScrollPane/raw/master/script/jquery.jscrollpane.min.js
//
// About: License
//
// Copyright (c) 2010 Kelvin Luck
// Dual licensed under the MIT or GPL Version 2 licenses.
// http://jscrollpane.kelvinluck.com/MIT-LICENSE.txt
// http://jscrollpane.kelvinluck.com/GPL-LICENSE.txt
//
// About: Examples
//
// All examples and demos are available through the jScrollPane example site at:
// http://jscrollpane.kelvinluck.com/
//
// About: Support and Testing
//
// This plugin is tested on the browsers below and has been found to work reliably on them. If you run
// into a problem on one of the supported browsers then please visit the support section on the jScrollPane
// website (http://jscrollpane.kelvinluck.com/) for more information on getting support. You are also
// welcome to fork the project on GitHub if you can contribute a fix for a given issue. 
//
// jQuery Versions - tested in 1.4.2+ - reported to work in 1.3.x
// Browsers Tested - Firefox 3.6.8, Safari 5, Opera 10.6, Chrome 5.0, IE 6, 7, 8
//
// About: Release History
//
// 2.0.0beta6 - (in progress) scrollToElement horizontal support
// 2.0.0beta5 - (2010-10-18) jQuery 1.4.3 support, various bug fixes
// 2.0.0beta4 - (2010-09-17) clickOnTrack support, bug fixes
// 2.0.0beta3 - (2010-08-27) Horizontal mousewheel, mwheelIntent, keyboard support, bug fixes
// 2.0.0beta2 - (2010-08-21) Bug fixes
// 2.0.0beta1 - (2010-08-17) Rewrite to follow modern best practices and enable horizontal scrolling, initially hidden
//							 elements and dynamically sized elements.
// 1.x - (2006-12-31 - 2010-07-31) Initial version, hosted at googlecode, deprecated

(function($,window,undefined){

	$.fn.jScrollPane = function(settings)
	{
		// JScrollPane "class" - public methods are available through $('selector').data('jsp')
		function JScrollPane(elem, s)
		{

			var settings, jsp = this, pane, paneWidth, paneHeight, container, contentWidth, contentHeight,
				percentInViewH, percentInViewV, isScrollableV, isScrollableH, verticalDrag, dragMaxY,
				verticalDragPosition, horizontalDrag, dragMaxX, horizontalDragPosition,
				verticalBar, verticalTrack, scrollbarWidth, verticalTrackHeight, verticalDragHeight, arrowUp, arrowDown,
				horizontalBar, horizontalTrack, horizontalTrackWidth, horizontalDragWidth, arrowLeft, arrowRight,
				reinitialiseInterval, originalPadding, originalPaddingTotalWidth, previousPaneWidth,
				wasAtTop = true, wasAtLeft = true, wasAtBottom = false, wasAtRight = false,
				mwEvent = $.fn.mwheelIntent ? 'mwheelIntent.jsp' : 'mousewheel.jsp';

			originalPadding = elem.css('paddingTop') + ' ' +
								elem.css('paddingRight') + ' ' +
								elem.css('paddingBottom') + ' ' +
								elem.css('paddingLeft');
			originalPaddingTotalWidth = (parseInt(elem.css('paddingLeft')) || 0) +
										(parseInt(elem.css('paddingRight')) || 0);

			initialise(s);

			function initialise(s)
			{

				var clonedElem, tempWrapper, /*firstChild, lastChild, */isMaintainingPositon, lastContentX, lastContentY,
						hasContainingSpaceChanged;

				settings = s;

				if (pane == undefined) {

					elem.css(
						{
							'overflow': 'hidden',
							'padding': 0
						}
					);
					// TODO: Deal with where width/ height is 0 as it probably means the element is hidden and we should
					// come back to it later and check once it is unhidden...
					paneWidth = elem.innerWidth() + originalPaddingTotalWidth;
					paneHeight = elem.innerHeight();

					elem.width(paneWidth);
					
					pane = $('<div class="jspPane" />').wrap(
						$('<div class="jspContainer" />')
							.css({
								'width': paneWidth + 'px',
								'height': paneHeight + 'px'
							}
						)
					);

					elem.wrapInner(pane.parent());
					// Need to get the vars after being added to the document, otherwise they reference weird
					// disconnected orphan elements...
					container = elem.find('>.jspContainer');
					pane = container.find('>.jspPane');
					pane.css('padding', originalPadding);

					/*
					// Move any margins from the first and last children up to the container so they can still
					// collapse with neighbouring elements as they would before jScrollPane 
					firstChild = pane.find(':first-child');
					lastChild = pane.find(':last-child');
					elem.css(
						{
							'margin-top': firstChild.css('margin-top'),
							'margin-bottom': lastChild.css('margin-bottom')
						}
					);
					firstChild.css('margin-top', 0);
					lastChild.css('margin-bottom', 0);
					*/
				} else {
					elem.css('width', '');

					hasContainingSpaceChanged = elem.outerWidth() + originalPaddingTotalWidth != paneWidth || elem.outerHeight() != paneHeight;

					if (hasContainingSpaceChanged) {
						//BeansBox changes
						paneWidth = jQuery(window).width();//elem.innerWidth() + originalPaddingTotalWidth;
						paneHeight = elem.innerHeight();
						container.css({
							'width': paneWidth + 'px',
							'height': paneHeight + 'px'
						});
					}

					previousPaneWidth = pane.innerWidth();

					if (!hasContainingSpaceChanged && pane.outerWidth() == contentWidth && pane.outerHeight() == contentHeight) {
						// Nothing has changed since we last initialised
						if (isScrollableH || isScrollableV) { // If we had already set a width then re-set it
							pane.css('width', previousPaneWidth + 'px');
							elem.css('width', (previousPaneWidth + originalPaddingTotalWidth) + 'px');
						}
						// Then abort...
						return;
					}
					
					pane.css('width', '');
					elem.css('width', (paneWidth ) + 'px');

					container.find('>.jspVerticalBar,>.jspHorizontalBar').remove().end();				
				}

				// Unfortunately it isn't that easy to find out the width of the element as it will always report the
				// width as allowed by its container, regardless of overflow settings.
				// A cunning workaround is to clone the element, set its position to absolute and place it in a narrow
				// container. Now it will push outwards to its maxium real width...
				clonedElem = pane.clone().css('position', 'absolute');
				tempWrapper = $('<div style="width:1px; position: relative;" />').append(clonedElem);
				$('body').append(tempWrapper);
				contentWidth = Math.max(pane.outerWidth(), clonedElem.outerWidth());
				tempWrapper.remove();
				
				contentHeight = pane.outerHeight();
				percentInViewH = contentWidth / paneWidth;
				percentInViewV = contentHeight / paneHeight;
				isScrollableV = percentInViewV > 1;

				isScrollableH = percentInViewH > 1;

				//console.log(paneWidth, paneHeight, contentWidth, contentHeight, percentInViewH, percentInViewV, isScrollableH, isScrollableV);

				if (!(isScrollableH || isScrollableV)) {
					elem.removeClass('jspScrollable');
					pane.css({
						'top': 0,
						'width': container.width() - originalPaddingTotalWidth
					});
					removeMousewheel();
					removeFocusHandler();
					removeKeyboardNav();
					removeClickOnTrack();
					unhijackInternalLinks();
				} else {
					elem.addClass('jspScrollable');

					isMaintainingPositon = settings.maintainPosition && (verticalDragPosition || horizontalDragPosition);
					if (isMaintainingPositon) {
						lastContentX = contentPositionX();
						lastContentY = contentPositionY();
					}

					initialiseVerticalScroll();
					initialiseHorizontalScroll();
					resizeScrollbars();

					if (isMaintainingPositon) {
						scrollToX(lastContentX);
						scrollToY(lastContentY);
					}

					initFocusHandler();
					initMousewheel();
					if (settings.enableKeyboardNavigation) {
						initKeyboardNav();
					}
					if (settings.clickOnTrack) {
						initClickOnTrack();
					}
					
					observeHash();
					if (settings.hijackInternalLinks) {
						hijackInternalLinks();
					}
				}

				if (settings.autoReinitialise && !reinitialiseInterval) {
					reinitialiseInterval = setInterval(
						function()
						{
							initialise(settings);
						},
						settings.autoReinitialiseDelay
					);
				} else if (!settings.autoReinitialise && reinitialiseInterval) {
					clearInterval(reinitialiseInterval)
				}

				elem.trigger('jsp-initialised', [isScrollableH || isScrollableV]);
			}

			function initialiseVerticalScroll()
			{
				if (isScrollableV) {

					container.append(
						$('<div class="jspVerticalBar" />').append(
							$('<div class="jspCap jspCapTop" />'),
							$('<div class="jspTrack" />').append(
								$('<div class="jspDrag" />').append(
									$('<div class="jspDragTop" />'),
									$('<div class="jspDragBottom" />')
								)
							),
							$('<div class="jspCap jspCapBottom" />')
						)
					);

					verticalBar = container.find('>.jspVerticalBar');
					verticalTrack = verticalBar.find('>.jspTrack');
					verticalDrag = verticalTrack.find('>.jspDrag');

					if (settings.showArrows) {
						arrowUp = $('<a class="jspArrow jspArrowUp" />').bind(
							'mousedown.jsp', getArrowScroll(0, -1)
						).bind('click.jsp', nil);
						arrowDown = $('<a class="jspArrow jspArrowDown" />').bind(
							'mousedown.jsp', getArrowScroll(0, 1)
						).bind('click.jsp', nil);
						if (settings.arrowScrollOnHover) {
							arrowUp.bind('mouseover.jsp', getArrowScroll(0, -1, arrowUp));
							arrowDown.bind('mouseover.jsp', getArrowScroll(0, 1, arrowDown));
						}

						appendArrows(verticalTrack, settings.verticalArrowPositions, arrowUp, arrowDown);
					}

					verticalTrackHeight = paneHeight;
					container.find('>.jspVerticalBar>.jspCap:visible,>.jspVerticalBar>.jspArrow').each(
						function()
						{
							verticalTrackHeight -= $(this).outerHeight();
						}
					);


					verticalDrag.hover(
						function()
						{
							verticalDrag.addClass('jspHover');
						},
						function()
						{
							verticalDrag.removeClass('jspHover');
						}
					).bind(
						'mousedown.jsp',
						function(e)
						{
							// Stop IE from allowing text selection
							$('html').bind('dragstart.jsp selectstart.jsp', function() { return false; });

							verticalDrag.addClass('jspActive');

							var startY = e.pageY - verticalDrag.position().top;

							$('html').bind(
								'mousemove.jsp',
								function(e)
								{
									positionDragY(e.pageY - startY, false);
								}
							).bind('mouseup.jsp mouseleave.jsp', cancelDrag);
							return false;
						}
					);
					sizeVerticalScrollbar();
				}
			}

			function sizeVerticalScrollbar()
			{
				verticalTrack.height(verticalTrackHeight + 'px');
				verticalDragPosition = 0;
				scrollbarWidth = settings.verticalGutter + verticalTrack.outerWidth();

				// Make the pane thinner to allow for the vertical scrollbar
				pane.width(paneWidth - scrollbarWidth - originalPaddingTotalWidth);

				// Add margin to the left of the pane if scrollbars are on that side (to position
				// the scrollbar on the left or right set it's left or right property in CSS)
				if (verticalBar.position().left == 0) {
					pane.css('margin-left', scrollbarWidth + 'px');
				}
			}

			function initialiseHorizontalScroll()
			{
				if (isScrollableH) {

					container.append(
						$('<div class="jspHorizontalBar" />').append(
							$('<div class="jspCap jspCapLeft" />'),
							$('<div class="jspTrack" />').append(
								$('<div class="jspDrag" />').append(
									$('<div class="jspDragLeft" />'),
									$('<div class="jspDragRight" />')
								)
							),
							$('<div class="jspCap jspCapRight" />')
						)
					);

					jQuery('.jspDrag').before('<div class="slider-bg-left"></div>');
					jQuery('.jspDrag').after('<div class="slider-bg-right"></div>');

					horizontalBar = container.find('>.jspHorizontalBar');
					horizontalTrack = horizontalBar.find('>.jspTrack');
					horizontalDrag = horizontalTrack.find('>.jspDrag');

					if (settings.showArrows) {
						arrowLeft = $('<a class="jspArrow jspArrowLeft" />').bind(
							'mousedown.jsp', getArrowScroll(-1, 0)
						).bind('click.jsp', nil);
						arrowRight = $('<a class="jspArrow jspArrowRight" />').bind(
							'mousedown.jsp', getArrowScroll(1, 0)
						).bind('click.jsp', nil);
						if (settings.arrowScrollOnHover) {
							arrowLeft.bind('mouseover.jsp', getArrowScroll(-1, 0, arrowLeft));
							arrowRight.bind('mouseover.jsp', getArrowScroll(1, 0, arrowRight));
						}
						appendArrows(horizontalTrack, settings.horizontalArrowPositions, arrowLeft, arrowRight);
					}

					horizontalDrag.hover(
						function()
						{
							horizontalDrag.addClass('jspHover');
						},
						function()
						{
							horizontalDrag.removeClass('jspHover');
						}
					).bind(
						'mousedown.jsp',
						function(e)
						{
							// Stop IE from allowing text selection
							$('html').bind('dragstart.jsp selectstart.jsp', function() { return false; });

							horizontalDrag.addClass('jspActive');

							var startX = e.pageX - horizontalDrag.position().left;

							$('html').bind(
								'mousemove.jsp',
								function(e)
								{
									positionDragX(e.pageX - startX, false);
								}
							).bind('mouseup.jsp mouseleave.jsp', cancelDrag);
							return false;
						}
					);
					horizontalTrackWidth = container.innerWidth();
					sizeHorizontalScrollbar();
				} else {
					// no horizontal scroll
				}
			}

			function sizeHorizontalScrollbar()
			{

				container.find('>.jspHorizontalBar>.jspCap:visible,>.jspHorizontalBar>.jspArrow').each(
					function()
					{
						horizontalTrackWidth -= $(this).outerWidth();
					}
				);

				horizontalTrack.width(horizontalTrackWidth + 'px');
				horizontalDragPosition = 0;
			}

			function resizeScrollbars()
			{
				if (isScrollableH && isScrollableV) {
					var horizontalTrackHeight = horizontalTrack.outerHeight(),
						verticalTrackWidth = verticalTrack.outerWidth();
					verticalTrackHeight -= horizontalTrackHeight;
					$(horizontalBar).find('>.jspCap:visible,>.jspArrow').each(
						function()
						{
							horizontalTrackWidth += $(this).outerWidth();
						}
					);
					horizontalTrackWidth -= verticalTrackWidth;
					paneHeight -= verticalTrackWidth;
					paneWidth -= horizontalTrackHeight;
					horizontalTrack.parent().append(
						$('<div class="jspCorner" />').css('width', horizontalTrackHeight + 'px')
					);
					sizeVerticalScrollbar();
					sizeHorizontalScrollbar();
				}
				// reflow content
				if (isScrollableH) {
					pane.width((container.outerWidth() - originalPaddingTotalWidth) + 'px');
				}
				contentHeight = pane.outerHeight();
				percentInViewV = contentHeight / paneHeight;

				if (isScrollableH) {
					horizontalDragWidth = 1 / percentInViewH * horizontalTrackWidth;
					if (horizontalDragWidth > settings.horizontalDragMaxWidth) {
						horizontalDragWidth = settings.horizontalDragMaxWidth;
					} else if (horizontalDragWidth < settings.horizontalDragMinWidth) {
						horizontalDragWidth = settings.horizontalDragMinWidth;
					}
					horizontalDrag.width(horizontalDragWidth + 'px');
					dragMaxX = horizontalTrackWidth - horizontalDragWidth;
					_positionDragX(horizontalDragPosition); // To update the state for the arrow buttons
				}
				if (isScrollableV) {
					verticalDragHeight = 1 / percentInViewV * verticalTrackHeight;
					if (verticalDragHeight > settings.verticalDragMaxHeight) {
						verticalDragHeight = settings.verticalDragMaxHeight;
					} else if (verticalDragHeight < settings.verticalDragMinHeight) {
						verticalDragHeight = settings.verticalDragMinHeight;
					}
					verticalDrag.height(verticalDragHeight + 'px');
					dragMaxY = verticalTrackHeight - verticalDragHeight;
					_positionDragY(verticalDragPosition); // To update the state for the arrow buttons
				}
				
				//BeansBox changes
				jQuery('.jspHorizontalBar .jspArrow').css('width','78px')
			}

			function appendArrows(ele, p, a1, a2)
			{
				var p1 = "before", p2 = "after", aTemp;
				
				// Sniff for mac... Is there a better way to determine whether the arrows would naturally appear
				// at the top or the bottom of the bar?
				if (p == "os") {
					p = /Mac/.test(navigator.platform) ? "after" : "split";
				}
				if (p == p1) {
					p2 = p;
				} else if (p == p2) {
					p1 = p;
					aTemp = a1;
					a1 = a2;
					a2 = aTemp;
				}

				ele[p1](a1)[p2](a2);
			}

			function getArrowScroll(dirX, dirY, ele) {
				return function()
				{
					arrowScroll(dirX, dirY, this, ele);
					this.blur();
					return false;
				}
			}

			function arrowScroll(dirX, dirY, arrow, ele)
			{
				arrow = $(arrow).addClass('jspActive');

				var eve, doScroll = function()
					{
						if (dirX != 0) {
							positionDragX(horizontalDragPosition + dirX * settings.arrowButtonSpeed, false);
						}
						if (dirY != 0) {
							positionDragY(verticalDragPosition + dirY * settings.arrowButtonSpeed, false);
						}
					},
					scrollInt = setInterval(doScroll, settings.arrowRepeatFreq);

				doScroll();

				eve = ele == undefined ? 'mouseup.jsp' : 'mouseout.jsp';
				ele = ele || $('html');
				ele.bind(
					eve,
					function()
					{
						arrow.removeClass('jspActive');
						clearInterval(scrollInt);
						ele.unbind(eve);
					}
				);
			}

			function initClickOnTrack()
			{
				removeClickOnTrack();
				if (isScrollableV) {
					verticalTrack.bind(
						'mousedown.jsp',
						function(e)
						{
							if (e.originalTarget == undefined || e.originalTarget == e.currentTarget) {
								var clickedTrack = $(this),
									scrollInt = setInterval(
										function()
										{
											var offset = clickedTrack.offset(), pos = e.pageY - offset.top;
											if (verticalDragPosition + verticalDragHeight < pos) {
												positionDragY(verticalDragPosition + settings.trackClickSpeed);
											} else if (pos < verticalDragPosition) {
												positionDragY(verticalDragPosition - settings.trackClickSpeed);
											} else {
												cancelClick();
											}
										},
										settings.trackClickRepeatFreq
									),
									cancelClick = function()
									{
										scrollInt && clearInterval(scrollInt);
										scrollInt = null;
										$(document).unbind('mouseup.jsp', cancelClick);
									};
								$(document).bind('mouseup.jsp', cancelClick);
								return false;
							}
						}
					);
				}
				if (isScrollableH) {
					horizontalTrack.bind(
						'mousedown.jsp',
						function(e)
						{
							if (e.originalTarget == undefined || e.originalTarget == e.currentTarget) {
								var clickedTrack = $(this),
									scrollInt = setInterval(
										function()
										{
											var offset = clickedTrack.offset(), pos = e.pageX - offset.left;
											if (horizontalDragPosition + horizontalDragWidth < pos) {
												positionDragX(horizontalDragPosition + settings.trackClickSpeed);
											} else if (pos < horizontalDragPosition) {
												positionDragX(horizontalDragPosition - settings.trackClickSpeed);
											} else {
												cancelClick();
											}
										},
										settings.trackClickRepeatFreq
									),
									cancelClick = function()
									{
										scrollInt && clearInterval(scrollInt);
										scrollInt = null;
										$(document).unbind('mouseup.jsp', cancelClick);
									};
								$(document).bind('mouseup.jsp', cancelClick);
								return false;
							}
						}
					);
				}
			}

			function removeClickOnTrack()
			{
				horizontalTrack && horizontalTrack.unbind('mousedown.jsp');
				verticalTrack && verticalTrack.unbind('mousedown.jsp');
			}

			function cancelDrag()
			{
				$('html').unbind('dragstart.jsp selectstart.jsp mousemove.jsp mouseup.jsp mouseleave.jsp');

				verticalDrag && verticalDrag.removeClass('jspActive');
				horizontalDrag && horizontalDrag.removeClass('jspActive');
			}

			function positionDragY(destY, animate)
			{
				if (!isScrollableV) {
					return;
				}
				if (destY < 0) {
					destY = 0;
				} else if (destY > dragMaxY) {
					destY = dragMaxY;
				}

				// can't just check if(animate) because false is a valid value that could be passed in...
				if (animate == undefined) {
					animate = settings.animateScroll;
				}
				if (animate) {
					jsp.animate(verticalDrag, 'top', destY,	_positionDragY);
				} else {
					verticalDrag.css('top', destY);
					_positionDragY(destY);
				}

			}

			function _positionDragY(destY)
			{
				if (destY == undefined) {
					destY = verticalDrag.position().top;
				}

				container.scrollTop(0);
				verticalDragPosition = destY;

				var isAtTop = verticalDragPosition == 0,
					isAtBottom = verticalDragPosition == dragMaxY,
					percentScrolled = destY/ dragMaxY,
					destTop = -percentScrolled * (contentHeight - paneHeight);

				if (wasAtTop != isAtTop || wasAtBottom != isAtBottom) {
					wasAtTop = isAtTop;
					wasAtBottom = isAtBottom;
					elem.trigger('jsp-arrow-change', [wasAtTop, wasAtBottom, wasAtLeft, wasAtRight]);
				}
				
				updateVerticalArrows(isAtTop, isAtBottom);
				pane.css('top', destTop);
				elem.trigger('jsp-scroll-y', [-destTop, isAtTop, isAtBottom]);
			}

			function positionDragX(destX, animate)
			{
				if (!isScrollableH) {
					return;
				}
				if (destX < 0) {
					destX = 0;
				} else if (destX > dragMaxX) {
					destX = dragMaxX;
				}

				if (animate == undefined) {
					animate = settings.animateScroll;
				}
				if (animate) {
					jsp.animate(horizontalDrag, 'left', destX,	_positionDragX);
				} else {
					horizontalDrag.css('left', destX);
					_positionDragX(destX);
				}
			}

			function _positionDragX(destX)
			{
				if (destX == undefined) {
					destX = horizontalDrag.position().left;
				}

				container.scrollTop(0);
				horizontalDragPosition = destX;

				var isAtLeft = horizontalDragPosition == 0,
					isAtRight = horizontalDragPosition == dragMaxX,
					percentScrolled = destX / dragMaxX,
					destLeft = -percentScrolled * (contentWidth - paneWidth);

				if (wasAtLeft != isAtLeft || wasAtRight != isAtRight) {
					wasAtLeft = isAtLeft;
					wasAtRight = isAtRight;
					elem.trigger('jsp-arrow-change', [wasAtTop, wasAtBottom, wasAtLeft, wasAtRight]);
				}
				
				updateHorizontalArrows(isAtLeft, isAtRight);
				pane.css('left', destLeft);
				elem.trigger('jsp-scroll-x', [-destLeft, isAtLeft, isAtRight]);
			}

			function updateVerticalArrows(isAtTop, isAtBottom)
			{
				if (settings.showArrows) {
					arrowUp[isAtTop ? 'addClass' : 'removeClass']('jspDisabled');
					arrowDown[isAtBottom ? 'addClass' : 'removeClass']('jspDisabled');
				}
			}

			function updateHorizontalArrows(isAtLeft, isAtRight)
			{
				if (settings.showArrows) {
					arrowLeft[isAtLeft ? 'addClass' : 'removeClass']('jspDisabled');
					arrowRight[isAtRight ? 'addClass' : 'removeClass']('jspDisabled');
				}
			}

			function scrollToY(destY, animate)
			{
				var percentScrolled = destY / (contentHeight - paneHeight);
				positionDragY(percentScrolled * dragMaxY, animate);
			}

			function scrollToX(destX, animate)
			{
				var percentScrolled = destX / (contentWidth - paneWidth);
				positionDragX(percentScrolled * dragMaxX, animate);
			}

			function scrollToElement(ele, stickToTop, animate)
			{
				var e, eleHeight, eleWidth, eleTop = 0, eleLeft = 0, viewportTop, maxVisibleEleTop, maxVisibleEleLeft, destY, destX;

				// Legal hash values aren't necessarily legal jQuery selectors so we need to catch any
				// errors from the lookup...
				try {
					e = $(ele);
				} catch (err) {
					return;
				}
				eleHeight = e.outerHeight();
				eleWidth= e.outerWidth();

				container.scrollTop(0);
				container.scrollLeft(0);
				
				// loop through parents adding the offset top of any elements that are relatively positioned between
				// the focused element and the jspPane so we can get the true distance from the top
				// of the focused element to the top of the scrollpane...
				while (!e.is('.jspPane')) {
					eleTop += e.position().top;
					eleLeft += e.position().left;
					e = e.offsetParent();
					if (/^body|html$/i.test(e[0].nodeName)) {
						// we ended up too high in the document structure. Quit!
						return;
					}
				}

				viewportTop = contentPositionY();
				maxVisibleEleTop = viewportTop + paneHeight;
				if (eleTop < viewportTop || stickToTop) { // element is above viewport
					destY = eleTop - settings.verticalGutter;
				} else if (eleTop + eleHeight > maxVisibleEleTop) { // element is below viewport
					destY = eleTop - paneHeight + eleHeight + settings.verticalGutter;
				}
				if (destY) {
					scrollToY(destY, animate);
				}
				
				viewportLeft = contentPositionX();
	            maxVisibleEleLeft = viewportLeft + paneWidth;
	            if (eleLeft < viewportLeft || stickToTop) { // element is to the left of viewport
	                destX = eleLeft - settings.horizontalGutter;
	            } else if (eleLeft + eleWidth > maxVisibleEleLeft) { // element is to the right viewport
	                destX = eleLeft - paneWidth + eleWidth + settings.horizontalGutter;
	            }
	            if (destX) {
	                scrollToX(destX, animate);
	            }

			}

			function contentPositionX()
			{
				return -pane.position().left;
			}

			function contentPositionY()
			{
				return -pane.position().top;
			}

			function initMousewheel()
			{
				container.unbind(mwEvent).bind(
					mwEvent,
					function (event, delta, deltaX, deltaY) {
						var dX = horizontalDragPosition, dY = verticalDragPosition;
						positionDragX(horizontalDragPosition + deltaX * settings.mouseWheelSpeed * paneWidth / (contentWidth - paneWidth), false)
						positionDragY(verticalDragPosition - deltaY * settings.mouseWheelSpeed * paneHeight / (contentHeight - paneHeight), false);
						// return true if there was no movement so rest of screen can scroll
						return dX == horizontalDragPosition && dY == verticalDragPosition;
					}
				);
			}

			function removeMousewheel()
			{
				container.unbind(mwEvent);
			}

			function nil()
			{
				return false;
			}

			function initFocusHandler()
			{
				pane.unbind('focus.jsp').bind(
					'focus.jsp',
					function(e)
					{
						if(e.target === pane[0]){return;}
						scrollToElement(e.target, false);
					}
				);
			}

			function removeFocusHandler()
			{

				pane.unbind('focus.jsp');
			}
			
			function initKeyboardNav()
			{
				var pressed, pressedTimer;
				elem.attr('tabindex', 0)
					.unbind('keydown.jsp')
					.bind(
						'keydown.jsp',
						function(e)
						{
							if(e.target !== elem[0]){
								return;
							}
							var dX = horizontalDragPosition, dY = verticalDragPosition, step = pressed ? 2 : 16;
							switch(e.keyCode) {
								case 40: // down
									positionDragY(verticalDragPosition + step, false);
									break;
								case 38: // up
									positionDragY(verticalDragPosition - step, false);
									break;
								case 34: // page down
								case 32: // space
									scrollToY(contentPositionY() + Math.max(32, paneHeight) - 16);
									break;
								case 33: // page up
									scrollToY(contentPositionY() - paneHeight + 16);
									break;
								case 35: // end
									scrollToY(contentHeight - paneHeight);
									break;
								case 36: // home
									scrollToY(0);
									break;
								case 39: // right
									positionDragX(horizontalDragPosition + step, false);
									break;
								case 37: // left
									positionDragX(horizontalDragPosition - step, false);
									break;
							}

							if( !(dX == horizontalDragPosition && dY == verticalDragPosition) ){
								pressed = true;
								clearTimeout(pressedTimer);
								pressedTimer = setTimeout(function(){
									pressed = false;
								}, 260);
								return false;
							}
						}
					);
				if(settings.hideFocus) {
					elem.css('outline', 'none');
					if('hideFocus' in container[0]){
						elem.attr('hideFocus', true);
					}
				} else {
					elem.css('outline', '');
					if('hideFocus' in container[0]){
						elem.attr('hideFocus', false);
					}
				}
			}
			
			function removeKeyboardNav()
			{
				elem.attr('tabindex', '-1')
					.removeAttr('tabindex')
					.unbind('keydown.jsp');
			}

			function observeHash()
			{
				if (location.hash && location.hash.length > 1) {
					var e, retryInt;
					try {
						e = $(location.hash);
					} catch (err) {
						return;
					}

					if (e.length && pane.find(e)) {
						// nasty workaround but it appears to take a little while before the hash has done its thing
						// to the rendered page so we just wait until the container's scrollTop has been messed up.
						if (container.scrollTop() == 0) {
							retryInt = setInterval(
								function()
								{
									if (container.scrollTop() > 0) {
										scrollToElement(location.hash, true);
										$(document).scrollTop(container.position().top);
										clearInterval(retryInt);
									}
								},
								50
							)
						} else {
							scrollToElement(location.hash, true);
							$(document).scrollTop(container.position().top);
						}
					}
				}
			}

			function unhijackInternalLinks()
			{
				$('a.jspHijack').unbind('click.jsp-hijack').removeClass('jspHijack');
			}

			function hijackInternalLinks()
			{
				unhijackInternalLinks();
				$('a[href^=#]').addClass('jspHijack').bind(
					'click.jsp-hijack',
					function()
					{
						var uriParts = this.href.split('#'), hash;
						if (uriParts.length > 1) {
							hash = uriParts[1];
							if (hash.length > 0 && pane.find('#' + hash).length > 0) {
								scrollToElement('#' + hash, true);
								// Need to return false otherwise things mess up... Would be nice to maybe also scroll
								// the window to the top of the scrollpane?
								return false;
							}
						}
					}
				)
			}

			// Public API
			$.extend(
				jsp,
				{
					// Reinitialises the scroll pane (if it's internal dimensions have changed since the last time it
					// was initialised). The settings object which is passed in will override any settings from the
					// previous time it was initialised - if you don't pass any settings then the ones from the previous
					// initialisation will be used.
					reinitialise: function(s)
					{
						s = $.extend({}, s, settings);
						initialise(s);
					},
					// Scrolls the specified element (a jQuery object, DOM node or jQuery selector string) into view so
					// that it can be seen within the viewport. If stickToTop is true then the element will appear at
					// the top of the viewport, if it is false then the viewport will scroll as little as possible to
					// show the element. You can also specify if you want animation to occur. If you don't provide this
					// argument then the animateScroll value from the settings object is used instead.
					scrollToElement: function(ele, stickToTop, animate)
					{
						scrollToElement(ele, stickToTop, animate);
					},
					// Scrolls the pane so that the specified co-ordinates within the content are at the top left
					// of the viewport. animate is optional and if not passed then the value of animateScroll from
					// the settings object this jScrollPane was initialised with is used.
					scrollTo: function(destX, destY, animate)
					{
						scrollToX(destX, animate);
						scrollToY(destY, animate);
					},
					// Scrolls the pane so that the specified co-ordinate within the content is at the left of the
					// viewport. animate is optional and if not passed then the value of animateScroll from the settings
					// object this jScrollPane was initialised with is used.
					scrollToX: function(destX, animate)
					{
						scrollToX(destX, animate);
					},
					// Scrolls the pane so that the specified co-ordinate within the content is at the top of the
					// viewport. animate is optional and if not passed then the value of animateScroll from the settings
					// object this jScrollPane was initialised with is used.
					scrollToY: function(destY, animate)
					{
						scrollToY(destY, animate);
					},
					// Scrolls the pane by the specified amount of pixels. animate is optional and if not passed then
					// the value of animateScroll from the settings object this jScrollPane was initialised with is used.
					scrollBy: function(deltaX, deltaY, animate)
					{
						jsp.scrollByX(deltaX, animate);
						jsp.scrollByY(deltaY, animate);
					},
					// Scrolls the pane by the specified amount of pixels. animate is optional and if not passed then
					// the value of animateScroll from the settings object this jScrollPane was initialised with is used.
					scrollByX: function(deltaX, animate)
					{
						var destX = contentPositionX() + deltaX,
							percentScrolled = destX / (contentWidth - paneWidth);
						positionDragX(percentScrolled * dragMaxX, animate);
					},
					// Scrolls the pane by the specified amount of pixels. animate is optional and if not passed then
					// the value of animateScroll from the settings object this jScrollPane was initialised with is used.
					scrollByY: function(deltaY, animate)
					{
						var destY = contentPositionY() + deltaY,
							percentScrolled = destY / (contentHeight - paneHeight);
						positionDragY(percentScrolled * dragMaxY, animate);
					},
					// This method is called when jScrollPane is trying to animate to a new position. You can override
					// it if you want to provide advanced animation functionality. It is passed the following arguments:
					//  * ele          - the element whose position is being animated
					//  * prop         - the property that is being animated
					//  * value        - the value it's being animated to
					//  * stepCallback - a function that you must execute each time you update the value of the property
					// You can use the default implementation (below) as a starting point for your own implementation.
					animate: function(ele, prop, value, stepCallback)
					{
						var params = {};
						params[prop] = value;
						ele.animate(
							params,
							{
								'duration'	: settings.animateDuration,
								'ease'		: settings.animateEase,
								'queue'		: false,
								'step'		: stepCallback
							}
						);
					},
					// Returns the current x position of the viewport with regards to the content pane.
					getContentPositionX: function()
					{
						return contentPositionX();
					},
					// Returns the current y position of the viewport with regards to the content pane.
					getContentPositionY: function()
					{
						return contentPositionY();
					},
					// Returns whether or not this scrollpane has a horizontal scrollbar.
					getIsScrollableH: function()
					{
						return isScrollableH;
					},
					// Returns whether or not this scrollpane has a vertical scrollbar.
					getIsScrollableV: function()
					{
						return isScrollableV;
					},
					// Gets a reference to the content pane. It is important that you use this method if you want to
					// edit the content of your jScrollPane as if you access the element directly then you may have some
					// problems (as your original element has had additional elements for the scrollbars etc added into
					// it).
					getContentPane: function()
					{
						return pane;
					},
					// Scrolls this jScrollPane down as far as it can currently scroll. If animate isn't passed then the
					// animateScroll value from settings is used instead.
					scrollToBottom: function(animate)
					{
						positionDragY(dragMaxY, animate);
					},
					// Hijacks the links on the page which link to content inside the scrollpane. If you have changed
					// the content of your page (e.g. via AJAX) and want to make sure any new anchor links to the
					// contents of your scroll pane will work then call this function.
					hijackInternalLinks: function()
					{
						hijackInternalLinks();
					}
				}
			);
		}

		// Pluginifying code...

		settings = $.extend({}, $.fn.jScrollPane.defaults, settings);

		var ret;
		this.each(
			function()
			{
				var elem = $(this), jspApi = elem.data('jsp');
				if (jspApi) {
					jspApi.reinitialise(settings);
				} else {
					jspApi = new JScrollPane(elem, settings);
					elem.data('jsp', jspApi);
				}
				ret = ret ? ret.add(elem) : elem;
			}
		)
		return ret;
	};

	$.fn.jScrollPane.defaults = {
		'showArrows'				: false,
		'maintainPosition'			: true,
		'clickOnTrack'				: true,
		'autoReinitialise'			: false,
		'autoReinitialiseDelay'		: 500,
		'verticalDragMinHeight'		: 0,
		'verticalDragMaxHeight'		: 99999,
		'horizontalDragMinWidth'	: 0,
		'horizontalDragMaxWidth'	: 99999,
		'animateScroll'				: false,
		'animateDuration'			: 300,
		'animateEase'				: 'linear',
		'hijackInternalLinks'		: false,
		'verticalGutter'			: 4,
		'horizontalGutter'			: 4,
		'mouseWheelSpeed'			: 30,
		'arrowButtonSpeed'			: 30,
		'arrowRepeatFreq'			: 100,
		'arrowScrollOnHover'		: false,
		'trackClickSpeed'			: 30,
		'trackClickRepeatFreq'		: 100,
		'verticalArrowPositions'	: 'split',
		'horizontalArrowPositions'	: 'split',
		'enableKeyboardNavigation'	: true,
		'hideFocus'					: false
	};

})(jQuery,this);
;
/*! Copyright (c) 2010 Brandon Aaron (http://brandonaaron.net)
 * Licensed under the MIT License (LICENSE.txt).
 *
 * Thanks to: http://adomas.org/javascript-mouse-wheel/ for some pointers.
 * Thanks to: Mathias Bank(http://www.mathias-bank.de) for a scope bug fix.
 * Thanks to: Seamus Leahy for adding deltaX and deltaY
 *
 * Version: 3.0.4
 * 
 * Requires: 1.2.2+
 */

(function($) {

var types = ['DOMMouseScroll', 'mousewheel'];

$.event.special.mousewheel = {
    setup: function() {
        if ( this.addEventListener ) {
            for ( var i=types.length; i; ) {
                this.addEventListener( types[--i], handler, false );
            }
        } else {
            this.onmousewheel = handler;
        }
    },
    
    teardown: function() {
        if ( this.removeEventListener ) {
            for ( var i=types.length; i; ) {
                this.removeEventListener( types[--i], handler, false );
            }
        } else {
            this.onmousewheel = null;
        }
    }
};

$.fn.extend({
    mousewheel: function(fn) {
        return fn ? this.bind("mousewheel", fn) : this.trigger("mousewheel");
    },
    
    unmousewheel: function(fn) {
        return this.unbind("mousewheel", fn);
    }
});


function handler(event) {
    var orgEvent = event || window.event, args = [].slice.call( arguments, 1 ), delta = 0, returnValue = true, deltaX = 0, deltaY = 0;
    event = $.event.fix(orgEvent);
    event.type = "mousewheel";
    
    // Old school scrollwheel delta
    if ( event.wheelDelta ) { delta = event.wheelDelta/120; }
    if ( event.detail     ) { delta = -event.detail/3; }
    
    // New school multidimensional scroll (touchpads) deltas
    deltaY = delta;
    
    // Gecko
    if ( orgEvent.axis !== undefined && orgEvent.axis === orgEvent.HORIZONTAL_AXIS ) {
        deltaY = 0;
        deltaX = -1*delta;
    }
    
    // Webkit
    if ( orgEvent.wheelDeltaY !== undefined ) { deltaY = orgEvent.wheelDeltaY/120; }
    if ( orgEvent.wheelDeltaX !== undefined ) { deltaX = -1*orgEvent.wheelDeltaX/120; }
    
    // Add event and delta to the front of the arguments
    args.unshift(event, delta, deltaX, deltaY);
    
    return $.event.handle.apply(this, args);
}

})(jQuery);;
/* ===========================================================================
 *
 * JQuery URL Parser
 * Version 1.0
 * Parses URLs and provides easy access to information within them.
 *
 * Author: Mark Perkins
 * Author email: mark@allmarkedup.com
 *
 * For full documentation and more go to http://projects.allmarkedup.com/jquery_url_parser/
 *
 * ---------------------------------------------------------------------------
 *
 * CREDITS:
 *
 * Parser based on the Regex-based URI parser by Steven Levithan.
 * For more information (including a detailed explaination of the differences
 * between the 'loose' and 'strict' pasing modes) visit http://blog.stevenlevithan.com/archives/parseuri
 *
 * ---------------------------------------------------------------------------
 *
 * LICENCE:
 *
 * Released under a MIT Licence. See licence.txt that should have been supplied with this file,
 * or visit http://projects.allmarkedup.com/jquery_url_parser/licence.txt
 *
 * ---------------------------------------------------------------------------
 * 
 * EXAMPLES OF USE:
 *
 * Get the domain name (host) from the current page URL
 * jQuery.url.attr("host")
 *
 * Get the query string value for 'item' for the current page
 * jQuery.url.param("item") // null if it doesn't exist
 *
 * Get the second segment of the URI of the current page
 * jQuery.url.segment(2) // null if it doesn't exist
 *
 * Get the protocol of a manually passed in URL
 * jQuery.url.setUrl("http://allmarkedup.com/").attr("protocol") // returns 'http'
 *
 */

jQuery.url = function()
{
	var segments = {};
	
	var parsed = {};
	
	/**
    * Options object. Only the URI and strictMode values can be changed via the setters below.
    */
  	var options = {
	
		url : window.location, // default URI is the page in which the script is running
		
		strictMode: false, // 'loose' parsing by default
	
		key: ["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","anchor"], // keys available to query 
		
		q: {
			name: "queryKey",
			parser: /(?:^|&)([^&=]*)=?([^&]*)/g
		},
		
		parser: {
			strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,  //less intuitive, more accurate to the specs
			loose:  /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/ // more intuitive, fails on relative paths and deviates from specs
		}
		
	};
	
    /**
     * Deals with the parsing of the URI according to the regex above.
 	 * Written by Steven Levithan - see credits at top.
     */		
	var parseUri = function()
	{
		str = decodeURI( options.url );
		
		var m = options.parser[ options.strictMode ? "strict" : "loose" ].exec( str );
		var uri = {};
		var i = 14;

		while ( i-- ) {
			uri[ options.key[i] ] = m[i] || "";
		}

		uri[ options.q.name ] = {};
		uri[ options.key[12] ].replace( options.q.parser, function ( $0, $1, $2 ) {
			if ($1) {
				uri[options.q.name][$1] = $2;
			}
		});

		return uri;
	};

    /**
     * Returns the value of the passed in key from the parsed URI.
  	 * 
	 * @param string key The key whose value is required
     */		
	var key = function( key )
	{
		if ( ! parsed.length )
		{
			setUp(); // if the URI has not been parsed yet then do this first...	
		} 
		if ( key == "base" )
		{
			if ( parsed.port !== null && parsed.port !== "" )
			{
				return parsed.protocol+"://"+parsed.host+":"+parsed.port+"/";	
			}
			else
			{
				return parsed.protocol+"://"+parsed.host+"/";
			}
		}
	
		return ( parsed[key] === "" ) ? null : parsed[key];
	};
	
	/**
     * Returns the value of the required query string parameter.
  	 * 
	 * @param string item The parameter whose value is required
     */		
	var param = function( item )
	{
		if ( ! parsed.length )
		{
			setUp(); // if the URI has not been parsed yet then do this first...	
		}
		return ( parsed.queryKey[item] === null ) ? null : parsed.queryKey[item];
	};

    /**
     * 'Constructor' (not really!) function.
     *  Called whenever the URI changes to kick off re-parsing of the URI and splitting it up into segments. 
     */	
	var setUp = function()
	{
		parsed = parseUri();
		
		getSegments();	
	};
	
    /**
     * Splits up the body of the URI into segments (i.e. sections delimited by '/')
     */
	var getSegments = function()
	{
		var p = parsed.path;
		segments = []; // clear out segments array
		segments = parsed.path.length == 1 ? {} : ( p.charAt( p.length - 1 ) == "/" ? p.substring( 1, p.length - 1 ) : path = p.substring( 1 ) ).split("/");
	};
	
	return {
		
	    /**
	     * Sets the parsing mode - either strict or loose. Set to loose by default.
	     *
	     * @param string mode The mode to set the parser to. Anything apart from a value of 'strict' will set it to loose!
	     */
		setMode : function( mode )
		{
			strictMode = mode == "strict" ? true : false;
			return this;
		},
		
		/**
	     * Sets URI to parse if you don't want to to parse the current page's URI.
		 * Calling the function with no value for newUri resets it to the current page's URI.
	     *
	     * @param string newUri The URI to parse.
	     */		
		setUrl : function( newUri )
		{
			options.url = newUri === undefined ? window.location : newUri;
			setUp();
			return this;
		},		
		
		/**
	     * Returns the value of the specified URI segment. Segments are numbered from 1 to the number of segments.
		 * For example the URI http://test.com/about/company/ segment(1) would return 'about'.
		 *
		 * If no integer is passed into the function it returns the number of segments in the URI.
	     *
	     * @param int pos The position of the segment to return. Can be empty.
	     */	
		segment : function( pos )
		{
			if ( ! parsed.length )
			{
				setUp(); // if the URI has not been parsed yet then do this first...	
			} 
			if ( pos === undefined )
			{
				return segments.length;
			}
			return ( segments[pos] === "" || segments[pos] === undefined ) ? null : segments[pos];
		},
		
		attr : key, // provides public access to private 'key' function - see above
		
		param : param // provides public access to private 'param' function - see above
		
	};
	
}();;
/*jslint browser: true */ /*global jQuery: true */

/**
 * jQuery Cookie plugin
 *
 * Copyright (c) 2010 Klaus Hartl (stilbuero.de)
 * Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 *
 */

// TODO JsDoc

/**
 * Create a cookie with the given key and value and other optional parameters.
 *
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Set the value of a cookie.
 * @example $.cookie('the_cookie', 'the_value', { expires: 7, path: '/', domain: 'jquery.com', secure: true });
 * @desc Create a cookie with all available options.
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Create a session cookie.
 * @example $.cookie('the_cookie', null);
 * @desc Delete a cookie by passing null as value. Keep in mind that you have to use the same path and domain
 *       used when the cookie was set.
 *
 * @param String key The key of the cookie.
 * @param String value The value of the cookie.
 * @param Object options An object literal containing key/value pairs to provide optional cookie attributes.
 * @option Number|Date expires Either an integer specifying the expiration date from now on in days or a Date object.
 *                             If a negative value is specified (e.g. a date in the past), the cookie will be deleted.
 *                             If set to null or omitted, the cookie will be a session cookie and will not be retained
 *                             when the the browser exits.
 * @option String path The value of the path atribute of the cookie (default: path of page that created the cookie).
 * @option String domain The value of the domain attribute of the cookie (default: domain of page that created the cookie).
 * @option Boolean secure If true, the secure attribute of the cookie will be set and the cookie transmission will
 *                        require a secure protocol (like HTTPS).
 * @type undefined
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */

/**
 * Get the value of a cookie with the given key.
 *
 * @example $.cookie('the_cookie');
 * @desc Get the value of a cookie.
 *
 * @param String key The key of the cookie.
 * @return The value of the cookie.
 * @type String
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */
jQuery.cookie = function (key, value, options) {

    // key and value given, set cookie...
    if (arguments.length > 1 && (value === null || typeof value !== "object")) {
        options = jQuery.extend({}, options);

        if (value === null) {
            options.expires = -1;
        }

        if (typeof options.expires === 'number') {
            var days = options.expires, t = options.expires = new Date();
            t.setDate(t.getDate() + days);
        }

        return (document.cookie = [
            encodeURIComponent(key), '=',
            options.raw ? String(value) : encodeURIComponent(String(value)),
            options.expires ? '; expires=' + options.expires.toUTCString() : '', // use expires attribute, max-age is not supported by IE
            options.path ? '; path=' + options.path : '',
            options.domain ? '; domain=' + options.domain : '',
            options.secure ? '; secure' : ''
        ].join(''));
    }

    // key and possibly options given, get cookie...
    options = value || {};
    var result, decode = options.raw ? function (s) { return s; } : decodeURIComponent;
    return (result = new RegExp('(?:^|; )' + encodeURIComponent(key) + '=([^;]*)').exec(document.cookie)) ? decode(result[1]) : null;
};
;
/**
* @author Remy Sharp
* @url http://remysharp.com/2007/01/25/jquery-tutorial-text-box-hints/
*/

(function ($) {

$.fn.hint = function (blurClass) {
  if (!blurClass) { 
    blurClass = 'blur';
  }
    
  return this.each(function () {
    // get jQuery version of 'this'
    var $input = $(this),
    
    // capture the rest of the variable to allow for reuse
      title = $input.attr('title'),
      $form = $(this.form),
      $win = $(window);

    function remove() {
      if ($input.val() === title && $input.hasClass(blurClass)) {
        $input.val('').removeClass(blurClass);
      }
    }

    // only apply logic if the element has the attribute
    if (title) { 
      // on blur, set value to title attr if text is blank
      $input.blur(function () {
        if (this.value === '') {
          $input.val(title).addClass(blurClass);
        }
      }).focus(remove).blur(); // now change all inputs to title
      
      // clear the pre-defined text when form is submitted
      $form.submit(remove);
      $win.unload(remove); // handles Firefox's autocomplete
    }
  });
};

})(jQuery);;
jQuery.fn.not_exists = function(){return jQuery(this).length==0;}

jQuery.fn.jqcollapse = function(o) {
 
 // Defaults
 var o = jQuery.extend( {
   slide: true,
   speed: 300,
   easing: ''
 },o);
 
 $(this).each(function(){
	 
	 var e = $(this).attr('id');
  
	 $('#'+e+' li > ul').each(function(i) {
	    var parent_li = $(this).parent('li');
	    var sub_ul = $(this).remove();
	    
	    // Create 'a' tag for parent if DNE

	    if (parent_li.children('a').not_exists()) {
	    	parent_li.wrapInner('<a/>');
	    }
	    
	    parent_li.find('a').addClass('jqcNode').css('cursor','pointer').click(function() {
	        if(o.slide==true){
	        	sub_ul.slideToggle(o.speed, o.easing);
	        }else{
	        	sub_ul.toggle();
	        }
	    });
	    parent_li.append(sub_ul);
	});
	
	//Hide all sub-lists
	 $('#'+e+' ul').hide();
	 
 });
 
};;
//v1.7
// Flash Player Version Detection
// Detect Client Browser type
// Copyright 2005-2007 Adobe Systems Incorporated.  All rights reserved.
var isIE  = (navigator.appVersion.indexOf("MSIE") != -1) ? true : false;
var isWin = (navigator.appVersion.toLowerCase().indexOf("win") != -1) ? true : false;
var isOpera = (navigator.userAgent.indexOf("Opera") != -1) ? true : false;

function ControlVersion()
{
	var version;
	var axo;
	var e;

	// NOTE : new ActiveXObject(strFoo) throws an exception if strFoo isn't in the registry

	try {
		// version will be set for 7.X or greater players
		axo = new ActiveXObject("ShockwaveFlash.ShockwaveFlash.7");
		version = axo.GetVariable("$version");
	} catch (e) {
	}

	if (!version)
	{
		try {
			// version will be set for 6.X players only
			axo = new ActiveXObject("ShockwaveFlash.ShockwaveFlash.6");
			
			// installed player is some revision of 6.0
			// GetVariable("$version") crashes for versions 6.0.22 through 6.0.29,
			// so we have to be careful. 
			
			// default to the first public version
			version = "WIN 6,0,21,0";

			// throws if AllowScripAccess does not exist (introduced in 6.0r47)		
			axo.AllowScriptAccess = "always";

			// safe to call for 6.0r47 or greater
			version = axo.GetVariable("$version");

		} catch (e) {
		}
	}

	if (!version)
	{
		try {
			// version will be set for 4.X or 5.X player
			axo = new ActiveXObject("ShockwaveFlash.ShockwaveFlash.3");
			version = axo.GetVariable("$version");
		} catch (e) {
		}
	}

	if (!version)
	{
		try {
			// version will be set for 3.X player
			axo = new ActiveXObject("ShockwaveFlash.ShockwaveFlash.3");
			version = "WIN 3,0,18,0";
		} catch (e) {
		}
	}

	if (!version)
	{
		try {
			// version will be set for 2.X player
			axo = new ActiveXObject("ShockwaveFlash.ShockwaveFlash");
			version = "WIN 2,0,0,11";
		} catch (e) {
			version = -1;
		}
	}
	
	return version;
}

// JavaScript helper required to detect Flash Player PlugIn version information
function GetSwfVer(){
	// NS/Opera version >= 3 check for Flash plugin in plugin array
	var flashVer = -1;
	
	if (navigator.plugins != null && navigator.plugins.length > 0) {
		if (navigator.plugins["Shockwave Flash 2.0"] || navigator.plugins["Shockwave Flash"]) {
			var swVer2 = navigator.plugins["Shockwave Flash 2.0"] ? " 2.0" : "";
			var flashDescription = navigator.plugins["Shockwave Flash" + swVer2].description;
			var descArray = flashDescription.split(" ");
			var tempArrayMajor = descArray[2].split(".");			
			var versionMajor = tempArrayMajor[0];
			var versionMinor = tempArrayMajor[1];
			var versionRevision = descArray[3];
			if (versionRevision == "") {
				versionRevision = descArray[4];
			}
			if (versionRevision[0] == "d") {
				versionRevision = versionRevision.substring(1);
			} else if (versionRevision[0] == "r") {
				versionRevision = versionRevision.substring(1);
				if (versionRevision.indexOf("d") > 0) {
					versionRevision = versionRevision.substring(0, versionRevision.indexOf("d"));
				}
			}
			var flashVer = versionMajor + "." + versionMinor + "." + versionRevision;
		}
	}
	// MSN/WebTV 2.6 supports Flash 4
	else if (navigator.userAgent.toLowerCase().indexOf("webtv/2.6") != -1) flashVer = 4;
	// WebTV 2.5 supports Flash 3
	else if (navigator.userAgent.toLowerCase().indexOf("webtv/2.5") != -1) flashVer = 3;
	// older WebTV supports Flash 2
	else if (navigator.userAgent.toLowerCase().indexOf("webtv") != -1) flashVer = 2;
	else if ( isIE && isWin && !isOpera ) {
		flashVer = ControlVersion();
	}	
	return flashVer;
}

// When called with reqMajorVer, reqMinorVer, reqRevision returns true if that version or greater is available
function DetectFlashVer(reqMajorVer, reqMinorVer, reqRevision)
{
	versionStr = GetSwfVer();
	if (versionStr == -1 ) {
		return false;
	} else if (versionStr != 0) {
		if(isIE && isWin && !isOpera) {
			// Given "WIN 2,0,0,11"
			tempArray         = versionStr.split(" "); 	// ["WIN", "2,0,0,11"]
			tempString        = tempArray[1];			// "2,0,0,11"
			versionArray      = tempString.split(",");	// ['2', '0', '0', '11']
		} else {
			versionArray      = versionStr.split(".");
		}
		var versionMajor      = versionArray[0];
		var versionMinor      = versionArray[1];
		var versionRevision   = versionArray[2];

        	// is the major.revision >= requested major.revision AND the minor version >= requested minor
		if (versionMajor > parseFloat(reqMajorVer)) {
			return true;
		} else if (versionMajor == parseFloat(reqMajorVer)) {
			if (versionMinor > parseFloat(reqMinorVer))
				return true;
			else if (versionMinor == parseFloat(reqMinorVer)) {
				if (versionRevision >= parseFloat(reqRevision))
					return true;
			}
		}
		return false;
	}
}

function AC_AddExtension(src, ext)
{
  if (src.indexOf('?') != -1)
    return src.replace(/\?/, ext+'?'); 
  else
    return src + ext;
}

function AC_Generateobj(objAttrs, params, embedAttrs) 
{ 
  var str = '';
  if (isIE && isWin && !isOpera)
  {
    str += '<object ';
    for (var i in objAttrs)
    {
      str += i + '="' + objAttrs[i] + '" ';
    }
    str += '>';
    for (var i in params)
    {
      str += '<param name="' + i + '" value="' + params[i] + '" /> ';
    }
    str += '</object>';
  }
  else
  {
    str += '<embed ';
    for (var i in embedAttrs)
    {
      str += i + '="' + embedAttrs[i] + '" ';
    }
    str += '> </embed>';
  }

  document.write(str);
}

function AC_FL_RunContent(){
  var ret = 
    AC_GetArgs
    (  arguments, ".swf", "movie", "clsid:d27cdb6e-ae6d-11cf-96b8-444553540000"
     , "application/x-shockwave-flash"
    );
  AC_Generateobj(ret.objAttrs, ret.params, ret.embedAttrs);
}

function AC_SW_RunContent(){
  var ret = 
    AC_GetArgs
    (  arguments, ".dcr", "src", "clsid:166B1BCA-3F9C-11CF-8075-444553540000"
     , null
    );
  AC_Generateobj(ret.objAttrs, ret.params, ret.embedAttrs);
}

function AC_GetArgs(args, ext, srcParamName, classid, mimeType){
  var ret = new Object();
  ret.embedAttrs = new Object();
  ret.params = new Object();
  ret.objAttrs = new Object();
  for (var i=0; i < args.length; i=i+2){
    var currArg = args[i].toLowerCase();    

    switch (currArg){	
      case "classid":
        break;
      case "pluginspage":
        ret.embedAttrs[args[i]] = args[i+1];
        break;
      case "src":
      case "movie":	
        args[i+1] = AC_AddExtension(args[i+1], ext);
        ret.embedAttrs["src"] = args[i+1];
        ret.params[srcParamName] = args[i+1];
        break;
      case "onafterupdate":
      case "onbeforeupdate":
      case "onblur":
      case "oncellchange":
      case "onclick":
      case "ondblclick":
      case "ondrag":
      case "ondragend":
      case "ondragenter":
      case "ondragleave":
      case "ondragover":
      case "ondrop":
      case "onfinish":
      case "onfocus":
      case "onhelp":
      case "onmousedown":
      case "onmouseup":
      case "onmouseover":
      case "onmousemove":
      case "onmouseout":
      case "onkeypress":
      case "onkeydown":
      case "onkeyup":
      case "onload":
      case "onlosecapture":
      case "onpropertychange":
      case "onreadystatechange":
      case "onrowsdelete":
      case "onrowenter":
      case "onrowexit":
      case "onrowsinserted":
      case "onstart":
      case "onscroll":
      case "onbeforeeditfocus":
      case "onactivate":
      case "onbeforedeactivate":
      case "ondeactivate":
      case "type":
      case "codebase":
      case "id":
        ret.objAttrs[args[i]] = args[i+1];
        break;
      case "width":
      case "height":
      case "align":
      case "vspace": 
      case "hspace":
      case "class":
      case "title":
      case "accesskey":
      case "name":
      case "tabindex":
        ret.embedAttrs[args[i]] = ret.objAttrs[args[i]] = args[i+1];
        break;
      default:
        ret.embedAttrs[args[i]] = ret.params[args[i]] = args[i+1];
    }
  }
  ret.objAttrs["classid"] = classid;
  if (mimeType) ret.embedAttrs["type"] = mimeType;
  return ret;
}
;
jQuery.autocomplete = function(input, options) {
	// Create a link to self
	var me = this;

	// Create jQuery object for input element
	var $input = $(input).attr("autocomplete", "off");

	// Apply inputClass if necessary
	if (options.inputClass) $input.addClass(options.inputClass);

	// Create results
	var results = document.createElement("div");
	// Create jQuery object for results
	var $results = $(results);
	$results.hide().addClass(options.resultsClass).css("position", "absolute");
	if( options.width > 0 ) $results.css("width", options.width);

	// Add to body element
	$("body").append(results);

	input.autocompleter = me;

	var timeout = null;
	var prev = "";
	var active = -1;
	var cache = {};
	var keyb = false;
	var hasFocus = false;
	var lastKeyPressCode = null;

	// flush cache
	function flushCache(){
		cache = {};
		cache.data = {};
		cache.length = 0;
	};

	// flush cache
	flushCache();

	// if there is a data array supplied
	if( options.data != null ){
		var sFirstChar = "", stMatchSets = {}, row = [];

		// no url was specified, we need to adjust the cache length to make sure it fits the local data store
		if( typeof options.url != "string" ) options.cacheLength = 1;

		// loop through the array and create a lookup structure
		for( var i=0; i < options.data.length; i++ ){
			// if row is a string, make an array otherwise just reference the array
			row = ((typeof options.data[i] == "string") ? [options.data[i]] : options.data[i]);

			// if the length is zero, don't add to list
			if( row[0].length > 0 ){
				// get the first character
				sFirstChar = row[0].substring(0, 1).toLowerCase();
				// if no lookup array for this character exists, look it up now
				if( !stMatchSets[sFirstChar] ) stMatchSets[sFirstChar] = [];
				// if the match is a string
				stMatchSets[sFirstChar].push(row);
			}
		}

		// add the data items to the cache
		for( var k in stMatchSets ){
			// increase the cache size
			options.cacheLength++;
			// add to the cache
			addToCache(k, stMatchSets[k]);
		}
	}

	$input
	.keydown(function(e) {
		// track last key pressed
		lastKeyPressCode = e.keyCode;
		switch(e.keyCode) {
			case 38: // up
				e.preventDefault();
				moveSelect(-1);
				break;
			case 40: // down
				e.preventDefault();
				moveSelect(1);
				break;
			case 9:  // tab
			case 13: // return
				if( selectCurrent() ){
					// make sure to blur off the current field
					$input.get(0).blur();
					e.preventDefault();
				}
				break;
			default:
				active = -1;
				if (timeout) clearTimeout(timeout);
				timeout = setTimeout(function(){onChange();}, options.delay);
				break;
		}
	})
	.focus(function(){
		// track whether the field has focus, we shouldn't process any results if the field no longer has focus
		hasFocus = true;
	})
	.blur(function() {
		// track whether the field has focus
		hasFocus = false;
		hideResults();
	});

	hideResultsNow();

	function onChange() {
		// ignore if the following keys are pressed: [del] [shift] [capslock]
		if( lastKeyPressCode == 46 || (lastKeyPressCode > 8 && lastKeyPressCode < 32) ) return $results.hide();
		var v = $input.val();
		if (v == prev) return;
		prev = v;
		if (v.length >= options.minChars) {
			$input.addClass(options.loadingClass);
			requestData(v);
		} else {
			$input.removeClass(options.loadingClass);
			$results.hide();
		}
	};

 	function moveSelect(step) {

		var lis = $("li", results);
		if (!lis) return;

		active += step;

		if (active < 0) {
			active = 0;
		} else if (active >= lis.size()) {
			active = lis.size() - 1;
		}

		lis.removeClass("ac_over");

		$(lis[active]).addClass("ac_over");

		// Weird behaviour in IE
		// if (lis[active] && lis[active].scrollIntoView) {
		// 	lis[active].scrollIntoView(false);
		// }

	};

	function selectCurrent() {
		var li = $("li.ac_over", results)[0];
		if (!li) {
			var $li = $("li", results);
			if (options.selectOnly) {
				if ($li.length == 1) li = $li[0];
			} else if (options.selectFirst) {
				li = $li[0];
			}
		}
		if (li) {
			selectItem(li);
			return true;
		} else {
			return false;
		}
	};

	function selectItem(li) {
		if (!li) {
			li = document.createElement("li");
			li.extra = [];
			li.selectValue = "";
		}
		var v = $.trim(li.selectValue ? li.selectValue : li.innerHTML);
		input.lastSelected = v;
		prev = v;
		$results.html("");
		$input.val(v);
		hideResultsNow();
		if (options.onItemSelect) setTimeout(function() { options.onItemSelect(li) }, 1);
	};

	// selects a portion of the input string
	function createSelection(start, end){
		// get a reference to the input element
		var field = $input.get(0);
		if( field.createTextRange ){
			var selRange = field.createTextRange();
			selRange.collapse(true);
			selRange.moveStart("character", start);
			selRange.moveEnd("character", end);
			selRange.select();
		} else if( field.setSelectionRange ){
			field.setSelectionRange(start, end);
		} else {
			if( field.selectionStart ){
				field.selectionStart = start;
				field.selectionEnd = end;
			}
		}
		field.focus();
	};

	// fills in the input box w/the first match (assumed to be the best match)
	function autoFill(sValue){
		// if the last user key pressed was backspace, don't autofill
		if( lastKeyPressCode != 8 ){
			// fill in the value (keep the case the user has typed)
			$input.val($input.val() + sValue.substring(prev.length));
			// select the portion of the value not typed by the user (so the next character will erase)
			createSelection(prev.length, sValue.length);
		}
	};

	function showResults() {
		// get the position of the input field right now (in case the DOM is shifted)
		var pos = findPos(input);
		// either use the specified width, or autocalculate based on form element
		var iWidth = (options.width > 0) ? options.width : $input.width();
		// reposition
		$results.css({
			width: parseInt(iWidth) + "px",
			top: (pos.y + input.offsetHeight) + "px",
			left: pos.x + "px"
		}).show();
	};

	function hideResults() {
		if (timeout) clearTimeout(timeout);
		timeout = setTimeout(hideResultsNow, 200);
	};

	function hideResultsNow() {
		if (timeout) clearTimeout(timeout);
		$input.removeClass(options.loadingClass);
		if ($results.is(":visible")) {
			$results.hide();
		}
		if (options.mustMatch) {
			var v = $input.val();
			if (v != input.lastSelected) {
				selectItem(null);
			}
		}
	};

	function receiveData(q, data) {
		if (data) {
			$input.removeClass(options.loadingClass);
			results.innerHTML = "";

			// if the field no longer has focus or if there are no matches, do not display the drop down
			if( !hasFocus || data.length == 0 ) return hideResultsNow();

			if ($.browser.msie) {
				// we put a styled iframe behind the calendar so HTML SELECT elements don't show through
				$results.append(document.createElement('iframe'));
			}
			results.appendChild(dataToDom(data));
			// autofill in the complete box w/the first match as long as the user hasn't entered in more data
			if( options.autoFill && ($input.val().toLowerCase() == q.toLowerCase()) ) autoFill(data[0][0]);
			showResults();
		} else {
			hideResultsNow();
		}
	};

	function parseData(data) {
		if (!data) return null;
		var parsed = [];
		var rows = data.split(options.lineSeparator);
		for (var i=0; i < rows.length; i++) {
			var row = $.trim(rows[i]);
			if (row) {
				parsed[parsed.length] = row.split(options.cellSeparator);
			}
		}
		return parsed;
	};

	function dataToDom(data) {
		var ul = document.createElement("ul");
		var num = data.length;

		// limited results to a max number
		if( (options.maxItemsToShow > 0) && (options.maxItemsToShow < num) ) num = options.maxItemsToShow;

		for (var i=0; i < num; i++) {
			var row = data[i];
			if (!row) continue;
			var li = document.createElement("li");
			if (options.formatItem) {
				li.innerHTML = options.formatItem(row, i, num);
				li.selectValue = row[0];
			} else {
				li.innerHTML = row[0];
				li.selectValue = row[0];
			}
			var extra = null;
			if (row.length > 1) {
				extra = [];
				for (var j=1; j < row.length; j++) {
					extra[extra.length] = row[j];
				}
			}
			li.extra = extra;
			ul.appendChild(li);
			$(li).hover(
				function() { $("li", ul).removeClass("ac_over"); $(this).addClass("ac_over"); active = $("li", ul).indexOf($(this).get(0)); },
				function() { $(this).removeClass("ac_over"); }
			).click(function(e) { e.preventDefault(); e.stopPropagation(); selectItem(this) });
		}
		return ul;
	};

	function requestData(q) {
		if (!options.matchCase) q = q.toLowerCase();
		var data = options.cacheLength ? loadFromCache(q) : null;
		// recieve the cached data
		if (data) {
			receiveData(q, data);
		// if an AJAX url has been supplied, try loading the data now
		} else if( (typeof options.url == "string") && (options.url.length > 0) ){
			$.get(makeUrl(q), function(data) {
				data = parseData(data);
				addToCache(q, data);
				receiveData(q, data);
			});
		// if there's been no data found, remove the loading class
		} else {
			$input.removeClass(options.loadingClass);
		}
	};

	function makeUrl(q) {
		var url = options.url + "?q=" + encodeURI(q);
		for (var i in options.extraParams) {
			url += "&" + i + "=" + encodeURI(options.extraParams[i]);
		}
		return url;
	};

	function loadFromCache(q) {
		if (!q) return null;
		if (cache.data[q]) return cache.data[q];
		if (options.matchSubset) {
			for (var i = q.length - 1; i >= options.minChars; i--) {
				var qs = q.substr(0, i);
				var c = cache.data[qs];
				if (c) {
					var csub = [];
					for (var j = 0; j < c.length; j++) {
						var x = c[j];
						var x0 = x[0];
						if (matchSubset(x0, q)) {
							csub[csub.length] = x;
						}
					}
					return csub;
				}
			}
		}
		return null;
	};

	function matchSubset(s, sub) {
		if (!options.matchCase) s = s.toLowerCase();
		var i = s.indexOf(sub);
		if (i == -1) return false;
		return i == 0 || options.matchContains;
	};

	this.flushCache = function() {
		flushCache();
	};

	this.setExtraParams = function(p) {
		options.extraParams = p;
	};

	this.findValue = function(){
		var q = $input.val();

		if (!options.matchCase) q = q.toLowerCase();
		var data = options.cacheLength ? loadFromCache(q) : null;
		if (data) {
			findValueCallback(q, data);
		} else if( (typeof options.url == "string") && (options.url.length > 0) ){
			$.get(makeUrl(q), function(data) {
				data = parseData(data)
				addToCache(q, data);
				findValueCallback(q, data);
			});
		} else {
			// no matches
			findValueCallback(q, null);
		}
	}

	function findValueCallback(q, data){
		if (data) $input.removeClass(options.loadingClass);

		var num = (data) ? data.length : 0;
		var li = null;

		for (var i=0; i < num; i++) {
			var row = data[i];

			if( row[0].toLowerCase() == q.toLowerCase() ){
				li = document.createElement("li");
				if (options.formatItem) {
					li.innerHTML = options.formatItem(row, i, num);
					li.selectValue = row[0];
				} else {
					li.innerHTML = row[0];
					li.selectValue = row[0];
				}
				var extra = null;
				if( row.length > 1 ){
					extra = [];
					for (var j=1; j < row.length; j++) {
						extra[extra.length] = row[j];
					}
				}
				li.extra = extra;
			}
		}

		if( options.onFindValue ) setTimeout(function() { options.onFindValue(li) }, 1);
	}

	function addToCache(q, data) {
		if (!data || !q || !options.cacheLength) return;
		if (!cache.length || cache.length > options.cacheLength) {
			flushCache();
			cache.length++;
		} else if (!cache[q]) {
			cache.length++;
		}
		cache.data[q] = data;
	};

	function findPos(obj) {
		var curleft = obj.offsetLeft || 0;
		var curtop = obj.offsetTop || 0;
		while (obj = obj.offsetParent) {
			curleft += obj.offsetLeft
			curtop += obj.offsetTop
		}
		return {x:curleft,y:curtop};
	}
}

jQuery.fn.autocomplete = function(url, options, data) {
	// Make sure options exists
	options = options || {};
	// Set url as option
	options.url = url;
	// set some bulk local data
	options.data = ((typeof data == "object") && (data.constructor == Array)) ? data : null;

	// Set default values for required options
	options.inputClass = options.inputClass || "ac_input";
	options.resultsClass = options.resultsClass || "ac_results";
	options.lineSeparator = options.lineSeparator || "\n";
	options.cellSeparator = options.cellSeparator || "|";
	options.minChars = options.minChars || 1;
	options.delay = options.delay || 400;
	options.matchCase = options.matchCase || 0;
	options.matchSubset = options.matchSubset || 1;
	options.matchContains = options.matchContains || 0;
	options.cacheLength = options.cacheLength || 1;
	options.mustMatch = options.mustMatch || 0;
	options.extraParams = options.extraParams || {};
	options.loadingClass = options.loadingClass || "ac_loading";
	options.selectFirst = options.selectFirst || false;
	options.selectOnly = options.selectOnly || false;
	options.maxItemsToShow = options.maxItemsToShow || -1;
	options.autoFill = options.autoFill || false;
	options.width = parseInt(options.width, 10) || 0;

	this.each(function() {
		var input = this;
		new jQuery.autocomplete(input, options);
	});

	// Don't break the chain
	return this;
}

jQuery.fn.autocompleteArray = function(data, options) {
	return this.autocomplete(null, options, data);
}

jQuery.fn.indexOf = function(e){
	for( var i=0; i<this.length; i++ ){
		if( this[i] == e ) return i;
	}
	return -1;
};
;
